Public Class AdjustDailyTemp
    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Form Input/Output Variables
    Private AdjustDailyTemp_DailyTempInput As String
    Private AdjustDailyTemp_MonthlyTempInput As String
    Private AdjustDailyTemp_Output As String
    Private AdjustDailyTemp_OutputAdjustments As String
    'Initialize Global Form Variables
    Private AdjustDailyTemp_DailyTempRowStart As Integer
    Private AdjustDailyTemp_MonthlyTempRowStart As Integer
    Private AdjustDailyTemp_Header As String
    Private AdjustDailyTemp_Interpolate As Integer
    Private AdjustDailyTemp_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub AdjustDailyTemp_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Adjust Daily Temperature Data Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub DailyTempSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DailyTempSelect_Button.Click
        EntryError.SetError(DailyTempFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Daily Temperature Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If DailyTempFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = AdjustDailyTemp_DailyTempInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            AdjustDailyTemp_DailyTempInput = FileSelect.FileName
            DailyTempFileLocation.Text = AdjustDailyTemp_DailyTempInput
            DailyTempFileLocation.SelectionStart = DailyTempFileLocation.Text.Length
        End If
    End Sub

    Private Sub DailyTempDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DailyTempDataRowStart.TextChanged
        EntryError.SetError(DailyTempDataRowStart, "")
        If DailyTempDataRowStart.Text <> "" Then
            Try
                AdjustDailyTemp_DailyTempRowStart = DailyTempDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DailyTempDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub MonthlyTempSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MonthlyTempSelect_Button.Click
        EntryError.SetError(MonthlyTempFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Monthly Temperature Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If MonthlyTempFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = AdjustDailyTemp_MonthlyTempInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            AdjustDailyTemp_MonthlyTempInput = FileSelect.FileName
            MonthlyTempFileLocation.Text = AdjustDailyTemp_MonthlyTempInput
            MonthlyTempFileLocation.SelectionStart = MonthlyTempFileLocation.Text.Length
        End If
    End Sub

    Private Sub MonthlyTempDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MonthlyTempDataRowStart.TextChanged
        EntryError.SetError(MonthlyTempDataRowStart, "")
        If MonthlyTempDataRowStart.Text <> "" Then
            Try
                AdjustDailyTemp_MonthlyTempRowStart = MonthlyTempDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(MonthlyTempDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = AdjustDailyTemp_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            AdjustDailyTemp_Output = FileOutput.FileName
            OutputFileLocation.Text = AdjustDailyTemp_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        AdjustDailyTemp_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If AdjustDailyTemp_DailyTempInput = "" Then
            EntryError.SetError(DailyTempFileLocation, "Daily temperature dataset missing.")
            Exit Sub
        End If
        If DailyTempDataRowStart.Text = "" Then
            EntryError.SetError(DailyTempDataRowStart, "Required daily temperature data information missing.")
            Exit Sub
        End If
        If AdjustDailyTemp_MonthlyTempInput = "" Then
            EntryError.SetError(MonthlyTempFileLocation, "Monthly temperature dataset missing.")
            Exit Sub
        End If
        If MonthlyTempDataRowStart.Text = "" Then
            EntryError.SetError(MonthlyTempDataRowStart, "Required monthly temperature data information missing.")
            Exit Sub
        End If
        If AdjustDailyTemp_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If AdjustDailyTemp_Output = AdjustDailyTemp_DailyTempInput Or AdjustDailyTemp_Output = AdjustDailyTemp_MonthlyTempInput Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        If Interpolate.Checked = True Then
            AdjustDailyTemp_Interpolate = 1
        Else
            AdjustDailyTemp_Interpolate = 0
        End If
        AdjustDailyTemp_Exception = 0
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim AdjustedTempValue As Double
        Dim TempValue As Double
        Dim TempTotal As Double
        Dim DayCount As Integer
        Dim CurrentMonth As Integer
        Dim TimeThrough As Integer
        Dim MissingData As Integer
        Dim i As Integer
        Dim j As Integer
        Dim TotalMonths As Integer
        Dim Date1 As Date
        Dim Date2 As Date
        Dim DayDiff As Integer
        'Initialize Observation Arrays
        Dim DailyTempMonthlyMean(0) As Double
        Dim MonthlyAdjustment(0) As Double
        'Initialize Output Files and Set Headers
        Dim DecimalIndex As Integer
        DecimalIndex = AdjustDailyTemp_Output.IndexOf(".")
        AdjustDailyTemp_OutputAdjustments = AdjustDailyTemp_Output.Insert(DecimalIndex, " Adjustments")
        Dim AdjustDailyTemp_HeaderAdjustments As String = AdjustDailyTemp_Header + " Adjustments"
        FileOpen(1, AdjustDailyTemp_Output, OpenMode.Output)
        PrintLine(1, AdjustDailyTemp_Header)
        PrintLine(1, "")
        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & "Temp")
        FileOpen(2, AdjustDailyTemp_OutputAdjustments, OpenMode.Output)
        PrintLine(2, AdjustDailyTemp_HeaderAdjustments)
        PrintLine(2, "")
        PrintLine(2, "Year" & vbTab & "Month" & vbTab & "Adjustment")
        'Initialize StreamReaders
        Dim DailyTempInFile As New System.IO.StreamReader(AdjustDailyTemp_DailyTempInput)
        Dim DailyTempLine As String
        Dim DailyTempFields As String()
        Dim MonthlyTempInFile As New System.IO.StreamReader(AdjustDailyTemp_MonthlyTempInput)
        Dim MonthlyTempLine As String
        Dim MonthlyTempFields As String()
        'Initialize Progress Bar
        Dim RunProgress As Integer
        Dim RunComplete As Integer
        Dim DailyTempFileComplete As Integer
        Dim MonthlyTempFileComplete As Integer
        Dim Percent As Integer
        RunProgress = 0
        DailyTempFileComplete = 0
        MonthlyTempFileComplete = 0
        While DailyTempInFile.Peek <> -1
            DailyTempLine = DailyTempInFile.ReadLine()
            DailyTempFileComplete = DailyTempFileComplete + 1
        End While
        While MonthlyTempInFile.Peek <> -1
            MonthlyTempLine = MonthlyTempInFile.ReadLine()
            MonthlyTempFileComplete = MonthlyTempFileComplete + 1
        End While
        'Reset StreamReaders
        DailyTempInFile.DiscardBufferedData()
        DailyTempInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        MonthlyTempInFile.DiscardBufferedData()
        MonthlyTempInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set StreamReaders to Data Start
        If AdjustDailyTemp_DailyTempRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To AdjustDailyTemp_DailyTempRowStart - 2
                DailyTempLine = DailyTempInFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            DailyTempFileComplete = DailyTempFileComplete - (AdjustDailyTemp_DailyTempRowStart - 1)
        End If
        If AdjustDailyTemp_MonthlyTempRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To AdjustDailyTemp_MonthlyTempRowStart - 2
                MonthlyTempLine = MonthlyTempInFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            MonthlyTempFileComplete = MonthlyTempFileComplete - (AdjustDailyTemp_MonthlyTempRowStart - 1)
        End If
        'Finalize Progress Bar
        RunComplete = DailyTempFileComplete * 2 + MonthlyTempFileComplete
        'Calculate Monthly Means from Daily Temperature Data
        'Skip Column Headers
        DailyTempLine = DailyTempInFile.ReadLine()
        RunProgress = RunProgress + 1
        Try
            While DailyTempInFile.Peek <> -1
                DailyTempLine = DailyTempInFile.ReadLine()
                DailyTempFields = DailyTempLine.Split(ControlChars.Tab)
                Year = DailyTempFields(0)
                Month = DailyTempFields(1)
                Day = DailyTempFields(2)
                TempValue = DailyTempFields(3)
                If TempValue = -99 Then
                    MissingData = 1
                End If
                Select Case TimeThrough
                    Case 0
                        TimeThrough = 1
                        CurrentMonth = Month
                        TempTotal = TempValue
                        DayCount = 1
                    Case 1
                        If CurrentMonth <> Month Then
                            'New Month...Calculate Monthly Mean and Save to Array
                            If MissingData = 0 Then
                                DailyTempMonthlyMean(i) = TempTotal / DayCount
                            Else
                                DailyTempMonthlyMean(i) = -99
                            End If
                            'Reset for Next Month
                            i = i + 1
                            ReDim Preserve DailyTempMonthlyMean(i)
                            CurrentMonth = Month
                            TempTotal = TempValue
                            DayCount = 1
                            MissingData = 0
                        Else
                            TempTotal = TempTotal + TempValue
                            DayCount = DayCount + 1
                        End If
                End Select
                'Update Progress Bar
                RunProgress = RunProgress + 1
                Percent = (RunProgress / RunComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
            'Calculate Monthly Mean for Final Month and Save to Array
            If MissingData = 0 Then
                DailyTempMonthlyMean(i) = TempTotal / DayCount
            Else
                DailyTempMonthlyMean(i) = -99
            End If
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the daily temperature data at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Daily Temperature Data")
            Else
                MsgBox("Error reading the daily temperature data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Daily Temperature Data")
            End If
            FileClose(1)
            FileClose(2)
            DailyTempInFile.Close()
            MonthlyTempInFile.Close()
            AdjustDailyTemp_Exception = 1
            Exit Sub
        End Try
        'Calculate Monthly Adjustments
        i = 0
        'Skip Column Headers
        MonthlyTempLine = MonthlyTempInFile.ReadLine()
        RunProgress = RunProgress + 1
        Try
            While MonthlyTempInFile.Peek <> -1
                MonthlyTempLine = MonthlyTempInFile.ReadLine()
                MonthlyTempFields = MonthlyTempLine.Split(ControlChars.Tab)
                Year = MonthlyTempFields(0)
                Month = MonthlyTempFields(1)
                TempValue = MonthlyTempFields(2)
                If DailyTempMonthlyMean(i) = -99 Or TempValue = -99 Then
                    MonthlyAdjustment(i) = -99
                Else
                    MonthlyAdjustment(i) = TempValue - DailyTempMonthlyMean(i)
                End If
                'Output Monthly Adjustment
                PrintLine(2, Year & vbTab & Month & vbTab & Math.Round(MonthlyAdjustment(i), 4))
                'Continue Calculation
                i = i + 1
                ReDim Preserve MonthlyAdjustment(i)
                'Update Progress Bar
                RunProgress = RunProgress + 1
                Percent = (RunProgress / RunComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the monthly temperature data at or near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Daily Temperature Data")
            Else
                MsgBox("Error reading the monthly temperature data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Daily Temperature Data")
            End If
            FileClose(1)
            FileClose(2)
            DailyTempInFile.Close()
            MonthlyTempInFile.Close()
            AdjustDailyTemp_Exception = 1
            Exit Sub
        End Try
        TotalMonths = i
        'Reset Daily Temperature StreamReader for Adjustment
        DailyTempInFile.DiscardBufferedData()
        DailyTempInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Reset Daily Temperature StreamReader to Data Start
        If AdjustDailyTemp_DailyTempRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To AdjustDailyTemp_DailyTempRowStart - 2
                DailyTempLine = DailyTempInFile.ReadLine()
            Next Row
        End If
        'Perform Adjustments on the Daily Temperature Data
        TimeThrough = 0
        'Skip Column Headers
        DailyTempLine = DailyTempInFile.ReadLine()
        RunProgress = RunProgress + 1
        While DailyTempInFile.Peek <> -1
            DailyTempLine = DailyTempInFile.ReadLine()
            DailyTempFields = DailyTempLine.Split(ControlChars.Tab)
            Year = DailyTempFields(0)
            Month = DailyTempFields(1)
            Day = DailyTempFields(2)
            TempValue = DailyTempFields(3)
            If TimeThrough = 0 Then
                i = 0
                TimeThrough = 1
                CurrentMonth = Month
            End If
            If Month <> CurrentMonth Then
                'New Month
                i = i + 1
                CurrentMonth = Month
            End If
            Select Case AdjustDailyTemp_Interpolate
                Case 0
                    'No Interpolation
                    If TempValue = -99 Or MonthlyAdjustment(i) = -99 Then
                        AdjustedTempValue = -99
                    Else
                        AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                        AdjustedTempValue = Math.Round(AdjustedTempValue, 1)
                    End If
                Case 1
                    'Interpolate Between Months (Except Start, End, and Missing)
                    If TempValue = -99 Or MonthlyAdjustment(i) = -99 Then
                        AdjustedTempValue = -99
                    Else
                        If i = 0 Then
                            Select Case Day
                                Case Is < 15
                                    'First Month...Unable to Interpolate
                                    AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                Case Is = 15
                                    'No Interpolation Needed
                                    AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                    'Prepare for Interpolation
                                    Date1 = DateValue(Month & "/" & Day & "/" & Year)
                                    Date2 = Date1.AddMonths(1)
                                    DayDiff = DateDiff(DateInterval.Day, Date1, Date2)
                                    DayDiff = DayDiff - 1
                                    j = 1
                                Case Is > 15
                                    'Attempt Interpolation
                                    If MonthlyAdjustment(i + 1) <> -99 Then
                                        'Interpolate
                                        AdjustedTempValue = TempValue + (MonthlyAdjustment(i) + (j / DayDiff) * (MonthlyAdjustment(i + 1) - MonthlyAdjustment(i)))
                                        j = j + 1
                                    Else
                                        'Missing Data...Unable to Interpolate
                                        AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                    End If
                            End Select
                        End If
                        If i <> 0 And i <> TotalMonths - 1 Then
                            Select Case Day
                                Case Is < 15
                                    'Attempt Interpolation
                                    If MonthlyAdjustment(i - 1) <> -99 Then
                                        'Interpolate
                                        AdjustedTempValue = TempValue + (MonthlyAdjustment(i - 1) + (j / DayDiff) * (MonthlyAdjustment(i) - MonthlyAdjustment(i - 1)))
                                        j = j + 1
                                    Else
                                        'Missing Data...Unable to Interpolate
                                        AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                    End If
                                Case Is = 15
                                    'No Interpolation Needed
                                    AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                    'Prepare New Interpolation
                                    Date1 = DateValue(Month & "/" & Day & "/" & Year)
                                    Date2 = Date1.AddMonths(1)
                                    DayDiff = DateDiff(DateInterval.Day, Date1, Date2)
                                    DayDiff = DayDiff - 1
                                    j = 1
                                Case Is > 15
                                    'Attempt Interpolation
                                    If MonthlyAdjustment(i + 1) <> -99 Then
                                        'Interpolate
                                        AdjustedTempValue = TempValue + (MonthlyAdjustment(i) + (j / DayDiff) * (MonthlyAdjustment(i + 1) - MonthlyAdjustment(i)))
                                        j = j + 1
                                    Else
                                        'Missing Data...Unable to Interpolate
                                        AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                    End If
                            End Select
                        End If
                        If i = TotalMonths - 1 Then
                            Select Case Day
                                Case Is < 15
                                    'Attempt Interpolation
                                    If MonthlyAdjustment(i - 1) <> -99 Then
                                        'Interpolate
                                        AdjustedTempValue = TempValue + (MonthlyAdjustment(i - 1) + (j / DayDiff) * (MonthlyAdjustment(i) - MonthlyAdjustment(i - 1)))
                                        j = j + 1
                                    Else
                                        'Missing Data...Unable to Interpolate
                                        AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                    End If
                                Case Is = 15
                                    'No Interpolation Needed
                                    AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                                Case Is > 15
                                    'Last Month...Unable to Interpolate
                                    AdjustedTempValue = TempValue + MonthlyAdjustment(i)
                            End Select
                        End If
                        AdjustedTempValue = Math.Round(AdjustedTempValue, 1)
                    End If
            End Select
            'Output Adjusted Temperature Value
            PrintLine(1, Year & vbTab & Month & vbTab & Day & vbTab & AdjustedTempValue)
            'Update Progress Bar
            RunProgress = RunProgress + 1
            Percent = (RunProgress / RunComplete) * 100
            BackgroundRun.ReportProgress(Percent)
        End While
        'Close Files
        FileClose(1)
        FileClose(2)
        DailyTempInFile.Close()
        MonthlyTempInFile.Close()
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case AdjustDailyTemp_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub AdjustDailyTemp_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        AdjustDailyTempDisp = 0
    End Sub
End Class