Public Class AdjustStationSegments
    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private AdjustStationSegments_HistoricalInput As String
    Private AdjustStationSegments_ModernInput As String
    Private AdjustStationSegments_SunriseSunsetInput As String
    Private AdjustStationSegments_Output As String
    Private AdjustStationSegments_OutputAdjustments As String
    'Initialize Global Form Variables
    Private AdjustStationSegments_HistRowStart As Integer
    Private AdjustStationSegments_HistColumn1 As Integer
    Private AdjustStationSegments_HistColumn2 As Integer
    Private AdjustStationSegments_HistColumn3 As Integer
    Private AdjustStationSegments_ModRowStart As Integer
    Private AdjustStationSegments_TopHourCorrection As Integer = 20
    Private AdjustStationSegments_SolarTime As Double
    Private AdjustStationSegments_MinMax As Integer
    Private AdjustStationSegments_HourlyTwo As Integer
    Private AdjustStationSegments_HistoricalDegF As Integer
    Private AdjustStationSegments_HistoricalDegC As Integer
    Private AdjustStationSegments_ModernDegF As Integer
    Private AdjustStationSegments_ModernDegC As Integer
    Private AdjustStationSegments_SunriseSunsetDataStart As Integer
    Private AdjustStationSegments_AMTime As String
    Private AdjustStationSegments_PMTime As String
    Private AdjustStationSegments_EveTime As String
    Private AdjustStationSegments_Time1 As Integer
    Private AdjustStationSegments_Time2 As Integer
    Private AdjustStationSegments_Time3 As Integer
    Private AdjustStationSegments_Time2Enabled As Integer
    Private AdjustStationSegments_Header As String
    Private AdjustStationSegments_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub AdjustStationSegments_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Adjust Station Segments Tool
        HistoricalDegC.Text = Chr(176) & "C"
        HistoricalDegF.Text = Chr(176) & "F"
        ModernDegC.Text = Chr(176) & "C"
        ModernDegF.Text = Chr(176) & "F"
        Time1.SelectedIndex = 2
        Time2.SelectedIndex = 2
        Time3.SelectedIndex = 3
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub HourlyThree_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HourlyThree.CheckedChanged
        If HourlyThree.Checked = True Then
            Column3.Enabled = True
            If ModernInputGroup.Enabled = False Then
                ModernInputGroup.Enabled = True
            End If
            SecondObPanel.Enabled = False
            If Time2.Enabled = False Then
                Time2.Enabled = True
            End If
            If Time3.Enabled = False Then
                Time3.Enabled = True
            End If
            If Time1.SelectedIndex = 0 Or Time2.SelectedIndex = 6 Then
                'Enable Sunrise/Sunset Input
                SunriseSunsetFileLocation.Enabled = True
                SunriseSunsetBrowse.Enabled = True
                SunriseSunsetDataStart.Enabled = True
            Else
                'Disable Sunrise/Sunset Input
                SunriseSunsetFileLocation.Enabled = False
                SunriseSunsetBrowse.Enabled = False
                SunriseSunsetDataStart.Enabled = False
            End If
        End If
    End Sub

    Private Sub HourlyTwo_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HourlyTwo.CheckedChanged
        If HourlyTwo.Checked = True Then
            Column3.Enabled = False
            If ModernInputGroup.Enabled = False Then
                ModernInputGroup.Enabled = True
            End If
            SecondObPanel.Enabled = True
            If SecondObPM.Checked = True Then
                Time3.Enabled = False
                If Time1.SelectedIndex = 0 Or Time2.SelectedIndex = 6 Then
                    'Enable Sunrise/Sunset Input
                    SunriseSunsetFileLocation.Enabled = True
                    SunriseSunsetBrowse.Enabled = True
                    SunriseSunsetDataStart.Enabled = True
                Else
                    'Disable Sunrise/Sunset Input
                    SunriseSunsetFileLocation.Enabled = False
                    SunriseSunsetBrowse.Enabled = False
                    SunriseSunsetDataStart.Enabled = False
                End If
            End If
            If SecondObEve.Checked = True Then
                Time2.Enabled = False
                If Time1.SelectedIndex = 0 Then
                    'Enable Sunrise/Sunset Input
                    SunriseSunsetFileLocation.Enabled = True
                    SunriseSunsetBrowse.Enabled = True
                    SunriseSunsetDataStart.Enabled = True
                Else
                    'Disable Sunrise/Sunset Input
                    SunriseSunsetFileLocation.Enabled = False
                    SunriseSunsetBrowse.Enabled = False
                    SunriseSunsetDataStart.Enabled = False
                End If
            End If
        End If
    End Sub

    Private Sub HistoricalBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HistoricalBrowse.Click
        EntryError.SetError(HistoricalFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Historical Temperature File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If HistoricalFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = AdjustStationSegments_HistoricalInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            AdjustStationSegments_HistoricalInput = FileSelect.FileName
            HistoricalFileLocation.Text = AdjustStationSegments_HistoricalInput
            HistoricalFileLocation.SelectionStart = HistoricalFileLocation.Text.Length
        End If
    End Sub

    Private Sub HistDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HistDataRowStart.TextChanged
        EntryError.SetError(HistDataRowStart, "")
        If HistDataRowStart.Text <> "" Then
            Try
                AdjustStationSegments_HistRowStart = HistDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(HistDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Column1_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Column1.TextChanged
        EntryError.SetError(Column1, "")
        If Column1.Text <> "" Then
            Try
                AdjustStationSegments_HistColumn1 = Column1.Text
            Catch ex As Exception
                EntryError.SetError(Column1, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Column2_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Column2.TextChanged
        EntryError.SetError(Column2, "")
        If Column2.Text <> "" Then
            Try
                AdjustStationSegments_HistColumn2 = Column2.Text
            Catch ex As Exception
                EntryError.SetError(Column2, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Column3_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Column3.TextChanged
        EntryError.SetError(Column3, "")
        If Column3.Text <> "" Then
            Try
                AdjustStationSegments_HistColumn3 = Column3.Text
            Catch ex As Exception
                EntryError.SetError(Column3, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ModernBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ModernBrowse.Click
        EntryError.SetError(ModernFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Modern Temperature File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If ModernFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = AdjustStationSegments_ModernInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            AdjustStationSegments_ModernInput = FileSelect.FileName
            ModernFileLocation.Text = AdjustStationSegments_ModernInput
            ModernFileLocation.SelectionStart = ModernFileLocation.Text.Length
        End If
    End Sub

    Private Sub ModDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ModDataRowStart.TextChanged
        EntryError.SetError(ModDataRowStart, "")
        If ModDataRowStart.Text <> "" Then
            Try
                AdjustStationSegments_ModRowStart = ModDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(ModDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TopHourCorrection_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TopHourCorrection.TextChanged
        EntryError.SetError(TopHourCorrection, "")
        If TopHourCorrection.Text <> "" Then
            Try
                AdjustStationSegments_TopHourCorrection = TopHourCorrection.Text
            Catch ex As Exception
                EntryError.SetError(TopHourCorrection, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub SolarTimeCorrection_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SolarTimeCorrection.TextChanged
        EntryError.SetError(SolarTimeCorrection, "")
        If SolarTimeCorrection.Text <> "" Then
            Try
                AdjustStationSegments_SolarTime = SolarTimeCorrection.Text
            Catch ex As Exception
                EntryError.SetError(SolarTimeCorrection, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub SecondObPM_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SecondObPM.CheckedChanged
        If SecondObPM.Checked = True And SecondObPanel.Enabled = True Then
            Time2.Enabled = True
            Time3.Enabled = False
            If Time1.SelectedIndex = 0 Or Time2.SelectedIndex = 6 Then
                'Enable Sunrise/Sunset Input
                SunriseSunsetFileLocation.Enabled = True
                SunriseSunsetBrowse.Enabled = True
                SunriseSunsetDataStart.Enabled = True
            Else
                'Disable Sunrise/Sunset Input
                SunriseSunsetFileLocation.Enabled = False
                SunriseSunsetBrowse.Enabled = False
                SunriseSunsetDataStart.Enabled = False
            End If
        End If
    End Sub

    Private Sub SecondObEve_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SecondObEve.CheckedChanged
        If SecondObEve.Checked = True Then
            Time2.Enabled = False
            Time3.Enabled = True
            If Time1.SelectedIndex = 0 Then
                'Enable Sunrise/Sunset Input
                SunriseSunsetFileLocation.Enabled = True
                SunriseSunsetBrowse.Enabled = True
                SunriseSunsetDataStart.Enabled = True
            Else
                'Disable Sunrise/Sunset Input
                SunriseSunsetFileLocation.Enabled = False
                SunriseSunsetBrowse.Enabled = False
                SunriseSunsetDataStart.Enabled = False
            End If
        End If
    End Sub

    Private Sub Time1_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Time1.SelectedIndexChanged
        If Time1.SelectedIndex = 0 Then
            'Enable Sunrise/Sunset Input
            SunriseSunsetFileLocation.Enabled = True
            SunriseSunsetBrowse.Enabled = True
            SunriseSunsetDataStart.Enabled = True
        End If
        If Time1.SelectedIndex <> 0 And Time2.SelectedIndex <> 6 Then
            'Disable Sunrise/Sunset Input
            SunriseSunsetFileLocation.Enabled = False
            SunriseSunsetBrowse.Enabled = False
            SunriseSunsetDataStart.Enabled = False
        End If
    End Sub

    Private Sub Time2_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Time2.SelectedIndexChanged
        If Time2.SelectedIndex = 6 Then
            'Enable Sunrise/Sunset Input
            SunriseSunsetFileLocation.Enabled = True
            SunriseSunsetBrowse.Enabled = True
            SunriseSunsetDataStart.Enabled = True
        End If
        If Time2.SelectedIndex <> 6 And Time1.SelectedIndex <> 0 Then
            'Disable Sunrise/Sunset Input
            SunriseSunsetFileLocation.Enabled = False
            SunriseSunsetBrowse.Enabled = False
            SunriseSunsetDataStart.Enabled = False
        End If
    End Sub

    Private Sub SunriseSunsetBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SunriseSunsetBrowse.Click
        EntryError.SetError(SunriseSunsetFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Sunrise/Sunset File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If SunriseSunsetFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = AdjustStationSegments_SunriseSunsetInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            AdjustStationSegments_SunriseSunsetInput = FileSelect.FileName
            SunriseSunsetFileLocation.Text = AdjustStationSegments_SunriseSunsetInput
            SunriseSunsetFileLocation.SelectionStart = SunriseSunsetFileLocation.Text.Length
        End If
    End Sub

    Private Sub SunriseSunsetDataStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SunriseSunsetDataStart.TextChanged
        EntryError.SetError(SunriseSunsetDataStart, "")
        If SunriseSunsetDataStart.Text <> "" Then
            Try
                AdjustStationSegments_SunriseSunsetDataStart = SunriseSunsetDataStart.Text
            Catch ex As Exception
                EntryError.SetError(SunriseSunsetDataStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub OutputBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OutputBrowse.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = AdjustStationSegments_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            AdjustStationSegments_Output = FileOutput.FileName
            OutputFileLocation.Text = AdjustStationSegments_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        AdjustStationSegments_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If AdjustStationSegments_HistoricalInput = "" Then
            EntryError.SetError(HistoricalFileLocation, "Input historical dataset missing.")
            Exit Sub
        End If
        If HistDataRowStart.Text = "" Then
            EntryError.SetError(HistDataRowStart, "Required historical data information missing.")
            Exit Sub
        End If
        If Column1.Text = "" Then
            EntryError.SetError(Column1, "Required historical data information missing.")
            Exit Sub
        End If
        If Column2.Text = "" Then
            EntryError.SetError(Column2, "Required historical data information missing.")
            Exit Sub
        End If
        If Column3.Enabled = True Then
            If Column3.Text = "" Then
                EntryError.SetError(Column3, "Required historical data information missing.")
                Exit Sub
            End If
        End If
        If AdjustStationSegments_ModernInput = "" Then
            EntryError.SetError(ModernFileLocation, "Input modern dataset missing.")
            Exit Sub
        End If
        If ModDataRowStart.Text = "" Then
            EntryError.SetError(ModDataRowStart, "Required modern data information missing.")
            Exit Sub
        End If
        If TopHourCorrection.Text = "" Then
            EntryError.SetError(TopHourCorrection, "Required top of the hour correction information missing.")
            Exit Sub
        End If
        If SolarTimeCorrection.Text = "" Then
            EntryError.SetError(SolarTimeCorrection, "Required solar time correction information missing.")
            Exit Sub
        End If
        If Time1.SelectedIndex = 0 Or (Time2.SelectedIndex = 6 And Time2.Enabled = True) Then
            If AdjustStationSegments_SunriseSunsetInput = "" Then
                EntryError.SetError(SunriseSunsetFileLocation, "Input sunrise/sunset dataset missing.")
                Exit Sub
            End If
            If HistDataRowStart.Text = "" Then
                EntryError.SetError(HistDataRowStart, "Required sunrise/sunset data information missing.")
                Exit Sub
            End If
        End If
        If AdjustStationSegments_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If AdjustStationSegments_Output = AdjustStationSegments_HistoricalInput Or AdjustStationSegments_Output = AdjustStationSegments_ModernInput Or AdjustStationSegments_Output = AdjustStationSegments_SunriseSunsetInput Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        AdjustStationSegments_Exception = 0
        If HourlyTwo.Checked = True Then
            AdjustStationSegments_HourlyTwo = 1
        Else
            AdjustStationSegments_HourlyTwo = 0
        End If
        If HistoricalDegF.Checked = True Then
            AdjustStationSegments_HistoricalDegF = 1
        Else
            AdjustStationSegments_HistoricalDegF = 0
        End If
        If HistoricalDegC.Checked = True Then
            AdjustStationSegments_HistoricalDegC = 1
        Else
            AdjustStationSegments_HistoricalDegC = 0
        End If
        If ModernDegF.Checked = True Then
            AdjustStationSegments_ModernDegF = 1
        Else
            AdjustStationSegments_ModernDegF = 0
        End If
        If ModernDegC.Checked = True Then
            AdjustStationSegments_ModernDegC = 1
        Else
            AdjustStationSegments_ModernDegC = 0
        End If
        AdjustStationSegments_AMTime = Time1.SelectedItem
        AdjustStationSegments_PMTime = Time2.SelectedItem
        AdjustStationSegments_EveTime = Time3.SelectedItem
        Select Case Time1.SelectedIndex
            Case 0
                AdjustStationSegments_Time1 = 0
            Case 1
                AdjustStationSegments_Time1 = 1
            Case 2
                AdjustStationSegments_Time1 = 2
            Case 3
                AdjustStationSegments_Time1 = 3
            Case 4
                AdjustStationSegments_Time1 = 4
            Case 5
                AdjustStationSegments_Time1 = 5
            Case 6
                AdjustStationSegments_Time1 = 6
            Case 7
                AdjustStationSegments_Time1 = 7
        End Select
        Select Case Time2.SelectedIndex
            Case 0
                AdjustStationSegments_Time2 = 0
            Case 1
                AdjustStationSegments_Time2 = 1
            Case 2
                AdjustStationSegments_Time2 = 2
            Case 3
                AdjustStationSegments_Time2 = 3
            Case 4
                AdjustStationSegments_Time2 = 4
            Case 5
                AdjustStationSegments_Time2 = 5
            Case 6
                AdjustStationSegments_Time2 = 6
            Case 7
                AdjustStationSegments_Time2 = 7
        End Select
        Select Case Time3.SelectedIndex
            Case 0
                AdjustStationSegments_Time3 = 0
            Case 1
                AdjustStationSegments_Time3 = 1
            Case 2
                AdjustStationSegments_Time3 = 2
            Case 3
                AdjustStationSegments_Time3 = 3
            Case 4
                AdjustStationSegments_Time3 = 4
            Case 5
                AdjustStationSegments_Time3 = 5
        End Select
        If Time2.Enabled = True Then
            AdjustStationSegments_Time2Enabled = 1
        Else
            AdjustStationSegments_Time2Enabled = 0
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim SunriseHour(0) As Integer
        Dim SunriseMin(0) As Integer
        Dim SunsetHour(0) As Integer
        Dim SunsetMin(0) As Integer
        Dim i As Integer
        i = 0
        Dim TempAdjust As Double
        Dim TopHourTemp As Double
        Dim TopHourTemp2 As Double
        Dim SunriseTemp As Double
        Dim SunsetTemp As Double
        Dim AMTemp As Double
        Dim PMTemp As Double
        Dim EveTemp As Double
        Dim MaxTemp As Double
        Dim MinTemp As Double
        Dim AdjAMTemp As Double
        Dim AdjPMTemp As Double
        Dim AdjEveTemp As Double
        'Initialize Observation Arrays
        Dim AMTempTotal(11) As Double
        Dim PMTempTotal(11) As Double
        Dim EveTempTotal(11) As Double
        Dim MeanTempTotal(11) As Double
        Dim Count(11) As Double
        Dim AMTempModernAnomaly(11) As Double
        Dim PMTempModernAnomaly(11) As Double
        Dim EveTempModernAnomaly(11) As Double
        Dim AMTempHistoricalAnomaly(11) As Double
        Dim PMTempHistoricalAnomaly(11) As Double
        Dim EveTempHistoricalAnomaly(11) As Double
        Dim AMTempAdjustment(11) As Double
        Dim PMTempAdjustment(11) As Double
        Dim EveTempAdjustment(11) As Double
        Dim Response As MsgBoxResult
        'Initialize Output Files and Set Headers
        Dim DecimalIndex As Integer
        DecimalIndex = AdjustStationSegments_Output.IndexOf(".")
        AdjustStationSegments_OutputAdjustments = AdjustStationSegments_Output.Insert(DecimalIndex, " Adjustments")
        FileOpen(1, AdjustStationSegments_Output, OpenMode.Output)
        PrintLine(1, AdjustStationSegments_Header)
        PrintLine(1, "")
        FileOpen(2, AdjustStationSegments_OutputAdjustments, OpenMode.Output)
        Dim AdjustStationSegments_HeaderAdjustments As String = AdjustStationSegments_Header + " Adjustments"
        PrintLine(2, AdjustStationSegments_HeaderAdjustments)
        PrintLine(2, "")
        PrintLine(2, "Month" & vbTab & "Variable" & vbTab & "Adjustment")
        'Initialize StreamReaders
        Dim HistoricalInFile As New System.IO.StreamReader(AdjustStationSegments_HistoricalInput)
        Dim HistoricalLine As String
        Dim HistoricalFields As String()
        Dim ModernInFile As New System.IO.StreamReader(AdjustStationSegments_ModernInput)
        Dim ModernLine As String
        Dim ModernFields As String()
        If AdjustStationSegments_Time1 = 0 Or (AdjustStationSegments_Time2 = 6 And AdjustStationSegments_Time2Enabled = 1) Then
            Dim SunriseSunsetInFile As New System.IO.StreamReader(AdjustStationSegments_SunriseSunsetInput)
            Dim SunriseSunsetLine As String
            Dim SunriseSunsetFields As String()
            'Set SunriseSunset StreamReader to Data Start
            If AdjustStationSegments_SunriseSunsetDataStart <> 1 Then
                'Header Exists so Move LineInput Down
                Dim Row As Integer
                For Row = 0 To AdjustStationSegments_SunriseSunsetDataStart - 2
                    SunriseSunsetLine = SunriseSunsetInFile.ReadLine()
                Next Row
            End If
            'Skip Column Headers
            SunriseSunsetLine = SunriseSunsetInFile.ReadLine()
            Try
                While SunriseSunsetInFile.Peek <> -1
                    If i <> 0 Then
                        If AdjustStationSegments_Time1 <> 0 Then
                            'Only Sunset Data
                            ReDim Preserve SunsetHour(i)
                            ReDim Preserve SunsetMin(i)
                        Else
                            If AdjustStationSegments_Time2 = 6 And AdjustStationSegments_Time2Enabled = 1 Then
                                'Both Sunrise and Sunset Data
                                ReDim Preserve SunriseHour(i)
                                ReDim Preserve SunriseMin(i)
                                ReDim Preserve SunsetHour(i)
                                ReDim Preserve SunsetMin(i)
                            Else
                                'Only Sunrise Data
                                ReDim Preserve SunriseHour(i)
                                ReDim Preserve SunriseMin(i)
                            End If
                        End If
                    End If
                    SunriseSunsetLine = SunriseSunsetInFile.ReadLine()
                    SunriseSunsetFields = SunriseSunsetLine.Split(ControlChars.Tab)
                    Year = SunriseSunsetFields(0)
                    Month = SunriseSunsetFields(1)
                    Day = SunriseSunsetFields(2)
                    If AdjustStationSegments_Time1 <> 0 Then
                        'Only Sunset Data
                        SunsetHour(i) = SunriseSunsetFields(3)
                        SunsetMin(i) = SunriseSunsetFields(4)
                    Else
                        If AdjustStationSegments_Time2 = 6 And AdjustStationSegments_Time2Enabled = 1 Then
                            'Both Sunrise and Sunset Data
                            SunriseHour(i) = SunriseSunsetFields(3)
                            SunriseMin(i) = SunriseSunsetFields(4)
                            SunsetHour(i) = SunriseSunsetFields(5)
                            SunsetMin(i) = SunriseSunsetFields(6)
                        Else
                            'Only Sunrise Data
                            SunriseHour(i) = SunriseSunsetFields(3)
                            SunriseMin(i) = SunriseSunsetFields(4)
                        End If
                    End If
                    i = i + 1
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the sunrise/sunset data file at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Station Segments")
                Else
                    MsgBox("Error reading the sunrise/sunset data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Station Segments")
                End If
                HistoricalInFile.Close()
                ModernInFile.Close()
                SunriseSunsetInFile.Close()
                FileClose(1)
                FileClose(2)
                AdjustStationSegments_Exception = 1
                Exit Sub
            End Try
            SunriseSunsetInFile.Close()
        End If
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim HistoricalComplete As Integer
        Dim ModernComplete As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        HistoricalComplete = 0
        ModernComplete = 0
        FileComplete = 0
        While HistoricalInFile.Peek <> -1
            HistoricalLine = HistoricalInFile.ReadLine()
            HistoricalComplete = HistoricalComplete + 1
        End While
        While ModernInFile.Peek <> -1
            ModernLine = ModernInFile.ReadLine()
            ModernComplete = ModernComplete + 1
        End While
        'Reset StreamReaders
        HistoricalInFile.DiscardBufferedData()
        HistoricalInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        ModernInFile.DiscardBufferedData()
        ModernInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set Historical StreamReader to Data Start
        If AdjustStationSegments_HistRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To AdjustStationSegments_HistRowStart - 2
                HistoricalLine = HistoricalInFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            HistoricalComplete = HistoricalComplete - (AdjustStationSegments_HistRowStart - 1)
        End If
        'Set Modern StreamReader to Data Start
        If AdjustStationSegments_ModRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To AdjustStationSegments_ModRowStart - 2
                ModernLine = ModernInFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            ModernComplete = ModernComplete - (AdjustStationSegments_ModRowStart - 1)
        End If
        FileComplete = HistoricalComplete * 2 + ModernComplete
        'Find Monthly Modern Adjustments
        i = 0
        Year = 0
        'Process Modern Daily Data
        'Skip Header Columns
        ModernLine = ModernInFile.ReadLine()
        FileProgress = FileProgress + 1
        Try
            While ModernInFile.Peek <> -1
                ModernLine = ModernInFile.ReadLine()
                ModernFields = ModernLine.Split(ControlChars.Tab)
                Year = ModernFields(0)
                Month = ModernFields(1)
                Day = ModernFields(2)
                MinTemp = ModernFields(28)
                MaxTemp = ModernFields(27)
                AMTemp = -99
                PMTemp = -99
                EveTemp = -99
                If AdjustStationSegments_Time1 = 0 Then
                    If SunriseHour(i) <> -99 And SunriseMin(i) <> -99 Then
                        If ModernFields(SunriseHour(i) + 1) <> -99 And ModernFields(SunriseHour(i) + 2) <> -99 And ModernFields(SunriseHour(i) + 3) <> -99 And ModernFields(SunriseHour(i) + 4) <> -99 Then
                            'Find Sunrise Temperature
                            Select Case AdjustStationSegments_SolarTime
                                Case Is > AdjustStationSegments_TopHourCorrection
                                    'Extrapolate Specified Sunrise Hour to Top of the Hour
                                    TempAdjust = (ModernFields(SunriseHour(i) + 2) - ModernFields(SunriseHour(i) + 1)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                    TopHourTemp = ModernFields(SunriseHour(i) + 2) - TempAdjust
                                    'Extrapolate Specified Sunrise Hour + 1 to Top of the Hour
                                    TempAdjust = (ModernFields(SunriseHour(i) + 3) - ModernFields(SunriseHour(i) + 2)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                    TopHourTemp2 = ModernFields(SunriseHour(i) + 3) - TempAdjust
                                Case Is <= AdjustStationSegments_TopHourCorrection
                                    'Extrapolate Specified Sunrise Hour to Top of the Hour
                                    TempAdjust = (ModernFields(SunriseHour(i) + 3) - ModernFields(SunriseHour(i) + 2)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                    TopHourTemp = ModernFields(SunriseHour(i) + 2) + TempAdjust
                                    'Extrapolate Specified Sunrise Hour + 1 to Top of the Hour
                                    TempAdjust = (ModernFields(SunriseHour(i) + 4) - ModernFields(SunriseHour(i) + 3)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                    TopHourTemp2 = ModernFields(SunriseHour(i) + 3) + TempAdjust
                            End Select
                            'Extrapolate Solar Time Top of the Hour Temp to Sunrise
                            TempAdjust = (TopHourTemp2 - TopHourTemp) * (SunriseMin(i) / 60)
                            SunriseTemp = TopHourTemp + TempAdjust
                            AMTemp = SunriseTemp
                        End If
                    End If
                End If
                If AdjustStationSegments_Time1 = 1 And ModernFields(7) <> -99 And ModernFields(8) <> -99 And ModernFields(9) <> -99 Then
                    'Find 6 a.m. Temperature
                    Select Case AdjustStationSegments_SolarTime
                        Case Is > AdjustStationSegments_TopHourCorrection
                            'Extrapolate 6 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(8) - ModernFields(7)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                            TopHourTemp = ModernFields(8) - TempAdjust
                        Case Is <= AdjustStationSegments_TopHourCorrection
                            'Extrapolate 6 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(9) - ModernFields(8)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                            TopHourTemp = ModernFields(8) + TempAdjust
                    End Select
                    AMTemp = TopHourTemp
                End If
                If AdjustStationSegments_Time1 = 2 And ModernFields(8) <> -99 And ModernFields(9) <> -99 And ModernFields(10) <> -99 Then
                    'Find 7 a.m. Temperature
                    Select Case AdjustStationSegments_SolarTime
                        Case Is > AdjustStationSegments_TopHourCorrection
                            'Extrapolate 7 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(9) - ModernFields(8)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                            TopHourTemp = ModernFields(9) - TempAdjust
                        Case Is <= AdjustStationSegments_TopHourCorrection
                            'Extrapolate 7 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(10) - ModernFields(9)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                            TopHourTemp = ModernFields(9) + TempAdjust
                    End Select
                    AMTemp = TopHourTemp
                End If
                If AdjustStationSegments_Time1 = 3 And ModernFields(9) <> -99 And ModernFields(10) <> -99 And ModernFields(11) <> -99 Then
                    'Find 8 a.m. Temperature
                    Select Case AdjustStationSegments_SolarTime
                        Case Is > AdjustStationSegments_TopHourCorrection
                            'Extrapolate 8 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(10) - ModernFields(9)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                            TopHourTemp = ModernFields(10) - TempAdjust
                        Case Is <= AdjustStationSegments_TopHourCorrection
                            'Extrapolate 8 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(11) - ModernFields(10)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                            TopHourTemp = ModernFields(10) + TempAdjust
                    End Select
                    AMTemp = TopHourTemp
                End If
                If AdjustStationSegments_Time1 = 4 And ModernFields(10) <> -99 And ModernFields(11) <> -99 And ModernFields(12) <> -99 Then
                    'Find 9 a.m. Temperature
                    Select Case AdjustStationSegments_SolarTime
                        Case Is > AdjustStationSegments_TopHourCorrection
                            'Extrapolate 9 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(11) - ModernFields(10)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                            TopHourTemp = ModernFields(11) - TempAdjust
                        Case Is <= AdjustStationSegments_TopHourCorrection
                            'Extrapolate 9 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(12) - ModernFields(11)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                            TopHourTemp = ModernFields(11) + TempAdjust
                    End Select
                    AMTemp = TopHourTemp
                End If
                If AdjustStationSegments_Time1 = 5 And ModernFields(11) <> -99 And ModernFields(12) <> -99 And ModernFields(13) <> -99 Then
                    'Find 10 a.m. Temperature
                    Select Case AdjustStationSegments_SolarTime
                        Case Is > AdjustStationSegments_TopHourCorrection
                            'Extrapolate 10 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(12) - ModernFields(11)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                            TopHourTemp = ModernFields(12) - TempAdjust
                        Case Is <= AdjustStationSegments_TopHourCorrection
                            'Extrapolate 10 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(13) - ModernFields(12)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                            TopHourTemp = ModernFields(12) + TempAdjust
                    End Select
                    AMTemp = TopHourTemp
                End If
                If AdjustStationSegments_Time1 = 6 And ModernFields(12) <> -99 And ModernFields(13) <> -99 And ModernFields(14) <> -99 Then
                    'Find 11 a.m. Temperature
                    Select Case AdjustStationSegments_SolarTime
                        Case Is > AdjustStationSegments_TopHourCorrection
                            'Extrapolate 11 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(13) - ModernFields(12)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                            TopHourTemp = ModernFields(13) - TempAdjust
                        Case Is <= AdjustStationSegments_TopHourCorrection
                            'Extrapolate 11 a.m. Temp to Top of the Hour
                            TempAdjust = (ModernFields(14) - ModernFields(13)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                            TopHourTemp = ModernFields(13) + TempAdjust
                    End Select
                    AMTemp = TopHourTemp
                End If
                If AdjustStationSegments_Time1 = 7 And ModernFields(28) <> -99 Then
                    'Find Minimum Temperature
                    AMTemp = ModernFields(28)
                End If
                If AdjustStationSegments_HourlyTwo = 0 Or (AdjustStationSegments_HourlyTwo = 1 And AdjustStationSegments_Time2Enabled = 1) Then
                    If AdjustStationSegments_Time2 = 0 And ModernFields(13) <> -99 And ModernFields(14) <> -99 And ModernFields(15) <> -99 Then
                        'Find 12 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 12 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(14) - ModernFields(13)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(14) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 12 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(15) - ModernFields(14)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(14) + TempAdjust
                        End Select
                        PMTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time2 = 1 And ModernFields(14) <> -99 And ModernFields(15) <> -99 And ModernFields(16) <> -99 Then
                        'Find 1 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 1 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(15) - ModernFields(14)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(15) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 1 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(16) - ModernFields(15)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(15) + TempAdjust
                        End Select
                        PMTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time2 = 2 And ModernFields(15) <> -99 And ModernFields(16) <> -99 And ModernFields(17) <> -99 Then
                        'Find 2 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 2 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(16) - ModernFields(15)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(16) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 2 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(17) - ModernFields(16)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(16) + TempAdjust
                        End Select
                        PMTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time2 = 3 And ModernFields(16) <> -99 And ModernFields(17) <> -99 And ModernFields(18) <> -99 Then
                        'Find 3 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 3 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(17) - ModernFields(16)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(17) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 3 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(18) - ModernFields(17)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(17) + TempAdjust
                        End Select
                        PMTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time2 = 4 And ModernFields(17) <> -99 And ModernFields(18) <> -99 And ModernFields(19) <> -99 Then
                        'Find 4 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 4 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(18) - ModernFields(17)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(18) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 4 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(19) - ModernFields(18)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(18) + TempAdjust
                        End Select
                        PMTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time2 = 5 And ModernFields(18) <> -99 And ModernFields(19) <> -99 And ModernFields(20) <> -99 Then
                        'Find 5 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 5 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(19) - ModernFields(18)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(19) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 5 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(20) - ModernFields(19)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(19) + TempAdjust
                        End Select
                        PMTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time2 = 6 Then
                        If SunsetHour(i) <> -99 And SunsetMin(i) <> -99 Then
                            If ModernFields(SunsetHour(i) + 1) <> -99 And ModernFields(SunsetHour(i) + 2) <> -99 And ModernFields(SunsetHour(i) + 3) <> -99 And ModernFields(SunsetHour(i) + 4) <> -99 Then
                                'Find Sunset Temperature
                                Select Case AdjustStationSegments_SolarTime
                                    Case Is > AdjustStationSegments_TopHourCorrection
                                        'Extrapolate Specified Sunset Hour to Top of the Hour
                                        TempAdjust = (ModernFields(SunsetHour(i) + 2) - ModernFields(SunsetHour(i) + 1)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                        TopHourTemp = ModernFields(SunsetHour(i) + 2) - TempAdjust
                                        'Extrapolate Specified Sunset Hour + 1 to Top of the Hour
                                        TempAdjust = (ModernFields(SunsetHour(i) + 3) - ModernFields(SunsetHour(i) + 2)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                        TopHourTemp2 = ModernFields(SunsetHour(i) + 3) - TempAdjust
                                    Case Is <= AdjustStationSegments_TopHourCorrection
                                        'Extrapolate Specified Sunset Hour to Top of the Hour
                                        TempAdjust = (ModernFields(SunsetHour(i) + 3) - ModernFields(SunsetHour(i) + 2)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                        TopHourTemp = ModernFields(SunsetHour(i) + 2) + TempAdjust
                                        'Extrapolate Specified Sunset Hour + 1 to Top of the Hour
                                        TempAdjust = (ModernFields(SunsetHour(i) + 4) - ModernFields(SunsetHour(i) + 3)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                        TopHourTemp2 = ModernFields(SunsetHour(i) + 3) + TempAdjust
                                End Select
                                'Extrapolate Solar Time Top of the Hour Temp to Sunset
                                TempAdjust = (TopHourTemp2 - TopHourTemp) * (SunsetMin(i) / 60)
                                SunsetTemp = TopHourTemp + TempAdjust
                                PMTemp = SunsetTemp
                            End If
                        End If
                    End If
                    If AdjustStationSegments_Time2 = 7 And ModernFields(27) <> -99 Then
                        'Find Maximum Temperature
                        PMTemp = ModernFields(27)
                    End If
                End If
                If AdjustStationSegments_HourlyTwo = 0 Or (AdjustStationSegments_HourlyTwo = 1 And AdjustStationSegments_Time2Enabled = 0) Then
                    If AdjustStationSegments_Time3 = 0 And ModernFields(19) <> -99 And ModernFields(20) <> -99 And ModernFields(21) <> -99 Then
                        'Find 6 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 6 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(20) - ModernFields(19)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(20) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 6 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(21) - ModernFields(20)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(20) + TempAdjust
                        End Select
                        EveTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time3 = 1 And ModernFields(20) <> -99 And ModernFields(21) <> -99 And ModernFields(22) <> -99 Then
                        'Find 7 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 7 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(21) - ModernFields(20)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(21) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 7 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(22) - ModernFields(21)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(21) + TempAdjust
                        End Select
                        EveTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time3 = 2 And ModernFields(21) <> -99 And ModernFields(22) <> -99 And ModernFields(23) <> -99 Then
                        'Find 8 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 8 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(22) - ModernFields(21)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(22) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 8 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(23) - ModernFields(22)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(22) + TempAdjust
                        End Select
                        EveTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time3 = 3 And ModernFields(22) <> -99 And ModernFields(23) <> -99 And ModernFields(24) <> -99 Then
                        'Find 9 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 9 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(23) - ModernFields(22)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(23) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 9 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(24) - ModernFields(23)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(23) + TempAdjust
                        End Select
                        EveTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time3 = 4 And ModernFields(23) <> -99 And ModernFields(24) <> -99 And ModernFields(25) <> -99 Then
                        'Find 10 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 10 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(24) - ModernFields(23)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(24) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 10 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(25) - ModernFields(24)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(24) + TempAdjust
                        End Select
                        EveTemp = TopHourTemp
                    End If
                    If AdjustStationSegments_Time3 = 5 And ModernFields(24) <> -99 And ModernFields(25) <> -99 And ModernFields(26) <> -99 Then
                        'Find 11 p.m. Temperature
                        Select Case AdjustStationSegments_SolarTime
                            Case Is > AdjustStationSegments_TopHourCorrection
                                'Extrapolate 11 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(25) - ModernFields(24)) * ((AdjustStationSegments_SolarTime - AdjustStationSegments_TopHourCorrection) / 60)
                                TopHourTemp = ModernFields(25) - TempAdjust
                            Case Is <= AdjustStationSegments_TopHourCorrection
                                'Extrapolate 11 p.m. Temp to Top of the Hour
                                TempAdjust = (ModernFields(26) - ModernFields(25)) * ((AdjustStationSegments_TopHourCorrection - AdjustStationSegments_SolarTime) / 60)
                                TopHourTemp = ModernFields(25) + TempAdjust
                        End Select
                        EveTemp = TopHourTemp
                    End If
                End If
                If AdjustStationSegments_HourlyTwo = 0 And AMTemp <> -99 And PMTemp <> -99 And EveTemp <> -99 Then
                    'Thrice Daily Observations
                    'Convert Units If Necessary
                    If AdjustStationSegments_ModernDegC = 1 And AdjustStationSegments_HistoricalDegF = 1 Then
                        'Convert from DegC to DegF
                        AMTemp = DegCDegF(AMTemp)
                        PMTemp = DegCDegF(PMTemp)
                        EveTemp = DegCDegF(EveTemp)
                    End If
                    If AdjustStationSegments_ModernDegF = 1 And AdjustStationSegments_HistoricalDegC = 1 Then
                        'Convert from DegF to DegC
                        AMTemp = DegFDegC(AMTemp)
                        PMTemp = DegFDegC(PMTemp)
                        EveTemp = DegFDegC(EveTemp)
                    End If
                    'Set Totals
                    AMTempTotal(Month - 1) = AMTemp + AMTempTotal(Month - 1)
                    PMTempTotal(Month - 1) = PMTemp + PMTempTotal(Month - 1)
                    EveTempTotal(Month - 1) = EveTemp + EveTempTotal(Month - 1)
                    MeanTempTotal(Month - 1) = ((AMTemp + PMTemp + EveTemp) / 3) + MeanTempTotal(Month - 1)
                    Count(Month - 1) = Count(Month - 1) + 1
                End If
                If AdjustStationSegments_HourlyTwo = 1 Then
                    'Twice Daily Observations
                    If AdjustStationSegments_Time2Enabled = 0 Then
                        'Second Observation Is Evening
                        PMTemp = EveTemp
                    End If
                    If AMTemp <> -99 And PMTemp <> -99 Then
                        'Convert Units If Necessary
                        If AdjustStationSegments_ModernDegC = 1 And AdjustStationSegments_HistoricalDegF = 1 Then
                            'Convert from DegC to DegF
                            AMTemp = DegCDegF(AMTemp)
                            PMTemp = DegCDegF(PMTemp)
                        End If
                        If AdjustStationSegments_ModernDegF = 1 And AdjustStationSegments_HistoricalDegC = 1 Then
                            'Convert from DegF to DegC
                            AMTemp = DegFDegC(AMTemp)
                            PMTemp = DegFDegC(PMTemp)
                        End If
                        'Set Totals
                        AMTempTotal(Month - 1) = AMTemp + AMTempTotal(Month - 1)
                        PMTempTotal(Month - 1) = PMTemp + PMTempTotal(Month - 1)
                        MeanTempTotal(Month - 1) = ((AMTemp + PMTemp) / 2) + MeanTempTotal(Month - 1)
                        Count(Month - 1) = Count(Month - 1) + 1
                    End If
                End If
                i = i + 1
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the modern data file at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Station Segments")
            Else
                MsgBox("Error reading the modern data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Station Segments")
            End If
            HistoricalInFile.Close()
            ModernInFile.Close()
            FileClose(1)
            FileClose(2)
            AdjustStationSegments_Exception = 1
            Exit Sub
        End Try
        'Calculate Modern Monthly Anomalies
        For i = 0 To 11
            AMTempModernAnomaly(i) = (AMTempTotal(i) / Count(i)) - (MeanTempTotal(i) / Count(i))
            PMTempModernAnomaly(i) = (PMTempTotal(i) / Count(i)) - (MeanTempTotal(i) / Count(i))
            EveTempModernAnomaly(i) = (EveTempTotal(i) / Count(i)) - (MeanTempTotal(i) / Count(i))
        Next i
        'Reset Totals
        For i = 0 To 11
            AMTempTotal(i) = 0
            PMTempTotal(i) = 0
            EveTempTotal(i) = 0
            MeanTempTotal(i) = 0
            Count(i) = 0
        Next i
        Year = 0
        'Process Historical Daily Data
        'Skip Header Columns
        HistoricalLine = HistoricalInFile.ReadLine()
        FileProgress = FileProgress + 1
        Try
            While HistoricalInFile.Peek <> -1
                HistoricalLine = HistoricalInFile.ReadLine()
                HistoricalFields = HistoricalLine.Split(ControlChars.Tab)
                Year = HistoricalFields(0)
                Month = HistoricalFields(1)
                Day = HistoricalFields(2)
                AMTemp = HistoricalFields(AdjustStationSegments_HistColumn1 - 1)
                PMTemp = HistoricalFields(AdjustStationSegments_HistColumn2 - 1)
                If AdjustStationSegments_HourlyTwo = 0 Then
                    'Thrice Daily Observations
                    EveTemp = HistoricalFields(AdjustStationSegments_HistColumn3 - 1)
                    If AMTemp <> -99 And PMTemp <> -99 And EveTemp <> -99 Then
                        'Check for Potential Unrealistic Temperature Values
                        If AdjustStationSegments_HistoricalDegF = 1 Then
                            If AMTemp < -40 Or AMTemp > 140 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & AMTemp & " detected in the morning temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    FileClose(2)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                            If PMTemp < -40 Or PMTemp > 140 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & PMTemp & " detected in the afternoon temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    FileClose(2)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                            If EveTemp < -40 Or EveTemp > 140 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & EveTemp & " detected in the evening temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    FileClose(2)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                        End If
                        If AdjustStationSegments_HistoricalDegC = 1 Then
                            If AMTemp < -40 Or AMTemp > 60 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & AMTemp & " detected in the morning temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    FileClose(2)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                            If PMTemp < -40 Or PMTemp > 60 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & PMTemp & " detected in the afternoon temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    FileClose(2)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                            If EveTemp < -40 Or EveTemp > 60 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & EveTemp & " detected in the evening temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    FileClose(2)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                        End If
                        'Set Totals
                        AMTempTotal(Month - 1) = AMTemp + AMTempTotal(Month - 1)
                        PMTempTotal(Month - 1) = PMTemp + PMTempTotal(Month - 1)
                        EveTempTotal(Month - 1) = EveTemp + EveTempTotal(Month - 1)
                        MeanTempTotal(Month - 1) = ((AMTemp + PMTemp + EveTemp) / 3) + MeanTempTotal(Month - 1)
                        Count(Month - 1) = Count(Month - 1) + 1
                    End If
                End If
                If AdjustStationSegments_HourlyTwo = 1 Then
                    'Twice Daily Observations
                    If AMTemp <> -99 And PMTemp <> -99 Then
                        'Check for Potential Unrealistic Temperature Values
                        If AdjustStationSegments_HistoricalDegF = 1 Then
                            If AMTemp < -40 Or AMTemp > 140 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & AMTemp & " detected in the a.m. temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                            If PMTemp < -40 Or PMTemp > 140 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & PMTemp & " detected in the p.m. temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                        End If
                        If AdjustStationSegments_HistoricalDegC = 1 Then
                            If AMTemp < -40 Or AMTemp > 60 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & AMTemp & " detected in the a.m. temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                            If PMTemp < -40 Or PMTemp > 60 Then
                                Response = MsgBox("Potential unrealistic temperature value of " & PMTemp & " detected in the p.m. temperature on the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".  Do you want to continue with the station adjustments?", MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, "HOB Tools - Adjust Station Segments")
                                If Response = MsgBoxResult.No Then
                                    HistoricalInFile.Close()
                                    ModernInFile.Close()
                                    FileClose(1)
                                    AdjustStationSegments_Exception = 2
                                    Exit Sub
                                End If
                            End If
                        End If
                        'Set Totals
                        AMTempTotal(Month - 1) = AMTemp + AMTempTotal(Month - 1)
                        PMTempTotal(Month - 1) = PMTemp + PMTempTotal(Month - 1)
                        MeanTempTotal(Month - 1) = ((AMTemp + PMTemp) / 2) + MeanTempTotal(Month - 1)
                        Count(Month - 1) = Count(Month - 1) + 1
                    End If
                End If
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the historical data file at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Station Segments")
            Else
                MsgBox("Error reading the historical data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Station Segments")
            End If
            ModernInFile.Close()
            HistoricalInFile.Close()
            FileClose(1)
            FileClose(2)
            AdjustStationSegments_Exception = 1
            Exit Sub
        End Try
        'Calculate Historical Monthly Anomalies
        For i = 0 To 11
            AMTempHistoricalAnomaly(i) = (AMTempTotal(i) / Count(i)) - (MeanTempTotal(i) / Count(i))
            PMTempHistoricalAnomaly(i) = (PMTempTotal(i) / Count(i)) - (MeanTempTotal(i) / Count(i))
            EveTempHistoricalAnomaly(i) = (EveTempTotal(i) / Count(i)) - (MeanTempTotal(i) / Count(i))
        Next i
        'Calculate Monthly Adjustments
        For i = 0 To 11
            AMTempAdjustment(i) = AMTempModernAnomaly(i) - AMTempHistoricalAnomaly(i)
            PMTempAdjustment(i) = PMTempModernAnomaly(i) - PMTempHistoricalAnomaly(i)
            EveTempAdjustment(i) = EveTempModernAnomaly(i) - EveTempHistoricalAnomaly(i)
        Next i
        'Output Derived Adjustments
        Select Case AdjustStationSegments_HourlyTwo
            Case 0
                'Thrice Daily Observations
                If Count(0) <> 0 Then
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(0), 4))
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(0), 4))
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(0), 4))
                Else
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(1) <> 0 Then
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(1), 4))
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(1), 4))
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(1), 4))
                Else
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(2) <> 0 Then
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(2), 4))
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(2), 4))
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(2), 4))
                Else
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(3) <> 0 Then
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(3), 4))
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(3), 4))
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(3), 4))
                Else
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(4) <> 0 Then
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(4), 4))
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(4), 4))
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(4), 4))
                Else
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(5) <> 0 Then
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(5), 4))
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(5), 4))
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(5), 4))
                Else
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(6) <> 0 Then
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(6), 4))
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(6), 4))
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(6), 4))
                Else
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(7) <> 0 Then
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(7), 4))
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(7), 4))
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(7), 4))
                Else
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(8) <> 0 Then
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(8), 4))
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(8), 4))
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(8), 4))
                Else
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(9) <> 0 Then
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(9), 4))
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(9), 4))
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(9), 4))
                Else
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(10)<>0 then
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(10), 4))
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(10), 4))
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(10), 4))
                Else
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
                If Count(11) <> 0 Then
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(11), 4))
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(11), 4))
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & Math.Round(EveTempAdjustment(11), 4))
                Else
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_EveTime & vbTab & vbTab & "NA")
                End If
            Case 1
                'Twice Daily Observations
                If AdjustStationSegments_Time2Enabled = 0 Then
                    'Second Observation Is Evening
                    AdjustStationSegments_PMTime = AdjustStationSegments_EveTime
                End If
                If Count(0) <> 0 Then
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(0), 4))
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(0), 4))
                Else
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jan" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(1) <> 0 Then
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(1), 4))
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(1), 4))
                Else
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Feb" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(2) <> 0 Then
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(2), 4))
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(2), 4))
                Else
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Mar" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(3) <> 0 Then
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(3), 4))
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(3), 4))
                Else
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Apr" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(4) <> 0 Then
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(4), 4))
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(4), 4))
                Else
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "May" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(5) <> 0 Then
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(5), 4))
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(5), 4))
                Else
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jun" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(6) <> 0 Then
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(6), 4))
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(6), 4))
                Else
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Jul" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(7) <> 0 Then
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(7), 4))
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(7), 4))
                Else
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Aug" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(8) <> 0 Then
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(8), 4))
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(8), 4))
                Else
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Sep" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(9) <> 0 Then
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(9), 4))
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(9), 4))
                Else
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Oct" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(10) <> 0 Then
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(10), 4))
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(10), 4))
                Else
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Nov" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
                If Count(11) <> 0 Then
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & Math.Round(AMTempAdjustment(11), 4))
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & Math.Round(PMTempAdjustment(11), 4))
                Else
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_AMTime & vbTab & vbTab & "NA")
                    PrintLine(2, "Dec" & vbTab & AdjustStationSegments_PMTime & vbTab & vbTab & "NA")
                End If
        End Select
        'Adjust Historical Daily Data
        Select Case AdjustStationSegments_HourlyTwo
            Case 0
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & AdjustStationSegments_AMTime & vbTab & AdjustStationSegments_PMTime & vbTab & AdjustStationSegments_EveTime)
            Case 1
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & AdjustStationSegments_AMTime & vbTab & AdjustStationSegments_PMTime)
        End Select
        'Reset Historical StreamReader
        HistoricalInFile.DiscardBufferedData()
        HistoricalInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set Historical StreamReader to Data Start
        If AdjustStationSegments_HistRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To AdjustStationSegments_HistRowStart - 2
                HistoricalLine = HistoricalInFile.ReadLine()
            Next Row
        End If
        'Process Historical Daily Data
        'Skip Header Columns
        HistoricalLine = HistoricalInFile.ReadLine()
        FileProgress = FileProgress + 1
        Try
            While HistoricalInFile.Peek <> -1
                HistoricalLine = HistoricalInFile.ReadLine()
                HistoricalFields = HistoricalLine.Split(ControlChars.Tab)
                Year = HistoricalFields(0)
                Month = HistoricalFields(1)
                Day = HistoricalFields(2)
                AMTemp = HistoricalFields(AdjustStationSegments_HistColumn1 - 1)
                PMTemp = HistoricalFields(AdjustStationSegments_HistColumn2 - 1)
                If AdjustStationSegments_HourlyTwo = 0 Then
                    'Thrice Daily Observations
                    EveTemp = HistoricalFields(AdjustStationSegments_HistColumn3 - 1)
                    'Adjust Temperatures
                    Select Case AMTemp
                        Case Is = -99
                            AdjAMTemp = -99
                        Case Else
                            AdjAMTemp = AMTemp + AMTempAdjustment(Month - 1)
                            AdjAMTemp = Math.Round(AdjAMTemp, 1)
                    End Select
                    Select Case PMTemp
                        Case Is = -99
                            AdjPMTemp = -99
                        Case Else
                            AdjPMTemp = PMTemp + PMTempAdjustment(Month - 1)
                            AdjPMTemp = Math.Round(AdjPMTemp, 1)
                    End Select
                    Select Case EveTemp
                        Case Is = -99
                            AdjEveTemp = -99
                        Case Else
                            AdjEveTemp = EveTemp + EveTempAdjustment(Month - 1)
                            AdjEveTemp = Math.Round(AdjEveTemp, 1)
                    End Select
                    'Output Adjusted Temperatures
                    PrintLine(1, Year & vbTab & Month & vbTab & Day & vbTab & AdjAMTemp & vbTab & AdjPMTemp & vbTab & AdjEveTemp)
                End If
                If AdjustStationSegments_HourlyTwo = 1 Then
                    'Twice Daily Observations
                    'Adjust Temperatures
                    Select Case AMTemp
                        Case Is = -99
                            AdjAMTemp = -99
                        Case Else
                            AdjAMTemp = AMTemp + AMTempAdjustment(Month - 1)
                            AdjAMTemp = Math.Round(AdjAMTemp, 1)
                    End Select
                    Select Case PMTemp
                        Case Is = -99
                            AdjPMTemp = -99
                        Case Else
                            AdjPMTemp = PMTemp + PMTempAdjustment(Month - 1)
                            AdjPMTemp = Math.Round(AdjPMTemp, 1)
                    End Select
                    'Output Adjusted Temperatures
                    PrintLine(1, Year & vbTab & Month & vbTab & Day & vbTab & AdjAMTemp & vbTab & AdjPMTemp)
                End If
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            MsgBox("Error reading the historical data file at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Adjust Station Segments")
            ModernInFile.Close()
            HistoricalInFile.Close()
            FileClose(1)
            FileClose(2)
            AdjustStationSegments_Exception = 1
            Exit Sub
        End Try
        'Close Files
        FileClose(1)
        FileClose(2)
        ModernInFile.Close()
        HistoricalInFile.Close()
    End Sub

    Private Sub BackgroundWorker_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case AdjustStationSegments_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
            Case 2
                Status_Label.Text = "Status:  Run Aborted"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub AdjustStationSegments_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        AdjustStationSegmentsDisp = 0
    End Sub
End Class