Public Class ColdWaves

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private ColdWaves_Input As String
    Private ColdWaves_Output As String
    'Initialize Global Form Variables
    Private ColdWaves_RowStart As Integer
    Private ColdWaves_TempColumn As Integer
    Private ColdWaves_ErrorBars As Integer
    Private ColdWaves_ErrorColumn As Integer
    Private ColdWaves_TempChange As Double
    Private ColdWaves_SpecificValue As Integer
    Private ColdWaves_Percentile As Integer
    Private ColdWaves_ComputeMonth As Integer
    Private ColdWaves_Month As Integer
    Private ColdWaves_ComputeRange As Integer
    Private ColdWaves_StartMonth As Integer
    Private ColdWaves_EndMonth As Integer
    Private ColdWaves_Header As String
    Private ColdWaves_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ColdWaves_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Cold Wave Analysis Tool
        Month.SelectedIndex = 0
        StartMonth.SelectedIndex = 0
        EndMonth.SelectedIndex = 0
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ColdWaves_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ColdWaves_Input = FileSelect.FileName
            InputFileLocation.Text = ColdWaves_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                ColdWaves_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TempColumnEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TempColumnEntry.TextChanged
        EntryError.SetError(TempColumnEntry, "")
        If TempColumnEntry.Text <> "" Then
            Try
                ColdWaves_TempColumn = TempColumnEntry.Text
            Catch ex As Exception
                EntryError.SetError(TempColumnEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ErrorBars_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorBars.CheckedChanged
        Select Case ErrorBars.Checked
            Case True
                ErrorColumn_Label.Enabled = True
                ErrorColumn.Enabled = True
            Case False
                ErrorColumn_Label.Enabled = False
                ErrorColumn.Enabled = False
        End Select
    End Sub

    Private Sub ErrorColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorColumn.TextChanged
        EntryError.SetError(ErrorColumn, "")
        If ErrorColumn.Text <> "" Then
            Try
                ColdWaves_ErrorColumn = ErrorColumn.Text
            Catch ex As Exception
                EntryError.SetError(ErrorColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TempChangeEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TempChangeEntry.TextChanged
        EntryError.SetError(TempChangeEntry, "")
        If TempChangeEntry.Text <> "" Then
            Try
                ColdWaves_TempChange = TempChangeEntry.Text
            Catch ex As Exception
                EntryError.SetError(TempChangeEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ComputeMonth_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComputeMonth.CheckedChanged
        If ComputeMonth.Checked = True Then
            Month.Enabled = True
            StartMonth.Enabled = False
            EndMonth.Enabled = False
        End If
    End Sub

    Private Sub Month_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Month.SelectedIndexChanged
        EntryError.SetError(Month, "")
        ColdWaves_Month = Month.SelectedIndex
    End Sub

    Private Sub ComputeRange_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComputeRange.CheckedChanged
        If ComputeRange.Checked = True Then
            Month.Enabled = False
            StartMonth.Enabled = True
            EndMonth.Enabled = True
        End If
    End Sub

    Private Sub StartMonth_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartMonth.SelectedIndexChanged
        EntryError.SetError(StartMonth, "")
        EntryError.SetError(EndMonth, "")
        ColdWaves_StartMonth = StartMonth.SelectedIndex
    End Sub

    Private Sub EndMonth_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndMonth.SelectedIndexChanged
        EntryError.SetError(StartMonth, "")
        EntryError.SetError(EndMonth, "")
        ColdWaves_EndMonth = EndMonth.SelectedIndex
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ColdWaves_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ColdWaves_Output = FileOutput.FileName
            OutputFileLocation.Text = ColdWaves_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ColdWaves_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ColdWaves_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If TempColumnEntry.Text = "" Then
            EntryError.SetError(TempColumnEntry, "Required input data information missing.")
            Exit Sub
        End If
        If ErrorBars.Checked = True Then
            If ErrorColumn.Text = "" Then
                EntryError.SetError(ErrorColumn, "Required input data information missing.")
                Exit Sub
            End If
        End If
        If TempChangeEntry.Text = "" Then
            EntryError.SetError(TempChangeEntry, "Required analysis information missing.")
            Exit Sub
        End If
        If ComputeMonth.Checked = True Then
            If Month.SelectedIndex = 0 Then
                EntryError.SetError(Month, "Required analysis month not selected.")
                Exit Sub
            End If
        End If
        If ComputeRange.Checked = True Then
            If StartMonth.SelectedIndex = 0 Then
                EntryError.SetError(StartMonth, "Required analysis start month not selected.")
                Exit Sub
            End If
            If EndMonth.SelectedIndex = 0 Then
                EntryError.SetError(EndMonth, "Required analysis end month not selected.")
                Exit Sub
            End If
            If StartMonth.SelectedIndex = EndMonth.SelectedIndex Then
                EntryError.SetError(EndMonth, "Start and end months cannot be the same.")
                Exit Sub
            End If
        End If
        If ColdWaves_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If ColdWaves_Input = ColdWaves_Output Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        ColdWaves_Exception = 0
        If SpecificValue.Checked = True Then
            ColdWaves_SpecificValue = 1
        Else
            ColdWaves_SpecificValue = 0
        End If
        If Percentile.Checked = True Then
            ColdWaves_Percentile = 1
        Else
            ColdWaves_Percentile = 0
        End If
        If ComputeMonth.Checked = True Then
            ColdWaves_ComputeMonth = 1
        Else
            ColdWaves_ComputeMonth = 0
        End If
        If ComputeRange.Checked = True Then
            ColdWaves_ComputeRange = 1
        Else
            ColdWaves_ComputeRange = 0
        End If
        If ErrorBars.Checked = True Then
            ColdWaves_ErrorBars = 1
        Else
            ColdWaves_ErrorBars = 0
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim MonthCheck As Integer
        Dim StoreOnOff As Integer
        Dim AtStartMonth As Integer
        Dim DepressionCheck As Integer
        Dim EvenOddCheck As Double
        Dim i As Integer
        Dim TimeThrough As Integer
        Dim TempDay0 As Double
        Dim TempDay0Err As Double
        Dim TempDay0LowErr As Double
        Dim TempDay0HighErr As Double
        Dim TempDay1 As Double
        Dim TempDay1Err As Double
        Dim TempDay1LowErr As Double
        Dim TempDay1HighErr As Double
        Dim TempDepression(0) As Double
        Dim TempChange As Double
        Dim CriticalTempDepression As Double
        Dim Count As Integer
        Dim CountLowErr As Integer
        Dim CountHighErr As Integer
        Dim Flags As Integer
        TimeThrough = 0
        CriticalTempDepression = 0
        Count = 0
        Flags = 0
        'Initialize Output File and Set Header
        FileOpen(1, ColdWaves_Output, OpenMode.Output)
        PrintLine(1, ColdWaves_Header)
        PrintLine(1, "")
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ColdWaves_Input)
        Dim Line As String
        Dim Fields As String()
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set StreamReader to Data Start
        If ColdWaves_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ColdWaves_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = (FileComplete - (ColdWaves_RowStart - 1)) * 2
        End If
        'Find Critical Temperature Change Value
        'Skip Header Columns
        Line = InFile.ReadLine()
        FileProgress = FileProgress + 1
        If ColdWaves_Percentile = 1 Then
            Try
                While InFile.Peek <> -1
                    Line = InFile.ReadLine()
                    Fields = Line.Split(ControlChars.Tab)
                    If TimeThrough = 1 Then
                        'Set Day One
                        Year = Fields(0)
                        Month = Fields(1)
                        Day = Fields(2)
                        TempDay1 = Fields(ColdWaves_TempColumn - 1)
                        If TempDay0 <> -99 And TempDay1 <> -99 Then
                            TempChange = TempDay0 - TempDay1
                        Else
                            TempChange = 0
                        End If
                        If TempChange > 0 Then
                            'Temperature Depression Occurred...Save to Array if Needed
                            If ColdWaves_ComputeMonth = 1 Then
                                If ColdWaves_Month = Month Then
                                    TempDepression(Count) = TempChange
                                    Count = Count + 1
                                    ReDim Preserve TempDepression(Count)
                                End If
                            End If
                            If ColdWaves_ComputeRange = 1 Then
                                MonthCheck = ColdWaves_EndMonth - ColdWaves_StartMonth
                                Select Case MonthCheck
                                    Case Is > 0
                                        If Month > ColdWaves_StartMonth And Month <= ColdWaves_EndMonth Then
                                            TempDepression(Count) = TempChange
                                            Count = Count + 1
                                            ReDim Preserve TempDepression(Count)
                                        End If
                                    Case Is < 0
                                        If Month > ColdWaves_StartMonth And Month <= 12 Then
                                            TempDepression(Count) = TempChange
                                            Count = Count + 1
                                            ReDim Preserve TempDepression(Count)
                                        End If
                                        If Month >= 1 And Month <= ColdWaves_EndMonth Then
                                            TempDepression(Count) = TempChange
                                            Count = Count + 1
                                            ReDim Preserve TempDepression(Count)
                                        End If
                                End Select
                            End If
                        End If
                        'Reset for Next Day
                        TempDay0 = TempDay1
                    End If
                    If TimeThrough = 0 Then
                        'Set Initial Conditions
                        Year = Fields(0)
                        Month = Fields(1)
                        Day = Fields(2)
                        TempDay0 = Fields(ColdWaves_TempColumn - 1)
                        TimeThrough = 1
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the input file at or near the following Year/Month/Day:  " & Fields(0) & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Cold Wave Analysis")
                Else
                    MsgBox("Error reading the input file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Cold Wave Analysis")
                End If
                InFile.Close()
                FileClose(1)
                ColdWaves_Exception = 1
                Exit Sub
            End Try
            'Sort Temperature Depression Array
            Array.Sort(TempDepression)
            'Change Entered Whole Percent into Decimal
            ColdWaves_TempChange = ColdWaves_TempChange / 100
            ColdWaves_TempChange = Math.Round(ColdWaves_TempChange, 2)
            'Calculate Percentile and Set Critical Value
            EvenOddCheck = Count Mod (1 / ColdWaves_TempChange)
            If EvenOddCheck = 0 Then
                'Percentile At i
                i = Count * ColdWaves_TempChange
                CriticalTempDepression = TempDepression(i - 1)
            Else
                'Percentile In Between i and i+1
                i = Math.Truncate(Count * ColdWaves_TempChange)
                CriticalTempDepression = TempDepression(i - 1)
            End If
            'Output Critical Value
            PrintLine(1, "Critical Temperature Depression = " & Math.Round(CriticalTempDepression, 1))
            PrintLine(1, "")
            'Reset StreamReader
            InFile.DiscardBufferedData()
            InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
            'Set StreamReader to Data Start
            If ColdWaves_RowStart <> 1 Then
                'Header Exists so Move LineInput Down
                Dim Row As Integer
                For Row = 0 To ColdWaves_RowStart - 2
                    Line = InFile.ReadLine()
                Next Row
            End If
            'Reset Variables
            TimeThrough = 0
            Count = 0
        Else
            'Output Critical Value
            CriticalTempDepression = ColdWaves_TempChange
            PrintLine(1, "Critical Temperature Depression = " & CriticalTempDepression)
            PrintLine(1, "")
        End If
        'Locate Storm Events
        'Skip Header Columns
        Line = InFile.ReadLine()
        FileProgress = FileProgress + 1
        Try
            If ColdWaves_ComputeMonth = 1 Then
                'Perform Monthly Analysis
                Select Case ColdWaves_ErrorBars
                    Case 0
                        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "ColdWvs" & vbTab & "Flags")
                    Case 1
                        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "ColdWvs" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                End Select
                While InFile.Peek <> -1
                    Line = InFile.ReadLine()
                    Fields = Line.Split(ControlChars.Tab)
                    If TimeThrough = 1 Then
                        'Not First Time Through While Loop So Check For Month Change
                        If Month <> Fields(1) Then
                            'New Month...Save Previous Month's Data
                            If ColdWaves_Month = Month Then
                                Select Case ColdWaves_ErrorBars
                                    Case 0
                                        PrintLine(1, Year & vbTab & Month & vbTab & Count & vbTab & Flags)
                                    Case 1
                                        PrintLine(1, Year & vbTab & Month & vbTab & Count & vbTab & CountLowErr & vbTab & CountHighErr & vbTab & Flags)
                                End Select
                            End If
                            'Reset to New Month
                            Year = Fields(0)
                            Month = Fields(1)
                            Day = Fields(2)
                            Count = 0
                            CountLowErr = 0
                            CountHighErr = 0
                            Flags = 0
                            'Restart Monthly Count
                            TempDay0 = Fields(ColdWaves_TempColumn - 1)
                            If ColdWaves_ErrorBars = 1 Then
                                TempDay0Err = Fields(ColdWaves_ErrorColumn - 1)
                                TempDay0LowErr = TempDay0 - TempDay0Err
                                TempDay0HighErr = TempDay0 + TempDay0Err
                            End If
                            If TempDay0 = -99 Then
                                Select Case ColdWaves_ErrorBars
                                    Case 0
                                        Flags = 1
                                    Case 1
                                        Flags = 2
                                End Select
                            End If
                        Else
                            Day = Fields(2)
                            'Continue Monthly Count
                            'Set Day One
                            TempDay1 = Fields(ColdWaves_TempColumn - 1)
                            If ColdWaves_ErrorBars = 1 Then
                                TempDay1Err = Fields(ColdWaves_ErrorColumn - 1)
                                TempDay1LowErr = TempDay1 - TempDay1Err
                                TempDay1HighErr = TempDay1 + TempDay1Err
                            End If
                            If TempDay1 = -99 Then
                                Select Case ColdWaves_ErrorBars
                                    Case 0
                                        Flags = Flags + 1
                                    Case 1
                                        Flags = Flags + 2
                                End Select
                            End If
                            If TempDay0 <> -99 And TempDay1 <> -99 Then
                                TempChange = TempDay0 - TempDay1
                            Else
                                TempChange = 0
                            End If
                            If TempChange > 0 Then
                                'Temperature Depression Occurred...Check if Above Critical Value
                                If TempChange > CriticalTempDepression Then
                                    Count = Count + 1
                                End If
                            End If
                            If ColdWaves_ErrorBars = 1 Then
                                'Low Error
                                If TempDay0LowErr <> -99 And TempDay1LowErr <> -99 Then
                                    TempChange = TempDay0LowErr - TempDay1LowErr
                                Else
                                    TempChange = 0
                                End If
                                If TempChange > 0 Then
                                    'Temperature Depression Occurred...Check if Above Critical Value
                                    If TempChange > CriticalTempDepression Then
                                        CountLowErr = CountLowErr + 1
                                    End If
                                End If
                                'High Error
                                If TempDay0HighErr <> -99 And TempDay1HighErr <> -99 Then
                                    TempChange = TempDay0HighErr - TempDay1HighErr
                                Else
                                    TempChange = 0
                                End If
                                If TempChange > 0 Then
                                    'Temperature Depression Occurred...Check if Above Critical Value
                                    If TempChange > CriticalTempDepression Then
                                        CountHighErr = CountHighErr + 1
                                    End If
                                End If
                            End If
                            'Reset for Next Day
                            TempDay0 = TempDay1
                            If ColdWaves_ErrorBars = 1 Then
                                TempDay0LowErr = TempDay1LowErr
                                TempDay0HighErr = TempDay1HighErr
                            End If
                        End If
                    End If
                    If TimeThrough = 0 Then
                        'Set Initial Conditions
                        Year = Fields(0)
                        Month = Fields(1)
                        Day = Fields(2)
                        TempDay0 = Fields(ColdWaves_TempColumn - 1)
                        If ColdWaves_ErrorBars = 1 Then
                            TempDay0Err = Fields(ColdWaves_ErrorColumn - 1)
                            TempDay0LowErr = TempDay0 - TempDay0Err
                            TempDay0HighErr = TempDay0 + TempDay0Err
                        End If
                        If TempDay0 = -99 Then
                            Select Case ColdWaves_ErrorBars
                                Case 0
                                    Flags = Flags + 1
                                Case 1
                                    Flags = Flags + 2
                            End Select
                        End If
                        TimeThrough = 1
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
                'Print Final Monthly Count
                If ColdWaves_Month = Month Then
                    Select Case ColdWaves_ErrorBars
                        Case 0
                            PrintLine(1, Year & vbTab & Month & vbTab & Count & vbTab & Flags)
                        Case 1
                            PrintLine(1, Year & vbTab & Month & vbTab & Count & vbTab & CountLowErr & vbTab & CountHighErr & vbTab & Flags)
                    End Select
                End If
            End If
            If ColdWaves_ComputeRange = 1 Then
                'Perform Range Analysis
                Select Case ColdWaves_ErrorBars
                    Case 0
                        PrintLine(1, "Year" & vbTab & "ColdWvs" & vbTab & "Flags")
                    Case 1
                        PrintLine(1, "Year" & vbTab & "ColdWvs" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                End Select
                While InFile.Peek <> -1
                    Line = InFile.ReadLine()
                    Fields = Line.Split(ControlChars.Tab)
                    Year = Fields(0)
                    Month = Fields(1)
                    Day = Fields(2)
                    If TimeThrough = 0 Then
                        'Set Initial Conditions
                        DepressionCheck = 0
                        'Check If Initial Month Is After Start Month but Before or At End Month
                        MonthCheck = ColdWaves_EndMonth - ColdWaves_StartMonth
                        Select Case MonthCheck
                            Case Is > 0
                                If Month > ColdWaves_StartMonth And Month <= ColdWaves_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                End If
                            Case Is < 0
                                If Month > ColdWaves_StartMonth And Month <= 12 Then
                                    'Begin Store
                                    StoreOnOff = 1
                                End If
                                If Month >= 1 And Month <= ColdWaves_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    'Reset Year
                                    Year = Year - 1
                                End If
                        End Select
                        If StoreOnOff = 1 Then
                            'Reset Flags
                            MonthCheck = ColdWaves_StartMonth
                            Do
                                If MonthCheck = Month Then
                                    Exit Do
                                End If
                                If MonthCheck = 1 Or MonthCheck = 3 Or MonthCheck = 5 Or MonthCheck = 7 Or MonthCheck = 8 Or MonthCheck = 10 Or MonthCheck = 12 Then
                                    Flags = Flags + 31
                                End If
                                If MonthCheck = 4 Or MonthCheck = 6 Or MonthCheck = 9 Or MonthCheck = 11 Then
                                    Flags = Flags + 30
                                End If
                                If MonthCheck = 2 Then
                                    If Date.IsLeapYear(Year) = True Then
                                        Flags = Flags + 29
                                    Else
                                        Flags = Flags + 28
                                    End If
                                End If
                                If MonthCheck = 12 Then
                                    MonthCheck = 1
                                    Year = Year + 1
                                Else
                                    MonthCheck = MonthCheck + 1
                                End If
                            Loop
                            'Double Value If Error Bars are Selected
                            If ColdWaves_ErrorBars = 1 Then
                                Flags = Flags * 2
                            End If
                        End If
                        TimeThrough = 1
                    End If
                    'Check for End Month
                    If ColdWaves_EndMonth = 12 Then
                        MonthCheck = 1
                    Else
                        MonthCheck = ColdWaves_EndMonth + 1
                    End If
                    If Month = MonthCheck And StoreOnOff = 1 And AtStartMonth = 0 Then
                        'End Month Reached...End Store
                        StoreOnOff = 0
                        'Save Data
                        If ColdWaves_EndMonth = 12 Then
                            'New Year Read So Go Back One for Output
                            Year = Year - 1
                        End If
                        Select Case ColdWaves_ErrorBars
                            Case 0
                                PrintLine(1, Year & vbTab & Count & vbTab & Flags)
                            Case 1
                                PrintLine(1, Year & vbTab & Count & vbTab & CountLowErr & vbTab & CountHighErr & vbTab & Flags)
                        End Select
                        'Reset for Next Range
                        Year = Fields(0)
                        Month = Fields(1)
                        Day = Fields(2)
                        DepressionCheck = 0
                        Count = 0
                        CountLowErr = 0
                        CountHighErr = 0
                        Flags = 0
                        'Restart Range Count
                        TempDay0 = Fields(ColdWaves_TempColumn - 1)
                        If ColdWaves_ErrorBars = 1 Then
                            TempDay0Err = Fields(ColdWaves_ErrorColumn - 1)
                            TempDay0LowErr = TempDay0 - TempDay0Err
                            TempDay0HighErr = TempDay0 + TempDay0Err
                        End If
                        If TempDay0 = -99 Then
                            Select Case ColdWaves_ErrorBars
                                Case 0
                                    Flags = 1
                                Case 1
                                    Flags = 2
                            End Select
                        End If
                    End If
                    'Check for StartMonth
                    If Month = ColdWaves_StartMonth Then
                        'Start Month Reached...Begin Store
                        StoreOnOff = 1
                        'Force No Save of Start Month
                        AtStartMonth = 1
                    Else
                        AtStartMonth = 0
                    End If
                    If StoreOnOff = 1 Then
                        Select Case DepressionCheck
                            Case 0
                                'Start Count
                                TempDay0 = Fields(ColdWaves_TempColumn - 1)
                                If ColdWaves_ErrorBars = 1 Then
                                    TempDay0Err = Fields(ColdWaves_ErrorColumn - 1)
                                    TempDay0LowErr = TempDay0 - TempDay0Err
                                    TempDay0HighErr = TempDay0 + TempDay0Err
                                End If
                                If TempDay0 = -99 Then
                                    Select Case ColdWaves_ErrorBars
                                        Case 0
                                            Flags = 1
                                        Case 1
                                            Flags = 2
                                    End Select
                                End If
                                DepressionCheck = 1
                            Case 1
                                'Continue Range Count
                                'Set Day One
                                TempDay1 = Fields(ColdWaves_TempColumn - 1)
                                If ColdWaves_ErrorBars = 1 Then
                                    TempDay1Err = Fields(ColdWaves_ErrorColumn - 1)
                                    TempDay1LowErr = TempDay1 - TempDay1Err
                                    TempDay1HighErr = TempDay1 + TempDay1Err
                                End If
                                If TempDay1 = -99 Then
                                    Select Case ColdWaves_ErrorBars
                                        Case 0
                                            Flags = Flags + 1
                                        Case 1
                                            Flags = Flags + 2
                                    End Select
                                End If
                                If TempDay0 <> -99 And TempDay1 <> -99 Then
                                    TempChange = TempDay0 - TempDay1
                                Else
                                    TempChange = 0
                                End If
                                If TempChange > 0 Then
                                    'Temperature Depression Occurred...Check if Above Critical Value
                                    If TempChange > CriticalTempDepression Then
                                        Count = Count + 1
                                    End If
                                End If
                                If ColdWaves_ErrorBars = 1 Then
                                    'Low Error
                                    If TempDay0LowErr <> -99 And TempDay1LowErr <> -99 Then
                                        TempChange = TempDay0LowErr - TempDay1LowErr
                                    Else
                                        TempChange = 0
                                    End If
                                    If TempChange > 0 Then
                                        'Temperature Depression Occurred...Check if Above Critical Value
                                        If TempChange > CriticalTempDepression Then
                                            CountLowErr = CountLowErr + 1
                                        End If
                                    End If
                                    'High Error
                                    If TempDay0HighErr <> -99 And TempDay1HighErr <> -99 Then
                                        TempChange = TempDay0HighErr - TempDay1HighErr
                                    Else
                                        TempChange = 0
                                    End If
                                    If TempChange > 0 Then
                                        'Temperature Depression Occurred...Check if Above Critical Value
                                        If TempChange > CriticalTempDepression Then
                                            CountHighErr = CountHighErr + 1
                                        End If
                                    End If
                                End If
                                'Reset for Next Day
                                TempDay0 = TempDay1
                                If ColdWaves_ErrorBars = 1 Then
                                    TempDay0LowErr = TempDay1LowErr
                                    TempDay0HighErr = TempDay1HighErr
                                End If
                        End Select
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
                'Finalize Output File
                If StoreOnOff = 1 Then
                    'Tabulate Final Flags
                    If Month <> ColdWaves_EndMonth Then
                        If Month = 12 Then
                            MonthCheck = 1
                        Else
                            MonthCheck = Month + 1
                        End If
                        Do
                            If MonthCheck = 1 Then
                                Year = Year + 1
                                If ColdWaves_ErrorBars = 1 Then
                                    Flags = Flags + 62
                                Else
                                    Flags = Flags + 31
                                End If
                            End If
                            If MonthCheck = 3 Or MonthCheck = 5 Or MonthCheck = 7 Or MonthCheck = 8 Or MonthCheck = 10 Or MonthCheck = 12 Then
                                If ColdWaves_ErrorBars = 1 Then
                                    Flags = Flags + 62
                                Else
                                    Flags = Flags + 31
                                End If
                            End If
                            If MonthCheck = 4 Or MonthCheck = 6 Or MonthCheck = 9 Or MonthCheck = 11 Then
                                If ColdWaves_ErrorBars = 1 Then
                                    Flags = Flags + 60
                                Else
                                    Flags = Flags + 30
                                End If
                            End If
                            If MonthCheck = 2 Then
                                If Date.IsLeapYear(Year) = True Then
                                    If ColdWaves_ErrorBars = 1 Then
                                        Flags = Flags + 58
                                    Else
                                        Flags = Flags + 29
                                    End If
                                Else
                                    If ColdWaves_ErrorBars = 1 Then
                                        Flags = Flags + 56
                                    Else
                                        Flags = Flags + 28
                                    End If
                                End If
                            End If
                            If MonthCheck = ColdWaves_EndMonth Then
                                Exit Do
                            Else
                                If MonthCheck = 12 Then
                                    MonthCheck = 1
                                Else
                                    MonthCheck = MonthCheck + 1
                                End If
                            End If
                        Loop
                    End If
                    'Print Final Range Count
                    Select Case ColdWaves_ErrorBars
                        Case 0
                            PrintLine(1, Year & vbTab & Count & vbTab & Flags)
                        Case 1
                            PrintLine(1, Year & vbTab & Count & vbTab & CountLowErr & vbTab & CountHighErr & vbTab & Flags)
                    End Select
                End If
            End If
        Catch ex As Exception
            MsgBox("Error reading the input file at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Cold Wave Analysis")
            InFile.Close()
            FileClose(1)
            ColdWaves_Exception = 1
            Exit Sub
        End Try
        'Close Files
        InFile.Close()
        FileClose(1)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ColdWaves_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ColdWaves_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ColdWavesDisp = 0
    End Sub
End Class