﻿Imports ZedGraph
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices
Public Class DifferencePlot

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input Variable
    Private DifferencePlot_Input As String
    'Initialize Global Form Variables
    Private DifferencePlot_RowStart As Integer
    Private DifferencePlot_Station1 As Integer
    Private DifferencePlot_Station2 As Integer
    Private DifferencePlot_YAxisLabel As String
    Private DifferencePlot_StartYear1 As Integer
    Private DifferencePlot_StartMonth1 As Integer
    Private DifferencePlot_StartDay1 As Integer
    Private DifferencePlot_EndYear1 As Integer
    Private DifferencePlot_EndMonth1 As Integer
    Private DifferencePlot_EndDay1 As Integer
    Private DifferencePlot_StartYear2 As Integer
    Private DifferencePlot_StartMonth2 As Integer
    Private DifferencePlot_StartDay2 As Integer
    Private DifferencePlot_EndYear2 As Integer
    Private DifferencePlot_EndMonth2 As Integer
    Private DifferencePlot_EndDay2 As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub DifferencePlot_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Difference Series Plot Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = DifferencePlot_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            DifferencePlot_Input = FileSelect.FileName
            InputFileLocation.Text = DifferencePlot_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStartEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStartEntry.TextChanged
        EntryError.SetError(DataRowStartEntry, "")
        If DataRowStartEntry.Text <> "" Then
            Try
                DifferencePlot_RowStart = DataRowStartEntry.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStartEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Station1Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Station1Entry.TextChanged
        EntryError.SetError(Station1Entry, "")
        If Station1Entry.Text <> "" Then
            Try
                DifferencePlot_Station1 = Station1Entry.Text
            Catch ex As Exception
                EntryError.SetError(Station1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Station2Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Station2Entry.TextChanged
        EntryError.SetError(Station2Entry, "")
        If Station2Entry.Text <> "" Then
            Try
                DifferencePlot_Station2 = Station2Entry.Text
            Catch ex As Exception
                EntryError.SetError(Station2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub YAxisEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles YAxisEntry.TextChanged
        EntryError.SetError(YAxisEntry, "")
        DifferencePlot_YAxisLabel = YAxisEntry.Text
    End Sub

    Private Sub DailyFormat_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DailyFormat.CheckedChanged
        If DailyFormat.Checked = True Then
            StartMonth1Entry.Enabled = True
            StartDay1Entry.Enabled = True
            EndMonth1Entry.Enabled = True
            EndDay1Entry.Enabled = True
            StartMonth2Entry.Enabled = True
            StartDay2Entry.Enabled = True
            EndMonth2Entry.Enabled = True
            EndDay2Entry.Enabled = True
        End If
    End Sub

    Private Sub MonthlyFormat_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MonthlyFormat.CheckedChanged
        If MonthlyFormat.Checked = True Then
            StartMonth1Entry.Enabled = True
            StartDay1Entry.Enabled = False
            EndMonth1Entry.Enabled = True
            EndDay1Entry.Enabled = False
            StartMonth2Entry.Enabled = True
            StartDay2Entry.Enabled = False
            EndMonth2Entry.Enabled = True
            EndDay2Entry.Enabled = False
        End If
    End Sub

    Private Sub AnnualFormat_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AnnualFormat.CheckedChanged
        If AnnualFormat.Checked = True Then
            StartMonth1Entry.Enabled = False
            StartDay1Entry.Enabled = False
            EndMonth1Entry.Enabled = False
            EndDay1Entry.Enabled = False
            StartMonth2Entry.Enabled = False
            StartDay2Entry.Enabled = False
            EndMonth2Entry.Enabled = False
            EndDay2Entry.Enabled = False
        End If
    End Sub

    Private Sub StartYear1Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartYear1Entry.TextChanged
        EntryError.SetError(StartYear1Entry, "")
        If StartYear1Entry.Text <> "" Then
            Try
                DifferencePlot_StartYear1 = StartYear1Entry.Text
            Catch ex As Exception
                EntryError.SetError(StartYear1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartMonth1Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartMonth1Entry.TextChanged
        EntryError.SetError(StartMonth1Entry, "")
        If StartMonth1Entry.Text <> "" Then
            Try
                DifferencePlot_StartMonth1 = StartMonth1Entry.Text
                If DifferencePlot_StartMonth1 > 12 Or DifferencePlot_StartMonth1 < 1 Then
                    EntryError.SetError(StartMonth1Entry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartMonth1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartDay1Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartDay1Entry.TextChanged
        EntryError.SetError(StartDay1Entry, "")
        If StartDay1Entry.Text <> "" Then
            Try
                DifferencePlot_StartDay1 = StartDay1Entry.Text
                If DifferencePlot_StartDay1 < 1 Then
                    EntryError.SetError(StartDay1Entry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartDay1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndYear1Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndYear1Entry.TextChanged
        EntryError.SetError(EndYear1Entry, "")
        If EndYear1Entry.Text <> "" Then
            Try
                DifferencePlot_EndYear1 = EndYear1Entry.Text
            Catch ex As Exception
                EntryError.SetError(EndYear1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndMonth1Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndMonth1Entry.TextChanged
        EntryError.SetError(EndMonth1Entry, "")
        If EndMonth1Entry.Text <> "" Then
            Try
                DifferencePlot_EndMonth1 = EndMonth1Entry.Text
                If DifferencePlot_EndMonth1 > 12 Or DifferencePlot_EndMonth1 < 1 Then
                    EntryError.SetError(EndMonth1Entry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndMonth1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndDay1Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndDay1Entry.TextChanged
        EntryError.SetError(EndDay1Entry, "")
        If EndDay1Entry.Text <> "" Then
            Try
                DifferencePlot_EndDay1 = EndDay1Entry.Text
                If DifferencePlot_EndDay1 < 1 Then
                    EntryError.SetError(EndDay1Entry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndDay1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartYear2Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartYear2Entry.TextChanged
        EntryError.SetError(StartYear2Entry, "")
        If StartYear2Entry.Text <> "" Then
            Try
                DifferencePlot_StartYear2 = StartYear2Entry.Text
            Catch ex As Exception
                EntryError.SetError(StartYear2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartMonth2Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartMonth2Entry.TextChanged
        EntryError.SetError(StartMonth2Entry, "")
        If StartMonth2Entry.Text <> "" Then
            Try
                DifferencePlot_StartMonth2 = StartMonth2Entry.Text
                If DifferencePlot_StartMonth2 > 12 Or DifferencePlot_StartMonth2 < 1 Then
                    EntryError.SetError(StartMonth2Entry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartMonth2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartDay2Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartDay2Entry.TextChanged
        EntryError.SetError(StartDay2Entry, "")
        If StartDay2Entry.Text <> "" Then
            Try
                DifferencePlot_StartDay2 = StartDay2Entry.Text
                If DifferencePlot_StartDay2 < 1 Then
                    EntryError.SetError(StartDay2Entry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartDay2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndYear2Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndYear2Entry.TextChanged
        EntryError.SetError(EndYear2Entry, "")
        If EndYear2Entry.Text <> "" Then
            Try
                DifferencePlot_EndYear2 = EndYear2Entry.Text
            Catch ex As Exception
                EntryError.SetError(EndYear2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndMonth2Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndMonth2Entry.TextChanged
        EntryError.SetError(EndMonth2Entry, "")
        If EndMonth2Entry.Text <> "" Then
            Try
                DifferencePlot_EndMonth2 = EndMonth2Entry.Text
                If DifferencePlot_EndMonth2 > 12 Or DifferencePlot_EndMonth2 < 1 Then
                    EntryError.SetError(EndMonth2Entry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndMonth2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndDay2Entry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndDay2Entry.TextChanged
        EntryError.SetError(EndDay2Entry, "")
        If EndDay2Entry.Text <> "" Then
            Try
                DifferencePlot_EndDay2 = EndDay2Entry.Text
                If DifferencePlot_EndDay2 < 1 Then
                    EntryError.SetError(EndDay2Entry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndDay2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Plot_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Plot.Click
        DrawDifferencePlot()
    End Sub

    Private Sub DrawDifferencePlot()
        'Check for User Input Errors
        If DifferencePlot_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStartEntry.Text = "" Then
            EntryError.SetError(DataRowStartEntry, "Required input data information missing.")
            Exit Sub
        End If
        If Station1Entry.Text = "" Then
            EntryError.SetError(Station1Entry, "Required input data information missing.")
            Exit Sub
        End If
        If Station2Entry.Text = "" Then
            EntryError.SetError(Station2Entry, "Required input data information missing.")
            Exit Sub
        End If
        If YAxisEntry.Text = "" Then
            EntryError.SetError(YAxisEntry, "Required y-axis information missing.")
            Exit Sub
        End If
        If StartYear1Entry.Text = "" Then
            EntryError.SetError(StartYear1Entry, "Required start year missing.")
            Exit Sub
        End If
        If StartYear2Entry.Text = "" Then
            EntryError.SetError(StartYear2Entry, "Required start year missing.")
        End If
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            If StartMonth1Entry.Text = "" Then
                EntryError.SetError(StartMonth1Entry, "Required start month missing.")
                Exit Sub
            End If
            If StartMonth2Entry.Text = "" Then
                EntryError.SetError(StartMonth2Entry, "Required start month missing.")
                Exit Sub
            End If
        End If
        If DailyFormat.Checked = True Then
            If StartDay1Entry.Text = "" Then
                EntryError.SetError(StartDay1Entry, "Required start day missing.")
                Exit Sub
            End If
            If StartDay2Entry.Text = "" Then
                EntryError.SetError(StartDay2Entry, "Required start day missing.")
                Exit Sub
            End If
            Select Case DifferencePlot_StartMonth1
                Case 1, 3, 5, 7, 8, 10, 12
                    If DifferencePlot_StartDay1 > 31 Then
                        EntryError.SetError(StartDay1Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 4, 6, 9, 11
                    If DifferencePlot_StartDay1 > 30 Then
                        EntryError.SetError(StartDay1Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 2
                    If Date.IsLeapYear(DifferencePlot_StartYear1) = True Then
                        If DifferencePlot_StartDay1 > 29 Then
                            EntryError.SetError(StartDay1Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    Else
                        If DifferencePlot_StartDay1 > 28 Then
                            EntryError.SetError(StartDay1Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    End If
            End Select
            Select Case DifferencePlot_StartMonth2
                Case 1, 3, 5, 7, 8, 10, 12
                    If DifferencePlot_StartDay2 > 31 Then
                        EntryError.SetError(StartDay2Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 4, 6, 9, 11
                    If DifferencePlot_StartDay2 > 30 Then
                        EntryError.SetError(StartDay2Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 2
                    If Date.IsLeapYear(DifferencePlot_StartYear2) = True Then
                        If DifferencePlot_StartDay2 > 29 Then
                            EntryError.SetError(StartDay2Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    Else
                        If DifferencePlot_StartDay2 > 28 Then
                            EntryError.SetError(StartDay2Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    End If
            End Select
        End If
        If EndYear1Entry.Text = "" Then
            EntryError.SetError(EndYear1Entry, "Required end year missing.")
            Exit Sub
        End If
        If EndYear2Entry.Text = "" Then
            EntryError.SetError(EndYear2Entry, "Required end year missing.")
            Exit Sub
        End If
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            If EndMonth1Entry.Text = "" Then
                EntryError.SetError(EndMonth1Entry, "Required end month missing.")
                Exit Sub
            End If
            If EndMonth2Entry.Text = "" Then
                EntryError.SetError(EndMonth2Entry, "Required end month missing.")
                Exit Sub
            End If
        End If
        If DailyFormat.Checked = True Then
            If EndDay1Entry.Text = "" Then
                EntryError.SetError(EndDay1Entry, "Required end day missing.")
                Exit Sub
            End If
            If EndDay2Entry.Text = "" Then
                EntryError.SetError(EndDay2Entry, "Required end day missing.")
                Exit Sub
            End If
            Select Case DifferencePlot_EndMonth1
                Case 1, 3, 5, 7, 8, 10, 12
                    If DifferencePlot_EndDay1 > 31 Then
                        EntryError.SetError(EndDay1Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 4, 6, 9, 11
                    If DifferencePlot_EndDay1 > 30 Then
                        EntryError.SetError(EndDay1Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 2
                    If Date.IsLeapYear(DifferencePlot_EndYear1) = True Then
                        If DifferencePlot_EndDay1 > 29 Then
                            EntryError.SetError(EndDay1Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    Else
                        If DifferencePlot_EndDay1 > 28 Then
                            EntryError.SetError(EndDay1Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    End If
            End Select
            Select Case DifferencePlot_EndMonth2
                Case 1, 3, 5, 7, 8, 10, 12
                    If DifferencePlot_EndDay2 > 31 Then
                        EntryError.SetError(EndDay2Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 4, 6, 9, 11
                    If DifferencePlot_EndDay2 > 30 Then
                        EntryError.SetError(EndDay2Entry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 2
                    If Date.IsLeapYear(DifferencePlot_EndYear2) = True Then
                        If DifferencePlot_EndDay2 > 29 Then
                            EntryError.SetError(EndDay1Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    Else
                        If DifferencePlot_EndDay2 > 28 Then
                            EntryError.SetError(EndDay1Entry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    End If
            End Select
        End If
        'Start Plot
        Plot.Text = "Plotting..."
        Plot.Enabled = False
        'Initialize Variables
        Dim i As Integer
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim StationName1 As String
        Dim StationName2 As String
        Dim Segment1Start As Date
        Dim Segment1End As Date
        Dim Segment2Start As Date
        Dim Segment2End As Date
        Dim CurrentDate As Date
        Dim StoreOnOff As Integer
        StoreOnOff = 0
        Dim SegmentNumber As Integer
        Dim Station1Value As Double
        Dim Station2Value As Double
        Dim DiffValue As Double
        Dim Segment1Diff(0) As Double
        Dim Segment2Diff(0) As Double
        Dim Segment1Data As New PointPairList
        Dim Segment1Series As LineItem
        Dim Segment2Data As New PointPairList
        Dim Segment2Series As LineItem
        Dim TDist As New MathNet.Numerics.Distributions.StudentsTDistribution
        Dim SumSegment1 As Double
        Dim SumSegment1Sq As Double
        Dim MeanSegment1 As Double
        Dim VarSegment1 As Double
        Dim NewVarSegment1 As Double
        Dim SumSegment2 As Double
        Dim SumSegment2Sq As Double
        Dim MeanSegment2 As Double
        Dim VarSegment2 As Double
        Dim NewVarSegment2 As Double
        Dim DF As Double
        Dim EstDiff As Double
        Dim TestStat As Double
        Dim p As Double
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(DifferencePlot_Input)
        Dim Line As String
        Dim Fields As String()
        'Set StreamReader to Data Start
        If DifferencePlot_RowStart <> 2 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To DifferencePlot_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
        End If
        If DailyFormat.Checked = True Then
            'Set Segment Start and End Dates
            Segment1Start = DateValue(DifferencePlot_StartMonth1 & "/" & DifferencePlot_StartDay1 & "/" & DifferencePlot_StartYear1)
            Segment1End = DateValue(DifferencePlot_EndMonth1 & "/" & DifferencePlot_EndDay1 & "/" & DifferencePlot_EndYear1)
            Segment2Start = DateValue(DifferencePlot_StartMonth2 & "/" & DifferencePlot_StartDay2 & "/" & DifferencePlot_StartYear2)
            Segment2End = DateValue(DifferencePlot_EndMonth2 & "/" & DifferencePlot_EndDay2 & "/" & DifferencePlot_EndYear2)
        End If
        If MonthlyFormat.Checked = True Then
            'Set Segment Start and End Dates
            Segment1Start = DateValue(DifferencePlot_StartMonth1 & "/1/" & DifferencePlot_StartYear1)
            Segment1End = DateValue(DifferencePlot_EndMonth1 & "/1/" & DifferencePlot_EndYear1)
            Segment2Start = DateValue(DifferencePlot_StartMonth2 & "/1/" & DifferencePlot_StartYear2)
            Segment2End = DateValue(DifferencePlot_EndMonth2 & "/1/" & DifferencePlot_EndYear2)
        End If
        'Validate Segment 2 Occurs After Segment 1
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            If Segment2Start < Segment1End Then
                MsgBox("Segment 2 should occur after Segment 1.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
                Plot.Text = "Plot"
                Plot.Enabled = True
                InFile.Close()
                Exit Sub
            End If
        Else
            If DifferencePlot_StartYear2 < DifferencePlot_EndYear1 Then
                MsgBox("Segment 2 should occur after Segment 1.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
                Plot.Text = "Plot"
                Plot.Enabled = True
                InFile.Close()
                Exit Sub
            End If
        End If
        'Input Station Names
        Line = InFile.ReadLine()
        Fields = Line.Split(ControlChars.Tab)
        Try
            StationName1 = Fields(DifferencePlot_Station1 - 1)
            StationName2 = Fields(DifferencePlot_Station2 - 1)
        Catch ex As Exception
            MsgBox("Error reading the input data near the header.  Make sure the row where the data start and the station columns are correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
            Plot.Text = "Plot"
            Plot.Enabled = True
            InFile.Close()
            Exit Sub
        End Try
        i = 0
        'Input Segment Data
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            'Daily or Monthly Format
            Try
                While InFile.Peek <> -1
                    'Read Station Data
                    Line = InFile.ReadLine()
                    Fields = Line.Split(ControlChars.Tab)
                    Year = Fields(0)
                    Month = Fields(1)
                    Select Case DailyFormat.Checked
                        Case True
                            Day = Fields(2)
                            CurrentDate = DateValue(Month & "/" & Day & "/" & Year)
                        Case False
                            CurrentDate = DateValue(Month & "/1/" & Year)
                    End Select
                    If CurrentDate = Segment1Start Then
                        'Segment 1 Start Date Reached
                        StoreOnOff = 1
                        SegmentNumber = 1
                    End If
                    If CurrentDate = Segment2Start Then
                        'Segment 2 Start Date Reached
                        StoreOnOff = 1
                        SegmentNumber = 2
                    End If
                    If StoreOnOff = 1 Then
                        Station1Value = Fields(DifferencePlot_Station1 - 1)
                        Station2Value = Fields(DifferencePlot_Station2 - 1)
                        DiffValue = Station1Value - Station2Value
                        If Station1Value <> -99 And Station2Value <> -99 Then
                            'Store Station Difference
                            Select Case SegmentNumber
                                Case 1
                                    ReDim Preserve Segment1Diff(i)
                                    Segment1Diff(i) = DiffValue
                                    Segment1Data.Add(CurrentDate.ToOADate, DiffValue)
                                Case 2
                                    ReDim Preserve Segment2Diff(i)
                                    Segment2Diff(i) = DiffValue
                                    Segment2Data.Add(CurrentDate.ToOADate, DiffValue)
                            End Select
                            i = i + 1
                        Else
                            'Denote Station Difference as Missing
                            Select Case SegmentNumber
                                Case 1
                                    Segment1Data.Add(CurrentDate.ToOADate, PointPair.Missing)
                                Case 2
                                    Segment2Data.Add(CurrentDate.ToOADate, PointPair.Missing)
                            End Select
                        End If
                    End If
                    If CurrentDate = Segment1End Or CurrentDate = Segment2End Then
                        'End Date Reached
                        StoreOnOff = 0
                        i = 0
                    End If
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    Select Case DailyFormat.Checked
                        Case True
                            MsgBox("Error reading the input data near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
                        Case False
                            MsgBox("Error reading the input data near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
                    End Select
                Else
                    MsgBox("Error reading the input data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
                End If
                Plot.Text = "Plot"
                Plot.Enabled = True
                InFile.Close()
                Exit Sub
            End Try
        Else
            'Annual Format
            Try
                While InFile.Peek <> -1
                    'Read Station Temperature Data
                    Line = InFile.ReadLine()
                    Fields = Line.Split(ControlChars.Tab)
                    Year = Fields(0)
                    If Year = DifferencePlot_StartYear1 Then
                        'Segment 1 Start Year Reached
                        StoreOnOff = 1
                        SegmentNumber = 1
                    End If
                    If Year = DifferencePlot_StartYear2 Then
                        'Segment 2 Start Year Reached
                        StoreOnOff = 1
                        SegmentNumber = 2
                    End If
                    If StoreOnOff = 1 Then
                        Station1Value = Fields(DifferencePlot_Station1 - 1)
                        Station2Value = Fields(DifferencePlot_Station2 - 1)
                        DiffValue = Station1Value - Station2Value
                        If Station1Value <> -99 And Station2Value <> -99 Then
                            'Store Station Difference
                            Select Case SegmentNumber
                                Case 1
                                    ReDim Preserve Segment1Diff(i)
                                    Segment1Diff(i) = DiffValue
                                    Segment1Data.Add(Year, DiffValue)
                                Case 2
                                    ReDim Preserve Segment2Diff(i)
                                    Segment2Diff(i) = DiffValue
                                    Segment2Data.Add(Year, DiffValue)
                            End Select
                            i = i + 1
                        Else
                            'Denote Station Difference as Missing
                            Select Case SegmentNumber
                                Case 1
                                    Segment1Data.Add(Year, PointPair.Missing)
                                Case 2
                                    Segment2Data.Add(Year, PointPair.Missing)
                            End Select
                        End If
                    End If
                    If Year = DifferencePlot_EndYear1 Or Year = DifferencePlot_EndYear2 Then
                        'End Year Reached
                        StoreOnOff = 0
                        i = 0
                    End If
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the input data near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
                Else
                    MsgBox("Error reading the input data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
                End If
                Plot.Text = "Plot"
                Plot.Enabled = True
                InFile.Close()
                Exit Sub
            End Try
        End If
        If Segment1Diff.Length = 1 Then
            'No Station Overlap in Segment 1 Due to Missing Data
            MsgBox("Unable to perform difference series due to missing station data in Segment 1.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
            Plot.Text = "Plot"
            Plot.Enabled = True
            InFile.Close()
            Exit Sub
        End If
        If Segment2Diff.Length = 1 Then
            'No Station Overlap in Segment 2 Due to Missing Data
            MsgBox("Unable to perform difference series due to missing station data in Segment 2.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Difference Series Plot")
            Plot.Text = "Plot"
            Plot.Enabled = True
            InFile.Close()
            Exit Sub
        End If
        'Calculate Segment Means
        For i = 0 To Segment1Diff.Length - 1
            SumSegment1 = SumSegment1 + Segment1Diff(i)
            SumSegment1Sq = SumSegment1Sq + Segment1Diff(i) ^ 2
        Next i
        For i = 0 To Segment2Diff.Length - 1
            SumSegment2 = SumSegment2 + Segment2Diff(i)
            SumSegment2Sq = SumSegment2Sq + Segment2Diff(i) ^ 2
        Next i
        MeanSegment1 = SumSegment1 / Segment1Diff.Length
        MeanSegment2 = SumSegment2 / Segment2Diff.Length
        EstDiff = MeanSegment1 - MeanSegment2
        'Calculate Segment Variances
        VarSegment1 = (SumSegment1Sq - SumSegment1 ^ 2 / Segment1Diff.Length) / (Segment1Diff.Length - 1)
        VarSegment2 = (SumSegment2Sq - SumSegment2 ^ 2 / Segment2Diff.Length) / (Segment2Diff.Length - 1)
        NewVarSegment1 = VarSegment1 / Segment1Diff.Length
        NewVarSegment2 = VarSegment2 / Segment2Diff.Length
        'Calculate Degrees of Freedom for T-Test
        DF = (NewVarSegment1 + NewVarSegment2) ^ 2 / ((NewVarSegment1 ^ 2 / (Segment1Diff.Length - 1)) + (NewVarSegment2 ^ 2 / (Segment2Diff.Length - 1)))
        DF = Math.Truncate(DF)
        'Perform Two-Sample T-Test (Two-Tailed)
        TestStat = EstDiff / (NewVarSegment1 + NewVarSegment2) ^ 0.5
        TDist.SetDistributionParameters(DF)
        p = TDist.CumulativeDistribution(TestStat)
        If TestStat > 0 Then
            p = 1 - p
        End If
        p = p * 2
        'Set Difference Series Chart Properties
        Dim DifferencePane As GraphPane = DifferenceChart.GraphPane
        DifferencePane.CurveList.Clear()
        DifferenceChart.ZoomOutAll(DifferencePane)
        DifferenceChart.MasterPane.Border.IsVisible = False
        DifferencePane.Border.IsVisible = False
        DifferencePane.Chart.Border.IsVisible = False
        'Set Title
        If DailyFormat.Checked = True Then
            'Daily Format
            DifferencePane.Title.Text = StationName1 & " vs. " & StationName2 & ControlChars.CrLf & Segment1Start.Month & "/" & Segment1Start.Day & "/" & Segment1Start.Year & " - " & Segment2End.Month & "/" & Segment2End.Day & "/" & Segment2End.Year
        End If
        If MonthlyFormat.Checked = True Then
            'Daily Format
            DifferencePane.Title.Text = StationName1 & " vs. " & StationName2 & ControlChars.CrLf & Segment1Start.Month & "/" & Segment1Start.Year & " - " & Segment2End.Month & "/" & Segment2End.Year
        End If
        If AnnualFormat.Checked = True Then
            'Annual Format
            DifferencePane.Title.Text = StationName1 & " vs. " & StationName2 & ControlChars.CrLf & DifferencePlot_StartYear1 & " - " & DifferencePlot_EndYear2
        End If
        DifferencePane.Title.FontSpec.Size = 16
        DifferencePane.Title.FontSpec.IsBold = True
        'Set X-Axis
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            DifferencePane.XAxis.Type = AxisType.Date
            DifferencePane.XAxis.Scale.Min = Segment1Start.ToOADate
            DifferencePane.XAxis.Scale.Max = Segment2End.ToOADate
            Select Case DailyFormat.Checked
                Case True
                    DifferencePane.XAxis.Scale.MajorUnit = DateUnit.Day
                Case False
                    DifferencePane.XAxis.Scale.MajorUnit = DateUnit.Month
            End Select
        Else
            DifferencePane.XAxis.Scale.Min = DifferencePlot_StartYear1
            DifferencePane.XAxis.Scale.Max = DifferencePlot_EndYear2
        End If
        DifferencePane.XAxis.Scale.FontSpec.Size = 12
        DifferencePane.XAxis.Scale.FontSpec.IsBold = True
        DifferencePane.XAxis.Scale.LabelGap = 0.2
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            DifferencePane.XAxis.Title.Text = "Date"
        Else
            DifferencePane.XAxis.Title.Text = "Year"
        End If
        DifferencePane.XAxis.Title.FontSpec.Size = 14
        DifferencePane.XAxis.Title.FontSpec.IsBold = True
        DifferencePane.XAxis.MajorTic.PenWidth = 2
        DifferencePane.XAxis.MajorTic.IsInside = False
        DifferencePane.XAxis.MajorTic.IsOpposite = False
        DifferencePane.XAxis.MajorTic.Size = 10
        DifferencePane.XAxis.MinorTic.IsAllTics = False
        DifferencePane.X2Axis.MajorTic.IsAllTics = False
        DifferencePane.X2Axis.MinorTic.IsAllTics = False
        'Set Y-Axis
        DifferencePane.YAxis.Scale.FontSpec.Size = 12
        DifferencePane.YAxis.Scale.FontSpec.IsBold = True
        DifferencePane.YAxis.Scale.LabelGap = 0.2
        DifferencePane.YAxis.Title.Text = DifferencePlot_YAxisLabel
        DifferencePane.YAxis.Title.FontSpec.Size = 14
        DifferencePane.YAxis.Title.FontSpec.IsBold = True
        DifferencePane.YAxis.MajorGrid.IsZeroLine = False
        DifferencePane.YAxis.MajorTic.PenWidth = 2
        DifferencePane.YAxis.MajorTic.IsInside = False
        DifferencePane.YAxis.MajorTic.IsOpposite = False
        DifferencePane.YAxis.MajorTic.Size = 10
        DifferencePane.YAxis.MinorTic.IsAllTics = False
        DifferencePane.Y2Axis.MajorTic.IsAllTics = False
        DifferencePane.Y2Axis.MinorTic.IsAllTics = False
        'Generate Difference Series Plot
        Segment1Series = DifferencePane.AddCurve("Segment 1", Segment1Data, Color.Blue, SymbolType.None)
        Segment1Series.Line.SmoothTension = 0
        Segment1Series.Line.Width = 2
        Segment2Series = DifferencePane.AddCurve("Segment 2", Segment2Data, Color.Red, SymbolType.None)
        Segment2Series.Line.SmoothTension = 0
        Segment2Series.Line.Width = 2
        'Set Legend
        DifferencePane.Legend.IsVisible = True
        DifferencePane.Legend.Border.IsVisible = False
        DifferencePane.Legend.Position = LegendPos.BottomCenter
        DifferencePane.Legend.FontSpec.Size = 10
        DifferencePane.Legend.FontSpec.IsBold = True
        'Output Test Results
        EstDiff = Math.Round(EstDiff, 2)
        p = Math.Round(p, 4)
        If p = 0 Then
            TestResults.Text = "Difference = " & EstDiff & vbCrLf & vbCrLf & "p < 0.0001"
        Else
            TestResults.Text = "Difference = " & EstDiff & vbCrLf & vbCrLf & "p = " & p
        End If
        'Calculate Axis Scale Ranges and Refresh Chart
        DifferencePane.AxisChange()
        DifferenceChart.Refresh()
        If DifferenceChart.Visible = False Then
            DifferenceChart.Visible = True
            CopyBitmap.Enabled = True
            CopyMetafile.Enabled = True
            SaveChart.Enabled = True
            PrintChart.Enabled = True
        End If
        'End Run
        InFile.Close()
        Plot.Text = "Plot"
        Plot.Enabled = True
    End Sub

    Private Sub CopyBitmap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyBitmap.Click
        Clipboard.SetImage(DifferenceChart.MasterPane.GetImage(2307, 1500, 288, False))
    End Sub

    Private Sub CopyMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyMetafile.Click
        Dim g As Graphics = Me.CreateGraphics()
        Dim hdc As System.IntPtr = g.GetHdc()
        Dim metaFile As New Imaging.Metafile(hdc, Imaging.EmfType.EmfOnly)
        g.ReleaseHdc(hdc)
        g.Dispose()
        Dim gMeta As Graphics = Graphics.FromImage(metaFile)
        DifferenceChart.MasterPane.Draw(gMeta)
        gMeta.Dispose()
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, metaFile)
    End Sub

    Private Sub SaveChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveChart.Click
        Dim SaveDifferencePlot As New SaveFileDialog
        Dim OutputImage As String
        SaveDifferencePlot.Title = "Save Difference Series Plot"
        SaveDifferencePlot.Filter = "Bitmap (*.bmp)|*.bmp|GIF (*.gif)|*.gif|JPEG (*.jpg)|*.jpg|PNG (*.png)|*.png|TIFF (*.tif)|*.tif"
        If SaveDifferencePlot.ShowDialog = Windows.Forms.DialogResult.OK Then
            OutputImage = SaveDifferencePlot.FileName
            If SaveDifferencePlot.FileName.EndsWith("bmp") Then
                DifferenceChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Bmp)
            End If
            If SaveDifferencePlot.FileName.EndsWith("gif") Then
                DifferenceChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Gif)
            End If
            If SaveDifferencePlot.FileName.EndsWith("jpg") Then
                DifferenceChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Jpeg)
            End If
            If SaveDifferencePlot.FileName.EndsWith("png") Then
                DifferenceChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Png)
            End If
            If SaveDifferencePlot.FileName.EndsWith("tif") Then
                DifferenceChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Tiff)
            End If
        End If
    End Sub

    Private Sub PrintChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrintChart.Click
        DifferenceChart.DoPrint()
    End Sub

    Private Sub DifferenceChart_ContextMenuBuilder(ByVal sender As ZedGraph.ZedGraphControl, ByVal menuStrip As System.Windows.Forms.ContextMenuStrip, ByVal mousePt As System.Drawing.Point, ByVal objState As ZedGraph.ZedGraphControl.ContextMenuObjectState) Handles DifferenceChart.ContextMenuBuilder
        'Remove Specific Functions from Difference Series Chart Context Menu for Use Elsewhere
        Dim Item As ToolStripMenuItem
        Dim CopyItem As ToolStripMenuItem
        Dim PageSetupItem As ToolStripMenuItem
        Dim PrintItem As ToolStripMenuItem
        Dim SaveAsItem As ToolStripMenuItem
        Dim SetScaleItem As ToolStripMenuItem
        For Each Item In menuStrip.Items
            If Item.Tag = "copy" Then
                CopyItem = Item
            End If
            If Item.Tag = "page_setup" Then
                PageSetupItem = Item
            End If
            If Item.Tag = "print" Then
                PrintItem = Item
            End If
            If Item.Tag = "save_as" Then
                SaveAsItem = Item
            End If
            If Item.Tag = "set_default" Then
                SetScaleItem = Item
            End If
        Next Item
        menuStrip.Items.Remove(CopyItem)
        menuStrip.Items.Remove(PageSetupItem)
        menuStrip.Items.Remove(PrintItem)
        menuStrip.Items.Remove(SaveAsItem)
        menuStrip.Items.Remove(SetScaleItem)
    End Sub

    Private Sub DifferencePlot_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        DifferencePlotDisp = 0
    End Sub
End Class