﻿Public Class DoubleMass

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Form Input/Output Variables
    Private DoubleMass_Input As String
    'Initialize Global Form Variables
    Private DoubleMass_RowStart As Integer
    Private DoubleMass_Station1 As Integer
    Private DoubleMass_Station2 As Integer
    Private DoubleMass_StationName1 As String
    Private DoubleMass_StationName2 As String
    Private DoubleMass_SegmentPoints As Integer
    Private DoubleMass_TolerancePrediction As Integer
    Private DoubleMass_Tolerance As Double
    Private DoubleMass_BreakpointYear As Integer
    Private DoubleMass_BreakpointMonth As Integer
    Private DoubleMass_Header As String
    Private DoubleMass_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub DoubleMass_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'Initialize Double Mass Analysis Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
        DoubleMass_TolerancePrediction = 1
    End Sub

    Private Sub Select_Button_Click(sender As System.Object, e As System.EventArgs) Handles Select_Button.Click
        EntryError.SetError(FileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If FileLocation.Text <> "" Then
            FileSelect.InitialDirectory = DoubleMass_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            DoubleMass_Input = FileSelect.FileName
            FileLocation.Text = DoubleMass_Input
            FileLocation.SelectionStart = FileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(sender As System.Object, e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                DoubleMass_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Station1Entry_TextChanged(sender As System.Object, e As System.EventArgs) Handles Station1Entry.TextChanged
        EntryError.SetError(Station1Entry, "")
        If Station1Entry.Text <> "" Then
            Try
                DoubleMass_Station1 = Station1Entry.Text
            Catch ex As Exception
                EntryError.SetError(Station1Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Station2Entry_TextChanged(sender As System.Object, e As System.EventArgs) Handles Station2Entry.TextChanged
        EntryError.SetError(Station2Entry, "")
        If Station2Entry.Text <> "" Then
            Try
                DoubleMass_Station2 = Station2Entry.Text
            Catch ex As Exception
                EntryError.SetError(Station2Entry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartYearEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles StartYearEntry.TextChanged
        EntryError.SetError(StartYearEntry, "")
        If StartYearEntry.Text <> "" Then
            Try
                DoubleMass_StartYear = StartYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(StartYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartMonthEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles StartMonthEntry.TextChanged
        EntryError.SetError(StartMonthEntry, "")
        If StartMonthEntry.Text <> "" Then
            Try
                DoubleMass_StartMonth = StartMonthEntry.Text
                If DoubleMass_StartMonth > 12 Or DoubleMass_StartMonth < 1 Then
                    EntryError.SetError(StartMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndYearEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles EndYearEntry.TextChanged
        EntryError.SetError(EndYearEntry, "")
        If EndYearEntry.Text <> "" Then
            Try
                DoubleMass_EndYear = EndYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(EndYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndMonthEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles EndMonthEntry.TextChanged
        EntryError.SetError(EndMonthEntry, "")
        If EndMonthEntry.Text <> "" Then
            Try
                DoubleMass_EndMonth = EndMonthEntry.Text
                If DoubleMass_EndMonth > 12 Or DoubleMass_EndMonth < 1 Then
                    EntryError.SetError(EndMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub InputFormat1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles InputFormat1.CheckedChanged
        Select Case InputFormat1.Checked
            Case True
                StartMonthEntry_Label.Enabled = True
                StartMonthEntry.Enabled = True
                EndMonthEntry_Label.Enabled = True
                EndMonthEntry.Enabled = True
                If AnalyzeBreakpoint.Checked = True Then
                    BreakpointMonthEntry_Label.Enabled = True
                    BreakpointMonthEntry.Enabled = True
                End If
            Case False
                StartMonthEntry_Label.Enabled = False
                StartMonthEntry.Enabled = False
                EndMonthEntry_Label.Enabled = False
                EndMonthEntry.Enabled = False
                If AnalyzeBreakpoint.Checked = True Then
                    BreakpointMonthEntry_Label.Enabled = False
                    BreakpointMonthEntry.Enabled = False
                End If
        End Select
    End Sub

    Private Sub DetectBreakpoint_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles DetectBreakpoint.CheckedChanged
        Select Case DetectBreakpoint.Checked
            Case True
                SegmentPointsEntry_Label.Enabled = True
                SegmentPointsEntry.Enabled = True
                ToleranceLevelEntry_Label.Enabled = True
                TolerancePredictionInterval.Enabled = True
                ToleranceSpecificValue.Enabled = True
                If ToleranceSpecificValue.Checked = True Then
                    ToleranceLevelEntry.Enabled = True
                End If
            Case False
                SegmentPointsEntry_Label.Enabled = False
                SegmentPointsEntry.Enabled = False
                ToleranceLevelEntry_Label.Enabled = False
                TolerancePredictionInterval.Enabled = False
                ToleranceSpecificValue.Enabled = False
                ToleranceLevelEntry.Enabled = False
        End Select
    End Sub

    Private Sub SegmentPointsEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles SegmentPointsEntry.TextChanged
        EntryError.SetError(SegmentPointsEntry, "")
        If SegmentPointsEntry.Text <> "" Then
            Try
                DoubleMass_SegmentPoints = SegmentPointsEntry.Text
            Catch ex As Exception
                EntryError.SetError(SegmentPointsEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TolerancePredictionInterval_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles TolerancePredictionInterval.CheckedChanged
        Select Case TolerancePredictionInterval.Checked
            Case True
                DoubleMass_TolerancePrediction = 1
                ToleranceLevelEntry.Enabled = False
            Case False
                DoubleMass_TolerancePrediction = 0
                ToleranceLevelEntry.Enabled = True
        End Select
    End Sub

    Private Sub ToleranceLevelEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles ToleranceLevelEntry.TextChanged
        EntryError.SetError(ToleranceLevelEntry, "")
        If ToleranceLevelEntry.Text <> "" Then
            Try
                DoubleMass_Tolerance = ToleranceLevelEntry.Text
            Catch ex As Exception
                EntryError.SetError(ToleranceLevelEntry, "Numerical values are the only valid characters.")
            End Try
            If DoubleMass_Tolerance < 0 Then
                'Transform Tolerance Value to Positive
                DoubleMass_Tolerance = DoubleMass_Tolerance * -1
            End If
        End If
    End Sub

    Private Sub AnalyzeBreakpoint_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles AnalyzeBreakpoint.CheckedChanged
        Select Case AnalyzeBreakpoint.Checked
            Case True
                SuspectedBreakpoint_Label.Enabled = True
                BreakpointYearEntry_Label.Enabled = True
                BreakpointYearEntry.Enabled = True
                If InputFormat1.Checked = True Then
                    BreakpointMonthEntry_Label.Enabled = True
                    BreakpointMonthEntry.Enabled = True
                End If
            Case False
                SuspectedBreakpoint_Label.Enabled = False
                BreakpointYearEntry_Label.Enabled = False
                BreakpointYearEntry.Enabled = False
                BreakpointMonthEntry_Label.Enabled = False
                BreakpointMonthEntry.Enabled = False
        End Select
    End Sub

    Private Sub BreakpointYearEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles BreakpointYearEntry.TextChanged
        EntryError.SetError(BreakpointYearEntry, "")
        If BreakpointYearEntry.Text <> "" Then
            Try
                DoubleMass_BreakpointYear = BreakpointYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(BreakpointYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub BreakpointMonthEntry_TextChanged(sender As System.Object, e As System.EventArgs) Handles BreakpointMonthEntry.TextChanged
        EntryError.SetError(BreakpointMonthEntry, "")
        If BreakpointMonthEntry.Text <> "" Then
            Try
                DoubleMass_BreakpointMonth = BreakpointMonthEntry.Text
            Catch ex As Exception
                EntryError.SetError(BreakpointMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(sender As System.Object, e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = DoubleMass_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            DoubleMass_Output = FileOutput.FileName
            OutputFileLocation.Text = DoubleMass_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(sender As System.Object, e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        DoubleMass_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(sender As System.Object, e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If DoubleMass_Input = "" Then
            EntryError.SetError(FileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input information missing.")
            Exit Sub
        End If
        If StartYearEntry.Text = "" Then
            EntryError.SetError(StartYearEntry, "Required analysis information missing.")
            Exit Sub
        End If
        If InputFormat1.Checked = True Then
            If StartMonthEntry.Text = "" Then
                EntryError.SetError(StartMonthEntry, "Required analysis information missing.")
                Exit Sub
            End If
        End If
        If EndYearEntry.Text = "" Then
            EntryError.SetError(EndYearEntry, "Required analysis information missing.")
            Exit Sub
        End If
        If InputFormat1.Checked = True Then
            If EndMonthEntry.Text = "" Then
                EntryError.SetError(EndMonthEntry, "Required analysis information missing.")
                Exit Sub
            End If
        End If
        If DetectBreakpoint.Checked = True Then
            If SegmentPointsEntry.Text = "" Then
                EntryError.SetError(SegmentPointsEntry, "Required analysis information missing.")
                Exit Sub
            End If
            If DoubleMass_SegmentPoints < 2 Then
                EntryError.SetError(SegmentPointsEntry, "At least two points required per segment.")
                Exit Sub
            End If
            If ToleranceSpecificValue.Checked = True And ToleranceLevelEntry.Text = "" Then
                EntryError.SetError(ToleranceLevelEntry, "Required analysis information missing.")
                Exit Sub
            End If
        End If
        If AnalyzeBreakpoint.Checked = True Then
            If BreakpointYearEntry.Text = "" Then
                EntryError.SetError(BreakpointYearEntry, "Required analysis information missing.")
                Exit Sub
            End If
            If InputFormat1.Checked = True Then
                If BreakpointMonthEntry.Text = "" Then
                    EntryError.SetError(BreakpointMonthEntry, "Required analysis information missing.")
                    Exit Sub
                End If
            End If
        End If
        If DoubleMass_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If DoubleMass_Output = DoubleMass_Input Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        DoubleMass_Exception = 0
        If InputFormat1.Checked = True Then
            DoubleMass_InputFormat = 1
        End If
        If InputFormat2.Checked = True Then
            DoubleMass_InputFormat = 2
        End If
        If DetectBreakpoint.Checked = True Then
            DoubleMass_BreakpointType = 1
        End If
        If AnalyzeBreakpoint.Checked = True Then
            DoubleMass_BreakpointType = 2
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(sender As System.Object, e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim FoundStart As Integer
        Dim StoreOnOff As Integer
        Dim Station1Value As Double
        Dim Station2Value As Double
        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim EndDataFlag As Integer
        Dim r As Double
        Dim p As Double
        Dim SXY As Double
        Dim SXX As Double
        Dim SYY As Double
        Dim SSE As Double
        Dim SumX As Double
        Dim SumY As Double
        Dim SumXY As Double
        Dim SumXSq As Double
        Dim SumYSq As Double
        Dim TotalPoints As Integer
        Dim StartPoint As Integer
        Dim EndPoint As Integer
        Dim SuspectedBreakpoint As Integer
        Dim MeanX As Double
        Dim StdDevX As Double
        Dim B0 As Double
        Dim B1 As Double
        Dim StdErr As Double
        Dim TValue As Double
        Dim YEstimated As Double
        Dim EstimateError As Double
        Dim Breakpoints As Integer
        Dim SegmentNumber As Integer
        Dim TestStat As Double
        Dim NDist As New MathNet.Numerics.Distributions.NormalDistribution
        Dim TDist As New MathNet.Numerics.Distributions.StudentsTDistribution
        FoundStart = 0
        i = 0
        j = 0
        k = 0
        EndDataFlag = 0
        'Initialize Output File and Set Header
        FileOpen(1, DoubleMass_Output, OpenMode.Output)
        PrintLine(1, DoubleMass_Header)
        'Initialize Observation Arrays
        Dim IndData(0) As Double
        Dim CumIndData(0) As Double
        Dim DepData(0) As Double
        Dim CumDepData(0) As Double
        Dim YearData(0) As Integer
        Dim MonthData(0) As Integer
        Dim SegmentEnd() As Integer
        Dim SegmentSlope() As Double
        Dim SegmentStdErr() As Double
        Dim SetNumber(0) As Integer
        'Initialize StreamReaders
        Dim InFile As New System.IO.StreamReader(DoubleMass_Input)
        Dim Line As String
        Dim Fields As String()
        'Set StreamReader to Data Start
        If DoubleMass_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To DoubleMass_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
        End If
        'Input Station Names
        Line = InFile.ReadLine()
        Fields = Line.Split(ControlChars.Tab)
        Try
            DoubleMass_StationName1 = Fields(DoubleMass_Station1 - 1)
            DoubleMass_StationName2 = Fields(DoubleMass_Station2 - 1)
        Catch ex As Exception
            MsgBox("Error reading the input data near the header.  Make sure the row where the data start and the station columns are correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
            InFile.Close()
            FileClose(1)
            DoubleMass_Exception = 1
            Exit Sub
        End Try
        'Initialize Progress Bar
        Dim Percent As Integer
        Dim RunProgress As Integer
        Dim RunComplete As Integer
        Select Case DoubleMass_InputFormat
            Case 1
                'Monthly Format
                'Total Months for Known Complete Years
                RunComplete = DoubleMass_EndYear - DoubleMass_StartYear - 1
                RunComplete = RunComplete * 12
                'Total Months for Potentially Incomplete Years
                RunComplete = RunComplete + (12 - DoubleMass_StartMonth + 1)
                RunComplete = RunComplete + DoubleMass_EndMonth
            Case 2
                'Annual Format
                RunComplete = DoubleMass_EndYear - DoubleMass_StartYear + 1
        End Select
        RunComplete = RunComplete + 1
        'Input Data
        Try
            While InFile.Peek <> -1
                Line = InFile.ReadLine()
                Fields = Line.Split(ControlChars.Tab)
                Year = Fields(0)
                If DoubleMass_InputFormat = 1 Then
                    Month = Fields(1)
                End If
                If Year = DoubleMass_StartYear Then
                    If DoubleMass_InputFormat = 1 Then
                        If Month = DoubleMass_StartMonth Then
                            'Monthly Format...Start Year/Month Reached...Turn On Store
                            StoreOnOff = 1
                            FoundStart = 1
                        End If
                    Else
                        'Annual Format...Start Year Reached...Turn On Store
                        StoreOnOff = 1
                        FoundStart = 1
                    End If
                End If
                If StoreOnOff = 1 Then
                    'Store Station Data in Array
                    Station1Value = Fields(DoubleMass_Station1 - 1)
                    Station2Value = Fields(DoubleMass_Station2 - 1)
                    If Station1Value <> -99 And Station2Value <> -99 Then
                        ReDim Preserve YearData(i)
                        ReDim Preserve MonthData(i)
                        ReDim Preserve IndData(i)
                        ReDim Preserve DepData(i)
                        ReDim Preserve CumIndData(i)
                        ReDim Preserve CumDepData(i)
                        YearData(i) = Year
                        If DoubleMass_InputFormat = 1 Then
                            MonthData(i) = Month
                        End If
                        IndData(i) = Station1Value
                        DepData(i) = Station2Value
                        If i = 0 Then
                            CumIndData(i) = Station1Value
                            CumDepData(i) = Station2Value
                            SumX = Station1Value
                            SumY = Station2Value
                        Else
                            CumIndData(i) = CumIndData(i - 1) + Station1Value
                            CumDepData(i) = CumDepData(i - 1) + Station2Value
                            SumX = SumX + Station1Value
                            SumY = SumY + Station2Value
                        End If
                        i = i + 1
                    End If
                    'Update Progress Bar
                    RunProgress = RunProgress + 1
                    Percent = (RunProgress / RunComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End If
                If Year = DoubleMass_EndYear Then
                    If DoubleMass_InputFormat = 1 Then
                        If Month = DoubleMass_EndMonth Then
                            'Monthly Format...End Year/Month Reached...Turn Off Store
                            StoreOnOff = 0
                        End If
                    Else
                        'Annual Format...End Year Reached...Turn Off Store
                        StoreOnOff = 0
                    End If
                End If
            End While
        Catch ex As Exception
            If Year <> 0 Then
                If DoubleMass_InputFormat = 1 Then
                    MsgBox("Error reading the input data near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
                Else
                    MsgBox("Error reading the input data near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
                End If
            Else
                MsgBox("Error reading the input data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
            End If
            InFile.Close()
            FileClose(1)
            DoubleMass_Exception = 1
            Exit Sub
        End Try
        If FoundStart = 0 Then
            If DoubleMass_InputFormat = 1 Then
                MsgBox("Unable to find start year/month.  Recheck the start year/month.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
            Else
                MsgBox("Unable to find start year.  Recheck the start year.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
            End If
            InFile.Close()
            FileClose(1)
            DoubleMass_Exception = 1
            Exit Sub
        End If
        'Perform Analysis
        If DoubleMass_BreakpointType = 1 Then
            'Detect Breakpoints Sequence
            'Calculate Monthly Correlation Between Independent and Dependent Data
            For j = 0 To i - 1
                SXY = SXY + (IndData(j) - (SumX / i)) * (DepData(j) - (SumY / i))
                SXX = SXX + ((IndData(j) - (SumX / i)) ^ 2)
                SYY = SYY + ((DepData(j) - (SumY / i)) ^ 2)
            Next j
            r = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
            r = Math.Round(r, 3)
            'Output Correlation Value
            PrintLine(1, "")
            PrintLine(1, "r = " & r)
            PrintLine(1, "")
            Select Case DoubleMass_InputFormat
                Case 1
                    'Data in Monthly Format
                    PrintLine(1, "Year" & vbTab & "Month" & vbTab & DoubleMass_StationName1 & vbTab & DoubleMass_StationName2 & vbTab & "Flag")
                Case 2
                    'Data in Annual Format
                    PrintLine(1, "Year" & vbTab & DoubleMass_StationName1 & vbTab & DoubleMass_StationName2 & vbTab & "Flag")
            End Select
            'Flag Breakpoints
            StartPoint = 0
            EndPoint = DoubleMass_SegmentPoints - 1
            If i - 1 < EndPoint Then
                MsgBox("There are not enough points available in the dataset to analyze a segment that is " & DoubleMass_SegmentPoints & " points in length.  Please reduce the initial number of points to analyze per segment.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
                InFile.Close()
                FileClose(1)
                DoubleMass_Exception = 1
                Exit Sub
            End If
            Do
                'Derive Regression Line for Segment
                SumX = 0
                SumY = 0
                SumXY = 0
                SumXSq = 0
                SumYSq = 0
                SXY = 0
                SXX = 0
                TotalPoints = EndPoint - StartPoint + 1
                For j = StartPoint To EndPoint
                    SumX = SumX + CumIndData(j)
                    SumY = SumY + CumDepData(j)
                    SumXY = SumXY + CumIndData(j) * CumDepData(j)
                    SumXSq = SumXSq + CumIndData(j) ^ 2
                    SumYSq = SumYSq + CumDepData(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / TotalPoints
                SXX = SumXSq - SumX ^ 2 / TotalPoints
                'Calculate Mean
                MeanX = SumX / TotalPoints
                'Calculate Standard Deviation
                StdDevX = SXX / (TotalPoints - 1)
                StdDevX = Math.Sqrt(StdDevX)
                'Calculate Slope B1
                B1 = SXY / SXX
                'Calculate Intercept B0
                B0 = (SumY - B1 * SumX) / TotalPoints
                'Calculate Standard Error of Slope Estimate
                SSE = SumYSq - B0 * SumY - B1 * SumXY
                StdErr = ((SSE / (TotalPoints - 2)) ^ 0.5) / (SXX) ^ 0.5
                'Estimate EndPoint+1
                If i - 1 < EndPoint + 1 Then
                    'End of Data Exit Do Loop
                    Exit Do
                Else
                    YEstimated = B0 + B1 * CumIndData(EndPoint + 1)
                End If
                'Calculate Prediction Interval for Tolerance Level...If Necessary
                If DoubleMass_TolerancePrediction = 1 Then
                    TValue = GetTValue(TotalPoints - 1)
                    DoubleMass_Tolerance = MeanX + TValue * StdDevX * Math.Sqrt(1 + (1 / TotalPoints))
                End If
                'Check for Breach of Tolerance Level
                EstimateError = YEstimated - CumDepData(EndPoint + 1)
                If EstimateError < 0 Then
                    'Transform EstimateError to Positive Value for Comparison
                    EstimateError = EstimateError * -1
                End If
                If EstimateError > DoubleMass_Tolerance Then
                    'Tolerance Level Breached
                    'Save EndPoint to SegmentEnd Array
                    ReDim Preserve SegmentEnd(Breakpoints)
                    SegmentEnd(Breakpoints) = EndPoint
                    'Save Slope of Line Segment to SegmentSlope Array
                    ReDim Preserve SegmentSlope(Breakpoints)
                    SegmentSlope(Breakpoints) = Math.Round(B1, 4)
                    'Save Standard Error of Slope Estimate to SegmentStdErr Array
                    ReDim Preserve SegmentStdErr(Breakpoints)
                    SegmentStdErr(Breakpoints) = Math.Round(StdErr, 4)
                    Breakpoints = Breakpoints + 1
                    If i - 1 = EndPoint + 1 Then
                        'End of Data Next, So Report Segment Slope and Segment Slope with End of Data Point
                        EndPoint = EndPoint + 1
                        'Turn On Special End Data Flag
                        EndDataFlag = 1
                    Else
                        'Reset StartPoint
                        StartPoint = EndPoint + 1
                        'Reset EndPoint
                        EndPoint = StartPoint + DoubleMass_SegmentPoints - 1
                        If i - 1 < EndPoint Then
                            'Short Segment so Reset EndPoint to End of Data
                            EndPoint = i - 1
                        End If
                    End If
                Else
                    'Increase Size of Line Segment
                    EndPoint = EndPoint + 1
                End If
            Loop
            'Save EndPoint to SegmentEnd Array Even If No Breakpoints
            ReDim Preserve SegmentEnd(Breakpoints)
            SegmentEnd(Breakpoints) = EndPoint
            'Save Slope of Line Segment to SegmentSlope Array Even If No Breakpoints
            ReDim Preserve SegmentSlope(Breakpoints)
            SegmentSlope(Breakpoints) = Math.Round(B1, 4)
            'Save Standard Error of Slope Estimate to SegmentStdErr Array Even If No Breakpoints
            ReDim Preserve SegmentStdErr(Breakpoints)
            SegmentStdErr(Breakpoints) = Math.Round(StdErr, 4)
            Select Case Breakpoints
                Case 0
                    'No Breakpoints Detected...Print Results
                    For j = StartPoint To EndPoint
                        Select Case DoubleMass_InputFormat
                            Case 1
                                'Data in Monthly Format
                                PrintLine(1, YearData(j) & vbTab & MonthData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                            Case 2
                                'Data in Annual Format
                                PrintLine(1, YearData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                        End Select
                    Next j
                Case Is > 0
                    'Breakpoints Detected...Run Slope Test Sequence and Print Results
                    StartPoint = 0
                    For SegmentNumber = 0 To Breakpoints - 1
                        EndPoint = SegmentEnd(SegmentNumber)
                        'Print Segment to EndPoint - 1
                        For j = StartPoint To EndPoint - 1
                            Select Case DoubleMass_InputFormat
                                Case 1
                                    'Data in Monthly Format
                                    PrintLine(1, YearData(j) & vbTab & MonthData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                                Case 2
                                    'Data in Annual Format
                                    PrintLine(1, YearData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                            End Select
                        Next j
                        'Test for Difference in Slopes (Two-Tailed)
                        TestStat = (SegmentSlope(SegmentNumber) - SegmentSlope(SegmentNumber + 1)) / ((SegmentStdErr(SegmentNumber) ^ 2 + SegmentStdErr(SegmentNumber + 1) ^ 2) ^ 0.5)
                        Try
                            p = NDist.CumulativeDistribution(TestStat)
                        Catch ex As Exception
                            'Test Statistic Infinitely Large...Set p to Zero
                            p = 0
                        End Try
                        If TestStat > 0 Then
                            p = 1 - p
                        End If
                        p = p * 2
                        p = Math.Round(p, 4)
                        'Print Data at EndPoint
                        Select Case EndDataFlag
                            Case 0
                                'No Special Flag Required
                                If p = 0 Then
                                    Select Case DoubleMass_InputFormat
                                        Case 1
                                            'Data in Monthly Format
                                            PrintLine(1, YearData(EndPoint) & vbTab & MonthData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                        Case 2
                                            'Data in Annual Format
                                            PrintLine(1, YearData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                    End Select
                                Else
                                    Select Case DoubleMass_InputFormat
                                        Case 1
                                            'Data in Monthly Format
                                            PrintLine(1, YearData(EndPoint) & vbTab & MonthData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                        Case 2
                                            'Data in Annual Format
                                            PrintLine(1, YearData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                    End Select
                                End If
                            Case 1
                                'Special Flag Required At Last BreakPoint
                                If SegmentNumber = Breakpoints - 1 Then
                                    If p = 0 Then
                                        Select Case DoubleMass_InputFormat
                                            Case 1
                                                'Data in Monthly Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & MonthData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "# p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                            Case 2
                                                'Data in Annual Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "# p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                        End Select
                                    Else
                                        Select Case DoubleMass_InputFormat
                                            Case 1
                                                'Data in Monthly Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & MonthData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "# p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                            Case 2
                                                'Data in Annual Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "# p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                        End Select
                                    End If
                                Else
                                    If p = 0 Then
                                        Select Case DoubleMass_InputFormat
                                            Case 1
                                                'Data in Monthly Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & MonthData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                            Case 2
                                                'Data in Annual Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                        End Select
                                    Else
                                        Select Case DoubleMass_InputFormat
                                            Case 1
                                                'Data in Monthly Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & MonthData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                            Case 2
                                                'Data in Annual Format
                                                PrintLine(1, YearData(EndPoint) & vbTab & Math.Round(CumIndData(EndPoint), 2) & vbTab & Math.Round(CumDepData(EndPoint), 2) & vbTab & "* p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                                        End Select
                                    End If
                                End If
                        End Select
                        'Reset StartPoint
                        StartPoint = EndPoint + 1
                    Next SegmentNumber
                    'Print Final Segment
                    For j = StartPoint To i - 1
                        Select Case DoubleMass_InputFormat
                            Case 1
                                'Data in Monthly Format
                                PrintLine(1, YearData(j) & vbTab & MonthData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                            Case 2
                                'Data in Annual Format
                                PrintLine(1, YearData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                        End Select
                    Next j
            End Select
        End If
        If DoubleMass_BreakpointType = 2 Then
            'Analyze Suspected Breakpoint Sequence
            ReDim SetNumber(i - 1)
            SegmentNumber = 0
            For j = 0 To i - 1
                SetNumber(j) = SegmentNumber
                'Check if Suspected Breakpoint
                Select Case DoubleMass_InputFormat
                    Case 1
                        'Data in Monthly Format
                        If DoubleMass_BreakpointYear = YearData(j) Then
                            If DoubleMass_BreakpointMonth = MonthData(j) Then
                                'Suspected Breakpoint Reached
                                SuspectedBreakpoint = j
                                SegmentNumber = 1
                            End If
                        End If
                    Case 2
                        'Data in Annual Format
                        If DoubleMass_BreakpointYear = YearData(j) Then
                            'Suspected Breakpoint Reached
                            SuspectedBreakpoint = j
                            SegmentNumber = 1
                        End If
                End Select
            Next j
            'Calculate Monthly Correlation Between Independent and Dependent Data
            For j = 0 To i - 1
                SXY = SXY + (IndData(j) - (CumIndData(i - 1) / i)) * (DepData(j) - (CumDepData(i - 1) / i))
                SXX = SXX + ((IndData(j) - (CumIndData(i - 1) / i)) ^ 2)
                SYY = SYY + ((DepData(j) - (CumDepData(i - 1) / i)) ^ 2)
            Next j
            r = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
            r = Math.Round(r, 3)
            'Output Correlation Value
            PrintLine(1, "")
            PrintLine(1, "r = " & r)
            PrintLine(1, "")
            Select Case DoubleMass_InputFormat
                Case 1
                    'Data in Monthly Format
                    PrintLine(1, "Year" & vbTab & "Month" & vbTab & DoubleMass_StationName1 & vbTab & DoubleMass_StationName2 & vbTab & "Flag")
                Case 2
                    'Data in Annual Format
                    PrintLine(1, "Year" & vbTab & DoubleMass_StationName1 & vbTab & DoubleMass_StationName2 & vbTab & "Flag")
            End Select
            'Set First Segment StartPoint and EndPoint
            StartPoint = 0
            EndPoint = SuspectedBreakpoint
            'Prepare Segment Slope and Standard Error Arrays
            ReDim SegmentSlope(1)
            ReDim SegmentStdErr(1)
            For j = 0 To 1
                'Derive Regression Line for Segment
                SumX = 0
                SumY = 0
                SumXY = 0
                SumXSq = 0
                SumYSq = 0
                SXY = 0
                SXX = 0
                TotalPoints = EndPoint - StartPoint + 1
                For k = StartPoint To EndPoint
                    SumX = SumX + CumIndData(k)
                    SumY = SumY + CumDepData(k)
                    SumXY = SumXY + CumIndData(k) * CumDepData(k)
                    SumXSq = SumXSq + CumIndData(k) ^ 2
                    SumYSq = SumYSq + CumDepData(k) ^ 2
                Next k
                SXY = SumXY - SumX * SumY / TotalPoints
                SXX = SumXSq - SumX ^ 2 / TotalPoints
                'Calculate Slope B1
                B1 = SXY / SXX
                'Calculate Intercept B0
                B0 = (SumY - B1 * SumX) / TotalPoints
                'Calculate Standard Error of Slope Estimate
                SSE = SumYSq - B0 * SumY - B1 * SumXY
                StdErr = ((SSE / (TotalPoints - 2)) ^ 0.5) / (SXX) ^ 0.5
                'Save Slope and Standard Error to Arrays
                SegmentSlope(j) = Math.Round(B1, 4)
                SegmentStdErr(j) = Math.Round(StdErr, 4)
                'Reset StartPoint and EndPoint for Second Segment
                StartPoint = SuspectedBreakpoint + 1
                EndPoint = i - 1
            Next j
            'Test for Difference in Slopes (Two-Tailed)
            TestStat = (SegmentSlope(0) - SegmentSlope(1)) / ((SegmentStdErr(0) ^ 2 + SegmentStdErr(1) ^ 2) ^ 0.5)
            Try
                p = NDist.CumulativeDistribution(TestStat)
            Catch ex As Exception
                'Test Statistic Infinitely Large...Set p to Zero
                p = 0
            End Try
            If TestStat > 0 Then
                p = 1 - p
            End If
            p = p * 2
            p = Math.Round(p, 4)
            SegmentNumber = 0
            'Print Results
            For j = 0 To i - 1
                If j = SuspectedBreakpoint Then
                    'Flag Required
                    If p = 0 Then
                        Select Case DoubleMass_InputFormat
                            Case 1
                                'Data in Monthly Format
                                PrintLine(1, YearData(j) & vbTab & MonthData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & "* p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                            Case 2
                                'Data in Annual Format
                                PrintLine(1, YearData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & "* p < 0.0001, Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                        End Select
                    Else
                        Select Case DoubleMass_InputFormat
                            Case 1
                                'Data in Monthly Format
                                PrintLine(1, YearData(j) & vbTab & MonthData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & "* p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                            Case 2
                                'Data in Annual Format
                                PrintLine(1, YearData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & "* p = " & p & ", Slope 1 = " & SegmentSlope(SegmentNumber) & ", Slope 2 = " & SegmentSlope(SegmentNumber + 1))
                        End Select
                    End If
                    SegmentNumber = 1
                Else
                    'No Flag
                    Select Case DoubleMass_InputFormat
                        Case 1
                            'Data in Monthly Format
                            PrintLine(1, YearData(j) & vbTab & MonthData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                        Case 2
                            'Data in Annual Format
                            PrintLine(1, YearData(j) & vbTab & Math.Round(CumIndData(j), 2) & vbTab & Math.Round(CumDepData(j), 2) & vbTab & " ")
                    End Select
                End If
            Next j
        End If
        'Update Progress Bar
        RunProgress = RunProgress + 1
        Percent = (RunProgress / RunComplete) * 100
        BackgroundRun.ReportProgress(Percent)
        'Close Files
        InFile.Close()
        FileClose(1)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(sender As Object, e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(sender As Object, e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case DoubleMass_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
                'Show Figure
                If DoubleMassFigDisp = 0 Then
                    DoubleMassFigDisp = 1
                    Dim frmDoubleMassFig As New DoubleMassFig
                    frmDoubleMassFig.MdiParent = MainParent
                    frmDoubleMassFig.Show()
                Else
                    MsgBox("Only one time series can be displayed at a time.  Please close the displayed time series, and run the Double Mass Analysis tool again.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Double Mass Analysis")
                End If
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub DoubleMass_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        DoubleMassDisp = 0
    End Sub
End Class