﻿Imports ZedGraph
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Public Class DoubleMassFig

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub DoubleMassFig_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        HOBToolsHelp.HelpNamespace = HOBToolsUG
        'Call Sub to Draw Double Mass Plot
        DrawDoubleMassPlot()
    End Sub

    Private Sub DrawDoubleMassPlot()
        Dim Header As String
        Dim StationName1 As String
        Dim StationName2 As String
        Dim RegularPointList As New PointPairList()
        Dim NotSigPointList As New PointPairList()
        Dim SigPointList As New PointPairList()
        Dim FlagFields As Char()
        Dim PValueField As String
        Dim p As Double
        Dim i As Integer
        Dim j As Integer
        Dim x As Double
        Dim y As Double
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(DoubleMass_Output)
        Dim Line As String
        Dim Fields As String()
        'Read Header
        Header = InFile.ReadLine()
        'Skip to Column Headers
        Line = InFile.ReadLine()
        Line = InFile.ReadLine()
        Line = InFile.ReadLine()
        'Import Station Names
        Line = InFile.ReadLine()
        Fields = Line.Split(ControlChars.Tab)
        Select Case DoubleMass_InputFormat
            Case 1
                'Data in Monthly Format
                StationName1 = Fields(2)
                StationName2 = Fields(3)
            Case 2
                'Data in Annual Format
                StationName1 = Fields(1)
                StationName2 = Fields(2)
        End Select
        'Import Data
        While InFile.Peek <> -1
            Line = InFile.ReadLine
            Fields = Line.Split(ControlChars.Tab)
            Select Case DoubleMass_InputFormat
                Case 1
                    'Data in Monthly Format
                    i = 2
                Case 2
                    'Data in Annual Format
                    i = 1
            End Select
            x = Fields(i)
            y = Fields(i + 1)
            If Fields(i + 2) <> " " Then
                'Detected Breakpoint
                'Extract P-Value
                FlagFields = Fields(i + 2).ToCharArray
                PValueField = ""
                j = 6
                Do
                    PValueField = PValueField + FlagFields(j)
                    j = j + 1
                    If FlagFields(j) = "," Then
                        Exit Do
                    End If
                Loop
                p = PValueField
                'Check P-Value for Significance
                If p < 0.05 Then
                    'Breakpoint Significant
                    SigPointList.Add(x, y)
                Else
                    'Breakpoint Not Significant
                    NotSigPointList.Add(x, y)
                End If
            Else
                'Regular Data Point
                RegularPointList.Add(x, y)
            End If
        End While
        InFile.Close()
        'Set Double Mass Chart Properties
        Dim DoubleMassPane As GraphPane = DoubleMassChart.GraphPane
        DoubleMassPane.CurveList.Clear()
        DoubleMassChart.ZoomOutAll(DoubleMassPane)
        DoubleMassChart.MasterPane.Border.IsVisible = False
        DoubleMassPane.Border.IsVisible = False
        DoubleMassPane.Chart.Border.IsVisible = False
        'Title
        Select Case DoubleMass_InputFormat
            Case 1
                'Data in Monthly Format
                DoubleMassPane.Title.Text = Header & ControlChars.CrLf & DoubleMass_StartMonth & "/" & DoubleMass_StartYear & " - " & DoubleMass_EndMonth & "/" & DoubleMass_EndYear
            Case 2
                'Data in Annual Format
                DoubleMassPane.Title.Text = Header & ControlChars.CrLf & DoubleMass_StartYear & " - " & DoubleMass_EndYear
        End Select
        DoubleMassPane.Title.FontSpec.Size = 16
        DoubleMassPane.Title.FontSpec.IsBold = True
        'X-Axis
        DoubleMassPane.XAxis.Scale.FontSpec.Size = 12
        DoubleMassPane.XAxis.Scale.FontSpec.IsBold = True
        DoubleMassPane.XAxis.Scale.LabelGap = 0.2
        DoubleMassPane.XAxis.Title.Text = StationName1
        DoubleMassPane.XAxis.Title.FontSpec.Size = 14
        DoubleMassPane.XAxis.Title.FontSpec.IsBold = True
        DoubleMassPane.XAxis.MajorTic.PenWidth = 2
        DoubleMassPane.XAxis.MajorTic.IsInside = False
        DoubleMassPane.XAxis.MajorTic.IsOpposite = False
        DoubleMassPane.XAxis.MinorTic.IsAllTics = False
        DoubleMassPane.X2Axis.MajorTic.IsAllTics = False
        DoubleMassPane.X2Axis.MinorTic.IsAllTics = False
        'Y-Axis
        DoubleMassPane.YAxis.Scale.FontSpec.Size = 12
        DoubleMassPane.YAxis.Scale.FontSpec.IsBold = True
        DoubleMassPane.YAxis.Scale.LabelGap = 0.2
        DoubleMassPane.YAxis.Title.Text = StationName2
        DoubleMassPane.YAxis.Title.FontSpec.Size = 14
        DoubleMassPane.YAxis.Title.FontSpec.IsBold = True
        DoubleMassPane.YAxis.MajorGrid.IsZeroLine = False
        DoubleMassPane.YAxis.MajorTic.PenWidth = 2
        DoubleMassPane.YAxis.MajorTic.IsInside = False
        DoubleMassPane.YAxis.MajorTic.IsOpposite = False
        DoubleMassPane.YAxis.MinorTic.IsAllTics = False
        DoubleMassPane.Y2Axis.MajorTic.IsAllTics = False
        DoubleMassPane.Y2Axis.MinorTic.IsAllTics = False
        'Legend
        DoubleMassPane.Legend.Border.IsVisible = False
        DoubleMassPane.Legend.Position = LegendPos.BottomCenter
        DoubleMassPane.Legend.IsHStack = False
        DoubleMassPane.Legend.Location.AlignV = AlignV.Top
        DoubleMassPane.Legend.Location.AlignH = AlignH.Left
        DoubleMassPane.Legend.FontSpec.Size = 10
        DoubleMassPane.Legend.FontSpec.IsBold = True
        'Generate Scatter Plots
        Dim RegularPointPlot As LineItem
        Dim NotSigPointPlot As LineItem
        Dim SigPointPlot As LineItem
        SigPointPlot = DoubleMassPane.AddCurve("Significant Breakpoint", SigPointList, Color.Red, SymbolType.Diamond)
        SigPointPlot.Line.IsVisible = False
        SigPointPlot.Symbol.Size = 8
        SigPointPlot.Symbol.Fill = New Fill(Brushes.Red)
        NotSigPointPlot = DoubleMassPane.AddCurve("Potential Breakpoint (Not Significant)", NotSigPointList, Color.Orange, SymbolType.Circle)
        NotSigPointPlot.Line.IsVisible = False
        NotSigPointPlot.Symbol.Size = 6
        NotSigPointPlot.Symbol.Fill = New Fill(Brushes.Orange)
        RegularPointPlot = DoubleMassPane.AddCurve("Line Segment", RegularPointList, Color.Blue, SymbolType.Circle)
        RegularPointPlot.Line.IsVisible = False
        RegularPointPlot.Symbol.Size = 6
        RegularPointPlot.Symbol.Fill = New Fill(Brushes.Blue)
        'Calculate Axis Scale Ranges and Refresh Chart
        DoubleMassPane.AxisChange()
        DoubleMassChart.Refresh()
    End Sub

    Private Sub CopyBitmap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyBitmap.Click
        Clipboard.SetImage(DoubleMassChart.MasterPane.GetImage(2307, 1500, 288, False))
    End Sub

    Private Sub CopyMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyMetafile.Click
        Dim g As Graphics = Me.CreateGraphics()
        Dim hdc As System.IntPtr = g.GetHdc()
        Dim metaFile As New Imaging.Metafile(hdc, Imaging.EmfType.EmfOnly)
        g.ReleaseHdc(hdc)
        g.Dispose()
        Dim gMeta As Graphics = Graphics.FromImage(metaFile)
        DoubleMassChart.MasterPane.Draw(gMeta)
        gMeta.Dispose()
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, metaFile)
    End Sub

    Private Sub SaveChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveChart.Click
        Dim SaveDoubleMassPlot As New SaveFileDialog
        Dim OutputImage As String
        SaveDoubleMassPlot.Title = "Save Double Mass Plot"
        SaveDoubleMassPlot.Filter = "Bitmap (*.bmp)|*.bmp|GIF (*.gif)|*.gif|JPEG (*.jpg)|*.jpg|PNG (*.png)|*.png|TIFF (*.tif)|*.tif"
        If SaveDoubleMassPlot.ShowDialog = Windows.Forms.DialogResult.OK Then
            OutputImage = SaveDoubleMassPlot.FileName
            If SaveDoubleMassPlot.FileName.EndsWith("bmp") Then
                DoubleMassChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Bmp)
            End If
            If SaveDoubleMassPlot.FileName.EndsWith("gif") Then
                DoubleMassChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Gif)
            End If
            If SaveDoubleMassPlot.FileName.EndsWith("jpg") Then
                DoubleMassChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Jpeg)
            End If
            If SaveDoubleMassPlot.FileName.EndsWith("png") Then
                DoubleMassChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Png)
            End If
            If SaveDoubleMassPlot.FileName.EndsWith("tif") Then
                DoubleMassChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Tiff)
            End If
        End If
    End Sub

    Private Sub PrintChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrintChart.Click
        DoubleMassChart.DoPrint()
    End Sub

    Private Sub DoubleMassChart_ContextMenuBuilder(ByVal sender As ZedGraph.ZedGraphControl, ByVal menuStrip As System.Windows.Forms.ContextMenuStrip, ByVal mousePt As System.Drawing.Point, ByVal objState As ZedGraph.ZedGraphControl.ContextMenuObjectState) Handles DoubleMassChart.ContextMenuBuilder
        'Remove Specific Functions from Double Mass Chart Context Menu for Use Elsewhere
        Dim Item As ToolStripMenuItem
        Dim CopyItem As ToolStripMenuItem
        Dim PageSetupItem As ToolStripMenuItem
        Dim PrintItem As ToolStripMenuItem
        Dim SaveAsItem As ToolStripMenuItem
        Dim SetScaleItem As ToolStripMenuItem
        For Each Item In menuStrip.Items
            If Item.Tag = "copy" Then
                CopyItem = Item
            End If
            If Item.Tag = "page_setup" Then
                PageSetupItem = Item
            End If
            If Item.Tag = "print" Then
                PrintItem = Item
            End If
            If Item.Tag = "save_as" Then
                SaveAsItem = Item
            End If
            If Item.Tag = "set_default" Then
                SetScaleItem = Item
            End If
        Next Item
        menuStrip.Items.Remove(CopyItem)
        menuStrip.Items.Remove(PageSetupItem)
        menuStrip.Items.Remove(PrintItem)
        menuStrip.Items.Remove(SaveAsItem)
        menuStrip.Items.Remove(SetScaleItem)
    End Sub

    Private Sub DoubleMassFig_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        DoubleMassFigDisp = 0
    End Sub
End Class