Public Class EstimatePrecip

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Form Input/Output Variables
    Private EstimatePrecip_AllPrecipInput As String
    Private EstimatePrecip_ThresholdPrecipInput As String
    Private EstimatePrecip_Output As String
    Private EstimatePrecip_OutputStats As String
    'Initialize Global Form Variables
    Private EstimatePrecip_AllPrecipRowStart As Integer
    Private EstimatePrecip_ThresholdPrecipRowStart As Integer
    Private EstimatePrecip_AdjStartYear As Integer
    Private EstimatePrecip_AdjStartMonth As Integer
    Private EstimatePrecip_AdjEndYear As Integer
    Private EstimatePrecip_AdjEndMonth As Integer
    Private EstimatePrecip_DerStartYear As Integer
    Private EstimatePrecip_DerStartMonth As Integer
    Private EstimatePrecip_DerEndYear As Integer
    Private EstimatePrecip_DerEndMonth As Integer
    Private EstimatePrecip_PrecipType As Integer
    Private EstimatePrecip_InputFormat As Double
    Private EstimatePrecip_Header As String
    Private EstimatePrecip_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub EstimatePrecip_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Estimate Precipitation Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub AllInputSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AllInputSelect_Button.Click
        EntryError.SetError(AllFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select the All Precipitation Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If AllFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = EstimatePrecip_AllPrecipInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            EstimatePrecip_AllPrecipInput = FileSelect.FileName
            AllFileLocation.Text = EstimatePrecip_AllPrecipInput
            AllFileLocation.SelectionStart = AllFileLocation.Text.Length
        End If
    End Sub

    Private Sub AllDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AllDataRowStart.TextChanged
        EntryError.SetError(AllDataRowStart, "")
        If AllDataRowStart.Text <> "" Then
            Try
                EstimatePrecip_AllPrecipRowStart = AllDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(AllDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ThresholdInputSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ThresholdInputSelect_Button.Click
        EntryError.SetError(ThresholdFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select the Threshold Precipitation Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If ThresholdFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = EstimatePrecip_ThresholdPrecipInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            EstimatePrecip_ThresholdPrecipInput = FileSelect.FileName
            ThresholdFileLocation.Text = EstimatePrecip_ThresholdPrecipInput
            ThresholdFileLocation.SelectionStart = ThresholdFileLocation.Text.Length
        End If
    End Sub

    Private Sub ThresholdDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ThresholdDataRowStart.TextChanged
        EntryError.SetError(ThresholdDataRowStart, "")
        If ThresholdDataRowStart.Text <> "" Then
            Try
                EstimatePrecip_ThresholdPrecipRowStart = ThresholdDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(ThresholdDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub InputFormat1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InputFormat1.CheckedChanged
        Select Case InputFormat1.Checked
            Case True
                AdjStartDate_Label.Text = "Year/Month to Start Adjustment:"
                AdjStartMonthEntry_Label.Enabled = True
                AdjStartMonthEntry.Enabled = True
                AdjEndDate_Label.Text = "Year/Month to End Adjustment:"
                AdjEndMonthEntry_Label.Enabled = True
                AdjEndMonthEntry.Enabled = True
                DerStartDate_Label.Text = "Year/Month to Start Derivation:"
                DerStartMonthEntry_Label.Enabled = True
                DerStartMonthEntry.Enabled = True
                DerEndDate_Label.Text = "Year/Month to End Derivation:"
                DerEndMonthEntry_Label.Enabled = True
                DerEndMonthEntry.Enabled = True
            Case False
                AdjStartDate_Label.Text = "Year to Start Adjustment:"
                AdjStartMonthEntry_Label.Enabled = False
                AdjStartMonthEntry.Enabled = False
                AdjEndDate_Label.Text = "Year to End Adjustment:"
                AdjEndMonthEntry_Label.Enabled = False
                AdjEndMonthEntry.Enabled = False
                DerStartDate_Label.Text = "Year to Start Derivation:"
                DerStartMonthEntry_Label.Enabled = False
                DerStartMonthEntry.Enabled = False
                DerEndDate_Label.Text = "Year to End Derivation:"
                DerEndMonthEntry_Label.Enabled = False
                DerEndMonthEntry.Enabled = False
        End Select
    End Sub

    Private Sub AdjStartYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AdjStartYearEntry.TextChanged
        EntryError.SetError(AdjStartYearEntry, "")
        If AdjStartYearEntry.Text <> "" Then
            Try
                EstimatePrecip_AdjStartYear = AdjStartYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(AdjStartYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub AdjStartMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AdjStartMonthEntry.TextChanged
        EntryError.SetError(AdjStartMonthEntry, "")
        If AdjStartMonthEntry.Text <> "" Then
            Try
                EstimatePrecip_AdjStartMonth = AdjStartMonthEntry.Text
                If EstimatePrecip_AdjStartMonth > 12 Or EstimatePrecip_AdjStartMonth < 1 Then
                    EntryError.SetError(AdjStartMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(AdjStartMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub AdjEndYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AdjEndYearEntry.TextChanged
        EntryError.SetError(AdjEndYearEntry, "")
        If AdjEndYearEntry.Text <> "" Then
            Try
                EstimatePrecip_AdjEndYear = AdjEndYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(AdjEndYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub AdjEndMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AdjEndMonthEntry.TextChanged
        EntryError.SetError(AdjEndMonthEntry, "")
        If AdjEndMonthEntry.Text <> "" Then
            Try
                EstimatePrecip_AdjEndMonth = AdjEndMonthEntry.Text
                If EstimatePrecip_AdjEndMonth > 12 Or EstimatePrecip_AdjEndMonth < 1 Then
                    EntryError.SetError(AdjEndMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(AdjEndMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DerStartYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DerStartYearEntry.TextChanged
        EntryError.SetError(DerStartYearEntry, "")
        If DerStartYearEntry.Text <> "" Then
            Try
                EstimatePrecip_DerStartYear = DerStartYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(DerStartYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DerStartMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DerStartMonthEntry.TextChanged
        EntryError.SetError(DerStartMonthEntry, "")
        If DerStartMonthEntry.Text <> "" Then
            Try
                EstimatePrecip_DerStartMonth = DerStartMonthEntry.Text
                If EstimatePrecip_DerStartMonth > 12 Or EstimatePrecip_DerStartMonth < 1 Then
                    EntryError.SetError(DerStartMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(DerStartMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DerEndYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DerEndYearEntry.TextChanged
        EntryError.SetError(DerEndYearEntry, "")
        If DerEndYearEntry.Text <> "" Then
            Try
                EstimatePrecip_DerEndYear = DerEndYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(DerEndYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DerEndMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DerEndMonthEntry.TextChanged
        EntryError.SetError(DerEndMonthEntry, "")
        If DerEndMonthEntry.Text <> "" Then
            Try
                EstimatePrecip_DerEndMonth = DerEndMonthEntry.Text
                If EstimatePrecip_DerEndMonth > 12 Or EstimatePrecip_DerEndMonth < 1 Then
                    EntryError.SetError(DerEndMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(DerEndMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = EstimatePrecip_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            EstimatePrecip_Output = FileOutput.FileName
            OutputFileLocation.Text = EstimatePrecip_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        EstimatePrecip_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for Errors
        If EstimatePrecip_AllPrecipInput = "" Then
            EntryError.SetError(AllFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If AllDataRowStart.Text = "" Then
            EntryError.SetError(AllDataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If EstimatePrecip_ThresholdPrecipInput = "" Then
            EntryError.SetError(ThresholdFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If ThresholdDataRowStart.Text = "" Then
            EntryError.SetError(ThresholdDataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If AdjStartYearEntry.Text = "" Then
            EntryError.SetError(AdjStartYearEntry, "Required estimation information missing.")
            Exit Sub
        End If
        If InputFormat1.Checked = True Then
            If AdjStartMonthEntry.Text = "" Then
                EntryError.SetError(AdjStartMonthEntry, "Required estimation information missing.")
                Exit Sub
            End If
        End If
        If AdjEndYearEntry.Text = "" Then
            EntryError.SetError(AdjEndYearEntry, "Required estimation information missing.")
            Exit Sub
        End If
        If InputFormat1.Checked = True Then
            If AdjEndMonthEntry.Text = "" Then
                EntryError.SetError(AdjEndMonthEntry, "Required estimation information missing.")
                Exit Sub
            End If
        End If
        If DerStartYearEntry.Text = "" Then
            EntryError.SetError(DerStartYearEntry, "Required estimation information missing.")
            Exit Sub
        End If
        If InputFormat1.Checked = True Then
            If DerStartMonthEntry.Text = "" Then
                EntryError.SetError(DerStartMonthEntry, "Required estimation information missing.")
                Exit Sub
            End If
        End If
        If DerEndYearEntry.Text = "" Then
            EntryError.SetError(DerEndYearEntry, "Required estimation information missing.")
            Exit Sub
        End If
        If InputFormat1.Checked = True Then
            If DerEndMonthEntry.Text = "" Then
                EntryError.SetError(DerEndMonthEntry, "Required estimation information missing.")
                Exit Sub
            End If
        End If
        If EstimatePrecip_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If EstimatePrecip_Output = EstimatePrecip_AllPrecipInput Or EstimatePrecip_Output = EstimatePrecip_ThresholdPrecipInput Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        EstimatePrecip_Exception = 0
        If Amounts.Checked = True Then
            EstimatePrecip_PrecipType = 1
        End If
        If Days.Checked = True Then
            EstimatePrecip_PrecipType = 2
        End If
        If InputFormat1.Checked = True Then
            EstimatePrecip_InputFormat = 1
        End If
        If InputFormat2.Checked = True Then
            EstimatePrecip_InputFormat = 2
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim FoundStart As Integer
        Dim Count As Integer
        Dim JanCount As Integer
        Dim FebCount As Integer
        Dim MarCount As Integer
        Dim AprCount As Integer
        Dim MayCount As Integer
        Dim JunCount As Integer
        Dim JulCount As Integer
        Dim AugCount As Integer
        Dim SepCount As Integer
        Dim OctCount As Integer
        Dim NovCount As Integer
        Dim DecCount As Integer
        Dim Start As Integer
        Dim i As Integer
        Dim DifferenceTotal As Double
        Dim Difference As Double
        Dim JanDifference As Double
        Dim FebDifference As Double
        Dim MarDifference As Double
        Dim AprDifference As Double
        Dim MayDifference As Double
        Dim JunDifference As Double
        Dim JulDifference As Double
        Dim AugDifference As Double
        Dim SepDifference As Double
        Dim OctDifference As Double
        Dim NovDifference As Double
        Dim DecDifference As Double
        Dim EstimateValue As Double
        Dim EstimateError As Double
        Dim TotalError As Double
        Dim JanLowError As Double
        Dim JanHighError As Double
        Dim JanAvgError As Double
        Dim FebLowError As Double
        Dim FebHighError As Double
        Dim FebAvgError As Double
        Dim MarLowError As Double
        Dim MarHighError As Double
        Dim MarAvgError As Double
        Dim AprLowError As Double
        Dim AprHighError As Double
        Dim AprAvgError As Double
        Dim MayLowError As Double
        Dim MayHighError As Double
        Dim MayAvgError As Double
        Dim JunLowError As Double
        Dim JunHighError As Double
        Dim JunAvgError As Double
        Dim JulLowError As Double
        Dim JulHighError As Double
        Dim JulAvgError As Double
        Dim AugLowError As Double
        Dim AugHighError As Double
        Dim AugAvgError As Double
        Dim SepLowError As Double
        Dim SepHighError As Double
        Dim SepAvgError As Double
        Dim OctLowError As Double
        Dim OctHighError As Double
        Dim OctAvgError As Double
        Dim NovLowError As Double
        Dim NovHighError As Double
        Dim NovAvgError As Double
        Dim DecLowError As Double
        Dim DecHighError As Double
        Dim DecAvgError As Double
        Dim LowError As Double
        Dim HighError As Double
        Dim AvgError As Double
        Dim NewPrecip As Double
        FoundStart = 0
        Count = 0
        JanCount = 0
        FebCount = 0
        MarCount = 0
        AprCount = 0
        MayCount = 0
        JunCount = 0
        JulCount = 0
        AugCount = 0
        SepCount = 0
        OctCount = 0
        NovCount = 0
        DecCount = 0
        'Initialize Observation Arrays
        Dim AllPrecip(0) As Double
        Dim JanAllPrecip(0) As Double
        Dim FebAllPrecip(0) As Double
        Dim MarAllPrecip(0) As Double
        Dim AprAllPrecip(0) As Double
        Dim MayAllPrecip(0) As Double
        Dim JunAllPrecip(0) As Double
        Dim JulAllPrecip(0) As Double
        Dim AugAllPrecip(0) As Double
        Dim SepAllPrecip(0) As Double
        Dim OctAllPrecip(0) As Double
        Dim NovAllPrecip(0) As Double
        Dim DecAllPrecip(0) As Double
        Dim ThresholdPrecip(0) As Double
        Dim JanThresholdPrecip(0) As Double
        Dim FebThresholdPrecip(0) As Double
        Dim MarThresholdPrecip(0) As Double
        Dim AprThresholdPrecip(0) As Double
        Dim MayThresholdPrecip(0) As Double
        Dim JunThresholdPrecip(0) As Double
        Dim JulThresholdPrecip(0) As Double
        Dim AugThresholdPrecip(0) As Double
        Dim SepThresholdPrecip(0) As Double
        Dim OctThresholdPrecip(0) As Double
        Dim NovThresholdPrecip(0) As Double
        Dim DecThresholdPrecip(0) As Double
        'Initialize Output Files and Set Headers
        Dim DecimalIndex As Integer
        DecimalIndex = EstimatePrecip_Output.IndexOf(".")
        EstimatePrecip_OutputStats = EstimatePrecip_Output.Insert(DecimalIndex, " Statistics")
        Dim AdjustPrecip_HeaderStats As String = EstimatePrecip_Header + " Statistics"
        FileOpen(1, EstimatePrecip_Output, OpenMode.Output)
        PrintLine(1, EstimatePrecip_Header)
        PrintLine(1, "")
        Select Case EstimatePrecip_InputFormat
            Case 1
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Precip")
            Case 2
                PrintLine(1, "Year" & vbTab & "Precip")
        End Select
        FileOpen(2, EstimatePrecip_OutputStats, OpenMode.Output)
        PrintLine(2, AdjustPrecip_HeaderStats)
        PrintLine(2, "")
        'Initialize StreamReaders
        Dim AllPrecipInFile As New System.IO.StreamReader(EstimatePrecip_AllPrecipInput)
        Dim AllPrecipLine As String
        Dim AllPrecipFields As String()
        Dim ThresholdPrecipInFile As New System.IO.StreamReader(EstimatePrecip_ThresholdPrecipInput)
        Dim ThresholdPrecipLine As String
        Dim ThresholdPrecipFields As String()
        'Set StreamReaders to Data Start
        If EstimatePrecip_AllPrecipRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To EstimatePrecip_AllPrecipRowStart - 2
                AllPrecipLine = AllPrecipInFile.ReadLine()
            Next Row
        End If
        If EstimatePrecip_ThresholdPrecipRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To EstimatePrecip_ThresholdPrecipRowStart - 2
                ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
            Next Row
        End If
        'Initialize Progress Bar
        Dim Percent As Integer
        Dim AdjustmentRows As Integer
        Dim DerivationRows As Integer
        Dim RunComplete As Integer
        Dim RunProgress As Integer
        RunProgress = 0
        Select Case EstimatePrecip_InputFormat
            Case 1
                'Get Complete Years for Adjustment Period
                AdjustmentRows = (EstimatePrecip_AdjEndYear - 1) - (EstimatePrecip_AdjStartYear + 1) + 1
                AdjustmentRows = AdjustmentRows * 12
                'Add Months from Incomplete Years for Adjustment Period
                AdjustmentRows = AdjustmentRows + (12 - (EstimatePrecip_AdjStartMonth - 1))
                AdjustmentRows = AdjustmentRows + EstimatePrecip_AdjEndMonth
                'Get Complete Years for Derivation Period
                DerivationRows = (EstimatePrecip_DerEndYear - 1) - (EstimatePrecip_DerStartYear + 1) + 1
                DerivationRows = DerivationRows * 12
                'Add Months from Incomplete Years for Derivation Period
                DerivationRows = DerivationRows + (12 - (EstimatePrecip_DerStartMonth - 1))
                DerivationRows = DerivationRows + EstimatePrecip_DerEndMonth
            Case 2
                'No Monthly Data
                AdjustmentRows = (EstimatePrecip_AdjEndYear - EstimatePrecip_AdjStartYear) + 1
                DerivationRows = (EstimatePrecip_DerEndYear - EstimatePrecip_DerStartYear) + 1
        End Select
        'Finalize RunComplete
        RunComplete = AdjustmentRows + DerivationRows
        'Derive and Test Modern Difference(s)
        'Set All Precipitation Data File to Derivation Start Date
        'Skip Column Headers
        AllPrecipLine = AllPrecipInFile.ReadLine()
        Try
            While AllPrecipInFile.Peek <> -1
                AllPrecipLine = AllPrecipInFile.ReadLine()
                AllPrecipFields = AllPrecipLine.Split(ControlChars.Tab)
                Year = AllPrecipFields(0)
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        Month = AllPrecipFields(1)
                        If Year = EstimatePrecip_DerStartYear Then
                            If Month = EstimatePrecip_DerStartMonth Then
                                FoundStart = 1
                                Exit While
                            End If
                        End If
                    Case 2
                        'Data in Seasonal/Annual Format
                        If Year = EstimatePrecip_DerStartYear Then
                            FoundStart = 1
                            Exit While
                        End If
                End Select
            End While
        Catch ex As Exception
            If Year <> 0 Then
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        MsgBox("Error reading the all precipitation data file near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                    Case 2
                        'Data in Seasonal/Annual Format
                        MsgBox("Error reading the all precipitation data file near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                End Select
            Else
                MsgBox("Error reading the all precipitation data file near the header.  Make sure the row where the all precipitation date file starts is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            End If
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End Try
        If FoundStart = 0 Then
            MsgBox("Unable to find derivation start date in the all precipitation data file.  Recheck the derivation start date.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End If
        FoundStart = 0
        Year = 0
        'Set Threshold Precipitation Data File to Derivation Start Date
        'Skip Column Headers
        ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
        Try
            While ThresholdPrecipInFile.Peek <> -1
                ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
                ThresholdPrecipFields = ThresholdPrecipLine.Split(ControlChars.Tab)
                Year = ThresholdPrecipFields(0)
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        Month = ThresholdPrecipFields(1)
                        If Year = EstimatePrecip_DerStartYear Then
                            If Month = EstimatePrecip_DerStartMonth Then
                                FoundStart = 1
                                Exit While
                            End If
                        End If
                    Case 2
                        'Data in Seasonal/Annual Format
                        If Year = EstimatePrecip_DerStartYear Then
                            FoundStart = 1
                            Exit While
                        End If
                End Select
            End While
        Catch ex As Exception
            If Year <> 0 Then
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        MsgBox("Error reading the threshold data file near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                    Case 2
                        'Data in Seasonal/Annual Format
                        MsgBox("Error reading the threshold data file near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                End Select
            Else
                MsgBox("Error reading the threshold data file near the header.  Make sure the row where the threshold data file starts is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            End If
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End Try
        If FoundStart = 0 Then
            MsgBox("Unable to find derivation start date in the threshold data file.  Recheck the derivation start date.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End If
        'Read Derivation Period into Arrays
        Try
            'Process Data at Start Date
            Year = AllPrecipFields(0)
            Select Case EstimatePrecip_InputFormat
                Case 1
                    'Data in Monthly Format
                    Month = AllPrecipFields(1)
                    Select Case Month
                        Case 1
                            JanAllPrecip(0) = AllPrecipFields(2)
                            JanThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 2
                            FebAllPrecip(0) = AllPrecipFields(2)
                            FebThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 3
                            MarAllPrecip(0) = AllPrecipFields(2)
                            MarThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 4
                            AprAllPrecip(0) = AllPrecipFields(2)
                            AprThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 5
                            MayAllPrecip(0) = AllPrecipFields(2)
                            MayThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 6
                            JunAllPrecip(0) = AllPrecipFields(2)
                            JunThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 7
                            JulAllPrecip(0) = AllPrecipFields(2)
                            JulThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 8
                            AugAllPrecip(0) = AllPrecipFields(2)
                            AugThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 9
                            SepAllPrecip(0) = AllPrecipFields(2)
                            SepThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 10
                            OctAllPrecip(0) = AllPrecipFields(2)
                            OctThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 11
                            NovAllPrecip(0) = AllPrecipFields(2)
                            NovThresholdPrecip(0) = ThresholdPrecipFields(2)
                        Case 12
                            DecAllPrecip(0) = AllPrecipFields(2)
                            DecThresholdPrecip(0) = ThresholdPrecipFields(2)
                    End Select
                Case 2
                    'Data in Seasonal/Annual Format
                    AllPrecip(0) = AllPrecipFields(1)
                    ThresholdPrecip(0) = ThresholdPrecipFields(1)
            End Select
            RunProgress = RunProgress + 1
            'Continue Processing Additional Dates
            While AllPrecipInFile.Peek <> -1
                AllPrecipLine = AllPrecipInFile.ReadLine()
                AllPrecipFields = AllPrecipLine.Split(ControlChars.Tab)
                ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
                ThresholdPrecipFields = ThresholdPrecipLine.Split(ControlChars.Tab)
                Year = AllPrecipFields(0)
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        Month = AllPrecipFields(1)
                        Select Case Month
                            Case 1
                                JanCount = JanCount + 1
                                ReDim Preserve JanAllPrecip(JanCount)
                                ReDim Preserve JanThresholdPrecip(JanCount)
                                JanAllPrecip(JanCount) = AllPrecipFields(2)
                                JanThresholdPrecip(JanCount) = ThresholdPrecipFields(2)
                            Case 2
                                FebCount = FebCount + 1
                                ReDim Preserve FebAllPrecip(FebCount)
                                ReDim Preserve FebThresholdPrecip(FebCount)
                                FebAllPrecip(FebCount) = AllPrecipFields(2)
                                FebThresholdPrecip(FebCount) = ThresholdPrecipFields(2)
                            Case 3
                                MarCount = MarCount + 1
                                ReDim Preserve MarAllPrecip(MarCount)
                                ReDim Preserve MarThresholdPrecip(MarCount)
                                MarAllPrecip(MarCount) = AllPrecipFields(2)
                                MarThresholdPrecip(MarCount) = ThresholdPrecipFields(2)
                            Case 4
                                AprCount = AprCount + 1
                                ReDim Preserve AprAllPrecip(AprCount)
                                ReDim Preserve AprThresholdPrecip(AprCount)
                                AprAllPrecip(AprCount) = AllPrecipFields(2)
                                AprThresholdPrecip(AprCount) = ThresholdPrecipFields(2)
                            Case 5
                                MayCount = MayCount + 1
                                ReDim Preserve MayAllPrecip(MayCount)
                                ReDim Preserve MayThresholdPrecip(MayCount)
                                MayAllPrecip(MayCount) = AllPrecipFields(2)
                                MayThresholdPrecip(MayCount) = ThresholdPrecipFields(2)
                            Case 6
                                JunCount = JunCount + 1
                                ReDim Preserve JunAllPrecip(JunCount)
                                ReDim Preserve JunThresholdPrecip(JunCount)
                                JunAllPrecip(JunCount) = AllPrecipFields(2)
                                JunThresholdPrecip(JunCount) = ThresholdPrecipFields(2)
                            Case 7
                                JulCount = JulCount + 1
                                ReDim Preserve JulAllPrecip(JulCount)
                                ReDim Preserve JulThresholdPrecip(JulCount)
                                JulAllPrecip(JulCount) = AllPrecipFields(2)
                                JulThresholdPrecip(JulCount) = ThresholdPrecipFields(2)
                            Case 8
                                AugCount = AugCount + 1
                                ReDim Preserve AugAllPrecip(AugCount)
                                ReDim Preserve AugThresholdPrecip(AugCount)
                                AugAllPrecip(AugCount) = AllPrecipFields(2)
                                AugThresholdPrecip(AugCount) = ThresholdPrecipFields(2)
                            Case 9
                                SepCount = SepCount + 1
                                ReDim Preserve SepAllPrecip(SepCount)
                                ReDim Preserve SepThresholdPrecip(SepCount)
                                SepAllPrecip(SepCount) = AllPrecipFields(2)
                                SepThresholdPrecip(SepCount) = ThresholdPrecipFields(2)
                            Case 10
                                OctCount = OctCount + 1
                                ReDim Preserve OctAllPrecip(OctCount)
                                ReDim Preserve OctThresholdPrecip(OctCount)
                                OctAllPrecip(OctCount) = AllPrecipFields(2)
                                OctThresholdPrecip(OctCount) = ThresholdPrecipFields(2)
                            Case 11
                                NovCount = NovCount + 1
                                ReDim Preserve NovAllPrecip(NovCount)
                                ReDim Preserve NovThresholdPrecip(NovCount)
                                NovAllPrecip(NovCount) = AllPrecipFields(2)
                                NovThresholdPrecip(NovCount) = ThresholdPrecipFields(2)
                            Case 12
                                DecCount = DecCount + 1
                                ReDim Preserve DecAllPrecip(DecCount)
                                ReDim Preserve DecThresholdPrecip(DecCount)
                                DecAllPrecip(DecCount) = AllPrecipFields(2)
                                DecThresholdPrecip(DecCount) = ThresholdPrecipFields(2)
                        End Select
                    Case 2
                        'Data in Seasonal/Annual Format
                        Count = Count + 1
                        ReDim Preserve AllPrecip(Count)
                        ReDim Preserve ThresholdPrecip(Count)
                        AllPrecip(Count) = AllPrecipFields(1)
                        ThresholdPrecip(Count) = ThresholdPrecipFields(1)
                End Select
                'Update Progress Bar
                RunProgress = RunProgress + 1
                Percent = (RunProgress / RunComplete) * 100
                BackgroundRun.ReportProgress(Percent)
                'Check for End Date
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        If Year = EstimatePrecip_DerEndYear Then
                            If Month = EstimatePrecip_DerEndMonth Then
                                Exit While
                            End If
                        End If
                    Case 2
                        'Data in Seasonal/Annual Format
                        If Year = EstimatePrecip_DerEndYear Then
                            Exit While
                        End If
                End Select
            End While
        Catch ex As Exception
            Select Case EstimatePrecip_InputFormat
                Case 1
                    'Data in Monthly Format
                    MsgBox("Error reading the all precipitation or threshold precipitation data file near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                Case 2
                    'Data in Seasonal/Annual Format
                    MsgBox("Error reading the all precipitation or threshold precipitation data file near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            End Select
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End Try
        'Derive Difference(s) Using 2nd Half of the Derivation Period for Testing
        If EstimatePrecip_InputFormat = 1 Then
            'Data in Monthly Format
            'January
            Start = JanCount / 2
            For i = Start To JanCount
                If JanAllPrecip(i) <> -99 And JanThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (JanAllPrecip(i) - JanThresholdPrecip(i))
                End If
            Next i
            JanDifference = DifferenceTotal / (JanCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JanDifference = Math.Round(JanDifference, 2)
                Case 2
                    'Days
                    JanDifference = Math.Round(JanDifference)
            End Select
            DifferenceTotal = 0
            'February
            Start = FebCount / 2
            For i = Start To FebCount
                If FebAllPrecip(i) <> -99 And FebThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (FebAllPrecip(i) - FebThresholdPrecip(i))
                End If
            Next i
            FebDifference = DifferenceTotal / (FebCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    FebDifference = Math.Round(FebDifference, 2)
                Case 2
                    'Days
                    FebDifference = Math.Round(FebDifference)
            End Select
            DifferenceTotal = 0
            'March
            Start = MarCount / 2
            For i = Start To MarCount
                If MarAllPrecip(i) <> -99 And MarThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (MarAllPrecip(i) - MarThresholdPrecip(i))
                End If
            Next i
            MarDifference = DifferenceTotal / (MarCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    MarDifference = Math.Round(MarDifference, 2)
                Case 2
                    'Days
                    MarDifference = Math.Round(MarDifference)
            End Select
            DifferenceTotal = 0
            'April
            Start = AprCount / 2
            For i = Start To AprCount
                If AprAllPrecip(i) <> -99 And AprThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (AprAllPrecip(i) - AprThresholdPrecip(i))
                End If
            Next i
            AprDifference = DifferenceTotal / (AprCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    AprDifference = Math.Round(AprDifference, 2)
                Case 2
                    'Days
                    AprDifference = Math.Round(AprDifference)
            End Select
            DifferenceTotal = 0
            'May
            Start = MayCount / 2
            For i = Start To MayCount
                If MayAllPrecip(i) <> -99 And MayThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (MayAllPrecip(i) - MayThresholdPrecip(i))
                End If
            Next i
            MayDifference = DifferenceTotal / (MayCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    MayDifference = Math.Round(MayDifference, 2)
                Case 2
                    'Days
                    MayDifference = Math.Round(MayDifference)
            End Select
            DifferenceTotal = 0
            'June
            Start = JunCount / 2
            For i = Start To JunCount
                If JunAllPrecip(i) <> -99 And JunThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (JunAllPrecip(i) - JunThresholdPrecip(i))
                End If
            Next i
            JunDifference = DifferenceTotal / (JunCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JunDifference = Math.Round(JunDifference, 2)
                Case 2
                    'Days
                    JunDifference = Math.Round(JunDifference)
            End Select
            DifferenceTotal = 0
            'July
            Start = JulCount / 2
            For i = Start To JulCount
                If JulAllPrecip(i) <> -99 And JulThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (JulAllPrecip(i) - JulThresholdPrecip(i))
                End If
            Next i
            JulDifference = DifferenceTotal / (JulCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JulDifference = Math.Round(JulDifference, 2)
                Case 2
                    'Days
                    JulDifference = Math.Round(JulDifference)
            End Select
            DifferenceTotal = 0
            'August
            Start = AugCount / 2
            For i = Start To AugCount
                If AugAllPrecip(i) <> -99 And AugThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (AugAllPrecip(i) - AugThresholdPrecip(i))
                End If
            Next i
            AugDifference = DifferenceTotal / (AugCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    AugDifference = Math.Round(AugDifference, 2)
                Case 2
                    'Days
                    AugDifference = Math.Round(AugDifference)
            End Select
            DifferenceTotal = 0
            'September
            Start = SepCount / 2
            For i = Start To SepCount
                If SepAllPrecip(i) <> -99 And SepThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (SepAllPrecip(i) - SepThresholdPrecip(i))
                End If
            Next i
            SepDifference = DifferenceTotal / (SepCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    SepDifference = Math.Round(SepDifference, 2)
                Case 2
                    'Days
                    SepDifference = Math.Round(SepDifference)
            End Select
            DifferenceTotal = 0
            'October
            Start = OctCount / 2
            For i = Start To OctCount
                If OctAllPrecip(i) <> -99 And OctThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (OctAllPrecip(i) - OctThresholdPrecip(i))
                End If
            Next i
            OctDifference = DifferenceTotal / (OctCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    OctDifference = Math.Round(OctDifference, 2)
                Case 2
                    'Days
                    OctDifference = Math.Round(OctDifference)
            End Select
            DifferenceTotal = 0
            'November
            Start = NovCount / 2
            For i = Start To NovCount
                If NovAllPrecip(i) <> -99 And NovThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (NovAllPrecip(i) - NovThresholdPrecip(i))
                End If
            Next i
            NovDifference = DifferenceTotal / (NovCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    NovDifference = Math.Round(NovDifference, 2)
                Case 2
                    'Days
                    NovDifference = Math.Round(NovDifference)
            End Select
            DifferenceTotal = 0
            'December
            Start = DecCount / 2
            For i = Start To DecCount
                If DecAllPrecip(i) <> -99 And DecThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (DecAllPrecip(i) - DecThresholdPrecip(i))
                End If
            Next i
            DecDifference = DifferenceTotal / (DecCount - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    DecDifference = Math.Round(DecDifference, 2)
                Case 2
                    'Days
                    DecDifference = Math.Round(DecDifference)
            End Select
            DifferenceTotal = 0
        End If
        If EstimatePrecip_InputFormat = 2 Then
            'Data in Seasonal/Annual Format
            DifferenceTotal = 0
            Start = Count / 2
            For i = Start To Count
                If AllPrecip(i) <> -99 And ThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (AllPrecip(i) - ThresholdPrecip(i))
                End If
            Next i
            Difference = DifferenceTotal / (Count - Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    Difference = Math.Round(Difference, 2)
                Case 2
                    'Days
                    Difference = Math.Round(Difference)
            End Select
            DifferenceTotal = 0
        End If
        'Ouput Derived Test Values
        PrintLine(2, "Initial Derived Differences:")
        PrintLine(2, "")
        Select Case EstimatePrecip_InputFormat
            Case 1
                'Data in Monthly Format
                PrintLine(2, "Month" & vbTab & "Difference")
                PrintLine(2, "1" & vbTab & JanDifference)
                PrintLine(2, "2" & vbTab & FebDifference)
                PrintLine(2, "3" & vbTab & MarDifference)
                PrintLine(2, "4" & vbTab & AprDifference)
                PrintLine(2, "5" & vbTab & MayDifference)
                PrintLine(2, "6" & vbTab & JunDifference)
                PrintLine(2, "7" & vbTab & JulDifference)
                PrintLine(2, "8" & vbTab & AugDifference)
                PrintLine(2, "9" & vbTab & SepDifference)
                PrintLine(2, "10" & vbTab & OctDifference)
                PrintLine(2, "11" & vbTab & NovDifference)
                PrintLine(2, "12" & vbTab & DecDifference)
                PrintLine(2, "")
            Case 2
                'Data in Seasonal/Annual Format
                PrintLine(2, "Difference = " & Difference)
                PrintLine(2, "")
        End Select
        'Test Derived Adjustment Values on 1st Half of the Derivation Period
        If EstimatePrecip_InputFormat = 1 Then
            'Data in Monthly Format
            'January
            JanLowError = 0
            JanHighError = 0
            Start = JanCount / 2
            Start = Start - 1
            For i = 0 To Start
                If JanThresholdPrecip(i) <> -99 And JanAllPrecip(i) <> -99 Then
                    EstimateValue = JanThresholdPrecip(i) + JanDifference
                    EstimateError = EstimateValue - JanAllPrecip(i)
                    Select Case EstimateError
                        Case Is < JanLowError
                            JanLowError = EstimateError
                        Case Is > JanHighError
                            JanHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            JanAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JanLowError = Math.Round(JanLowError, 2)
                    JanHighError = Math.Round(JanHighError, 2)
                    JanAvgError = Math.Round(JanAvgError, 2)
                Case 2
                    'Days
                    JanLowError = Math.Round(JanLowError)
                    JanHighError = Math.Round(JanHighError)
                    JanAvgError = Math.Round(JanAvgError)
            End Select
            TotalError = 0
            'February
            FebLowError = 0
            FebHighError = 0
            Start = FebCount / 2
            Start = Start - 1
            For i = 0 To Start
                If FebThresholdPrecip(i) <> -99 And FebAllPrecip(i) <> -99 Then
                    EstimateValue = FebThresholdPrecip(i) + FebDifference
                    EstimateError = EstimateValue - FebAllPrecip(i)
                    Select Case EstimateError
                        Case Is < FebLowError
                            FebLowError = EstimateError
                        Case Is > FebHighError
                            FebHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            FebAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    FebLowError = Math.Round(FebLowError, 2)
                    FebHighError = Math.Round(FebHighError, 2)
                    FebAvgError = Math.Round(FebAvgError, 2)
                Case 2
                    'Days
                    FebLowError = Math.Round(FebLowError)
                    FebHighError = Math.Round(FebHighError)
                    FebAvgError = Math.Round(FebAvgError)
            End Select
            TotalError = 0
            'March
            MarLowError = 0
            MarHighError = 0
            Start = MarCount / 2
            Start = Start - 1
            For i = 0 To Start
                If MarThresholdPrecip(i) <> -99 And MarAllPrecip(i) <> -99 Then
                    EstimateValue = MarThresholdPrecip(i) + MarDifference
                    EstimateError = EstimateValue - MarAllPrecip(i)
                    Select Case EstimateError
                        Case Is < MarLowError
                            MarLowError = EstimateError
                        Case Is > MarHighError
                            MarHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            MarAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    MarLowError = Math.Round(MarLowError, 2)
                    MarHighError = Math.Round(MarHighError, 2)
                    MarAvgError = Math.Round(MarAvgError, 2)
                Case 2
                    'Days
                    MarLowError = Math.Round(MarLowError)
                    MarHighError = Math.Round(MarHighError)
                    MarAvgError = Math.Round(MarAvgError)
            End Select
            TotalError = 0
            'April
            AprLowError = 0
            AprHighError = 0
            Start = AprCount / 2
            Start = Start - 1
            For i = 0 To Start
                If AprThresholdPrecip(i) <> -99 And AprAllPrecip(i) <> -99 Then
                    EstimateValue = AprThresholdPrecip(i) + AprDifference
                    EstimateError = EstimateValue - AprAllPrecip(i)
                    Select Case EstimateError
                        Case Is < AprLowError
                            AprLowError = EstimateError
                        Case Is > AprHighError
                            AprHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            AprAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    AprLowError = Math.Round(AprLowError, 2)
                    AprHighError = Math.Round(AprHighError, 2)
                    AprAvgError = Math.Round(AprAvgError, 2)
                Case 2
                    'Days
                    AprLowError = Math.Round(AprLowError)
                    AprHighError = Math.Round(AprHighError)
                    AprAvgError = Math.Round(AprAvgError)
            End Select
            TotalError = 0
            'May
            MayLowError = 0
            MayHighError = 0
            Start = MayCount / 2
            Start = Start - 1
            For i = 0 To Start
                If MayThresholdPrecip(i) <> -99 And MayAllPrecip(i) <> -99 Then
                    EstimateValue = MayThresholdPrecip(i) + MayDifference
                    EstimateError = EstimateValue - MayAllPrecip(i)
                    Select Case EstimateError
                        Case Is < MayLowError
                            MayLowError = EstimateError
                        Case Is > MayHighError
                            MayHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            MayAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    MayLowError = Math.Round(MayLowError, 2)
                    MayHighError = Math.Round(MayHighError, 2)
                    MayAvgError = Math.Round(MayAvgError, 2)
                Case 2
                    'Days
                    MayLowError = Math.Round(MayLowError)
                    MayHighError = Math.Round(MayHighError)
                    MayAvgError = Math.Round(MayAvgError)
            End Select
            TotalError = 0
            'June
            JunLowError = 0
            JunHighError = 0
            Start = JunCount / 2
            Start = Start - 1
            For i = 0 To Start
                If JunThresholdPrecip(i) <> -99 And JunAllPrecip(i) <> -99 Then
                    EstimateValue = JunThresholdPrecip(i) + JunDifference
                    EstimateError = EstimateValue - JunAllPrecip(i)
                    Select Case EstimateError
                        Case Is < JunLowError
                            JunLowError = EstimateError
                        Case Is > JunHighError
                            JunHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            JunAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JunLowError = Math.Round(JunLowError, 2)
                    JunHighError = Math.Round(JunHighError, 2)
                    JunAvgError = Math.Round(JunAvgError, 2)
                Case 2
                    'Days
                    JunLowError = Math.Round(JunLowError)
                    JunHighError = Math.Round(JunHighError)
                    JunAvgError = Math.Round(JunAvgError)
            End Select
            TotalError = 0
            'July
            JulLowError = 0
            JulHighError = 0
            Start = JulCount / 2
            Start = Start - 1
            For i = 0 To Start
                If JulThresholdPrecip(i) <> -99 And JulAllPrecip(i) <> -99 Then
                    EstimateValue = JulThresholdPrecip(i) + JulDifference
                    EstimateError = EstimateValue - JulAllPrecip(i)
                    Select Case EstimateError
                        Case Is < JulLowError
                            JulLowError = EstimateError
                        Case Is > JulHighError
                            JulHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            JulAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JulLowError = Math.Round(JulLowError, 2)
                    JulHighError = Math.Round(JulHighError, 2)
                    JulAvgError = Math.Round(JulAvgError, 2)
                Case 2
                    'Days
                    JulLowError = Math.Round(JulLowError)
                    JulHighError = Math.Round(JulHighError)
                    JulAvgError = Math.Round(JulAvgError)
            End Select
            TotalError = 0
            'August
            AugLowError = 0
            AugHighError = 0
            Start = AugCount / 2
            Start = Start - 1
            For i = 0 To Start
                If AugThresholdPrecip(i) <> -99 And AugAllPrecip(i) <> -99 Then
                    EstimateValue = AugThresholdPrecip(i) + AugDifference
                    EstimateError = EstimateValue - AugAllPrecip(i)
                    Select Case EstimateError
                        Case Is < AugLowError
                            AugLowError = EstimateError
                        Case Is > AugHighError
                            AugHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            AugAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    AugLowError = Math.Round(AugLowError, 2)
                    AugHighError = Math.Round(AugHighError, 2)
                    AugAvgError = Math.Round(AugAvgError, 2)
                Case 2
                    'Days
                    AugLowError = Math.Round(AugLowError)
                    AugHighError = Math.Round(AugHighError)
                    AugAvgError = Math.Round(AugAvgError)
            End Select
            TotalError = 0
            'September
            SepLowError = 0
            SepHighError = 0
            Start = SepCount / 2
            Start = Start - 1
            For i = 0 To Start
                If SepThresholdPrecip(i) <> -99 And SepAllPrecip(i) <> -99 Then
                    EstimateValue = SepThresholdPrecip(i) + SepDifference
                    EstimateError = EstimateValue - SepAllPrecip(i)
                    Select Case EstimateError
                        Case Is < SepLowError
                            SepLowError = EstimateError
                        Case Is > SepHighError
                            SepHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            SepAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    SepLowError = Math.Round(SepLowError, 2)
                    SepHighError = Math.Round(SepHighError, 2)
                    SepAvgError = Math.Round(SepAvgError, 2)
                Case 2
                    'Days
                    SepLowError = Math.Round(SepLowError)
                    SepHighError = Math.Round(SepHighError)
                    SepAvgError = Math.Round(SepAvgError)
            End Select
            TotalError = 0
            'October
            OctLowError = 0
            OctHighError = 0
            Start = OctCount / 2
            Start = Start - 1
            For i = 0 To Start
                If OctThresholdPrecip(i) <> -99 And OctAllPrecip(i) <> -99 Then
                    EstimateValue = OctThresholdPrecip(i) + OctDifference
                    EstimateError = EstimateValue - OctAllPrecip(i)
                    Select Case EstimateError
                        Case Is < OctLowError
                            OctLowError = EstimateError
                        Case Is > OctHighError
                            OctHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            OctAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    OctLowError = Math.Round(OctLowError, 2)
                    OctHighError = Math.Round(OctHighError, 2)
                    OctAvgError = Math.Round(OctAvgError, 2)
                Case 2
                    'Days
                    OctLowError = Math.Round(OctLowError)
                    OctHighError = Math.Round(OctHighError)
                    OctAvgError = Math.Round(OctAvgError)
            End Select
            TotalError = 0
            'November
            NovLowError = 0
            NovHighError = 0
            Start = NovCount / 2
            Start = Start - 1
            For i = 0 To Start
                If NovThresholdPrecip(i) <> -99 And NovAllPrecip(i) <> -99 Then
                    EstimateValue = NovThresholdPrecip(i) + NovDifference
                    EstimateError = EstimateValue - NovAllPrecip(i)
                    Select Case EstimateError
                        Case Is < NovLowError
                            NovLowError = EstimateError
                        Case Is > NovHighError
                            NovHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            NovAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    NovLowError = Math.Round(NovLowError, 2)
                    NovHighError = Math.Round(NovHighError, 2)
                    NovAvgError = Math.Round(NovAvgError, 2)
                Case 2
                    'Days
                    NovLowError = Math.Round(NovLowError)
                    NovHighError = Math.Round(NovHighError)
                    NovAvgError = Math.Round(NovAvgError)
            End Select
            TotalError = 0
            'December
            DecLowError = 0
            DecHighError = 0
            Start = DecCount / 2
            Start = Start - 1
            For i = 0 To Start
                If DecThresholdPrecip(i) <> -99 And DecAllPrecip(i) <> -99 Then
                    EstimateValue = DecThresholdPrecip(i) + DecDifference
                    EstimateError = EstimateValue - DecAllPrecip(i)
                    Select Case EstimateError
                        Case Is < DecLowError
                            DecLowError = EstimateError
                        Case Is > DecHighError
                            DecHighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            DecAvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    DecLowError = Math.Round(DecLowError, 2)
                    DecHighError = Math.Round(DecHighError, 2)
                    DecAvgError = Math.Round(DecAvgError, 2)
                Case 2
                    'Days
                    DecLowError = Math.Round(DecLowError)
                    DecHighError = Math.Round(DecHighError)
                    DecAvgError = Math.Round(DecAvgError)
            End Select
            TotalError = 0
        End If
        If EstimatePrecip_InputFormat = 2 Then
            'Data in Seasonal/Annual Format
            LowError = 0
            HighError = 0
            Start = Count / 2
            Start = Start - 1
            For i = 0 To Start
                If ThresholdPrecip(i) <> -99 And AllPrecip(i) <> -99 Then
                    EstimateValue = ThresholdPrecip(i) + Difference
                    EstimateError = EstimateValue - AllPrecip(i)
                    Select Case EstimateError
                        Case Is < LowError
                            LowError = EstimateError
                        Case Is > HighError
                            HighError = EstimateError
                    End Select
                    TotalError = TotalError + EstimateError
                End If
            Next i
            AvgError = TotalError / (Start + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    LowError = Math.Round(LowError, 2)
                    HighError = Math.Round(HighError, 2)
                    AvgError = Math.Round(AvgError, 2)
                Case 2
                    'Days
                    LowError = Math.Round(LowError)
                    HighError = Math.Round(HighError)
                    AvgError = Math.Round(AvgError)
            End Select
            TotalError = 0
        End If
        'Ouput Verification
        PrintLine(2, "Verification of Initial Derived Differences:")
        PrintLine(2, "")
        Select Case EstimatePrecip_InputFormat
            Case 1
                'Data in Monthly Format
                PrintLine(2, "Month" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "AvgErr")
                PrintLine(2, "1" & vbTab & JanLowError & vbTab & JanHighError & vbTab & JanAvgError)
                PrintLine(2, "2" & vbTab & FebLowError & vbTab & FebHighError & vbTab & FebAvgError)
                PrintLine(2, "3" & vbTab & MarLowError & vbTab & MarHighError & vbTab & MarAvgError)
                PrintLine(2, "4" & vbTab & AprLowError & vbTab & AprHighError & vbTab & AprAvgError)
                PrintLine(2, "5" & vbTab & MayLowError & vbTab & MayHighError & vbTab & MayAvgError)
                PrintLine(2, "6" & vbTab & JunLowError & vbTab & JunHighError & vbTab & JunAvgError)
                PrintLine(2, "7" & vbTab & JulLowError & vbTab & JulHighError & vbTab & JulAvgError)
                PrintLine(2, "8" & vbTab & AugLowError & vbTab & AugHighError & vbTab & AugAvgError)
                PrintLine(2, "9" & vbTab & SepLowError & vbTab & SepHighError & vbTab & SepAvgError)
                PrintLine(2, "10" & vbTab & OctLowError & vbTab & OctHighError & vbTab & OctAvgError)
                PrintLine(2, "11" & vbTab & NovLowError & vbTab & NovHighError & vbTab & NovAvgError)
                PrintLine(2, "12" & vbTab & DecLowError & vbTab & DecHighError & vbTab & DecAvgError)
                PrintLine(2, "")
            Case 2
                'Data in Seasonal/Annual Format
                PrintLine(2, "Low Error = " & LowError)
                PrintLine(2, "High Error = " & HighError)
                PrintLine(2, "Average Error = " & AvgError)
                PrintLine(2, "")
        End Select
        'Re-Derive Difference(s) Using All of the Derivation Period
        If EstimatePrecip_InputFormat = 1 Then
            'Data in Monthly Format
            'January
            For i = 0 To JanCount
                If JanAllPrecip(i) <> -99 And JanThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (JanAllPrecip(i) - JanThresholdPrecip(i))
                End If
            Next i
            JanDifference = DifferenceTotal / (JanCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JanDifference = Math.Round(JanDifference, 2)
                Case 2
                    'Days
                    JanDifference = Math.Round(JanDifference)
            End Select
            DifferenceTotal = 0
            'February
            For i = 0 To FebCount
                If FebAllPrecip(i) <> -99 And FebThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (FebAllPrecip(i) - FebThresholdPrecip(i))
                End If
            Next i
            FebDifference = DifferenceTotal / (FebCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    FebDifference = Math.Round(FebDifference, 2)
                Case 2
                    'Days
                    FebDifference = Math.Round(FebDifference)
            End Select
            DifferenceTotal = 0
            'March
            For i = 0 To MarCount
                If MarAllPrecip(i) <> -99 And MarThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (MarAllPrecip(i) - MarThresholdPrecip(i))
                End If
            Next i
            MarDifference = DifferenceTotal / (MarCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    MarDifference = Math.Round(MarDifference, 2)
                Case 2
                    'Days
                    MarDifference = Math.Round(MarDifference)
            End Select
            DifferenceTotal = 0
            'April
            For i = 0 To AprCount
                If AprAllPrecip(i) <> -99 And AprThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (AprAllPrecip(i) - AprThresholdPrecip(i))
                End If
            Next i
            AprDifference = DifferenceTotal / (AprCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    AprDifference = Math.Round(AprDifference, 2)
                Case 2
                    'Days
                    AprDifference = Math.Round(AprDifference)
            End Select
            DifferenceTotal = 0
            'May
            For i = 0 To MayCount
                If MayAllPrecip(i) <> -99 And MayThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (MayAllPrecip(i) - MayThresholdPrecip(i))
                End If
            Next i
            MayDifference = DifferenceTotal / (MayCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    MayDifference = Math.Round(MayDifference, 2)
                Case 2
                    'Days
                    MayDifference = Math.Round(MayDifference)
            End Select
            DifferenceTotal = 0
            'June
            For i = 0 To JunCount
                If JunAllPrecip(i) <> -99 And JunThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (JunAllPrecip(i) - JunThresholdPrecip(i))
                End If
            Next i
            JunDifference = DifferenceTotal / (JunCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JunDifference = Math.Round(JunDifference, 2)
                Case 2
                    'Days
                    JunDifference = Math.Round(JunDifference)
            End Select
            DifferenceTotal = 0
            'July
            For i = 0 To JulCount
                If JulAllPrecip(i) <> -99 And JulThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (JulAllPrecip(i) - JulThresholdPrecip(i))
                End If
            Next i
            JulDifference = DifferenceTotal / (JulCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    JulDifference = Math.Round(JulDifference, 2)
                Case 2
                    'Days
                    JulDifference = Math.Round(JulDifference)
            End Select
            DifferenceTotal = 0
            'August
            For i = 0 To AugCount
                If AugAllPrecip(i) <> -99 And AugThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (AugAllPrecip(i) - AugThresholdPrecip(i))
                End If
            Next i
            AugDifference = DifferenceTotal / (AugCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    AugDifference = Math.Round(AugDifference, 2)
                Case 2
                    'Days
                    AugDifference = Math.Round(AugDifference)
            End Select
            DifferenceTotal = 0
            'September
            For i = 0 To SepCount
                If SepAllPrecip(i) <> -99 And SepThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (SepAllPrecip(i) - SepThresholdPrecip(i))
                End If
            Next i
            SepDifference = DifferenceTotal / (SepCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    SepDifference = Math.Round(SepDifference, 2)
                Case 2
                    'Days
                    SepDifference = Math.Round(SepDifference)
            End Select
            DifferenceTotal = 0
            'October
            For i = 0 To OctCount
                If OctAllPrecip(i) <> -99 And OctThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (OctAllPrecip(i) - OctThresholdPrecip(i))
                End If
            Next i
            OctDifference = DifferenceTotal / (OctCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    OctDifference = Math.Round(OctDifference, 2)
                Case 2
                    'Days
                    OctDifference = Math.Round(OctDifference)
            End Select
            DifferenceTotal = 0
            'November
            For i = 0 To NovCount
                If NovAllPrecip(i) <> -99 And NovThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (NovAllPrecip(i) - NovThresholdPrecip(i))
                End If
            Next i
            NovDifference = DifferenceTotal / (NovCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    NovDifference = Math.Round(NovDifference, 2)
                Case 2
                    'Days
                    NovDifference = Math.Round(NovDifference)
            End Select
            DifferenceTotal = 0
            'December
            For i = 0 To DecCount
                If DecAllPrecip(i) <> -99 And DecThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (DecAllPrecip(i) - DecThresholdPrecip(i))
                End If
            Next i
            DecDifference = DifferenceTotal / (DecCount + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    DecDifference = Math.Round(DecDifference, 2)
                Case 2
                    'Days
                    DecDifference = Math.Round(DecDifference)
            End Select
            DifferenceTotal = 0
        End If
        If EstimatePrecip_InputFormat = 2 Then
            'Data in Seasonal/Annual Format
            DifferenceTotal = 0
            For i = 0 To Count
                If AllPrecip(i) <> -99 And ThresholdPrecip(i) <> -99 Then
                    DifferenceTotal = DifferenceTotal + (AllPrecip(i) - ThresholdPrecip(i))
                End If
            Next i
            Difference = DifferenceTotal / (Count + 1)
            Select Case EstimatePrecip_PrecipType
                Case 1
                    'Amounts
                    Difference = Math.Round(Difference, 2)
                Case 2
                    'Days
                    Difference = Math.Round(Difference)
            End Select
            DifferenceTotal = 0
        End If
        'Ouput Final Derived Differences
        PrintLine(2, "Final Derived Differences:")
        PrintLine(2, "")
        Select Case EstimatePrecip_InputFormat
            Case 1
                'Data in Monthly Format
                PrintLine(2, "Month" & vbTab & "Difference")
                PrintLine(2, "1" & vbTab & JanDifference)
                PrintLine(2, "2" & vbTab & FebDifference)
                PrintLine(2, "3" & vbTab & MarDifference)
                PrintLine(2, "4" & vbTab & AprDifference)
                PrintLine(2, "5" & vbTab & MayDifference)
                PrintLine(2, "6" & vbTab & JunDifference)
                PrintLine(2, "7" & vbTab & JulDifference)
                PrintLine(2, "8" & vbTab & AugDifference)
                PrintLine(2, "9" & vbTab & SepDifference)
                PrintLine(2, "10" & vbTab & OctDifference)
                PrintLine(2, "11" & vbTab & NovDifference)
                Print(2, "12" & vbTab & DecDifference)
            Case 2
                'Data in Seasonal/Annual Format
                Print(2, "Difference = " & Difference)
        End Select
        'Reset StreamReaders
        AllPrecipInFile.DiscardBufferedData()
        AllPrecipInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        ThresholdPrecipInFile.DiscardBufferedData()
        ThresholdPrecipInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        FoundStart = 0
        'Set StreamReaders to Data Start
        If EstimatePrecip_AllPrecipRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To EstimatePrecip_AllPrecipRowStart - 2
                AllPrecipLine = AllPrecipInFile.ReadLine()
            Next Row
        End If
        If EstimatePrecip_ThresholdPrecipRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To EstimatePrecip_ThresholdPrecipRowStart - 2
                ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
            Next Row
        End If
        'Set All Precipitation Data File to Adjustment Start Date
        'Skip Column Headers
        AllPrecipLine = AllPrecipInFile.ReadLine()
        Try
            While AllPrecipInFile.Peek <> -1
                AllPrecipLine = AllPrecipInFile.ReadLine()
                AllPrecipFields = AllPrecipLine.Split(ControlChars.Tab)
                Year = AllPrecipFields(0)
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        Month = AllPrecipFields(1)
                        If Year = EstimatePrecip_AdjStartYear Then
                            If Month = EstimatePrecip_AdjStartMonth Then
                                FoundStart = 1
                                Exit While
                            End If
                        End If
                    Case 2
                        'Data in Seasonal/Annual Format
                        If Year = EstimatePrecip_AdjStartYear Then
                            FoundStart = 1
                            Exit While
                        End If
                End Select
            End While
        Catch ex As Exception
            Select Case EstimatePrecip_InputFormat
                Case 1
                    'Data in Monthly Format
                    MsgBox("Error reading the all precipitation data file near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                Case 2
                    'Data in Seasonal/Annual Format
                    MsgBox("Error reading the all precipitation data file near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            End Select
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End Try
        If FoundStart = 0 Then
            MsgBox("Unable to find adjustment start date in the all precipitation data file.  Recheck the adjustment start date.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End If
        FoundStart = 0
        Year = 0
        'Set Threshold Precipitation Data File to Adjustment Start Date
        'Skip Column Headers
        ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
        Try
            While ThresholdPrecipInFile.Peek <> -1
                ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
                ThresholdPrecipFields = ThresholdPrecipLine.Split(ControlChars.Tab)
                Year = ThresholdPrecipFields(0)
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        Month = ThresholdPrecipFields(1)
                        If Year = EstimatePrecip_AdjStartYear Then
                            If Month = EstimatePrecip_AdjStartMonth Then
                                FoundStart = 1
                                Exit While
                            End If
                        End If
                    Case 2
                        'Data in Seasonal/Annual Format
                        If Year = EstimatePrecip_AdjStartYear Then
                            FoundStart = 1
                            Exit While
                        End If
                End Select
            End While
        Catch ex As Exception
            Select Case EstimatePrecip_InputFormat
                Case 1
                    'Data in Monthly Format
                    MsgBox("Error reading the threshold data file near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                Case 2
                    'Data in Seasonal/Annual Format
                    MsgBox("Error reading the threshold data file near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            End Select
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End Try
        If FoundStart = 0 Then
            MsgBox("Unable to find adjustment start date in the threshold data file.  Recheck the adjustment start date.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End If
        Try
            'Process Data at Start Date
            Year = AllPrecipFields(0)
            If EstimatePrecip_InputFormat = 1 Then
                'Data in Monthly Format
                Month = AllPrecipFields(1)
                Select Case Month
                    Case 1
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + JanDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 2
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + FebDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 3
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + MarDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 4
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + AprDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 5
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + MayDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 6
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + JunDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 7
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + JulDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 8
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + AugDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 9
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + SepDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 10
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + OctDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 11
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + NovDifference
                        Else
                            NewPrecip = -99
                        End If
                    Case 12
                        If ThresholdPrecipFields(2) <> -99 Then
                            NewPrecip = ThresholdPrecipFields(2) + DecDifference
                        Else
                            NewPrecip = -99
                        End If
                End Select
            End If
            If EstimatePrecip_InputFormat = 2 Then
                'Data in Seasonal/Annual Format
                If ThresholdPrecipFields(1) <> -99 Then
                    NewPrecip = ThresholdPrecipFields(1) + Difference
                Else
                    NewPrecip = -99
                End If
            End If
            'Output Estimated Data
            Select Case EstimatePrecip_InputFormat
                Case 1
                    'Data in Monthly Format
                    PrintLine(1, Year & vbTab & Month & vbTab & NewPrecip)
                Case 2
                    'Data in Seasonal/Annual Format
                    PrintLine(1, Year & vbTab & NewPrecip)
            End Select
            RunProgress = RunProgress + 1
            'Continue Processing Additional Dates
            While AllPrecipInFile.Peek <> -1
                AllPrecipLine = AllPrecipInFile.ReadLine()
                AllPrecipFields = AllPrecipLine.Split(ControlChars.Tab)
                ThresholdPrecipLine = ThresholdPrecipInFile.ReadLine()
                ThresholdPrecipFields = ThresholdPrecipLine.Split(ControlChars.Tab)
                Year = AllPrecipFields(0)
                If EstimatePrecip_InputFormat = 1 Then
                    'Data in Monthly Format
                    Month = AllPrecipFields(1)
                    Select Case Month
                        Case 1
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + JanDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 2
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + FebDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 3
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + MarDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 4
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + AprDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 5
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + MayDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 6
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + JunDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 7
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + JulDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 8
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + AugDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 9
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + SepDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 10
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + OctDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 11
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + NovDifference
                            Else
                                NewPrecip = -99
                            End If
                        Case 12
                            If ThresholdPrecipFields(2) <> -99 Then
                                NewPrecip = ThresholdPrecipFields(2) + DecDifference
                            Else
                                NewPrecip = -99
                            End If
                    End Select
                End If
                If EstimatePrecip_InputFormat = 2 Then
                    'Data in Seasonal/Annual Format
                    If ThresholdPrecipFields(1) <> -99 Then
                        NewPrecip = ThresholdPrecipFields(1) + Difference
                    Else
                        NewPrecip = -99
                    End If
                End If
                'Output Estimated Data
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        PrintLine(1, Year & vbTab & Month & vbTab & NewPrecip)
                    Case 2
                        'Data in Seasonal/Annual Format
                        PrintLine(1, Year & vbTab & NewPrecip)
                End Select
                'Update Progress Bar
                RunProgress = RunProgress + 1
                Percent = (RunProgress / RunComplete) * 100
                BackgroundRun.ReportProgress(Percent)
                'Check for End Date
                Select Case EstimatePrecip_InputFormat
                    Case 1
                        'Data in Monthly Format
                        If Year = EstimatePrecip_AdjEndYear Then
                            If Month = EstimatePrecip_AdjEndMonth Then
                                Exit While
                            End If
                        End If
                    Case 2
                        'Data in Seasonal/Annual Format
                        If Year = EstimatePrecip_AdjEndYear Then
                            Exit While
                        End If
                End Select
            End While
        Catch ex As Exception
            Select Case EstimatePrecip_InputFormat
                Case 1
                    'Data in Monthly Format
                    MsgBox("Error reading the all precipitation or threshold precipitation data file near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
                Case 2
                    'Data in Seasonal/Annual Format
                    MsgBox("Error reading the all precipitation or threshold precipitation data file near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Estimate Precipitation")
            End Select
            AllPrecipInFile.Close()
            ThresholdPrecipInFile.Close()
            FileClose(1)
            FileClose(2)
            EstimatePrecip_Exception = 1
            Exit Sub
        End Try
        'End of Adjustment Period Reached...Output Remaining Lines from All Precipitation Data File
        While AllPrecipInFile.Peek <> -1
            AllPrecipLine = AllPrecipInFile.ReadLine()
            PrintLine(1, AllPrecipLine)
        End While
        'Close Files
        AllPrecipInFile.Close()
        ThresholdPrecipInFile.Close()
        FileClose(1)
        FileClose(2)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case EstimatePrecip_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub EstimatePrecip_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        EstimatePrecipDisp = 0
    End Sub
End Class