Public Class ExtractGHCNDaily
    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private ExtractGHCNDaily_Input As String
    Private ExtractGHCNDaily_Output As String
    'Initialize Global Form Variables
    Private ExtractGHCNDaily_RowStart As Integer
    Private ExtractGHCNDaily_Header As String
    Private ExtractGHCNDaily_TempType As Integer
    Private ExtractGHCNDaily_TempUnits As Integer
    Private ExtractGHCNDaily_PrecipUnits As Integer
    Private ExtractGHCNDaily_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ExtractGHCNDaily_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Extract GHCN Daily Station Data Tool
        OutputDegC.Text = Chr(176) & "C"
        OutputDegF.Text = Chr(176) & "F"
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input File"
        FileSelect.Filter = "Daily Files (*.dly)|*.dly|Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ExtractGHCNDaily_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ExtractGHCNDaily_Input = FileSelect.FileName
            InputFileLocation.Text = ExtractGHCNDaily_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                ExtractGHCNDaily_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ExtractGHCNDaily_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ExtractGHCNDaily_Output = FileOutput.FileName
            OutputFileLocation.Text = ExtractGHCNDaily_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ExtractGHCNDaily_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ExtractGHCNDaily_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If ExtractGHCNDaily_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If ExtractGHCNDaily_Input = ExtractGHCNDaily_Output Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        ExtractGHCNDaily_Exception = 0
        If OutputMinMax.Checked = True Then
            ExtractGHCNDaily_TempType = 1
        End If
        If OutputMean.Checked = True Then
            ExtractGHCNDaily_TempType = 2
        End If
        If OutputDegC.Checked = True Then
            ExtractGHCNDaily_TempUnits = 1
        End If
        If OutputDegF.Checked = True Then
            ExtractGHCNDaily_TempUnits = 2
        End If
        If Outputinches.Checked = True Then
            ExtractGHCNDaily_PrecipUnits = 1
        End If
        If Outputcm.Checked = True Then
            ExtractGHCNDaily_PrecipUnits = 2
        End If
        If Outputmm.Checked = True Then
            ExtractGHCNDaily_PrecipUnits = 3
        End If
        Status_Label.Text = "Status:  Running..."
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim MonthMinData(30) As Double
        Dim MonthMaxData(30) As Double
        Dim MonthPrecipData(30) As Double
        Dim MonthSnowData(30) As Double
        Dim MonthSnowDepthData(30) As Double
        Dim DataValue As Double
        Dim MonthMeanData As Double
        Dim ObType As String
        Dim Year As Integer
        Dim YearProcessing As Integer
        Dim MonthProcessing As Integer
        Dim Month As Integer
        Dim TimeThrough As Integer
        Dim i As Integer
        Dim j As Integer
        TimeThrough = 0
        'Initialize Output File and Set Header
        FileOpen(1, ExtractGHCNDaily_Output, OpenMode.Output)
        PrintLine(1, ExtractGHCNDaily_Header)
        PrintLine(1, "")
        Select Case ExtractGHCNDaily_TempType
            Case 1
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & "Min" & vbTab & "Max" & vbTab & "Precip" & vbTab & "Snow" & vbTab & "SnowDepth")
            Case 2
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & "Mean" & vbTab & "Precip" & vbTab & "Snow" & vbTab & "Snow Depth")
        End Select
        'Set Arrays
        For i = 0 To 30
            MonthMinData(i) = -99
            MonthMaxData(i) = -99
            MonthPrecipData(i) = -99
            MonthSnowData(i) = -99
            MonthSnowDepthData(i) = -99
        Next i
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ExtractGHCNDaily_Input)
        Dim Line As String
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set StreamReader to Data Start
        If ExtractGHCNDaily_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ExtractGHCNDaily_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = FileComplete - (ExtractGHCNDaily_RowStart - 1)
        End If
        Try
            While InFile.Peek <> -1
                'Process Line
                Line = InFile.ReadLine()
                Line.ToCharArray()
                Year = Line(11) + Line(12) + Line(13) + Line(14)
                Month = Line(15) + Line(16)
                ObType = Line(17) + Line(18) + Line(19) + Line(20)
                If TimeThrough = 0 Then
                    TimeThrough = 1
                    YearProcessing = Year
                    MonthProcessing = Month
                End If
                If Year <> YearProcessing Or Month <> MonthProcessing Then
                    'Output Extracted Data
                    Select Case MonthProcessing
                        Case 1, 3, 5, 7, 8, 10, 12
                            For i = 0 To 30
                                If ExtractGHCNDaily_TempType = 1 Then
                                    'Output Min/Max Temperature Followed by Precipitation
                                    PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                Else
                                    'Output Mean Temperature Followed by Precipitation
                                    If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                                        MonthMeanData = -99
                                    Else
                                        MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                                    End If
                                    PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                End If
                            Next i
                        Case 4, 6, 9, 11
                            For i = 0 To 29
                                If ExtractGHCNDaily_TempType = 1 Then
                                    'Output Min/Max Temperature Followed by Precipitation
                                    PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                Else
                                    'Output Mean Temperature Followed by Precipitation
                                    If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                                        MonthMeanData = -99
                                    Else
                                        MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                                    End If
                                    PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                End If
                            Next i
                        Case 2
                            If Date.IsLeapYear(YearProcessing) = True Then
                                For i = 0 To 28
                                    If ExtractGHCNDaily_TempType = 1 Then
                                        'Output Min/Max Temperature Followed by Precipitation
                                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                    Else
                                        'Output Mean Temperature Followed by Precipitation
                                        If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                                            MonthMeanData = -99
                                        Else
                                            MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                                        End If
                                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                    End If
                                Next i
                            Else
                                For i = 0 To 27
                                    If ExtractGHCNDaily_TempType = 1 Then
                                        'Output Min/Max Temperature Followed by Precipitation
                                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                    Else
                                        'Output Mean Temperature Followed by Precipitation
                                        If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                                            MonthMeanData = -99
                                        Else
                                            MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                                        End If
                                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                                    End If
                                Next i
                            End If
                    End Select
                    'Set New Year and Month
                    YearProcessing = Year
                    MonthProcessing = Month
                    'Reset Arrays
                    For i = 0 To 30
                        MonthMinData(i) = -99
                        MonthMaxData(i) = -99
                        MonthPrecipData(i) = -99
                        MonthSnowData(i) = -99
                        MonthSnowDepthData(i) = -99
                    Next i
                End If
                j = 21
                Select Case ObType
                    Case Is = "TMAX"
                        'Extract Maximum Temperature Data (Tenths of Degree C)
                        For i = 0 To 30
                            DataValue = Line(j) + Line(j + 1) + Line(j + 2) + Line(j + 3) + Line(j + 4)
                            Select Case DataValue = -9999
                                Case True
                                    MonthMaxData(i) = -99
                                Case False
                                    'Divide by 10 to Remove Tenths Format
                                    DataValue = DataValue / 10
                                    'Change Units If Necessary
                                    If ExtractGHCNDaily_TempUnits = 2 Then
                                        'Convert to Degrees F
                                        DataValue = DegCDegF(DataValue)
                                    End If
                                    MonthMaxData(i) = DataValue
                            End Select
                            j = j + 8
                        Next i
                    Case Is = "TMIN"
                        'Extract Minimum Temperature Data (Tenths of Degree C)
                        For i = 0 To 30
                            DataValue = Line(j) + Line(j + 1) + Line(j + 2) + Line(j + 3) + Line(j + 4)
                            Select Case DataValue = -9999
                                Case True
                                    MonthMinData(i) = -99
                                Case False
                                    'Divide by 10 to Remove Tenths Format
                                    DataValue = DataValue / 10
                                    'Change Units If Necessary
                                    If ExtractGHCNDaily_TempUnits = 2 Then
                                        'Convert to Degrees F
                                        DataValue = DegCDegF(DataValue)
                                    End If
                                    MonthMinData(i) = DataValue
                            End Select
                            j = j + 8
                        Next i
                    Case Is = "PRCP"
                        'Extract Precipitation Data (Tenths of mm)
                        For i = 0 To 30
                            DataValue = Line(j) + Line(j + 1) + Line(j + 2) + Line(j + 3) + Line(j + 4)
                            Select Case DataValue = -9999
                                Case True
                                    MonthPrecipData(i) = -99
                                Case False
                                    'Divide by 10 to Remove Tenths Format
                                    DataValue = DataValue / 10
                                    'Change Units If Necessary
                                    If ExtractGHCNDaily_PrecipUnits = 1 Then
                                        'Convert to inches
                                        DataValue = DataValue / 25.4
                                    End If
                                    If ExtractGHCNDaily_PrecipUnits = 2 Then
                                        'Convert to cm
                                        DataValue = DataValue / 10
                                    End If
                                    MonthPrecipData(i) = DataValue
                            End Select
                            j = j + 8
                        Next i
                    Case Is = "SNOW"
                        'Extract Snow Data (Tenths of mm)
                        For i = 0 To 30
                            DataValue = Line(j) + Line(j + 1) + Line(j + 2) + Line(j + 3) + Line(j + 4)
                            Select Case DataValue = -9999
                                Case True
                                    MonthSnowData(i) = -99
                                Case False
                                    'Divide by 10 to Remove Tenths Format
                                    DataValue = DataValue / 10
                                    'Change Units If Necessary
                                    If ExtractGHCNDaily_PrecipUnits = 1 Then
                                        'Convert to inches
                                        DataValue = DataValue / 25.4
                                    End If
                                    If ExtractGHCNDaily_PrecipUnits = 2 Then
                                        'Convert to cm
                                        DataValue = DataValue / 10
                                    End If
                                    MonthSnowData(i) = DataValue
                            End Select
                            j = j + 8
                        Next i
                    Case Is = "SNWD"
                        'Extract Snow Depth Data (Tenths of mm)
                        For i = 0 To 30
                            DataValue = Line(j) + Line(j + 1) + Line(j + 2) + Line(j + 3) + Line(j + 4)
                            Select Case DataValue = -9999
                                Case True
                                    MonthSnowDepthData(i) = -99
                                Case False
                                    'Divide by 10 to Remove Tenths Format
                                    DataValue = DataValue / 10
                                    'Change Units If Necessary
                                    If ExtractGHCNDaily_PrecipUnits = 1 Then
                                        'Convert to inches
                                        DataValue = DataValue / 25.4
                                    End If
                                    If ExtractGHCNDaily_PrecipUnits = 2 Then
                                        'Convert to cm
                                        DataValue = DataValue / 10
                                    End If
                                    MonthSnowDepthData(i) = DataValue
                            End Select
                            j = j + 8
                        Next i
                End Select
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If ObType = "PRCP" Or ObType = "TMAX" Or ObType = "TMIN" Or ObType = "SNOW" Or ObType = "SNWD" Or ObType = "TOBS" Then
                MsgBox("Error reading the input data file at or near the following Observation Type/Year/Month:  " & ObType & "/" & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Extract GHCN Daily Station Data")
            Else
                MsgBox("Error reading the input data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Extract GHCN Daily Station Data")
            End If
            FileClose(1)
            InFile.Close()
            ExtractGHCNDaily_Exception = 1
            Exit Sub
        End Try
        'Output Final Month
        Select Case MonthProcessing
            Case 1, 3, 5, 7, 8, 10, 12
                For i = 0 To 30
                    If ExtractGHCNDaily_TempType = 1 Then
                        'Output Min/Max Temperature Followed by Precipitation
                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                    Else
                        'Output Mean Temperature Followed by Precipitation
                        If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                            MonthMeanData = -99
                        Else
                            MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                        End If
                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                    End If
                Next i
            Case 4, 6, 9, 11
                For i = 0 To 29
                    If ExtractGHCNDaily_TempType = 1 Then
                        'Output Min/Max Temperature Followed by Precipitation
                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                    Else
                        'Output Mean Temperature Followed by Precipitation
                        If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                            MonthMeanData = -99
                        Else
                            MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                        End If
                        PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                    End If
                Next i
            Case 2
                If Date.IsLeapYear(YearProcessing) = True Then
                    For i = 0 To 28
                        If ExtractGHCNDaily_TempType = 1 Then
                            'Output Min/Max Temperature Followed by Precipitation
                            PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                        Else
                            'Output Mean Temperature Followed by Precipitation
                            If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                                MonthMeanData = -99
                            Else
                                MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                            End If
                            PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                        End If
                    Next i
                Else
                    For i = 0 To 27
                        If ExtractGHCNDaily_TempType = 1 Then
                            'Output Min/Max Temperature Followed by Precipitation
                            PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMinData(i), 1) & vbTab & Math.Round(MonthMaxData(i), 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                        Else
                            'Output Mean Temperature Followed by Precipitation
                            If MonthMinData(i) = -99 Or MonthMaxData(i) = -99 Then
                                MonthMeanData = -99
                            Else
                                MonthMeanData = (MonthMinData(i) + MonthMaxData(i)) / 2
                            End If
                            PrintLine(1, YearProcessing & vbTab & MonthProcessing & vbTab & i + 1 & vbTab & Math.Round(MonthMeanData, 1) & vbTab & Math.Round(MonthPrecipData(i), 2) & vbTab & Math.Round(MonthSnowData(i), 2) & vbTab & Math.Round(MonthSnowDepthData(i), 2))
                        End If
                    Next i
                End If
        End Select
        'Close Files
        FileClose(1)
        InFile.Close()
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ExtractGHCNDaily_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ExtractGHCNDaily_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ExtractGHCNDailyDisp = 0
    End Sub
End Class