﻿Public Class ExtractIGRA

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private ExtractIGRA_Input As String
    Private ExtractIGRA_Output As String
    'Initialize Global Form Variables
    Private ExtractIGRA_RowStart As Integer
    Private ExtractIGRA_12Z As Integer
    Private ExtractIGRA_00Z As Integer
    Private ExtractIGRA_CAPE As Integer
    Private ExtractIGRA_CIN As Integer
    Private ExtractIGRA_FrzHgt As Integer
    Private ExtractIGRA_KI As Integer
    Private ExtractIGRA_LCLHgt As Integer
    Private ExtractIGRA_LFCHgt As Integer
    Private ExtractIGRA_LI As Integer
    Private ExtractIGRA_PW As Integer
    Private ExtractIGRA_SI As Integer
    Private ExtractIGRA_TT As Integer
    Private ExtractIGRA_Header As String
    Private ExtractIGRA_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ExtractIGRA_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'Initialize Extract IGRA Data Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
        ObTime.SelectedIndex = 0
    End Sub

    Private Sub FileSelect_Button_Click(sender As System.Object, e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ExtractIGRA_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ExtractIGRA_Input = FileSelect.FileName
            InputFileLocation.Text = ExtractIGRA_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(sender As System.Object, e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                ExtractIGRA_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ObTime_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles ObTime.SelectedIndexChanged
        Select Case ObTime.SelectedIndex
            Case 0
                '00Z Data Extraction
                ExtractIGRA_00Z = 1
                ExtractIGRA_12Z = 0
            Case 1
                '12Z Data Extraction
                ExtractIGRA_00Z = 0
                ExtractIGRA_12Z = 1
        End Select
    End Sub

    Private Sub ExtractCAPE_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractCAPE.CheckedChanged
        If ExtractCAPE.Checked = True Then
            ExtractIGRA_CAPE = 1
        Else
            ExtractIGRA_CAPE = 0
        End If
    End Sub

    Private Sub ExtractCIN_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractCIN.CheckedChanged
        If ExtractCIN.Checked = True Then
            ExtractIGRA_CIN = 1
        Else
            ExtractIGRA_CIN = 0
        End If
    End Sub

    Private Sub ExtractFrzHgt_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractFrzHgt.CheckedChanged
        If ExtractFrzHgt.Checked = True Then
            ExtractIGRA_FrzHgt = 1
        Else
            ExtractIGRA_FrzHgt = 0
        End If
    End Sub

    Private Sub ExtractKI_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractKI.CheckedChanged
        If ExtractKI.Checked = True Then
            ExtractIGRA_KI = 1
        Else
            ExtractIGRA_KI = 0
        End If
    End Sub

    Private Sub ExtractLCLHgt_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractLCLHgt.CheckedChanged
        If ExtractLCLHgt.Checked = True Then
            ExtractIGRA_LCLHgt = 1
        Else
            ExtractIGRA_LCLHgt = 0
        End If
    End Sub

    Private Sub ExtractLI_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractLI.CheckedChanged
        If ExtractLI.Checked = True Then
            ExtractIGRA_LI = 1
        Else
            ExtractIGRA_LI = 0
        End If
    End Sub

    Private Sub ExtractLFCHgt_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractLFCHgt.CheckedChanged
        If ExtractLFCHgt.Checked = True Then
            ExtractIGRA_LFCHgt = 1
        Else
            ExtractIGRA_LFCHgt = 0
        End If
    End Sub

    Private Sub ExtractPW_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractPW.CheckedChanged
        If ExtractPW.Checked = True Then
            ExtractIGRA_PW = 1
        Else
            ExtractIGRA_PW = 0
        End If
    End Sub

    Private Sub ExtractSI_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractSI.CheckedChanged
        If ExtractSI.Checked = True Then
            ExtractIGRA_SI = 1
        Else
            ExtractIGRA_SI = 0
        End If
    End Sub

    Private Sub ExtractTT_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ExtractTT.CheckedChanged
        If ExtractTT.Checked = True Then
            ExtractIGRA_TT = 1
        Else
            ExtractIGRA_TT = 0
        End If
    End Sub

    Private Sub FileOutput_Button_Click(sender As System.Object, e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ExtractIGRA_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ExtractIGRA_Output = FileOutput.FileName
            OutputFileLocation.Text = ExtractIGRA_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(sender As System.Object, e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ExtractIGRA_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(sender As System.Object, e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ExtractIGRA_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If ExtractCAPE.Checked = False And ExtractCIN.Checked = False And ExtractFrzHgt.Checked = False And ExtractKI.Checked = False And ExtractLCLHgt.Checked = False And ExtractLFCHgt.Checked = False And ExtractLI.Checked = False And ExtractPW.Checked = False And ExtractSI.Checked = False And ExtractTT.Checked = False Then
            EntryError.SetError(ExtractCAPE, "No data have been selected to extract.")
            Exit Sub
        End If
        If ExtractIGRA_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If ExtractIGRA_Input = ExtractIGRA_Output Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        ExtractIGRA_Exception = 0
        Status_Label.Text = "Status:  Running..."
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(sender As System.Object, e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim Hour As Integer
        Dim CAPE As Integer
        Dim CIN As Integer
        Dim FrzHgt As Integer
        Dim KI As Integer
        Dim LCLHgt As Integer
        Dim LFCHgt As Integer
        Dim LI As Integer
        Dim PW As Integer
        Dim SI As Integer
        Dim TT As Integer
        Dim PassedHeader As Integer
        PassedHeader = 0
        'Initialize Output File and Set Header
        FileOpen(1, ExtractIGRA_Output, OpenMode.Output)
        PrintLine(1, ExtractIGRA_Header)
        PrintLine(1, "")
        Print(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & "Hour")
        If ExtractIGRA_CAPE = 1 Then
            Print(1, vbTab & "CAPE")
        End If
        If ExtractIGRA_CIN = 1 Then
            Print(1, vbTab & "CIN")
        End If
        If ExtractIGRA_FrzHgt = 1 Then
            Print(1, vbTab & "FrzHgt")
        End If
        If ExtractIGRA_KI = 1 Then
            Print(1, vbTab & "KI")
        End If
        If ExtractIGRA_LCLHgt = 1 Then
            Print(1, vbTab & "LCLHgt")
        End If
        If ExtractIGRA_LFCHgt = 1 Then
            Print(1, vbTab & "LFCHgt")
        End If
        If ExtractIGRA_LI = 1 Then
            Print(1, vbTab & "LI")
        End If
        If ExtractIGRA_PW = 1 Then
            Print(1, vbTab & "PW")
        End If
        If ExtractIGRA_SI = 1 Then
            Print(1, vbTab & "SI")
        End If
        If ExtractIGRA_TT = 1 Then
            Print(1, vbTab & "TT")
        End If
        Print(1, vbCrLf)
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ExtractIGRA_Input)
        Dim Line As String
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set StreamReader to Data Start
        If ExtractIGRA_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ExtractIGRA_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = FileComplete - (ExtractIGRA_RowStart - 1)
        End If
        Try
            While InFile.Peek <> -1
                'Process Line
                Line = InFile.ReadLine()
                Line.ToCharArray()
                If Line(0) = "#" Then
                    PassedHeader = 1
                    Year = Line(6) + Line(7) + Line(8) + Line(9)
                    Month = Line(10) + Line(11)
                    Day = Line(12) + Line(13)
                    Hour = Line(14) + Line(15)
                    CAPE = Line(132) + Line(133) + Line(134) + Line(135) + Line(136) + Line(137)
                    CIN = Line(138) + Line(139) + Line(140) + Line(141) + Line(142) + Line(143)
                    FrzHgt = Line(66) + Line(67) + Line(68) + Line(69) + Line(70) + Line(71)
                    KI = Line(120) + Line(121) + Line(122) + Line(123) + Line(124) + Line(125)
                    LCLHgt = Line(78) + Line(79) + Line(80) + Line(81) + Line(82) + Line(83)
                    LFCHgt = Line(90) + Line(91) + Line(92) + Line(93) + Line(94) + Line(95)
                    LI = Line(108) + Line(109) + Line(110) + Line(111) + Line(112) + Line(113)
                    PW = Line(24) + Line(25) + Line(26) + Line(27) + Line(28) + Line(29)
                    SI = Line(114) + Line(115) + Line(116) + Line(117) + Line(118) + Line(119)
                    TT = Line(126) + Line(127) + Line(128) + Line(129) + Line(130) + Line(131)
                    If CAPE = -99999 Then
                        CAPE = -99
                    End If
                    If CIN = -99999 Then
                        CIN = -999
                    End If
                    If FrzHgt = -99999 Then
                        FrzHgt = -99
                    End If
                    If KI = -99999 Then
                        KI = -99
                    End If
                    If LCLHgt = -99999 Then
                        LCLHgt = -99
                    End If
                    If LFCHgt = -99999 Then
                        LFCHgt = -99
                    End If
                    If LI = -99999 Then
                        LI = -99
                    End If
                    If PW = -99999 Then
                        PW = -99
                    End If
                    If SI = -99999 Then
                        SI = -99
                    End If
                    If TT = -99999 Then
                        TT = -99
                    End If
                    If ((Hour = 11 Or Hour = 12 Or Hour = 13) And ExtractIGRA_12Z = 1) Or ((Hour = 23 Or Hour = 0 Or Hour = 1) And ExtractIGRA_00Z = 1) Then
                        'Output Data
                        Print(1, Year & vbTab & Month & vbTab & Day & vbTab & Hour)
                        If ExtractIGRA_CAPE = 1 Then
                            Print(1, vbTab & CAPE)
                        End If
                        If ExtractIGRA_CIN = 1 Then
                            Print(1, vbTab & CIN)
                        End If
                        If ExtractIGRA_FrzHgt = 1 Then
                            Print(1, vbTab & FrzHgt)
                        End If
                        If ExtractIGRA_KI = 1 Then
                            Print(1, vbTab & KI)
                        End If
                        If ExtractIGRA_LCLHgt = 1 Then
                            Print(1, vbTab & LCLHgt)
                        End If
                        If ExtractIGRA_LFCHgt = 1 Then
                            Print(1, vbTab & LFCHgt)
                        End If
                        If ExtractIGRA_LI = 1 Then
                            Print(1, vbTab & LI)
                        End If
                        If ExtractIGRA_PW = 1 Then
                            Print(1, vbTab & PW)
                        End If
                        If ExtractIGRA_SI = 1 Then
                            Print(1, vbTab & SI)
                        End If
                        If ExtractIGRA_TT = 1 Then
                            Print(1, vbTab & TT)
                        End If
                        Print(1, vbCrLf)
                    End If
                End If
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If PassedHeader = 1 Then
                MsgBox("Error reading the input data file at or near the following Year/Month/Day/Hour:  " & Year & "/" & Month & "/" & Day & "/" & Hour & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Extract IGRA Data")
            Else
                MsgBox("Error reading the input data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Extract IGRA Data")
            End If
            FileClose(1)
            InFile.Close()
            ExtractIGRA_Exception = 1
            Exit Sub
        End Try
        'Close Files
        FileClose(1)
        InFile.Close()
    End Sub

    Private Sub BackgroundRun_ProgressChanged(sender As Object, e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(sender As Object, e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ExtractIGRA_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ExtractIGRA_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ExtractIGRADisp = 0
    End Sub
End Class