Public Class ExtractUSHCNMonthly
    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private ExtractUSHCNMonthly_Input As String
    Private ExtractUSHCNMonthly_Output As String
    'Initialize Global Form Variables
    Private ExtractUSHCNMonthly_RowStart As Integer
    Private ExtractUSHCNMonthly_Header As String
    Private ExtractUSHCNMonthly_DataType As Integer
    Private ExtractUSHCNMonthly_OutputUnits As Integer
    Private ExtractUSHCNMonthly_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ExtractUSHCNMonthly_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Extract USHCN Monthly Station Data Tool
        Output1.Text = Chr(176) & "C"
        Output2.Text = Chr(176) & "F"
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ExtractUSHCNMonthly_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ExtractUSHCNMonthly_Input = FileSelect.FileName
            InputFileLocation.Text = ExtractUSHCNMonthly_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                ExtractUSHCNMonthly_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TempInput_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TempInput.CheckedChanged
        If TempInput.Checked = True Then
            'Change Output Units to Temperature Units
            Output1.Text = Chr(176) & "C"
            Output2.Text = Chr(176) & "F"
            Output3.Text = ""
            Output3.Enabled = False
        End If
    End Sub

    Private Sub PrecipInput_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrecipInput.CheckedChanged
        If PrecipInput.Checked = True Then
            'Change Output Units to Precipitation Units
            Output3.Enabled = True
            Output1.Text = "inches"
            Output2.Text = "cm"
            Output3.Text = "mm"
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ExtractUSHCNMonthly_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ExtractUSHCNMonthly_Output = FileOutput.FileName
            OutputFileLocation.Text = ExtractUSHCNMonthly_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ExtractUSHCNMonthly_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ExtractUSHCNMonthly_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If ExtractUSHCNMonthly_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If ExtractUSHCNMonthly_Input = ExtractUSHCNMonthly_Output Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        ExtractUSHCNMonthly_Exception = 0
        If TempInput.Checked = True Then
            ExtractUSHCNMonthly_DataType = 1
        End If
        If PrecipInput.Checked = True Then
            ExtractUSHCNMonthly_DataType = 2
        End If
        If Output1.Checked = True Then
            ExtractUSHCNMonthly_OutputUnits = 1
        End If
        If Output2.Checked = True Then
            ExtractUSHCNMonthly_OutputUnits = 2
        End If
        If Output3.Checked = True Then
            ExtractUSHCNMonthly_OutputUnits = 3
        End If
        Status_Label.Text = "Status:  Running..."
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim MonthlyData As Double
        Dim Total As Integer
        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        i = 0
        j = 0
        k = 0
        'Initialize Output File and Set Header
        FileOpen(1, ExtractUSHCNMonthly_Output, OpenMode.Output)
        PrintLine(1, ExtractUSHCNMonthly_Header)
        PrintLine(1, "")
        Select Case ExtractUSHCNMonthly_DataType
            Case 1
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Temp")
            Case 2
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Precip")
        End Select
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ExtractUSHCNMonthly_Input)
        Dim Line As String
        'Set StreamReader to Data Start
        If ExtractUSHCNMonthly_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ExtractUSHCNMonthly_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
        End If
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Reset StreamReader to Data Start
        If ExtractUSHCNMonthly_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ExtractUSHCNMonthly_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
        End If
        'Set Total
        Total = FileComplete * 12 - 1
        'Initialize Observation Arrays
        Dim DataYear(Total) As Integer
        Dim DataMonth(Total) As Integer
        Dim DataValue(Total) As Double
        Try
            While InFile.Peek <> -1
                'Process Line
                Line = InFile.ReadLine()
                Line.ToCharArray()
                Year = Line(12) + Line(13) + Line(14) + Line(15)
                'Run Extraction Sequence (Hundredths of Degree C or Tenths of mm)
                k = 17
                For j = 1 To 12
                    DataYear(i) = Year
                    DataMonth(i) = j
                    MonthlyData = Line(k) + Line(k + 1) + Line(k + 2) + Line(k + 3) + Line(k + 4)
                    Select Case MonthlyData = -9999
                        Case True
                            DataValue(i) = -99
                        Case False
                            Select Case ExtractUSHCNMonthly_DataType
                                Case 1
                                    'Temperature Data...Divide by 100 to Remove Hundredths Format
                                    MonthlyData = MonthlyData / 100
                                    DataValue(i) = MonthlyData
                                Case 2
                                    'Precipitation Data...Divide by 10 to Remove Tenths Format
                                    MonthlyData = MonthlyData / 10
                                    DataValue(i) = MonthlyData
                            End Select
                    End Select
                    i = i + 1
                    k = k + 9
                Next j
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the temperature data file at or near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Extract USHCN Monthly Station Data")
            Else
                MsgBox("Error reading the temperature data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Extract USHCN Monthly Station Data")
            End If
            InFile.Close()
            FileClose(1)
            ExtractUSHCNMonthly_Exception = 1
            Exit Sub
        End Try
        Select Case ExtractUSHCNMonthly_DataType
            Case 1
                'Output Temperature Data
                Select Case ExtractUSHCNMonthly_OutputUnits
                    Case 1
                        'Output Degrees C
                        For i = 0 To Total
                            PrintLine(1, DataYear(i) & vbTab & DataMonth(i) & vbTab & Math.Round(DataValue(i), 1))
                        Next i
                    Case 2
                        'Output Degrees F
                        For i = 0 To Total
                            If DataValue(i) <> -99 Then
                                DataValue(i) = DegCDegF(DataValue(i))
                            End If
                            PrintLine(1, DataYear(i) & vbTab & DataMonth(i) & vbTab & Math.Round(DataValue(i), 1))
                        Next i
                End Select
            Case 2
                'Output Precipitation Data
                Select Case ExtractUSHCNMonthly_OutputUnits
                    Case 1
                        'Output inches
                        For i = 0 To Total
                            If DataValue(i) <> -99 Then
                                DataValue(i) = DataValue(i) / 25.4
                            End If
                            PrintLine(1, DataYear(i) & vbTab & DataMonth(i) & vbTab & Math.Round(DataValue(i), 2))
                        Next i
                    Case 2
                        'Output cm
                        For i = 0 To Total
                            If DataValue(i) <> -99 Then
                                DataValue(i) = DataValue(i) / 10
                            End If
                            PrintLine(1, DataYear(i) & vbTab & DataMonth(i) & vbTab & Math.Round(DataValue(i), 2))
                        Next i
                    Case 3
                        'Output mm
                        For i = 0 To Total
                            PrintLine(1, DataYear(i) & vbTab & DataMonth(i) & vbTab & Math.Round(DataValue(i), 2))
                        Next i
                End Select
        End Select
        'Close Files
        InFile.Close()
        FileClose(1)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ExtractUSHCNMonthly_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ExtractUSHCNMonthly_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ExtractUSHCNMonthlyDisp = 0
    End Sub
End Class