Module _Module

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2012  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Main Program Variables
    Public MainParent As New Main
    Public HOBToolsDirectory As String
    Public HOBToolsUG As String
    'Initialize Form Display Variables
    Public DataEntryDisp As Integer
    Public AddMissingDatesDisp As Integer
    Public TransformMonthDisp As Integer
    Public ExtractGHCNDailyDisp As Integer
    Public ExtractGHCNMonthlyDisp As Integer
    Public ExtractIGRADisp As Integer
    Public ExtractUSHCNMonthlyDisp As Integer
    Public DoubleMassDisp As Integer
    Public DoubleMassFigDisp As Integer
    Public IntTempRangeDisp As Integer
    Public IntTempRangeFigDisp As Integer
    Public AdjustStationSegmentsDisp As Integer
    Public DailyMeanDisp As Integer
    Public AdjustDailyTempDisp As Integer
    Public ReconstructTempDisp As Integer
    Public ReconstructTempFigDisp As Integer
    Public TempUnitsDisp As Integer
    Public TempAnalysisDisp As Integer
    Public ScreenPrecipDisp As Integer
    Public ScreenPrecipFigDisp As Integer
    Public PrecipTotalsDisp As Integer
    Public PrecipPercentDisp As Integer
    Public ReconstructPrecipDisp As Integer
    Public EstimatePrecipDisp As Integer
    Public PrecipUnitsDisp As Integer
    Public ColdWavesDisp As Integer
    Public GrowSeasonDisp As Integer
    Public ReturnTimeDisp As Integer
    Public ReturnTimeFigDisp As Integer
    Public ThresholdAnalysisDisp As Integer
    Public DifferencePlotDisp As Integer
    Public SpaghettiPlotsDisp As Integer
    Public TimeSeriesPlotDisp As Integer
    Public UpdateDisp As Integer
    Public AboutDisp As Integer
    'Initialize Figure Variables
    Public DoubleMass_Output As String
    Public DoubleMass_InputFormat As Integer
    Public DoubleMass_StartYear As Integer
    Public DoubleMass_StartMonth As Integer
    Public DoubleMass_EndYear As Integer
    Public DoubleMass_EndMonth As Integer
    Public DoubleMass_AnalysisType As Integer
    Public DoubleMass_BreakpointType As Integer
    Public IntTempRange_Output As String
    Public IntTempRange_HistoricalDegF As Integer
    Public IntTempRange_HistoricalDegC As Integer
    Public IntTempRange_Header As String
    Public ReconstructTemp_DepStation As String
    Public ReconstructTemp_IndStation As String
    Public ReconstructTemp_OutputResiduals As String
    Public ScreenPrecip_Output As String
    Public ScreenPrecip_StartYear As Integer
    Public ScreenPrecip_StartMonth As Integer
    Public ScreenPrecip_StartDay As Integer
    Public ScreenPrecip_EndYear As Integer
    Public ScreenPrecip_EndMonth As Integer
    Public ScreenPrecip_EndDay As Integer
    Public ScreenPrecip_Header As String
    Public ReturnTime_Output As String
    Public ReturnTime_StartYear As Integer
    Public ReturnTime_EndYear As Integer
    Public ReturnTime_Header As String
    'Initialize Function Variables
    Public Season As String
    Public DegC As Double
    Public DegF As Double
    Public CriticalTValue As Double

    Function DegFDegC(ByVal DegF As Double) As Double
        'Convert DegF to DegC
        DegC = (DegF - 32) / 1.8
        Return DegC
    End Function

    Function DegCDegF(ByVal DegC As Double) As Double
        'Convert DegC to DegF
        DegF = 1.8 * DegC + 32
        Return DegF
    End Function

    Function GetTValue(ByVal DOF As Integer) As Double
        'T-Value Table for Alpha = 0.025 (Two-Tailed Test)
        Select Case DOF
            Case 1
                CriticalTValue = 12.7062047361747
            Case 2
                CriticalTValue = 4.30265272974946
            Case 3
                CriticalTValue = 3.18244630528371
            Case 4
                CriticalTValue = 2.77644510519779
            Case 5
                CriticalTValue = 2.57058183563631
            Case 6
                CriticalTValue = 2.44691185114497
            Case 7
                CriticalTValue = 2.36462425159278
            Case 8
                CriticalTValue = 2.30600413520417
            Case 9
                CriticalTValue = 2.2621571627982
            Case 10
                CriticalTValue = 2.22813885198627
            Case 11
                CriticalTValue = 2.20098516009164
            Case 12
                CriticalTValue = 2.1788128296672298
            Case 13
                CriticalTValue = 2.16036865646279
            Case 14
                CriticalTValue = 2.1447866879178
            Case 15
                CriticalTValue = 2.13144954555978
            Case 16
                CriticalTValue = 2.11990529922126
            Case 17
                CriticalTValue = 2.10981557783332
            Case 18
                CriticalTValue = 2.10092204024104
            Case 19
                CriticalTValue = 2.09302405440831
            Case 20
                CriticalTValue = 2.08596344726586
            Case 21
                CriticalTValue = 2.07961384472768
            Case 22
                CriticalTValue = 2.07387306790403
            Case 23
                CriticalTValue = 2.06865761041905
            Case 24
                CriticalTValue = 2.06389856162803
            Case 25
                CriticalTValue = 2.0595385527533
            Case 26
                CriticalTValue = 2.05552943864287
            Case 27
                CriticalTValue = 2.05183051648029
            Case 28
                CriticalTValue = 2.04840714179524
            Case 29
                CriticalTValue = 2.0452296421327
            Case 30
                CriticalTValue = 2.04227245630124
            Case 31
                CriticalTValue = 2.03951344639641
            Case 32
                CriticalTValue = 2.0369333434601
            Case 33
                CriticalTValue = 2.03451529744934
            Case 34
                CriticalTValue = 2.03224450931772
            Case 35
                CriticalTValue = 2.03010792825034
            Case 36
                CriticalTValue = 2.02809400098045
            Case 37
                CriticalTValue = 2.02619246302911
            Case 38
                CriticalTValue = 2.02439416391197
            Case 39
                CriticalTValue = 2.02269092003676
            Case 40
                CriticalTValue = 2.02107539030627
            Case 41
                CriticalTValue = 2.01954097044138
            Case 42
                CriticalTValue = 2.01808170281844
            Case 43
                CriticalTValue = 2.01669219922782
            Case 44
                CriticalTValue = 2.01536757444376
            Case 45
                CriticalTValue = 2.01410338888085
            Case 46
                CriticalTValue = 2.01289559891943
            Case 47
                CriticalTValue = 2.01174051372977
            Case 48
                CriticalTValue = 2.01063475762423
            Case 49
                CriticalTValue = 2.00957523712924
            Case 50
                CriticalTValue = 2.00855911210076
            Case 51
                CriticalTValue = 2.00758377031584
            Case 52
                CriticalTValue = 2.00664680506169
            Case 53
                CriticalTValue = 2.00574599531787
            Case 54
                CriticalTValue = 2.00487928818806
            Case 55
                CriticalTValue = 2.00404478328915
            Case 56
                CriticalTValue = 2.00324071884787
            Case 57
                CriticalTValue = 2.00246545929101
            Case 58
                CriticalTValue = 2.00171748414524
            Case 59
                CriticalTValue = 2.00099537808827
            Case 60
                CriticalTValue = 2.00029782201426
            Case 61
                CriticalTValue = 1.99962358499494
            Case 62
                CriticalTValue = 1.99897151703338
            Case 63
                CriticalTValue = 1.99834054252074
            Case 64
                CriticalTValue = 1.99772965431769
            Case 65
                CriticalTValue = 1.997137908392
            Case 66
                CriticalTValue = 1.99656441895231
            Case 67
                CriticalTValue = 1.9960083540253
            Case 68
                CriticalTValue = 1.99546893142984
            Case 69
                CriticalTValue = 1.99494541510724
            Case 70
                CriticalTValue = 1.99443711177119
            Case 71
                CriticalTValue = 1.99394336784563
            Case 72
                CriticalTValue = 1.99346356666187
            Case 73
                CriticalTValue = 1.99299712588986
            Case 74
                CriticalTValue = 1.99254349518093
            Case 75
                CriticalTValue = 1.99210215400224
            Case 76
                CriticalTValue = 1.99167260964466
            Case 77
                CriticalTValue = 1.99125439538839
            Case 78
                CriticalTValue = 1.99084706881169
            Case 79
                CriticalTValue = 1.99045021023013
            Case 80
                CriticalTValue = 1.99006342125445
            Case 81
                CriticalTValue = 1.9896863234569
            Case 82
                CriticalTValue = 1.98931855713657
            Case 83
                CriticalTValue = 1.98895978017516
            Case 84
                CriticalTValue = 1.98860966697571
            Case 85
                CriticalTValue = 1.98826790747722
            Case 86
                CriticalTValue = 1.98793420623902
            Case 87
                CriticalTValue = 1.98760828158907
            Case 88
                CriticalTValue = 1.98728986483117
            Case 89
                CriticalTValue = 1.98697869950628
            Case 90
                CriticalTValue = 1.98667454070377
            Case 91
                CriticalTValue = 1.98637715441862
            Case 92
                CriticalTValue = 1.98608631695113
            Case 93
                CriticalTValue = 1.98580181434582
            Case 94
                CriticalTValue = 1.9855234418666
            Case 95
                CriticalTValue = 1.9852510035055
            Case 96
                CriticalTValue = 1.98498431152246
            Case 97
                CriticalTValue = 1.98472318601398
            Case 98
                CriticalTValue = 1.98446745450848
            Case 99
                CriticalTValue = 1.98421695158642
            Case 100
                CriticalTValue = 1.98397151852355
            Case 101
                CriticalTValue = 1.98373100295561
            Case 102
                CriticalTValue = 1.98349525856288
            Case 103
                CriticalTValue = 1.98326414477346
            Case 104
                CriticalTValue = 1.98303752648373
            Case 105
                CriticalTValue = 1.98281527379505
            Case 106
                CriticalTValue = 1.9825972617655
            Case 107
                CriticalTValue = 1.98238337017569
            Case 108
                CriticalTValue = 1.98217348330773
            Case 109
                CriticalTValue = 1.98196748973648
            Case 110
                CriticalTValue = 1.98176528213237
            Case 111
                CriticalTValue = 1.9815667570749
            Case 112
                CriticalTValue = 1.98137181487631
            Case 113
                CriticalTValue = 1.98118035941466
            Case 114
                CriticalTValue = 1.98099229797586
            Case 115
                CriticalTValue = 1.98080754110391
            Case 116
                CriticalTValue = 1.98062600245909
            Case 117
                CriticalTValue = 1.9804475986834
            Case 118
                CriticalTValue = 1.98027224927295
            Case 119
                CriticalTValue = 1.98009987645692
            Case 120
                CriticalTValue = 1.97993040508242
            Case 121
                CriticalTValue = 1.97976376250537
            Case 122
                CriticalTValue = 1.97959987848662
            Case 123
                CriticalTValue = 1.97943868509329
            Case 124
                CriticalTValue = 1.97928011660484
            Case 125
                CriticalTValue = 1.97912410942378
            Case 126
                CriticalTValue = 1.97897060199061
            Case 127
                CriticalTValue = 1.97881953470284
            Case 128
                CriticalTValue = 1.97867084983782
            Case 129
                CriticalTValue = 1.97852449147925
            Case 130
                CriticalTValue = 1.97838040544701
            Case 131
                CriticalTValue = 1.97823853923037
            Case 132
                CriticalTValue = 1.97809884192412
            Case 133
                CriticalTValue = 1.97796126416772
            Case 134
                CriticalTValue = 1.97782575808712
            Case 135
                CriticalTValue = 1.97769227723924
            Case 136
                CriticalTValue = 1.97756077655893
            Case 137
                CriticalTValue = 1.97743121230817
            Case 138
                CriticalTValue = 1.97730354202764
            Case 139
                CriticalTValue = 1.97717772449033
            Case 140
                CriticalTValue = 1.97705371965709
            Case 141
                CriticalTValue = 1.97693148863425
            Case 142
                CriticalTValue = 1.97681099363285
            Case 143
                CriticalTValue = 1.97669219792979
            Case 144
                CriticalTValue = 1.97657506583044
            Case 145
                CriticalTValue = 1.97645956263291
            Case 146
                CriticalTValue = 1.97634565459381
            Case 147
                CriticalTValue = 1.97623330889532
            Case 148
                CriticalTValue = 1.97612249361374
            Case 149
                CriticalTValue = 1.97601317768919
            Case 150
                CriticalTValue = 1.97590533089661
            Case 151
                CriticalTValue = 1.97579892381793
            Case 152
                CriticalTValue = 1.97569392781527
            Case 153
                CriticalTValue = 1.97559031500524
            Case 154
                CriticalTValue = 1.97548805823434
            Case 155
                CriticalTValue = 1.97538713105511
            Case 156
                CriticalTValue = 1.97528750770344
            Case 157
                CriticalTValue = 1.97518916307659
            Case 158
                CriticalTValue = 1.97509207271208
            Case 159
                CriticalTValue = 1.97499621276747
            Case 160
                CriticalTValue = 1.9749015600008
            Case 161
                CriticalTValue = 1.97480809175178
            Case 162
                CriticalTValue = 1.97471578592379
            Case 163
                CriticalTValue = 1.97462462096636
            Case 164
                CriticalTValue = 1.97453457585847
            Case 165
                CriticalTValue = 1.97444563009238
            Case 166
                CriticalTValue = 1.97435776365803
            Case 167
                CriticalTValue = 1.97427095702805
            Case 168
                CriticalTValue = 1.97418519114332
            Case 169
                CriticalTValue = 1.97410044739897
            Case 170
                CriticalTValue = 1.97401670763097
            Case 171
                CriticalTValue = 1.9739339541031
            Case 172
                CriticalTValue = 1.9738521694945
            Case 173
                CriticalTValue = 1.97377133688752
            Case 174
                CriticalTValue = 1.97369143975607
            Case 175
                CriticalTValue = 1.97361246195438
            Case 176
                CriticalTValue = 1.9735343877061
            Case 177
                CriticalTValue = 1.9734572015938
            Case 178
                CriticalTValue = 1.97338088854882
            Case 179
                CriticalTValue = 1.97330543384147
            Case 180
                CriticalTValue = 1.97323082307154
            Case 181
                CriticalTValue = 1.97315704215916
            Case 182
                CriticalTValue = 1.9730840773359
            Case 183
                CriticalTValue = 1.97301191513626
            Case 184
                CriticalTValue = 1.97294054238936
            Case 185
                CriticalTValue = 1.97286994621089
            Case 186
                CriticalTValue = 1.97280011399544
            Case 187
                CriticalTValue = 1.97273103340891
            Case 188
                CriticalTValue = 1.9726626923813
            Case 189
                CriticalTValue = 1.97259507909967
            Case 190
                CriticalTValue = 1.97252818200132
            Case 191
                CriticalTValue = 1.97246198976721
            Case 192
                CriticalTValue = 1.97239649131558
            Case 193
                CriticalTValue = 1.97233167579575
            Case 194
                CriticalTValue = 1.97226753258213
            Case 195
                CriticalTValue = 1.97220405126844
            Case 196
                CriticalTValue = 1.97214122166204
            Case 197
                CriticalTValue = 1.9720790337785
            Case 198
                CriticalTValue = 1.97201747783631
            Case 199
                CriticalTValue = 1.97195654425175
            Case 200
                CriticalTValue = 1.97189622363391
            Case 201
                CriticalTValue = 1.97183650677986
            Case 202
                CriticalTValue = 1.97177738466999
            Case 203
                CriticalTValue = 1.97171884846345
            Case 204
                CriticalTValue = 1.97166088949376
            Case 205
                CriticalTValue = 1.97160349926451
            Case 206
                CriticalTValue = 1.97154666944523
            Case 207
                CriticalTValue = 1.97149039186733
            Case 208
                CriticalTValue = 1.97143465852024
            Case 209
                CriticalTValue = 1.97137946154754
            Case 210
                CriticalTValue = 1.97132479324333
            Case 211
                CriticalTValue = 1.97127064604859
            Case 212
                CriticalTValue = 1.97121701254775
            Case 213
                CriticalTValue = 1.97116388546525
            Case 214
                CriticalTValue = 1.9711112576623
            Case 215
                CriticalTValue = 1.97105912213364
            Case 216
                CriticalTValue = 1.97100747200447
            Case 217
                CriticalTValue = 1.97095630052739
            Case 218
                CriticalTValue = 1.97090560107948
            Case 219
                CriticalTValue = 1.97085536715947
            Case 220
                CriticalTValue = 1.9708055923849
            Case 221
                CriticalTValue = 1.97075627048947
            Case 222
                CriticalTValue = 1.97070739532039
            Case 223
                CriticalTValue = 1.97065896083581
            Case 224
                CriticalTValue = 1.97061096110236
            Case 225
                CriticalTValue = 1.9705633902927
            Case 226
                CriticalTValue = 1.97051624268316
            Case 227
                CriticalTValue = 1.97046951265148
            Case 228
                CriticalTValue = 1.97042319467452
            Case 229
                CriticalTValue = 1.97037728332615
            Case 230
                CriticalTValue = 1.97033177327508
            Case 231
                CriticalTValue = 1.97028665928279
            Case 232
                CriticalTValue = 1.97024193620157
            Case 233
                CriticalTValue = 1.97019759897253
            Case 234
                CriticalTValue = 1.97015364262368
            Case 235
                CriticalTValue = 1.9701100622681
            Case 236
                CriticalTValue = 1.97006685310212
            Case 237
                CriticalTValue = 1.97002401040355
            Case 238
                CriticalTValue = 1.96998152952994
            Case 239
                CriticalTValue = 1.96993940591696
            Case 240
                CriticalTValue = 1.96989763507669
            Case 241
                CriticalTValue = 1.9698562125961
            Case 242
                CriticalTValue = 1.96981513413544
            Case 243
                CriticalTValue = 1.96977439542675
            Case 244
                CriticalTValue = 1.96973399227238
            Case 245
                CriticalTValue = 1.96969392054355
            Case 246
                CriticalTValue = 1.96965417617892
            Case 247
                CriticalTValue = 1.96961475518327
            Case 248
                CriticalTValue = 1.96957565362609
            Case 249
                CriticalTValue = 1.96953686764035
            Case 250
                CriticalTValue = 1.96949839342115
            Case 251
                CriticalTValue = 1.96946022722454
            Case 252
                CriticalTValue = 1.96942236536627
            Case 253
                CriticalTValue = 1.9693848042206
            Case 254
                CriticalTValue = 1.96934754021918
            Case 255
                CriticalTValue = 1.96931056984987
            Case 256
                CriticalTValue = 1.96927388965569
            Case 257
                CriticalTValue = 1.96923749623368
            Case 258
                CriticalTValue = 1.9692013862339
            Case 259
                CriticalTValue = 1.9691655563584
            Case 260
                CriticalTValue = 1.96913000336017
            Case 261
                CriticalTValue = 1.96909472404221
            Case 262
                CriticalTValue = 1.96905971525654
            Case 263
                CriticalTValue = 1.96902497390328
            Case 264
                CriticalTValue = 1.96899049692972
            Case 265
                CriticalTValue = 1.96895628132945
            Case 266
                CriticalTValue = 1.96892232414145
            Case 267
                CriticalTValue = 1.96888862244929
            Case 268
                CriticalTValue = 1.96885517338024
            Case 269
                CriticalTValue = 1.96882197410447
            Case 270
                CriticalTValue = 1.96878902183426
            Case 271
                CriticalTValue = 1.96875631382325
            Case 272
                CriticalTValue = 1.96872384736559
            Case 273
                CriticalTValue = 1.96869161979529
            Case 274
                CriticalTValue = 1.96865962848544
            Case 275
                CriticalTValue = 1.9686278708475
            Case 276
                CriticalTValue = 1.9685963443306
            Case 277
                CriticalTValue = 1.96856504642087
            Case 278
                CriticalTValue = 1.96853397464078
            Case 279
                CriticalTValue = 1.96850312654844
            Case 280
                CriticalTValue = 1.96847249973702
            Case 281
                CriticalTValue = 1.96844209183408
            Case 282
                CriticalTValue = 1.96841190050099
            Case 283
                CriticalTValue = 1.96838192343229
            Case 284
                CriticalTValue = 1.96835215835516
            Case 285
                CriticalTValue = 1.96832260302877
            Case 286
                CriticalTValue = 1.96829325524379
            Case 287
                CriticalTValue = 1.96826411282181
            Case 288
                CriticalTValue = 1.96823517361478
            Case 289
                CriticalTValue = 1.96820643550454
            Case 290
                CriticalTValue = 1.96817789640221
            Case 291
                CriticalTValue = 1.9681495542478
            Case 292
                CriticalTValue = 1.96812140700962
            Case 293
                CriticalTValue = 1.96809345268382
            Case 294
                CriticalTValue = 1.96806568929396
            Case 295
                CriticalTValue = 1.96803811489047
            Case 296
                CriticalTValue = 1.96801072755026
            Case 297
                CriticalTValue = 1.96798352537622
            Case 298
                CriticalTValue = 1.96795650649682
            Case 299
                CriticalTValue = 1.96792966906567
            Case 300
                CriticalTValue = 1.96790301126109
            Case 301
                CriticalTValue = 1.96787653128569
            Case 302
                CriticalTValue = 1.96785022736602
            Case 303
                CriticalTValue = 1.96782409775209
            Case 304
                CriticalTValue = 1.96779814071704
            Case 305
                CriticalTValue = 1.96777235455677
            Case 306
                CriticalTValue = 1.9677467375895
            Case 307
                CriticalTValue = 1.96772128815549
            Case 308
                CriticalTValue = 1.96769600461661
            Case 309
                CriticalTValue = 1.96767088535601
            Case 310
                CriticalTValue = 1.96764592877782
            Case 311
                CriticalTValue = 1.96762113330675
            Case 312
                CriticalTValue = 1.96759649738777
            Case 313
                CriticalTValue = 1.96757201948582
            Case 314
                CriticalTValue = 1.96754769808547
            Case 315
                CriticalTValue = 1.96752353169058
            Case 316
                CriticalTValue = 1.96749951882406
            Case 317
                CriticalTValue = 1.96747565802749
            Case 318
                CriticalTValue = 1.96745194786088
            Case 319
                CriticalTValue = 1.96742838690237
            Case 320
                CriticalTValue = 1.96740497374793
            Case 321
                CriticalTValue = 1.9673817070111
            Case 322
                CriticalTValue = 1.96735858532268
            Case 323
                CriticalTValue = 1.96733560733053
            Case 324
                CriticalTValue = 1.96731277169922
            Case 325
                CriticalTValue = 1.96729007710986
            Case 326
                CriticalTValue = 1.96726752225977
            Case 327
                CriticalTValue = 1.96724510586227
            Case 328
                CriticalTValue = 1.96722282664644
            Case 329
                CriticalTValue = 1.96720068335686
            Case 330
                CriticalTValue = 1.96717867475337
            Case 331
                CriticalTValue = 1.96715679961087
            Case 332
                CriticalTValue = 1.96713505671906
            Case 333
                CriticalTValue = 1.96711344488223
            Case 334
                CriticalTValue = 1.96709196291906
            Case 335
                CriticalTValue = 1.96707060966236
            Case 336
                CriticalTValue = 1.9670493839589
            Case 337
                CriticalTValue = 1.96702828466919
            Case 338
                CriticalTValue = 1.96700731066724
            Case 339
                CriticalTValue = 1.96698646084044
            Case 340
                CriticalTValue = 1.96696573408929
            Case 341
                CriticalTValue = 1.96694512932722
            Case 342
                CriticalTValue = 1.96692464548044
            Case 343
                CriticalTValue = 1.9669042814877
            Case 344
                CriticalTValue = 1.96688403630016
            Case 345
                CriticalTValue = 1.96686390888117
            Case 346
                CriticalTValue = 1.96684389820609
            Case 347
                CriticalTValue = 1.96682400326215
            Case 348
                CriticalTValue = 1.96680422304826
            Case 349
                CriticalTValue = 1.96678455657484
            Case 350
                CriticalTValue = 1.96676500286365
            Case 351
                CriticalTValue = 1.96674556094765
            Case 352
                CriticalTValue = 1.96672622987082
            Case 353
                CriticalTValue = 1.966707008688
            Case 354
                CriticalTValue = 1.96668789646476
            Case 355
                CriticalTValue = 1.96666889227723
            Case 356
                CriticalTValue = 1.96664999521195
            Case 357
                CriticalTValue = 1.96663120436573
            Case 358
                CriticalTValue = 1.96661251884552
            Case 359
                CriticalTValue = 1.96659393776823
            Case 360
                CriticalTValue = 1.96657546026063
            Case 361
                CriticalTValue = 1.96655708545919
            Case 362
                CriticalTValue = 1.96653881250995
            Case 363
                CriticalTValue = 1.96652064056841
            Case 364
                CriticalTValue = 1.96650256879936
            Case 365
                CriticalTValue = 1.96648459637679
            Case 366
                CriticalTValue = 1.96646672248372
            Case 367
                CriticalTValue = 1.96644894631213
            Case 368
                CriticalTValue = 1.96643126706281
            Case 369
                CriticalTValue = 1.96641368394522
            Case 370
                CriticalTValue = 1.96639619617741
            Case 371
                CriticalTValue = 1.96637880298588
            Case 372
                CriticalTValue = 1.96636150360549
            Case 373
                CriticalTValue = 1.96634429727931
            Case 374
                CriticalTValue = 1.96632718325855
            Case 375
                CriticalTValue = 1.96631016080241
            Case 376
                CriticalTValue = 1.96629322917802
            Case 377
                CriticalTValue = 1.9662763876603
            Case 378
                CriticalTValue = 1.96625963553186
            Case 379
                CriticalTValue = 1.96624297208292
            Case 380
                CriticalTValue = 1.96622639661118
            Case 381
                CriticalTValue = 1.96620990842174
            Case 382
                CriticalTValue = 1.96619350682701
            Case 383
                CriticalTValue = 1.96617719114659
            Case 384
                CriticalTValue = 1.96616096070719
            Case 385
                CriticalTValue = 1.96614481484255
            Case 386
                CriticalTValue = 1.96612875289333
            Case 387
                CriticalTValue = 1.96611277420701
            Case 388
                CriticalTValue = 1.96609687813783
            Case 389
                CriticalTValue = 1.96608106404671
            Case 390
                CriticalTValue = 1.9660653313011
            Case 391
                CriticalTValue = 1.96604967927499
            Case 392
                CriticalTValue = 1.96603410734874
            Case 393
                CriticalTValue = 1.96601861490905
            Case 394
                CriticalTValue = 1.96600320134887
            Case 395
                CriticalTValue = 1.9659878660673
            Case 396
                CriticalTValue = 1.96597260846955
            Case 397
                CriticalTValue = 1.96595742796682
            Case 398
                CriticalTValue = 1.96594232397627
            Case 399
                CriticalTValue = 1.96592729592088
            Case 400
                CriticalTValue = 1.96591234322946
            Case 401
                CriticalTValue = 1.96589746533652
            Case 402
                CriticalTValue = 1.96588266168219
            Case 403
                CriticalTValue = 1.96586793171221
            Case 404
                CriticalTValue = 1.96585327487781
            Case 405
                CriticalTValue = 1.96583869063564
            Case 406
                CriticalTValue = 1.96582417844774
            Case 407
                CriticalTValue = 1.96580973778144
            Case 408
                CriticalTValue = 1.96579536810932
            Case 409
                CriticalTValue = 1.96578106890912
            Case 410
                CriticalTValue = 1.9657668396637
            Case 411
                CriticalTValue = 1.96575267986095
            Case 412
                CriticalTValue = 1.96573858899377
            Case 413
                CriticalTValue = 1.96572456655998
            Case 414
                CriticalTValue = 1.96571061206225
            Case 415
                CriticalTValue = 1.96569672500807
            Case 416
                CriticalTValue = 1.96568290490967
            Case 417
                CriticalTValue = 1.965669151284
            Case 418
                CriticalTValue = 1.9656554636526
            Case 419
                CriticalTValue = 1.96564184154164
            Case 420
                CriticalTValue = 1.96562828448177
            Case 421
                CriticalTValue = 1.96561479200814
            Case 422
                CriticalTValue = 1.96560136366032
            Case 423
                CriticalTValue = 1.96558799898222
            Case 424
                CriticalTValue = 1.9655746975221
            Case 425
                CriticalTValue = 1.96556145883246
            Case 426
                CriticalTValue = 1.96554828247002
            Case 427
                CriticalTValue = 1.96553516799566
            Case 428
                CriticalTValue = 1.96552211497438
            Case 429
                CriticalTValue = 1.96550912297525
            Case 430
                CriticalTValue = 1.96549619157135
            Case 431
                CriticalTValue = 1.96548332033974
            Case 432
                CriticalTValue = 1.96547050886142
            Case 433
                CriticalTValue = 1.96545775672123
            Case 434
                CriticalTValue = 1.9654450635079
            Case 435
                CriticalTValue = 1.96543242881391
            Case 436
                CriticalTValue = 1.96541985223552
            Case 437
                CriticalTValue = 1.96540733337266
            Case 438
                CriticalTValue = 1.96539487182897
            Case 439
                CriticalTValue = 1.96538246721166
            Case 440
                CriticalTValue = 1.96537011913156
            Case 441
                CriticalTValue = 1.96535782720302
            Case 442
                CriticalTValue = 1.9653455910439
            Case 443
                CriticalTValue = 1.96533341027551
            Case 444
                CriticalTValue = 1.96532128452259
            Case 445
                CriticalTValue = 1.96530921341326
            Case 446
                CriticalTValue = 1.96529719657897
            Case 447
                CriticalTValue = 1.96528523365449
            Case 448
                CriticalTValue = 1.96527332427787
            Case 449
                CriticalTValue = 1.96526146809037
            Case 450
                CriticalTValue = 1.96524966473647
            Case 451
                CriticalTValue = 1.96523791386378
            Case 452
                CriticalTValue = 1.96522621512307
            Case 453
                CriticalTValue = 1.96521456816819
            Case 454
                CriticalTValue = 1.96520297265605
            Case 455
                CriticalTValue = 1.96519142824656
            Case 456
                CriticalTValue = 1.96517993460266
            Case 457
                CriticalTValue = 1.96516849139022
            Case 458
                CriticalTValue = 1.96515709827804
            Case 459
                CriticalTValue = 1.96514575493784
            Case 460
                CriticalTValue = 1.96513446104415
            Case 461
                CriticalTValue = 1.96512321627439
            Case 462
                CriticalTValue = 1.96511202030873
            Case 463
                CriticalTValue = 1.96510087283015
            Case 464
                CriticalTValue = 1.96508977352434
            Case 465
                CriticalTValue = 1.96507872207972
            Case 466
                CriticalTValue = 1.96506771818738
            Case 467
                CriticalTValue = 1.96505676154106
            Case 468
                CriticalTValue = 1.96504585183714
            Case 469
                CriticalTValue = 1.96503498877459
            Case 470
                CriticalTValue = 1.96502417205493
            Case 471
                CriticalTValue = 1.96501340138226
            Case 472
                CriticalTValue = 1.96500267646315
            Case 473
                CriticalTValue = 1.96499199700668
            Case 474
                CriticalTValue = 1.9649813627244
            Case 475
                CriticalTValue = 1.96497077333028
            Case 476
                CriticalTValue = 1.96496022854071
            Case 477
                CriticalTValue = 1.96494972807444
            Case 478
                CriticalTValue = 1.96493927165262
            Case 479
                CriticalTValue = 1.9649288589987
            Case 480
                CriticalTValue = 1.96491848983845
            Case 481
                CriticalTValue = 1.96490816389994
            Case 482
                CriticalTValue = 1.96489788091348
            Case 483
                CriticalTValue = 1.96488764061164
            Case 484
                CriticalTValue = 1.96487744272919
            Case 485
                CriticalTValue = 1.96486728700309
            Case 486
                CriticalTValue = 1.96485717317249
            Case 487
                CriticalTValue = 1.96484710097867
            Case 488
                CriticalTValue = 1.96483707016505
            Case 489
                CriticalTValue = 1.96482708047714
            Case 490
                CriticalTValue = 1.96481713166254
            Case 491
                CriticalTValue = 1.9648072234709
            Case 492
                CriticalTValue = 1.96479735565394
            Case 493
                CriticalTValue = 1.96478752796536
            Case 494
                CriticalTValue = 1.9647777401609
            Case 495
                CriticalTValue = 1.96476799199823
            Case 496
                CriticalTValue = 1.96475828323702
            Case 497
                CriticalTValue = 1.96474861363887
            Case 498
                CriticalTValue = 1.96473898296729
            Case 499
                CriticalTValue = 1.96472939098769
            Case 500
                CriticalTValue = 1.96471983746737
            Case Else
                CriticalTValue = 1.9647
        End Select
        Return CriticalTValue
    End Function
End Module
