Public Class PrecipPercent

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private PrecipPercent_Input As String
    Private PrecipPercent_ModernInput As String
    Private PrecipPercent_Output As String
    'Initialize Global Form Variables
    Private PrecipPercent_RowStart As Integer
    Private PrecipPercent_HistPrecipColumn As Integer
    Private PrecipPercent_ModRowStart As Integer
    Private PrecipPercent_ModPrecipColumn As Integer
    Private PrecipPercent_Percentiles As Integer
    Private PrecipPercent_PercentNormal As Integer
    Private PrecipPercent_ComputeMonth As Integer
    Private PrecipPercent_ComputeRange As Integer
    Private PrecipPercent_CheckJan As Integer
    Private PrecipPercent_CheckFeb As Integer
    Private PrecipPercent_CheckMar As Integer
    Private PrecipPercent_CheckApr As Integer
    Private PrecipPercent_CheckMay As Integer
    Private PrecipPercent_CheckJun As Integer
    Private PrecipPercent_CheckJul As Integer
    Private PrecipPercent_CheckAug As Integer
    Private PrecipPercent_CheckSep As Integer
    Private PrecipPercent_CheckOct As Integer
    Private PrecipPercent_CheckNov As Integer
    Private PrecipPercent_CheckDec As Integer
    Private PrecipPercent_StartMonth As Integer
    Private PrecipPercent_EndMonth As Integer
    Private PrecipPercent_Header As String
    Private PrecipPercent_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub PrecipPercent_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Compute Precipitation Percentages Tool
        StartMonth.SelectedIndex = 0
        EndMonth.SelectedIndex = 0
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(HistoricalFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Historical Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If HistoricalFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = PrecipPercent_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            PrecipPercent_Input = FileSelect.FileName
            HistoricalFileLocation.Text = PrecipPercent_Input
            HistoricalFileLocation.SelectionStart = HistoricalFileLocation.Text.Length
        End If
    End Sub

    Private Sub HistDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HistDataRowStart.TextChanged
        EntryError.SetError(HistDataRowStart, "")
        If HistDataRowStart.Text <> "" Then
            Try
                PrecipPercent_RowStart = HistDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(HistDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub HistPrecipColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HistPrecipColumn.TextChanged
        EntryError.SetError(HistPrecipColumn, "")
        If HistPrecipColumn.Text <> "" Then
            Try
                PrecipPercent_HistPrecipColumn = HistPrecipColumn.Text
            Catch ex As Exception
                EntryError.SetError(HistPrecipColumn, "Numerical values are the only valid characters.")
                Exit Sub
            End Try
        End If
    End Sub

    Private Sub ModernSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ModernSelect_Button.Click
        EntryError.SetError(ModernFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Modern Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If ModernFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = PrecipPercent_ModernInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            PrecipPercent_ModernInput = FileSelect.FileName
            ModernFileLocation.Text = PrecipPercent_ModernInput
            ModernFileLocation.SelectionStart = ModernFileLocation.Text.Length
        End If
    End Sub

    Private Sub ModDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ModDataRowStart.TextChanged
        EntryError.SetError(ModDataRowStart, "")
        If ModDataRowStart.Text <> "" Then
            Try
                PrecipPercent_ModRowStart = ModDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(ModDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ModPrecipColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ModPrecipColumn.TextChanged
        EntryError.SetError(ModPrecipColumn, "")
        If ModPrecipColumn.Text <> "" Then
            Try
                PrecipPercent_ModPrecipColumn = ModPrecipColumn.Text
            Catch ex As Exception
                EntryError.SetError(ModPrecipColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ComputeMonth_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComputeMonth.CheckedChanged
        Select Case ComputeMonth.Checked
            Case True
                CheckJan.Enabled = True
                CheckFeb.Enabled = True
                CheckMar.Enabled = True
                CheckApr.Enabled = True
                CheckMay.Enabled = True
                CheckJun.Enabled = True
                CheckJul.Enabled = True
                CheckAug.Enabled = True
                CheckSep.Enabled = True
                CheckOct.Enabled = True
                CheckNov.Enabled = True
                CheckDec.Enabled = True
            Case False
                CheckJan.Enabled = False
                CheckFeb.Enabled = False
                CheckMar.Enabled = False
                CheckApr.Enabled = False
                CheckMay.Enabled = False
                CheckJun.Enabled = False
                CheckJul.Enabled = False
                CheckAug.Enabled = False
                CheckSep.Enabled = False
                CheckOct.Enabled = False
                CheckNov.Enabled = False
                CheckDec.Enabled = False
        End Select
    End Sub

    Private Sub CheckJan_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckJan.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckJan.Checked = True Then
            PrecipPercent_CheckJan = 1
        Else
            PrecipPercent_CheckJan = 0
        End If
    End Sub

    Private Sub CheckFeb_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckFeb.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckFeb.Checked = True Then
            PrecipPercent_CheckFeb = 1
        Else
            PrecipPercent_CheckFeb = 0
        End If
    End Sub

    Private Sub CheckMar_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckMar.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckMar.Checked = True Then
            PrecipPercent_CheckMar = 1
        Else
            PrecipPercent_CheckMar = 0
        End If
    End Sub

    Private Sub CheckApr_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckApr.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckApr.Checked = True Then
            PrecipPercent_CheckApr = 1
        Else
            PrecipPercent_CheckApr = 0
        End If
    End Sub

    Private Sub CheckMay_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckMay.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckMay.Checked = True Then
            PrecipPercent_CheckMay = 1
        Else
            PrecipPercent_CheckMay = 0
        End If
    End Sub

    Private Sub CheckJun_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckJun.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckJun.Checked = True Then
            PrecipPercent_CheckJun = 1
        Else
            PrecipPercent_CheckJun = 0
        End If
    End Sub

    Private Sub CheckJul_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckJul.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckJul.Checked = True Then
            PrecipPercent_CheckJul = 1
        Else
            PrecipPercent_CheckJul = 0
        End If
    End Sub

    Private Sub CheckAug_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckAug.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckAug.Checked = True Then
            PrecipPercent_CheckAug = 1
        Else
            PrecipPercent_CheckAug = 0
        End If
    End Sub

    Private Sub CheckSep_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckSep.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckSep.Checked = True Then
            PrecipPercent_CheckSep = 1
        Else
            PrecipPercent_CheckSep = 0
        End If
    End Sub

    Private Sub CheckOct_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckOct.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckOct.Checked = True Then
            PrecipPercent_CheckOct = 1
        Else
            PrecipPercent_CheckOct = 0
        End If
    End Sub

    Private Sub CheckNov_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckNov.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckNov.Checked = True Then
            PrecipPercent_CheckNov = 1
        Else
            PrecipPercent_CheckNov = 0
        End If
    End Sub

    Private Sub CheckDec_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckDec.CheckedChanged
        EntryError.SetError(CheckJun, "")
        If CheckDec.Checked = True Then
            PrecipPercent_CheckDec = 1
        Else
            PrecipPercent_CheckDec = 0
        End If
    End Sub

    Private Sub ComputeRange_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComputeRange.CheckedChanged
        Select Case ComputeRange.Checked
            Case True
                StartMonth.Enabled = True
                EndMonth.Enabled = True
            Case False
                StartMonth.Enabled = False
                EndMonth.Enabled = False
        End Select
    End Sub

    Private Sub StartMonth_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartMonth.SelectedIndexChanged
        EntryError.SetError(StartMonth, "")
        EntryError.SetError(EndMonth, "")
        PrecipPercent_StartMonth = StartMonth.SelectedIndex
    End Sub

    Private Sub EndMonth_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndMonth.SelectedIndexChanged
        EntryError.SetError(StartMonth, "")
        EntryError.SetError(EndMonth, "")
        PrecipPercent_EndMonth = EndMonth.SelectedIndex
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = PrecipPercent_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            PrecipPercent_Output = FileOutput.FileName
            OutputFileLocation.Text = PrecipPercent_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        PrecipPercent_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If PrecipPercent_Input = "" Then
            EntryError.SetError(HistoricalFileLocation, "Input historical dataset missing.")
            Exit Sub
        End If
        If HistDataRowStart.Text = "" Then
            EntryError.SetError(HistDataRowStart, "Required historical data information missing.")
            Exit Sub
        End If
        If HistPrecipColumn.Text = "" Then
            EntryError.SetError(HistPrecipColumn, "Required historical data information missing.")
            Exit Sub
        End If
        If PrecipPercent_ModernInput = "" Then
            EntryError.SetError(ModernFileLocation, "Input modern dataset missing.")
            Exit Sub
        End If
        If ModDataRowStart.Text = "" Then
            EntryError.SetError(ModDataRowStart, "Required modern data information missing.")
            Exit Sub
        End If
        If ModPrecipColumn.Text = "" Then
            EntryError.SetError(ModPrecipColumn, "Required modern data information missing.")
            Exit Sub
        End If
        If ComputeMonth.Checked = True Then
            If CheckJan.Checked = False And CheckFeb.Checked = False And CheckMar.Checked = False And CheckApr.Checked = False And CheckMay.Checked = False And CheckJun.Checked = False And CheckJul.Checked = False And CheckAug.Checked = False And CheckSep.Checked = False And CheckOct.Checked = False And CheckNov.Checked = False And CheckDec.Checked = False Then
                EntryError.SetError(CheckJun, "Required month(s) not selected.")
                Exit Sub
            End If
        End If
        If ComputeRange.Checked = True Then
            If StartMonth.SelectedIndex = 0 Then
                EntryError.SetError(StartMonth, "Required start month not selected.")
                Exit Sub
            End If
            If EndMonth.SelectedIndex = 0 Then
                EntryError.SetError(EndMonth, "Required end month not selected.")
                Exit Sub
            End If
            If StartMonth.SelectedIndex = EndMonth.SelectedIndex Then
                EntryError.SetError(EndMonth, "Start and end months cannot be the same.")
                Exit Sub
            End If
        End If
        If PrecipPercent_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If PrecipPercent_Output = PrecipPercent_Input Or PrecipPercent_Output = PrecipPercent_ModernInput Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        PrecipPercent_Exception = 0
        If PercentilesButton.Checked = True Then
            PrecipPercent_Percentiles = 1
        Else
            PrecipPercent_Percentiles = 0
        End If
        If PercentofNormalButton.Checked = True Then
            PrecipPercent_PercentNormal = 1
        Else
            PrecipPercent_PercentNormal = 0
        End If
        If ComputeMonth.Checked = True Then
            PrecipPercent_ComputeMonth = 1
        Else
            PrecipPercent_ComputeMonth = 0
        End If
        If ComputeRange.Checked = True Then
            PrecipPercent_ComputeRange = 1
        Else
            PrecipPercent_ComputeRange = 0
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Frequent Variables
        Dim Precip As Double
        Dim Year As Integer
        Dim Month As Integer
        Dim i As Integer
        Dim Total As Integer
        Dim Percentage As Double
        'Initialize Output File and Set Header
        FileOpen(1, PrecipPercent_Output, OpenMode.Output)
        PrintLine(1, PrecipPercent_Header)
        PrintLine(1, "")
        If PrecipPercent_Percentiles = 1 Then
            If PrecipPercent_ComputeMonth = 1 Then
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Percentile")
            Else
                PrintLine(1, "Year" & vbTab & "Percentile")
            End If
        End If
        If PrecipPercent_PercentNormal = 1 Then
            If PrecipPercent_ComputeMonth = 1 Then
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Percent of Normal")
            Else
                PrintLine(1, "Year" & vbTab & "Percent of Normal")
            End If
        End If
        'Initialize StreamReaders
        Dim HistoricalInFile As New System.IO.StreamReader(PrecipPercent_Input)
        Dim HistoricalLine As String
        Dim HistoricalFields As String()
        Dim ModernInFile As New System.IO.StreamReader(PrecipPercent_ModernInput)
        Dim ModernLine As String
        Dim ModernFields As String()
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim HistoricalComplete As Integer
        Dim ModernComplete As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        HistoricalComplete = 0
        ModernComplete = 0
        FileComplete = 0
        While HistoricalInFile.Peek <> -1
            HistoricalLine = HistoricalInFile.ReadLine()
            HistoricalComplete = HistoricalComplete + 1
        End While
        While ModernInFile.Peek <> -1
            ModernLine = ModernInFile.ReadLine()
            ModernComplete = ModernComplete + 1
        End While
        'Reset StreamReaders
        HistoricalInFile.DiscardBufferedData()
        HistoricalInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        ModernInFile.DiscardBufferedData()
        ModernInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set Historical StreamReader to Data Start
        If PrecipPercent_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To PrecipPercent_RowStart - 2
                HistoricalLine = HistoricalInFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            HistoricalComplete = HistoricalComplete - (PrecipPercent_RowStart - 1)
        End If
        'Set Modern StreamReader to Data Start
        If PrecipPercent_ModRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To PrecipPercent_ModRowStart - 2
                ModernLine = ModernInFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            ModernComplete = ModernComplete - (PrecipPercent_ModRowStart - 1)
        End If
        FileComplete = HistoricalComplete + ModernComplete
        'Skip Column Headers
        HistoricalLine = HistoricalInFile.ReadLine()
        FileProgress = FileProgress + 1
        ModernLine = ModernInFile.ReadLine()
        FileProgress = FileProgress + 1
        'Run Computation By Month
        If PrecipPercent_ComputeMonth = 1 Then
            'Initialize Variables
            Dim JanNormal(0) As Double
            Dim FebNormal(0) As Double
            Dim MarNormal(0) As Double
            Dim AprNormal(0) As Double
            Dim MayNormal(0) As Double
            Dim JunNormal(0) As Double
            Dim JulNormal(0) As Double
            Dim AugNormal(0) As Double
            Dim SepNormal(0) As Double
            Dim OctNormal(0) As Double
            Dim NovNormal(0) As Double
            Dim DecNormal(0) As Double
            Dim JanCount As Integer
            Dim FebCount As Integer
            Dim MarCount As Integer
            Dim AprCount As Integer
            Dim MayCount As Integer
            Dim JunCount As Integer
            Dim JulCount As Integer
            Dim AugCount As Integer
            Dim SepCount As Integer
            Dim OctCount As Integer
            Dim NovCount As Integer
            Dim DecCount As Integer
            JanCount = 0
            FebCount = 0
            MarCount = 0
            AprCount = 0
            MayCount = 0
            JunCount = 0
            JulCount = 0
            AugCount = 0
            SepCount = 0
            OctCount = 0
            NovCount = 0
            DecCount = 0
            'Read Modern Normals
            Try
                While ModernInFile.Peek <> -1
                    ModernLine = ModernInFile.ReadLine()
                    ModernFields = ModernLine.Split(ControlChars.Tab)
                    Year = ModernFields(0)
                    Month = ModernFields(1)
                    If ModernFields(PrecipPercent_ModPrecipColumn - 1) <> -99 Then
                        Select Case Month
                            Case 1
                                JanNormal(JanCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                JanCount = JanCount + 1
                                ReDim Preserve JanNormal(JanCount)
                            Case 2
                                FebNormal(FebCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                FebCount = FebCount + 1
                                ReDim Preserve FebNormal(FebCount)
                            Case 3
                                MarNormal(MarCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                MarCount = MarCount + 1
                                ReDim Preserve MarNormal(MarCount)
                            Case 4
                                AprNormal(AprCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                AprCount = AprCount + 1
                                ReDim Preserve AprNormal(AprCount)
                            Case 5
                                MayNormal(MayCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                MayCount = MayCount + 1
                                ReDim Preserve MayNormal(MayCount)
                            Case 6
                                JunNormal(JunCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                JunCount = JunCount + 1
                                ReDim Preserve JunNormal(JunCount)
                            Case 7
                                JulNormal(JulCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                JulCount = JulCount + 1
                                ReDim Preserve JulNormal(JulCount)
                            Case 8
                                AugNormal(AugCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                AugCount = AugCount + 1
                                ReDim Preserve AugNormal(AugCount)
                            Case 9
                                SepNormal(SepCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                SepCount = SepCount + 1
                                ReDim Preserve SepNormal(SepCount)
                            Case 10
                                OctNormal(OctCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                OctCount = OctCount + 1
                                ReDim Preserve OctNormal(OctCount)
                            Case 11
                                NovNormal(NovCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                NovCount = NovCount + 1
                                ReDim Preserve NovNormal(NovCount)
                            Case 12
                                DecNormal(DecCount) = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                                DecCount = DecCount + 1
                                ReDim Preserve DecNormal(DecCount)
                        End Select
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the modern data file at or near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                Else
                    MsgBox("Error reading the modern data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                End If
                HistoricalInFile.Close()
                ModernInFile.Close()
                FileClose(1)
                PrecipPercent_Exception = 1
                Exit Sub
            End Try
            'Compute Percentages
            Try
                While HistoricalInFile.Peek <> -1
                    HistoricalLine = HistoricalInFile.ReadLine()
                    HistoricalFields = HistoricalLine.Split(ControlChars.Tab)
                    Year = HistoricalFields(0)
                    Month = HistoricalFields(1)
                    Precip = HistoricalFields(PrecipPercent_HistPrecipColumn - 1)
                    If Month = 1 And PrecipPercent_CheckJan = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate January Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To JanCount - 1
                                    If JanNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((JanCount - Total) / JanCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate January Percent of Normal
                                Total = 0
                                For i = 0 To JanCount - 1
                                    Total = Total + JanNormal(i)
                                Next i
                                Percentage = (Precip / (Total / JanCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 2 And PrecipPercent_CheckFeb = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate February Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To FebCount - 1
                                    If FebNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((FebCount - Total) / FebCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate February Percent of Normal
                                Total = 0
                                For i = 0 To FebCount - 1
                                    Total = Total + FebNormal(i)
                                Next i
                                Percentage = (Precip / (Total / FebCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 3 And PrecipPercent_CheckMar = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate March Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To MarCount - 1
                                    If MarNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((MarCount - Total) / MarCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate March Percent of Normal
                                Total = 0
                                For i = 0 To MarCount - 1
                                    Total = Total + MarNormal(i)
                                Next i
                                Percentage = (Precip / (Total / MarCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 4 And PrecipPercent_CheckApr = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate April Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To AprCount - 1
                                    If AprNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((AprCount - Total) / AprCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate April Percent of Normal
                                Total = 0
                                For i = 0 To AprCount - 1
                                    Total = Total + AprNormal(i)
                                Next i
                                Percentage = (Precip / (Total / AprCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 5 And PrecipPercent_CheckMay = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate May Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To MayCount - 1
                                    If MayNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((MayCount - Total) / MayCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate May Percent of Normal
                                Total = 0
                                For i = 0 To MayCount - 1
                                    Total = Total + MayNormal(i)
                                Next i
                                Percentage = (Precip / (Total / MayCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 6 And PrecipPercent_CheckJun = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate June Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To JunCount - 1
                                    If JunNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((JunCount - Total) / JunCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate June Percent of Normal
                                Total = 0
                                For i = 0 To JunCount - 1
                                    Total = Total + JunNormal(i)
                                Next i
                                Percentage = (Precip / (Total / JunCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 7 And PrecipPercent_CheckJul = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate July Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To JulCount - 1
                                    If JulNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((JulCount - Total) / JulCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate July Percent of Normal
                                Total = 0
                                For i = 0 To JulCount - 1
                                    Total = Total + JulNormal(i)
                                Next i
                                Percentage = (Precip / (Total / JulCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 8 And PrecipPercent_CheckAug = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate August Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To AugCount - 1
                                    If AugNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((AugCount - Total) / AugCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate August Percent of Normal
                                Total = 0
                                For i = 0 To AugCount - 1
                                    Total = Total + AugNormal(i)
                                Next i
                                Percentage = (Precip / (Total / AugCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 9 And PrecipPercent_CheckSep = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate September Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To SepCount - 1
                                    If SepNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((SepCount - Total) / SepCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate September Percent of Normal
                                Total = 0
                                For i = 0 To SepCount - 1
                                    Total = Total + SepNormal(i)
                                Next i
                                Percentage = (Precip / (Total / SepCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 10 And PrecipPercent_CheckOct = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate October Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To OctCount - 1
                                    If OctNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((OctCount - Total) / OctCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate October Percent of Normal
                                Total = 0
                                For i = 0 To OctCount - 1
                                    Total = Total + OctNormal(i)
                                Next i
                                Percentage = (Precip / (Total / OctCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 11 And PrecipPercent_CheckNov = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate November Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To NovCount - 1
                                    If NovNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((NovCount - Total) / NovCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate November Percent of Normal
                                Total = 0
                                For i = 0 To NovCount - 1
                                    Total = Total + NovNormal(i)
                                Next i
                                Percentage = (Precip / (Total / NovCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    If Month = 12 And PrecipPercent_CheckDec = 1 Then
                        If Precip <> -99 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate December Percentile
                                'Find Number of Records Greater than Precip
                                Total = 0
                                For i = 0 To DecCount - 1
                                    If DecNormal(i) > Precip Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((DecCount - Total) / DecCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate December Percent of Normal
                                Total = 0
                                For i = 0 To DecCount - 1
                                    Total = Total + DecNormal(i)
                                Next i
                                Percentage = (Precip / (Total / DecCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        End If
                    End If
                    'Save to Output File
                    If Precip <> -99 Then
                        PrintLine(1, Year & vbTab & Month & vbTab & Percentage)
                    Else
                        PrintLine(1, Year & vbTab & Month & vbTab & "-99")
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the historical data file at or near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                Else
                    MsgBox("Error reading the historical data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                End If
                HistoricalInFile.Close()
                ModernInFile.Close()
                FileClose(1)
                PrecipPercent_Exception = 1
                Exit Sub
            End Try
        End If
        'Run Computation By Range
        If PrecipPercent_ComputeRange = 1 Then
            'Initialize Variables
            Dim PrecipTotal As Double
            Dim Normal(0) As Double
            Dim NormalCount As Integer
            Dim StoreOnOff As Integer
            Dim MissingData As Integer
            Dim TimeThrough As Integer
            Dim MonthCheck As Integer
            StoreOnOff = 0
            MissingData = 0
            NormalCount = 0
            TimeThrough = 0
            'Read Modern Normals
            Try
                While ModernInFile.Peek <> -1
                    ModernLine = ModernInFile.ReadLine()
                    ModernFields = ModernLine.Split(ControlChars.Tab)
                    Year = ModernFields(0)
                    Month = ModernFields(1)
                    Precip = ModernFields(PrecipPercent_ModPrecipColumn - 1)
                    If TimeThrough = 0 Then
                        'First Time Through While Loop...Check If Initial Month Is After Start Month but Before or At End Month
                        TimeThrough = 1
                        MonthCheck = PrecipPercent_EndMonth - PrecipPercent_StartMonth
                        Select Case MonthCheck
                            Case Is > 0
                                If Month > PrecipPercent_StartMonth And Month <= PrecipPercent_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    MissingData = 1
                                End If
                            Case Is < 0
                                If Month > PrecipPercent_StartMonth And Month <= 12 Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    MissingData = 1
                                End If
                                If Month >= 1 And Month <= PrecipPercent_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    MissingData = 1
                                End If
                        End Select
                    End If
                    'Check for Start Month
                    If Month = PrecipPercent_StartMonth Then
                        StoreOnOff = 1
                        If Precip <> -99 Then
                            PrecipTotal = Precip
                        Else
                            MissingData = 1
                        End If
                    End If
                    'Store Months In Between Start and End Months
                    If StoreOnOff = 1 Then
                        If Precip <> -99 Then
                            PrecipTotal = PrecipTotal + Precip
                        Else
                            MissingData = 1
                        End If
                    End If
                    'Check for End Month
                    If Month = PrecipPercent_EndMonth Then
                        StoreOnOff = 0
                        If Precip <> -99 Then
                            PrecipTotal = PrecipTotal + Precip
                        Else
                            MissingData = 1
                        End If
                        'Save Data to Array If None Missing
                        If MissingData <> 1 Then
                            Normal(NormalCount) = PrecipTotal
                            NormalCount = NormalCount + 1
                            ReDim Preserve Normal(NormalCount)
                        Else
                            MissingData = 0
                        End If
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
                'Force Reset of Switches
                StoreOnOff = 0
                MissingData = 0
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the modern data file at or near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                Else
                    MsgBox("Error reading the modern data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                End If
                HistoricalInFile.Close()
                ModernInFile.Close()
                FileClose(1)
                PrecipPercent_Exception = 1
                Exit Sub
            End Try
            TimeThrough = 0
            'Compute Percentages
            Try
                While HistoricalInFile.Peek <> -1
                    HistoricalLine = HistoricalInFile.ReadLine()
                    HistoricalFields = HistoricalLine.Split(ControlChars.Tab)
                    Year = HistoricalFields(0)
                    Month = HistoricalFields(1)
                    Precip = HistoricalFields(PrecipPercent_HistPrecipColumn - 1)
                    If TimeThrough = 0 Then
                        'First Time Through While Loop...Check If Initial Month Is After Start Month but Before or At End Month
                        TimeThrough = 1
                        MonthCheck = PrecipPercent_EndMonth - PrecipPercent_StartMonth
                        Select Case MonthCheck
                            Case Is > 0
                                If Month > PrecipPercent_StartMonth And Month <= PrecipPercent_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    MissingData = 1
                                End If
                            Case Is < 0
                                If Month > PrecipPercent_StartMonth And Month <= 12 Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    MissingData = 1
                                End If
                                If Month >= 1 And Month <= PrecipPercent_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    MissingData = 1
                                End If
                        End Select
                    End If
                    'Check for Start Month
                    If Month = PrecipPercent_StartMonth Then
                        StoreOnOff = 1
                        If Precip <> -99 Then
                            PrecipTotal = Precip
                        Else
                            MissingData = 1
                        End If
                    End If
                    'Store Months In Between Start and End Months
                    If StoreOnOff = 1 Then
                        If Precip <> -99 Then
                            PrecipTotal = PrecipTotal + Precip
                        Else
                            MissingData = 1
                        End If
                    End If
                    'Check for End Month
                    If Month = PrecipPercent_EndMonth Then
                        StoreOnOff = 0
                        If Precip <> -99 Then
                            PrecipTotal = PrecipTotal + Precip
                        Else
                            MissingData = 1
                        End If
                        'Calculate Percentage for Period If No Missing Months
                        If MissingData <> 1 Then
                            If PrecipPercent_Percentiles = 1 Then
                                'Calculate Range Percentile
                                'Find Number of Records Greater than PrecipTotal
                                Total = 0
                                For i = 0 To NormalCount - 1
                                    If Normal(i) > PrecipTotal Then
                                        Total = Total + 1
                                    End If
                                Next i
                                'Calculate Percentile
                                Percentage = ((NormalCount - Total) / NormalCount) * 100
                                Percentage = Math.Round(Percentage, 2)
                                'Save Percentile to Output File
                                PrintLine(1, Year & vbTab & Percentage)
                            End If
                            If PrecipPercent_PercentNormal = 1 Then
                                'Calculate Range Percent of Normal
                                Total = 0
                                For i = 0 To NormalCount - 1
                                    Total = Total + Normal(i)
                                Next i
                                Percentage = (PrecipTotal / (Total / NormalCount)) * 100
                                Percentage = Math.Round(Percentage, 2)
                            End If
                        Else
                            'Save Missing Data to Output File
                            PrintLine(1, Year & vbTab & "-99")
                            MissingData = 0
                        End If
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the historical data file at or near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                Else
                    MsgBox("Error reading the historical data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Compute Precipitation Percentages")
                End If
                HistoricalInFile.Close()
                ModernInFile.Close()
                FileClose(1)
                PrecipPercent_Exception = 1
                Exit Sub
            End Try
            If StoreOnOff = 1 Then
                'End Month Not Reached so Last Percentile is Missing
                'Reset Year if Necessary
                Do
                    If Month = 12 Then
                        Month = 1
                        Year = Year + 1
                    Else
                        Month = Month + 1
                    End If
                    If Month = PrecipPercent_EndMonth Then
                        Exit Do
                    End If
                Loop
                'Output Last Line
                Print(1, Year & vbTab & "-99")
            End If
        End If
        'Close Files
        HistoricalInFile.Close()
        ModernInFile.Close()
        FileClose(1)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case PrecipPercent_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub PrecipPercent_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        PrecipPercentDisp = 0
    End Sub
End Class