Public Class PrecipUnits
    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private PrecipUnits_Input As String
    Private PrecipUnits_Output As String
    'Initialize Global Form Variables
    Private PrecipUnits_RowStart As Integer
    Private PrecipUnits_DataColumn As Integer
    Private PrecipUnits_InputFormat As Integer
    Private PrecipUnits_Inputinches As Integer
    Private PrecipUnits_Inputcm As Integer
    Private PrecipUnits_Inputmm As Integer
    Private PrecipUnits_ErrorBars As Integer
    Private PrecipUnits_ErrorColumn As Integer
    Private PrecipUnits_Outputinches As Integer
    Private PrecipUnits_Outputcm As Integer
    Private PrecipUnits_Outputmm As Integer
    Private PrecipUnits_Header As String
    Private PrecipUnits_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub PrecipUnits_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Convert Precipitation Units Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Precipitation Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = PrecipUnits_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            PrecipUnits_Input = FileSelect.FileName
            InputFileLocation.Text = PrecipUnits_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                PrecipUnits_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DataColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataColumn.TextChanged
        EntryError.SetError(DataColumn, "")
        If DataColumn.Text <> "" Then
            Try
                PrecipUnits_DataColumn = DataColumn.Text
            Catch ex As Exception
                EntryError.SetError(DataColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Inputinches_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Inputinches.CheckedChanged
        EntryError.SetError(Outputmm, "")
    End Sub

    Private Sub Inputcm_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Inputcm.CheckedChanged
        EntryError.SetError(Outputmm, "")
    End Sub

    Private Sub Inputmm_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Inputmm.CheckedChanged
        EntryError.SetError(Outputmm, "")
    End Sub

    Private Sub ErrorBars_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorBars.CheckedChanged
        Select Case ErrorBars.Checked
            Case True
                ErrorColumn_Label.Enabled = True
                ErrorColumn.Enabled = True
            Case False
                ErrorColumn_Label.Enabled = False
                ErrorColumn.Enabled = False
        End Select
    End Sub

    Private Sub ErrorColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorColumn.TextChanged
        EntryError.SetError(ErrorColumn, "")
        If ErrorColumn.Text <> "" Then
            Try
                PrecipUnits_ErrorColumn = ErrorColumn.Text
            Catch ex As Exception
                EntryError.SetError(ErrorColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = PrecipUnits_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            PrecipUnits_Output = FileOutput.FileName
            OutputFileLocation.Text = PrecipUnits_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub Outputinches_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Outputinches.CheckedChanged
        EntryError.SetError(Outputmm, "")
    End Sub

    Private Sub Outputcm_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Outputcm.CheckedChanged
        EntryError.SetError(Outputmm, "")
    End Sub

    Private Sub Outputmm_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Outputmm.CheckedChanged
        EntryError.SetError(Outputmm, "")
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        PrecipUnits_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If PrecipUnits_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If DataColumn.Text = "" Then
            EntryError.SetError(DataColumn, "Required input data information missing.")
            Exit Sub
        End If
        If ErrorBars.Checked = True Then
            If ErrorColumn.Text = "" Then
                EntryError.SetError(ErrorColumn, "Required input data information missing.")
                Exit Sub
            End If
        End If
        If PrecipUnits_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If PrecipUnits_Output = PrecipUnits_Input Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If (Inputinches.Checked = True And Outputinches.Checked = True) Or (Inputcm.Checked = True And Outputcm.Checked = True) Or (Inputmm.Checked = True And Outputmm.Checked = True) Then
            EntryError.SetError(Outputmm, "Input and output units cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        PrecipUnits_Exception = 0
        If InputFormat1.Checked = True Then
            PrecipUnits_InputFormat = 1
        End If
        If InputFormat2.Checked = True Then
            PrecipUnits_InputFormat = 2
        End If
        If Inputinches.Checked = True Then
            PrecipUnits_Inputinches = 1
        Else
            PrecipUnits_Inputinches = 0
        End If
        If Inputcm.Checked = True Then
            PrecipUnits_Inputcm = 1
        Else
            PrecipUnits_Inputcm = 0
        End If
        If Inputmm.Checked = True Then
            PrecipUnits_Inputmm = 1
        Else
            PrecipUnits_Inputmm = 0
        End If
        If ErrorBars.Checked = True Then
            PrecipUnits_ErrorBars = 1
        Else
            PrecipUnits_ErrorBars = 0
        End If
        If Outputinches.Checked = True Then
            PrecipUnits_Outputinches = 1
        Else
            PrecipUnits_Outputinches = 0
        End If
        If Outputcm.Checked = True Then
            PrecipUnits_Outputcm = 1
        Else
            PrecipUnits_Outputcm = 0
        End If
        If Outputmm.Checked = True Then
            PrecipUnits_Outputmm = 1
        Else
            PrecipUnits_Outputmm = 0
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim PrecipValue As Double
        Dim ErrorValue As Double
        'Initialize Output File and Set Header
        FileOpen(1, PrecipUnits_Output, OpenMode.Output)
        PrintLine(1, PrecipUnits_Header)
        PrintLine(1, "")
        Select Case PrecipUnits_InputFormat
            Case 1
                'Data in Monthly Format
                If PrecipUnits_ErrorBars = 1 Then
                    PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Precip" & vbTab & "Error")
                Else
                    PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Precip")
                End If
            Case 2
                'Data in Seasonal/Annual Format
                If PrecipUnits_ErrorBars = 1 Then
                    PrintLine(1, "Year" & vbTab & "Precip" & vbTab & "Error")
                Else
                    PrintLine(1, "Year" & vbTab & "Precip")
                End If
        End Select
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(PrecipUnits_Input)
        Dim Line As String
        Dim Fields As String()
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set Input Precipitation StreamReader to Data Start
        If PrecipUnits_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To PrecipUnits_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = FileComplete - (PrecipUnits_RowStart - 1)
        End If
        'Skip Column Headers
        Line = InFile.ReadLine()
        FileProgress = FileProgress + 1
        'Convert Precipitation Units
        Try
            While InFile.Peek <> -1
                Line = InFile.ReadLine()
                Fields = Line.Split(ControlChars.Tab)
                Year = Fields(0)
                If PrecipUnits_InputFormat = 1 Then
                    Month = Fields(1)
                End If
                PrecipValue = Fields(PrecipUnits_DataColumn - 1)
                If PrecipUnits_ErrorBars = 1 Then
                    ErrorValue = Fields(PrecipUnits_ErrorColumn - 1)
                End If
                'Convert Precipitation Value
                If PrecipValue <> -99 Then
                    If PrecipUnits_Inputinches = 1 And PrecipUnits_Outputcm = 1 Then
                        'Convert from inches to centimeters
                        PrecipValue = PrecipValue * 2.54
                        If PrecipUnits_ErrorBars = 1 Then
                            If ErrorValue <> -99 Then
                                ErrorValue = ErrorValue * 2.54
                            End If
                        End If
                    End If
                    If PrecipUnits_Inputinches = 1 And PrecipUnits_Outputmm = 1 Then
                        'Convert from inches to millimeters
                        PrecipValue = PrecipValue * 25.4
                        If PrecipUnits_ErrorBars = 1 Then
                            If ErrorValue <> -99 Then
                                ErrorValue = ErrorValue * 25.4
                            End If
                        End If
                    End If
                    If PrecipUnits_Inputcm = 1 And PrecipUnits_Outputinches = 1 Then
                        'Convert from centimeters to inches
                        PrecipValue = PrecipValue / 2.54
                        If PrecipUnits_ErrorBars = 1 Then
                            If ErrorValue <> -99 Then
                                ErrorValue = ErrorValue / 2.54
                            End If
                        End If
                    End If
                    If PrecipUnits_Inputcm = 1 And PrecipUnits_Outputmm = 1 Then
                        'Convert from centimeters to millimeters
                        PrecipValue = PrecipValue * 10
                        If PrecipUnits_ErrorBars = 1 Then
                            If ErrorValue <> -99 Then
                                ErrorValue = ErrorValue * 10
                            End If
                        End If
                    End If
                    If PrecipUnits_Inputmm = 1 And PrecipUnits_Outputinches = 1 Then
                        'Convert from millimeters to inches
                        PrecipValue = PrecipValue / 25.4
                        If PrecipUnits_ErrorBars = 1 Then
                            If ErrorValue <> -99 Then
                                ErrorValue = ErrorValue / 25.4
                            End If
                        End If
                    End If
                    If PrecipUnits_Inputmm = 1 And PrecipUnits_Outputcm = 1 Then
                        'Convert from millimeters to centimeters
                        PrecipValue = PrecipValue / 10
                        If PrecipUnits_ErrorBars = 1 Then
                            If ErrorValue <> -99 Then
                                ErrorValue = ErrorValue / 10
                            End If
                        End If
                    End If
                    PrecipValue = Math.Round(PrecipValue, 2)
                    ErrorValue = Math.Round(ErrorValue, 2)
                End If
                'Output New Precipitation
                Select Case PrecipUnits_InputFormat
                    Case 1
                        'Data in Monthly Format
                        If PrecipUnits_ErrorBars = 1 Then
                            PrintLine(1, Year & vbTab & Month & vbTab & PrecipValue & vbTab & ErrorValue)
                        Else
                            PrintLine(1, Year & vbTab & Month & vbTab & PrecipValue)
                        End If
                    Case 2
                        'Data in Seasonal/Annual Format
                        If PrecipUnits_ErrorBars = 1 Then
                            PrintLine(1, Year & vbTab & PrecipValue & vbTab & ErrorValue)
                        Else
                            PrintLine(1, Year & vbTab & PrecipValue)
                        End If
                End Select
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                Select Case PrecipUnits_InputFormat
                    Case 1
                        'Data in Monthly Format
                        MsgBox("Error reading the precipitation data file at or near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Convert Precipitation Units")
                    Case 2
                        'Data in Seasonal/Annual Format
                        MsgBox("Error reading the precipitation data file at or near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Convert Precipitation Units")
                End Select
            Else
                MsgBox("Error reading the precipitation data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Convert Precipitation Units")
            End If
            InFile.Close()
            FileClose(1)
            PrecipUnits_Exception = 1
            Exit Sub
        End Try
        'Close Files
        InFile.Close()
        FileClose(1)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case PrecipUnits_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub PrecipUnits_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        PrecipUnitsDisp = 0
    End Sub
End Class