Public Class ReconstructPrecip

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private ReconstructPrecip_Input As String
    Private ReconstructPrecip_Output As String
    'Initialize Global Form Variables
    Private ReconstructPrecip_TotalStations As Integer
    Private ReconstructPrecip_RowStart As Integer
    Private ReconstructPrecip_Amounts As Integer
    Private ReconstructPrecip_Days As Integer
    Private ReconstructPrecip_ConsistentStations As Integer
    Private ReconstructPrecip_ConsistentNumber As Integer
    Private ReconstructPrecip_Header As String
    Private ReconstructPrecip_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ReconstructPrecip_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Reconstruct Precipitation Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub HistoricalSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InputSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input Precipitation File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ReconstructPrecip_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ReconstructPrecip_Input = FileSelect.FileName
            InputFileLocation.Text = ReconstructPrecip_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                ReconstructPrecip_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TotalStations_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TotalStations.TextChanged
        EntryError.SetError(TotalStations, "")
        If TotalStations.Text <> "" Then
            Try
                ReconstructPrecip_TotalStations = TotalStations.Text
            Catch ex As Exception
                EntryError.SetError(TotalStations, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ConsistentStations_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ConsistentStations.CheckedChanged
        If ConsistentStations.Checked = True Then
            ConsistentNumber.Enabled = True
        Else
            ConsistentNumber.Enabled = False
        End If
    End Sub

    Private Sub ConsistentNumber_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ConsistentNumber.TextChanged
        EntryError.SetError(ConsistentNumber, "")
        If ConsistentNumber.Text <> "" Then
            Try
                ReconstructPrecip_ConsistentNumber = ConsistentNumber.Text
            Catch ex As Exception
                EntryError.SetError(ConsistentNumber, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ReconstructPrecip_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ReconstructPrecip_Output = FileOutput.FileName
            OutputFileLocation.Text = ReconstructPrecip_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ReconstructPrecip_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ReconstructPrecip_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If TotalStations.Text = "" Then
            EntryError.SetError(TotalStations, "Required input data information missing.")
            Exit Sub
        End If
        If ConsistentStations.Checked = True Then
            If ConsistentNumber.Text = "" Then
                EntryError.SetError(ConsistentNumber, "Required number of stations missing.")
                Exit Sub
            End If
        End If
        If ReconstructPrecip_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If ReconstructPrecip_Input = ReconstructPrecip_Output Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        If Amounts.Checked = True Then
            ReconstructPrecip_Amounts = 1
        Else
            ReconstructPrecip_Amounts = 0
        End If
        If Days.Checked = True Then
            ReconstructPrecip_Days = 1
        Else
            ReconstructPrecip_Days = 0
        End If
        If ConsistentStations.Checked = True Then
            ReconstructPrecip_ConsistentStations = 1
        Else
            ReconstructPrecip_ConsistentStations = 0
        End If
        ReconstructPrecip_Exception = 0
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim i As Integer
        Dim n As Double
        Dim nInt As Integer
        Dim nDec As Double
        Dim TotalPrecip As Double
        Dim Count As Integer
        Dim PrecipDataAvailable As Integer
        Dim MeanPrecip As Double
        Dim Percent75 As Double
        Dim Percent25 As Double
        Dim IQR As Double
        Dim PrecipData(0) As Double
        Dim Difference(0) As Double
        Dim NewPrecipData(0) As Double
        'Initialize Output File and Set Header
        FileOpen(1, ReconstructPrecip_Output, OpenMode.Output)
        PrintLine(1, ReconstructPrecip_Header)
        PrintLine(1, "")
        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Precip" & vbTab & "Error")
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ReconstructPrecip_Input)
        Dim Line As String
        Dim Fields As String()
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set StreamReader to Data Start
        If ReconstructPrecip_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ReconstructPrecip_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = FileComplete - (ReconstructPrecip_RowStart - 1)
        End If
        'Skip Column Headers
        Line = InFile.ReadLine()
        FileProgress = FileProgress + 1
        'Reconstruct Monthly Precipitation
        Try
            While InFile.Peek <> -1
                Line = InFile.ReadLine()
                Fields = Line.Split(ControlChars.Tab)
                Year = Fields(0)
                Month = Fields(1)
                'Input Monthly Observations
                For i = 1 To ReconstructPrecip_TotalStations
                    If Fields(i + 1) <> -99 Then
                        ReDim Preserve PrecipData(Count)
                        PrecipData(Count) = Fields(i + 1)
                        TotalPrecip = TotalPrecip + Fields(i + 1)
                        Count = Count + 1
                        PrecipDataAvailable = 1
                    End If
                Next i
                'Calculate Mean
                MeanPrecip = TotalPrecip / Count
                If PrecipDataAvailable = 1 Then
                    Select Case ReconstructPrecip_ConsistentStations
                        Case 0
                            'Use All Stations for Reconstruction...Mean Already Calculated
                            If Count < 2 Then
                                'Not Enough Stations...Set Interquartile Range to Missing
                                IQR = -99
                            Else
                                Array.Sort(PrecipData)
                                'Calculate 75th Percentile
                                n = 0.75 * (Count - 1) + 1
                                nInt = Math.Truncate(n)
                                nDec = n - nInt
                                Percent75 = nDec * (PrecipData(nInt) - PrecipData(nInt - 1))
                                Percent75 = PrecipData(nInt - 1) + Percent75
                                'Calculate 25th Percentile
                                n = 0.25 * (Count - 1) + 1
                                nInt = Math.Truncate(n)
                                nDec = n - nInt
                                Percent25 = nDec * (PrecipData(nInt) - PrecipData(nInt - 1))
                                Percent25 = PrecipData(nInt - 1) + Percent25
                                'Calculate Interquartile Range
                                IQR = Percent75 - Percent25
                            End If
                        Case 1
                            'Use Consistent Number of Stations for Reconstruction...Recalculate Mean
                            If Count < ReconstructPrecip_ConsistentNumber Then
                                'Not Enough Stations...Set Mean and Interquartile Range to Missing
                                MeanPrecip = -99
                                IQR = -99
                            Else
                                'Find Differences
                                ReDim Difference(Count - 1)
                                For i = 0 To Count - 1
                                    Difference(i) = PrecipData(i) - MeanPrecip
                                    If Difference(i) < 0 Then
                                        Difference(i) = Difference(i) * -1
                                    End If
                                Next i
                                'Sort Arrays Using Difference Array as Key
                                Array.Sort(Difference, PrecipData)
                                'Recalculate Mean
                                TotalPrecip = 0
                                Count = 0
                                ReDim NewPrecipData(ReconstructPrecip_ConsistentNumber - 1)
                                For i = 1 To ReconstructPrecip_ConsistentNumber
                                    NewPrecipData(i - 1) = PrecipData(i - 1)
                                    TotalPrecip = TotalPrecip + PrecipData(i - 1)
                                    Count = Count + 1
                                Next i
                                MeanPrecip = TotalPrecip / Count
                                'Recalculate Interquartile Range
                                Array.Sort(NewPrecipData)
                                'Calculate 75th Percentile
                                n = 0.75 * (Count - 1) + 1
                                nInt = Math.Truncate(n)
                                nDec = n - nInt
                                Percent75 = nDec * (NewPrecipData(nInt) - NewPrecipData(nInt - 1))
                                Percent75 = PrecipData(nInt - 1) + Percent75
                                'Calculate 25th Percentile
                                n = 0.25 * (Count - 1) + 1
                                nInt = Math.Truncate(n)
                                nDec = n - nInt
                                Percent25 = nDec * (NewPrecipData(nInt) - NewPrecipData(nInt - 1))
                                Percent25 = PrecipData(nInt - 1) + Percent25
                                'Calculate Interquartile Range
                                IQR = Percent75 - Percent25
                            End If
                    End Select
                    'Finalize Mean and Interquartile Range Values
                    If ReconstructPrecip_Amounts = 1 Then
                        'Calculation in Amounts, Round to Hundredths
                        MeanPrecip = Math.Round(MeanPrecip, 2)
                        IQR = Math.Round(IQR, 2)
                    End If
                    If ReconstructPrecip_Days = 1 Then
                        'Calculation in Days, Round to Whole Value
                        MeanPrecip = Math.Round(MeanPrecip)
                        IQR = Math.Round(IQR)
                    End If
                Else
                    'Monthly Precipitation Data Missing
                    MeanPrecip = -99
                    IQR = -99
                End If
                'Output Result
                PrintLine(1, Year & vbTab & Month & vbTab & MeanPrecip & vbTab & IQR)
                'Reset for Next Month
                ReDim PrecipData(0)
                TotalPrecip = 0
                Count = 0
                PrecipDataAvailable = 0
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the input station data near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Precipitation")
            Else
                MsgBox("Error reading the input station data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Precipitation")
            End If
            InFile.Close()
            FileClose(1)
            ReconstructPrecip_Exception = 1
            Exit Sub
        End Try
        'Close Files
        FileClose(1)
        InFile.Close()
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ReconstructPrecip_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ReconstructPrecip_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ReconstructPrecipDisp = 0
    End Sub
End Class