Public Class ReconstructTemp
    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Form Input/Output Variables
    Private ReconstructTemp_ReconstructionInput As String
    Private ReconstructTemp_ScreeningInput As String
    Private ReconstructTemp_Output As String
    Private ReconstructTemp_OutputTestModels As String
    Private ReconstructTemp_OutputFinalModels As String
    'Initialize Temporary Residual Output Variables
    Private ReconstructTemp_ResidualsSet1 As String
    Private ReconstructTemp_ResidualsSet2 As String
    'Initialize Global Form Variables
    Private ReconstructTemp_ReconstructionRowStart As Integer
    Private ReconstructTemp_ReconstructionStations As Integer
    Private ReconstructTemp_ScreeningRowStart As Integer
    Private ReconstructTemp_ScreeningStations As Integer
    Private ReconstructTemp_Header As String
    Private ReconstructTemp_HeaderTestModels As String
    Private ReconstructTemp_HeaderFinalModels As String
    Private ReconstructTemp_Exception As Integer
    Private ReconstructTemp_JobNumber As Integer
    Private ReconstructTemp_RemoveOutliers As Integer
    Private ReconstructTemp_ReconstructionStationNames() As String
    Private ReconstructTemp_ReconstructionStationData(0, 0) As Double
    Private ReconstructTemp_ReconstructionStationDataCount As Integer
    Private ReconstructTemp_ScreeningStationData(0, 0) As Double
    Private ReconstructTemp_ScreeningStationDataCount As Integer
    Private ReconstructTemp_FinalReconstruction(,) As String
    Private ReconstructTemp_StationNumber As Integer
    Private ReconstructTemp_DepStartDate As Date
    Private ReconstructTemp_DepEndDate As Date
    Private ReconstructTemp_IndStartDate As Date
    Private ReconstructTemp_IndEndDate As Date
    Private ReconstructTemp_DepIndData(,) As Double
    Private ReconstructTemp_DepIndDataCount As Integer
    Private ReconstructTemp_RegressionResultsForwardCal As String
    Private ReconstructTemp_RegressionResultsForwardVer As String
    Private ReconstructTemp_RegressionResultsBackwardCal As String
    Private ReconstructTemp_RegressionResultsBackwardVer As String
    Private ReconstructTemp_RegressionResultsFinal As String
    Private ReconstructTemp_ResidualDates() As Date
    Private ReconstructTemp_ResidualDatesCount As Integer
    Private ReconstructTemp_RemoveResidualDates() As Date
    Private ReconstructTemp_RemoveResidualDatesCount As Integer
    Private ReconstructTemp_FinalB0(35) As Double
    Private ReconstructTemp_FinalB1(35) As Double
    Private ReconstructTemp_FinalStdErr(35) As Double

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ReconstructTemp_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Reconstruct Temperature Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub ReconstructionSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReconstructionSelect_Button.Click
        EntryError.SetError(ReconstructionFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Station Data for Reconstruction"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If ReconstructionFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ReconstructTemp_ReconstructionInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ReconstructTemp_ReconstructionInput = FileSelect.FileName
            ReconstructionFileLocation.Text = ReconstructTemp_ReconstructionInput
            ReconstructionFileLocation.SelectionStart = ReconstructionFileLocation.Text.Length
        End If
    End Sub

    Private Sub ReconstructionDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReconstructionDataRowStart.TextChanged
        EntryError.SetError(ReconstructionDataRowStart, "")
        If ReconstructionDataRowStart.Text <> "" Then
            Try
                ReconstructTemp_ReconstructionRowStart = ReconstructionDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(ReconstructionDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ReconstructionStations_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReconstructionStations.TextChanged
        EntryError.SetError(ReconstructionStations, "")
        If ReconstructionStations.Text <> "" Then
            Try
                ReconstructTemp_ReconstructionStations = ReconstructionStations.Text
            Catch ex As Exception
                EntryError.SetError(ReconstructionStations, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ScreeningSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ScreeningSelect_Button.Click
        EntryError.SetError(ScreeningFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Station Data for Inter-Station Screening"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If ScreeningFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ReconstructTemp_ScreeningInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ReconstructTemp_ScreeningInput = FileSelect.FileName
            ScreeningFileLocation.Text = ReconstructTemp_ScreeningInput
            ScreeningFileLocation.SelectionStart = ScreeningFileLocation.Text.Length
        End If
    End Sub

    Private Sub ScreeningDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ScreeningDataRowStart.TextChanged
        EntryError.SetError(ScreeningDataRowStart, "")
        If ScreeningDataRowStart.Text <> "" Then
            Try
                ReconstructTemp_ScreeningRowStart = ScreeningDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(ScreeningDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ScreeningStations_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ScreeningStations.TextChanged
        EntryError.SetError(ScreeningStations, "")
        If ScreeningStations.Text <> "" Then
            Try
                ReconstructTemp_ScreeningStations = ScreeningStations.Text
            Catch ex As Exception
                EntryError.SetError(ScreeningStations, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ReconstructTemp_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ReconstructTemp_Output = FileOutput.FileName
            OutputFileLocation.Text = ReconstructTemp_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ReconstructTemp_Header = HeaderInfo.Text
    End Sub

    Private Sub ResidualsList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ResidualsList.SelectedIndexChanged
        EntryError.SetError(ResidualsList, "")
    End Sub

    Private Sub Forward_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Forward.CheckedChanged
        'Read Residuals Set 1 Into Residuals List
        If Forward.Checked = True Then
            ResidualsList.Items.Clear()
            Dim ResidualsSet1InFile As New System.IO.StreamReader(ReconstructTemp_ResidualsSet1)
            Dim ResidualsSet1Line As String
            Dim ResidualsSet1Fields As String()
            While ResidualsSet1InFile.Peek <> -1
                ResidualsSet1Line = ResidualsSet1InFile.ReadLine()
                ResidualsSet1Fields = ResidualsSet1Line.Split(ControlChars.Tab)
                ResidualsList.Items.Add(ResidualsSet1Fields(0) & " Model " & ResidualsSet1Fields(1) & " Residual = " & ResidualsSet1Fields(5))
            End While
            ResidualsSet1InFile.Close()
        End If
    End Sub

    Private Sub Backward_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Backward.CheckedChanged
        'Read Residuals Set 2 Into Residuals List
        If Backward.Checked = True Then
            ResidualsList.Items.Clear()
            Dim ResidualsSet2InFile As New System.IO.StreamReader(ReconstructTemp_ResidualsSet2)
            Dim ResidualsSet2Line As String
            Dim ResidualsSet2Fields As String()
            While ResidualsSet2InFile.Peek <> -1
                ResidualsSet2Line = ResidualsSet2InFile.ReadLine()
                ResidualsSet2Fields = ResidualsSet2Line.Split(ControlChars.Tab)
                ResidualsList.Items.Add(ResidualsSet2Fields(0) & " Model " & ResidualsSet2Fields(1) & " Residual = " & ResidualsSet2Fields(5))
            End While
            ResidualsSet2InFile.Close()
        End If
    End Sub

    Private Sub ViewPlots_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ViewPlots.Click
        'Set Residuals File and Station Names
        If (Forward.Enabled = False And Backward.Enabled = False) Or (Forward.Enabled = True And Forward.Checked = True) Then
            ReconstructTemp_OutputResiduals = ReconstructTemp_ResidualsSet1
        End If
        If Backward.Enabled = True And Backward.Checked = True Then
            ReconstructTemp_OutputResiduals = ReconstructTemp_ResidualsSet2
        End If
        ReconstructTemp_DepStation = ReconstructTemp_ReconstructionStationNames(0)
        ReconstructTemp_IndStation = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
        'Show Figure
        If ReconstructTempFigDisp = 0 Then
            ReconstructTempFigDisp = 1
            Dim frmReconstructTempFig As New ReconstructTempFig
            frmReconstructTempFig.MdiParent = MainParent
            frmReconstructTempFig.Show()
        Else
            MsgBox("A plot of residuals is already open for display.  Please close this plot before launching a new one.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
        End If
    End Sub

    Private Sub Run1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run1.CheckedChanged
        If Run1.Checked = True And RemoveOutliers.Enabled = False Then
            RemoveOutliers.Enabled = True
        End If
    End Sub

    Private Sub Run2_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run2.CheckedChanged
        If Run2.Checked = True And RemoveOutliers.Enabled = True Then
            RemoveOutliers.Enabled = False
        End If
    End Sub

    Private Sub Run3_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run3.CheckedChanged
        If Run3.Checked = True Then
            If ReconstructTemp_StationNumber <> ReconstructTemp_ReconstructionStations - 1 Then
                RemoveOutliers.Enabled = True
            Else
                RemoveOutliers.Enabled = False
                Run.Text = "Finish Run"
            End If
        Else
            Run.Text = "Continue Run"
        End If
    End Sub

    Private Sub RemoveOutliers_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RemoveOutliers.CheckedChanged
        If RemoveOutliers.Checked = True Then
            Run1.Text = "Calibrate and Verify Models"
        Else
            Run1.Text = "Remove Selected Residuals / Calibrate and Verify Models"
        End If
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        If ReconstructTemp_JobNumber = 0 Then
            'First Run Sequence...Check for User Input Errors
            If ReconstructTemp_ReconstructionInput = "" Then
                EntryError.SetError(ReconstructionFileLocation, "Input data for reconstruction missing.")
                Exit Sub
            End If
            If ReconstructionDataRowStart.Text = "" Then
                EntryError.SetError(ReconstructionDataRowStart, "Required reconstruction data information missing.")
                Exit Sub
            End If
            If ReconstructionStations.Text = "" Then
                EntryError.SetError(ReconstructionStations, "Required reconstruction data information missing.")
                Exit Sub
            End If
            If ReconstructTemp_ScreeningInput = "" Then
                EntryError.SetError(ScreeningFileLocation, "Input data for inter-station screening missing.")
                Exit Sub
            End If
            If ScreeningDataRowStart.Text = "" Then
                EntryError.SetError(ScreeningDataRowStart, "Required screening data information missing.")
                Exit Sub
            End If
            If ScreeningStations.Text = "" Then
                EntryError.SetError(ScreeningStations, "Required screening data information missing.")
                Exit Sub
            End If
            If ReconstructTemp_Output = "" Then
                EntryError.SetError(OutputFileLocation, "Output dataset missing.")
                Exit Sub
            End If
            If ReconstructTemp_Output = ReconstructTemp_ReconstructionInput Or ReconstructTemp_Output = ReconstructTemp_ScreeningInput Then
                EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
                Exit Sub
            End If
            If HeaderInfo.Text = "" Then
                EntryError.SetError(HeaderInfo, "Required output header information missing.")
                Exit Sub
            End If
        End If
        'Start Run
        Run.Enabled = False
        If ReconstructTemp_JobNumber <> 0 Then
            If Run1.Checked = True Then
                Select Case RemoveOutliers.Checked
                    Case True
                        ReconstructTemp_JobNumber = 3
                    Case False
                        'Prepare Selected Residuals
                        Dim i As Integer
                        If ResidualsList.CheckedItems.Count <> 0 Then
                            Status_Label.Text = "Status:  Preparing Selected Residuals..."
                            ReconstructTemp_ResidualDatesCount = 0
                            ReconstructTemp_RemoveResidualDatesCount = 0
                            'Read Residuals File for Dates
                            If (Forward.Enabled = False And Backward.Enabled = False) Or (Forward.Enabled = True And Forward.Checked = True) Then
                                'Residuals File Set 1
                                Dim ResidualsSet1InFile As New System.IO.StreamReader(ReconstructTemp_ResidualsSet1)
                                Dim ResidualsSet1Line As String
                                Dim ResidualsSet1Fields As String()
                                While ResidualsSet1InFile.Peek <> -1
                                    ReDim Preserve ReconstructTemp_ResidualDates(ReconstructTemp_ResidualDatesCount)
                                    ResidualsSet1Line = ResidualsSet1InFile.ReadLine()
                                    ResidualsSet1Fields = ResidualsSet1Line.Split(ControlChars.Tab)
                                    ReconstructTemp_ResidualDates(ReconstructTemp_ResidualDatesCount) = ResidualsSet1Fields(6)
                                    ReconstructTemp_ResidualDatesCount = ReconstructTemp_ResidualDatesCount + 1
                                End While
                                ResidualsSet1InFile.Close()
                            End If
                            If Backward.Enabled = True And Backward.Checked = True Then
                                'Residuals File Set 2
                                Dim ResidualsSet2InFile As New System.IO.StreamReader(ReconstructTemp_ResidualsSet2)
                                Dim ResidualsSet2Line As String
                                Dim ResidualsSet2Fields As String()
                                While ResidualsSet2InFile.Peek <> -1
                                    ReDim Preserve ReconstructTemp_ResidualDates(ReconstructTemp_ResidualDatesCount)
                                    ResidualsSet2Line = ResidualsSet2InFile.ReadLine()
                                    ResidualsSet2Fields = ResidualsSet2Line.Split(ControlChars.Tab)
                                    ReconstructTemp_ResidualDates(ReconstructTemp_ResidualDatesCount) = ResidualsSet2Fields(6)
                                    ReconstructTemp_ResidualDatesCount = ReconstructTemp_ResidualDatesCount + 1
                                End While
                                ResidualsSet2InFile.Close()
                            End If
                            'Retrieve Dates of Selected Residuals
                            For i = 0 To ResidualsList.Items.Count - 1
                                If ResidualsList.GetItemChecked(i) = True Then
                                    ReDim Preserve ReconstructTemp_RemoveResidualDates(ReconstructTemp_RemoveResidualDatesCount)
                                    ReconstructTemp_RemoveResidualDates(ReconstructTemp_RemoveResidualDatesCount) = ReconstructTemp_ResidualDates(i)
                                    ReconstructTemp_RemoveResidualDatesCount = ReconstructTemp_RemoveResidualDatesCount + 1
                                End If
                            Next i
                            ReconstructTemp_JobNumber = 4
                        Else
                            EntryError.SetError(ResidualsList, "No residuals have been checked.")
                            Run.Enabled = True
                            Exit Sub
                        End If
                End Select
            End If
            If Run2.Checked = True Then
                ReconstructTemp_JobNumber = 6
            End If
            If Run3.Checked = True Then
                ReconstructTemp_JobNumber = 1
            End If
        End If
        If RemoveOutliers.Checked = True Then
            ReconstructTemp_RemoveOutliers = 1
        Else
            ReconstructTemp_RemoveOutliers = 0
        End If
        ReconstructTemp_Exception = 0
        Status_Label.Text = "Status:  Running..."
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Day As Integer
        Dim Month As Integer
        Dim Year As Integer
        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim l As Integer
        Dim FoundDate As Integer
        Dim CheckDate As Date
        Dim DepStore As Integer
        Dim IndStore As Integer
        Dim HalfPoint As Integer
        Dim IndModel1Set1(0) As Double
        Dim DepModel1Set1(0) As Double
        Dim Model1Set1Dates(0) As Date
        Dim DiffModel1(0) As Double
        Dim Model1Set1Count As Integer
        Dim IndModel2Set1(0) As Double
        Dim DepModel2Set1(0) As Double
        Dim Model2Set1Dates(0) As Date
        Dim DiffModel2(0) As Double
        Dim Model2Set1Count As Integer
        Dim IndModel3Set1(0) As Double
        Dim DepModel3Set1(0) As Double
        Dim Model3Set1Dates(0) As Date
        Dim DiffModel3(0) As Double
        Dim Model3Set1Count As Integer
        Dim IndModel1Set2(0) As Double
        Dim DepModel1Set2(0) As Double
        Dim Model1Set2Dates(0) As Date
        Dim Model1Set2Count As Integer
        Dim IndModel2Set2(0) As Double
        Dim DepModel2Set2(0) As Double
        Dim Model2Set2Dates(0) As Date
        Dim Model2Set2Count As Integer
        Dim IndModel3Set2(0) As Double
        Dim DepModel3Set2(0) As Double
        Dim Model3Set2Dates(0) As Date
        Dim Model3Set2Count As Integer
        Dim EvenOddCheck As Double
        Dim Model1Percentile10 As Double
        Dim Model1Percentile90 As Double
        Dim Model2Percentile10 As Double
        Dim Model2Percentile90 As Double
        Dim Model3Percentile10 As Double
        Dim Model3Percentile90 As Double
        Dim TotalTemp As Double
        Dim NetworkAvg As Double
        Dim TempCount As Integer
        Dim DepvsNetwork As Double
        Dim IndvsNetwork As Double
        Dim DayValue As Date
        Dim IndValue As Double
        Dim SumXY As Double
        Dim SumX As Double
        Dim SumY As Double
        Dim SumXSq As Double
        Dim SumYSq As Double
        Dim SumError As Double
        Dim SXY As Double
        Dim SXX As Double
        Dim SYY As Double
        Dim SSE As Double
        Dim SST As Double
        Dim SSR As Double
        Dim Model1B0 As Double
        Dim Model1B1 As Double
        Dim Model1StdErr As Double
        Dim Model1RSq As Double
        Dim Model1DurWat As Double
        Dim Model2B0 As Double
        Dim Model2B1 As Double
        Dim Model2StdErr As Double
        Dim Model2RSq As Double
        Dim Model2DurWat As Double
        Dim Model3B0 As Double
        Dim Model3B1 As Double
        Dim Model3StdErr As Double
        Dim Model3RSq As Double
        Dim Model3DurWat As Double
        Dim MonthPrint As String
        Dim PredictedTemp As Double
        Dim ActualTemp As Double
        Dim Residual As Double
        Dim Residualm1 As Double
        Dim VerActual() As Double
        Dim VerPredicted() As Double
        Dim CE As Double
        Dim Verr As Double
        Dim TransferOn As Integer
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        Select Case ReconstructTemp_JobNumber
            Case 0
                Select Case ReconstructTemp_RemoveOutliers
                    Case 0
                        'Start Reconstruction, Calibrate and Verify
                        FileComplete = 4 + 2 + 12
                    Case 1
                        'Start Reconstruction, Remove Outliers, Calibrate and Verify
                        FileComplete = 4 + 2 + 12 + 12
                End Select
            Case 1
                If ReconstructTemp_StationNumber = ReconstructTemp_ReconstructionStations - 1 Then
                    'Save and Transfer, Complete Reconstruction
                    FileComplete = 3
                Else
                    Select Case ReconstructTemp_RemoveOutliers
                        Case 0
                            'Save and Transfer, Move to Next Station, Calibrate and Verify
                            FileComplete = 2 + 2 + 12
                        Case 1
                            'Save and Transfer, Move to Next Station, Remove Outliers, Calibrate and Verify
                            FileComplete = 2 + 2 + 12 + 12
                    End Select
                End If
            Case 3
                'Remove Outliers, Calibrate and Verify Models
                FileComplete = 12 + 12
            Case 4
                'Remove Selected Residuals, Calibrate and Verify Models
                FileComplete = 1 + 12
            Case 6
                'Derive Final Models
                FileComplete = 12
        End Select
        If ReconstructTemp_JobNumber = 0 Then
            'First Run Sequence...Read In Reconstruction and Screening Datasets
            'Initialize StreamReaders
            Dim ReconstructionInFile As New System.IO.StreamReader(ReconstructTemp_ReconstructionInput)
            Dim ReconstructionLine As String
            Dim ReconstructionFields As String()
            Dim ScreeningInFile As New System.IO.StreamReader(ReconstructTemp_ScreeningInput)
            Dim ScreeningLine As String
            Dim ScreeningFields As String()
            'Set StreamReaders to Data Start
            If ReconstructTemp_ReconstructionRowStart <> 1 Then
                'Header Exists so Move LineInput Down
                Dim Row As Integer
                For Row = 0 To ReconstructTemp_ReconstructionRowStart - 2
                    ReconstructionLine = ReconstructionInFile.ReadLine()
                Next Row
            End If
            If ReconstructTemp_ScreeningRowStart <> 1 Then
                'Header Exists so Move LineInput Down
                Dim Row As Integer
                For Row = 0 To ReconstructTemp_ScreeningRowStart - 2
                    ScreeningLine = ScreeningInFile.ReadLine()
                Next Row
            End If
            'Start Row for Screening Data Includes Station Names for User Input Consistency But Not Needed...Move StreamReader Down One Line
            ScreeningLine = ScreeningInFile.ReadLine()
            'Read Reconstruction Station Names into Array
            Try
                ReconstructionLine = ReconstructionInFile.ReadLine()
                ReconstructionFields = ReconstructionLine.Split(ControlChars.Tab)
                ReDim ReconstructTemp_ReconstructionStationNames(ReconstructTemp_ReconstructionStations - 1)
                For i = 0 To ReconstructTemp_ReconstructionStations - 1
                    ReconstructTemp_ReconstructionStationNames(i) = ReconstructionFields(i + 3)
                Next i
            Catch ex As Exception
                MsgBox("Error reading the station names in the reconstruction data file.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
                ReconstructionInFile.Close()
                ScreeningInFile.Close()
                ReconstructTemp_Exception = 1
                Exit Sub
            End Try
            'Read Reconstruction Station Data into Matrix Array
            Try
                ReDim ReconstructTemp_ReconstructionStationData(ReconstructTemp_ReconstructionStations + 2, 0)
                While ReconstructionInFile.Peek <> -1
                    ReconstructionLine = ReconstructionInFile.ReadLine()
                    ReconstructionFields = ReconstructionLine.Split(ControlChars.Tab)
                    ReDim Preserve ReconstructTemp_ReconstructionStationData(ReconstructTemp_ReconstructionStations + 2, j)
                    ReconstructTemp_ReconstructionStationDataCount = ReconstructTemp_ReconstructionStationDataCount + 1
                    Year = ReconstructionFields(0)
                    Month = ReconstructionFields(1)
                    Day = ReconstructionFields(2)
                    For i = 0 To ReconstructTemp_ReconstructionStations + 2
                        ReconstructTemp_ReconstructionStationData(i, j) = ReconstructionFields(i)
                    Next i
                    j = j + 1
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the reconstruction data near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
                Else
                    MsgBox("Error reading the reconstruction data near the header.  Make sure the row where the reconstruction data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
                End If
                ReconstructionInFile.Close()
                ScreeningInFile.Close()
                ReconstructTemp_Exception = 1
                Exit Sub
            End Try
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            j = 0
            Year = 0
            'Read Screening Station Data into Matrix Array
            Try
                ReDim ReconstructTemp_ScreeningStationData(ReconstructTemp_ScreeningStations + 2, 0)
                While ScreeningInFile.Peek <> -1
                    ScreeningLine = ScreeningInFile.ReadLine()
                    ScreeningFields = ScreeningLine.Split(ControlChars.Tab)
                    ReDim Preserve ReconstructTemp_ScreeningStationData(ReconstructTemp_ScreeningStations + 2, j)
                    ReconstructTemp_ScreeningStationDataCount = ReconstructTemp_ScreeningStationDataCount + 1
                    Year = ScreeningFields(0)
                    Month = ScreeningFields(1)
                    Day = ScreeningFields(2)
                    For i = 0 To ReconstructTemp_ScreeningStations + 2
                        ReconstructTemp_ScreeningStationData(i, j) = ScreeningFields(i)
                    Next i
                    j = j + 1
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the inter-station screening data near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
                Else
                    MsgBox("Error reading the inter-station screening data near the header.  Make sure the row where the inter-station screening data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
                End If
                ReconstructionInFile.Close()
                ScreeningInFile.Close()
                ReconstructTemp_Exception = 1
                Exit Sub
            End Try
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Close StreamReaders
            ReconstructionInFile.Close()
            ScreeningInFile.Close()
            'Initialize Final Reconstruction Matrix Array
            ReDim ReconstructTemp_FinalReconstruction(5, ReconstructTemp_ReconstructionStationDataCount - 1)
            For i = 0 To ReconstructTemp_ReconstructionStationDataCount - 1
                ReconstructTemp_FinalReconstruction(0, i) = ReconstructTemp_ReconstructionStationData(0, i)
                ReconstructTemp_FinalReconstruction(1, i) = ReconstructTemp_ReconstructionStationData(1, i)
                ReconstructTemp_FinalReconstruction(2, i) = ReconstructTemp_ReconstructionStationData(2, i)
                ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                ReconstructTemp_FinalReconstruction(4, i) = 0
                If ReconstructTemp_ReconstructionStationData(3, i) <> -99 Then
                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(0)
                End If
            Next i
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Initialize Output Filenames and Set Headers
            Dim DecimalIndex As Integer = ReconstructTemp_Output.IndexOf(".")
            ReconstructTemp_OutputTestModels = ReconstructTemp_Output.Insert(DecimalIndex, " Test Models")
            ReconstructTemp_OutputFinalModels = ReconstructTemp_Output.Insert(DecimalIndex, " Final Models")
            ReconstructTemp_ResidualsSet1 = ReconstructTemp_Output.Insert(DecimalIndex, " Residuals Set 1")
            ReconstructTemp_ResidualsSet2 = ReconstructTemp_Output.Insert(DecimalIndex, " Residuals Set 2")
            ReconstructTemp_HeaderTestModels = ReconstructTemp_Header + " Test Models"
            ReconstructTemp_HeaderFinalModels = ReconstructTemp_Header + " Final Models"
            'Find Start and End Dates of Dependent Station
            For i = 0 To ReconstructTemp_ReconstructionStationDataCount - 1
                Select Case FoundDate
                    Case 0
                        If ReconstructTemp_ReconstructionStationData(3, i) <> -99 Then
                            'Start Date Found
                            FoundDate = 1
                            ReconstructTemp_DepStartDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                        End If
                    Case 1
                        'Temperature Observation On Previous Day...Check for Missing Value Indicating Possible End Date
                        If ReconstructTemp_ReconstructionStationData(3, i) = -99 Then
                            'Set as Possible End Date
                            FoundDate = 2
                            ReconstructTemp_DepEndDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                        End If
                    Case 2
                        'Temperature Observation Missing On Previous Day...Possible End Date
                        If ReconstructTemp_ReconstructionStationData(3, i) <> -99 Then
                            'Remove Possible End Date
                            FoundDate = 1
                            ReconstructTemp_DepEndDate = Nothing
                        End If
                End Select
                If i = ReconstructTemp_ReconstructionStationDataCount - 1 And ReconstructTemp_DepEndDate = Nothing Then
                    'End of Station Data...Set End Date as End of Data
                    ReconstructTemp_DepEndDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                End If
            Next i
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Next Job Number
            ReconstructTemp_JobNumber = 2
        End If
        If ReconstructTemp_JobNumber = 1 Then
            'Save and Transfer Sequence
            'Save Previous Models to Output Files
            If ReconstructTemp_StationNumber = 1 Then
                'First Save...Set Modes to Output and Output Headers
                FileOpen(2, ReconstructTemp_OutputTestModels, OpenMode.Output)
                PrintLine(2, ReconstructTemp_HeaderTestModels)
                PrintLine(2, "")
                FileOpen(3, ReconstructTemp_OutputFinalModels, OpenMode.Output)
                PrintLine(3, ReconstructTemp_HeaderFinalModels)
                PrintLine(3, "")
            Else
                FileOpen(2, ReconstructTemp_OutputTestModels, OpenMode.Append)
                PrintLine(2, "")
                PrintLine(2, "--------------------")
                PrintLine(2, "")
                FileOpen(3, ReconstructTemp_OutputFinalModels, OpenMode.Append)
                PrintLine(3, "")
                PrintLine(3, "--------------------")
                PrintLine(3, "")
            End If
            PrintLine(2, ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber))
            PrintLine(2, "")
            PrintLine(2, ReconstructTemp_RegressionResultsForwardCal)
            PrintLine(2, "")
            PrintLine(2, ReconstructTemp_RegressionResultsForwardVer)
            PrintLine(2, "")
            PrintLine(2, ReconstructTemp_RegressionResultsBackwardCal)
            PrintLine(2, "")
            PrintLine(2, ReconstructTemp_RegressionResultsBackwardVer)
            FileClose(2)
            PrintLine(3, ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber))
            PrintLine(3, "")
            PrintLine(3, ReconstructTemp_RegressionResultsFinal)
            FileClose(3)
            'Place Screened Independent and Dependent Data Into Main Reconstruction Matrix Array
            j = 0
            DepStore = 0
            IndStore = 0
            For i = 0 To ReconstructTemp_ReconstructionStationDataCount - 1
                CheckDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                If CheckDate = ReconstructTemp_DepStartDate Then
                    DepStore = 1
                End If
                If CheckDate = ReconstructTemp_IndStartDate Then
                    IndStore = 1
                End If
                If DepStore = 1 And IndStore = 1 Then
                    'Overlap Period...Store Screened Independent and Depedent Datasets
                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_DepIndData(3, j)
                    ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i) = ReconstructTemp_DepIndData(4, j)
                    j = j + 1
                End If
                If CheckDate = ReconstructTemp_DepEndDate Then
                    DepStore = 0
                End If
                If CheckDate = ReconstructTemp_IndEndDate Then
                    IndStore = 0
                End If
            Next i
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Use Final Models to Transfer Observations from Independent to Dependent Station
            DepStore = 0
            TransferOn = 0
            For i = 0 To ReconstructTemp_ReconstructionStationDataCount - 1
                CheckDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                If ReconstructTemp_DepStartDate = CheckDate And TransferOn = 0 Then
                    DepStore = 1
                End If
                If ReconstructTemp_IndStartDate = CheckDate Then
                    TransferOn = 1
                    If DepStore = 0 Then
                        'Set New Dependent Start Date
                        ReconstructTemp_DepStartDate = CheckDate
                        DepStore = 1
                    End If
                End If
                If TransferOn = 1 And ReconstructTemp_ReconstructionStationData(3, i) = -99 And ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i) <> -99 Then
                    'Transfer Observations
                    Select Case ReconstructTemp_ReconstructionStationData(1, i)
                        Case 1
                            'January
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(0) + ReconstructTemp_FinalB1(0) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(0)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(1) + ReconstructTemp_FinalB1(1) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(1)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(2) + ReconstructTemp_FinalB1(2) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(2)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 2
                            'February
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(3) + ReconstructTemp_FinalB1(3) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(3)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(4) + ReconstructTemp_FinalB1(4) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(4)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(5) + ReconstructTemp_FinalB1(5) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(5)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 3
                            'March
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(6) + ReconstructTemp_FinalB1(6) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(6)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(7) + ReconstructTemp_FinalB1(7) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(7)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(8) + ReconstructTemp_FinalB1(8) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(8)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 4
                            'April
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(9) + ReconstructTemp_FinalB1(9) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(9)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(10) + ReconstructTemp_FinalB1(10) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(10)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(11) + ReconstructTemp_FinalB1(11) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(11)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 5
                            'May
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(12) + ReconstructTemp_FinalB1(12) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(12)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(13) + ReconstructTemp_FinalB1(13) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(13)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(14) + ReconstructTemp_FinalB1(14) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(14)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 6
                            'June
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(15) + ReconstructTemp_FinalB1(15) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(15)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(16) + ReconstructTemp_FinalB1(16) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(16)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(17) + ReconstructTemp_FinalB1(17) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(17)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 7
                            'July
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(18) + ReconstructTemp_FinalB1(18) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(18)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(19) + ReconstructTemp_FinalB1(19) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(19)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(20) + ReconstructTemp_FinalB1(20) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(20)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 8
                            'August
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(21) + ReconstructTemp_FinalB1(21) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(21)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(22) + ReconstructTemp_FinalB1(22) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(22)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(23) + ReconstructTemp_FinalB1(23) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(23)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 9
                            'September
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(24) + ReconstructTemp_FinalB1(24) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(24)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(25) + ReconstructTemp_FinalB1(25) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(25)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(26) + ReconstructTemp_FinalB1(26) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(26)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 10
                            'October
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(27) + ReconstructTemp_FinalB1(27) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(27)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(28) + ReconstructTemp_FinalB1(28) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(28)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(29) + ReconstructTemp_FinalB1(29) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(29)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 11
                            'November
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(30) + ReconstructTemp_FinalB1(30) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(30)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(31) + ReconstructTemp_FinalB1(31) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(31)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(32) + ReconstructTemp_FinalB1(32) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(32)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                        Case 12
                            'December
                            Select Case ReconstructTemp_ReconstructionStationData(2, i)
                                Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                    'Model 1
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(33) + ReconstructTemp_FinalB1(33) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(33)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                    'Model 2
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(34) + ReconstructTemp_FinalB1(34) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(34)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                                Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                    'Model 3
                                    ReconstructTemp_ReconstructionStationData(3, i) = ReconstructTemp_FinalB0(35) + ReconstructTemp_FinalB1(35) * ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                                    ReconstructTemp_ReconstructionStationData(3, i) = Math.Round(ReconstructTemp_ReconstructionStationData(3, i), 1)
                                    ReconstructTemp_FinalReconstruction(3, i) = ReconstructTemp_ReconstructionStationData(3, i)
                                    ReconstructTemp_FinalReconstruction(4, i) = ReconstructTemp_FinalStdErr(35)
                                    ReconstructTemp_FinalReconstruction(5, i) = ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                            End Select
                    End Select
                End If
                If ReconstructTemp_IndEndDate = CheckDate Then
                    TransferOn = 0
                    If DepStore = 0 Then
                        'Set New Dependent End Date
                        ReconstructTemp_DepEndDate = CheckDate
                    End If
                End If
                If ReconstructTemp_DepEndDate = CheckDate Then
                    DepStore = 0
                End If
            Next i
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Reset Variables
            ReDim ReconstructTemp_DepIndData(4, 0)
            ReconstructTemp_DepIndDataCount = 0
            If ReconstructTemp_StationNumber = ReconstructTemp_ReconstructionStations - 1 Then
                'Output Final Reconstruction
                FileOpen(1, ReconstructTemp_Output, OpenMode.Output)
                PrintLine(1, ReconstructTemp_Header)
                PrintLine(1, "")
                PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & "Temp" & vbTab & "StdErr" & vbTab & "Station")
                For i = 0 To ReconstructTemp_ReconstructionStationDataCount - 1
                    PrintLine(1, ReconstructTemp_FinalReconstruction(0, i) & vbTab & ReconstructTemp_FinalReconstruction(1, i) & vbTab & ReconstructTemp_FinalReconstruction(2, i) & vbTab & ReconstructTemp_FinalReconstruction(3, i) & vbTab & ReconstructTemp_FinalReconstruction(4, i) & vbTab & ReconstructTemp_FinalReconstruction(5, i))
                Next i
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
                FileClose(1)
                'Reset Variables and Arrays for New Run
                System.IO.File.Delete(ReconstructTemp_ResidualsSet1)
                System.IO.File.Delete(ReconstructTemp_ResidualsSet2)
                ReDim ReconstructTemp_ReconstructionStationNames(0)
                ReDim ReconstructTemp_ReconstructionStationData(0, 0)
                ReconstructTemp_ReconstructionStationDataCount = 0
                ReDim ReconstructTemp_ScreeningStationData(0, 0)
                ReconstructTemp_ScreeningStationDataCount = 0
                ReDim ReconstructTemp_FinalReconstruction(0, 0)
                ReconstructTemp_StationNumber = 0
                ReconstructTemp_JobNumber = 0
                'Change Exception Flag to Denote End of Reconstruction
                ReconstructTemp_Exception = 2
            Else
                'Next Job Number
                ReconstructTemp_JobNumber = 2
            End If
        End If
        If ReconstructTemp_JobNumber = 2 Then
            'Independent and Dependent Stations Setup Sequence
            'Increment Station Number
            ReconstructTemp_StationNumber = ReconstructTemp_StationNumber + 1
            'Find Start and End Dates of Independent Station
            FoundDate = 0
            For i = 0 To ReconstructTemp_ReconstructionStationDataCount - 1
                Select Case FoundDate
                    Case 0
                        If ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i) <> -99 Then
                            'Start Date Found
                            FoundDate = 1
                            ReconstructTemp_IndStartDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                        End If
                    Case 1
                        'Temperature Observation On Previous Day...Check for Missing Value Indicating Possible End Date
                        If ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i) = -99 Then
                            'Set as Possible End Date
                            FoundDate = 2
                            ReconstructTemp_IndEndDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                        End If
                    Case 2
                        'Temperature Observation Missing On Previous Day...Possible End Date
                        If ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i) <> -99 Then
                            'Remove Possible End Date
                            FoundDate = 1
                            ReconstructTemp_IndEndDate = Nothing
                        End If
                End Select
                If i = ReconstructTemp_ReconstructionStationDataCount - 1 And ReconstructTemp_IndEndDate = Nothing Then
                    'End of Station Data...Set End Date as End of Data
                    ReconstructTemp_IndEndDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                End If
            Next i
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Place Dependent and Independent Datasets into Matrix Array
            j = 0
            DepStore = 0
            IndStore = 0
            For i = 0 To ReconstructTemp_ReconstructionStationDataCount - 1
                CheckDate = DateValue(ReconstructTemp_ReconstructionStationData(1, i) & "/" & ReconstructTemp_ReconstructionStationData(2, i) & "/" & ReconstructTemp_ReconstructionStationData(0, i))
                If CheckDate = ReconstructTemp_DepStartDate Then
                    DepStore = 1
                End If
                If CheckDate = ReconstructTemp_IndStartDate Then
                    IndStore = 1
                End If
                If DepStore = 1 And IndStore = 1 Then
                    'Overlap Period...Store Dependent and Independent Datasets
                    ReDim Preserve ReconstructTemp_DepIndData(4, j)
                    ReconstructTemp_DepIndDataCount = ReconstructTemp_DepIndDataCount + 1
                    ReconstructTemp_DepIndData(0, j) = ReconstructTemp_ReconstructionStationData(0, i)
                    ReconstructTemp_DepIndData(1, j) = ReconstructTemp_ReconstructionStationData(1, i)
                    ReconstructTemp_DepIndData(2, j) = ReconstructTemp_ReconstructionStationData(2, i)
                    ReconstructTemp_DepIndData(3, j) = ReconstructTemp_ReconstructionStationData(3, i)
                    ReconstructTemp_DepIndData(4, j) = ReconstructTemp_ReconstructionStationData(3 + ReconstructTemp_StationNumber, i)
                    j = j + 1
                End If
                If CheckDate = ReconstructTemp_DepEndDate Then
                    DepStore = 0
                End If
                If CheckDate = ReconstructTemp_IndEndDate Then
                    IndStore = 0
                End If
            Next i
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Next Job Number
            Select Case ReconstructTemp_RemoveOutliers
                Case 0
                    ReconstructTemp_JobNumber = 5
                Case 1
                    ReconstructTemp_JobNumber = 3
            End Select
        End If
        If ReconstructTemp_JobNumber = 3 Then
            'Remove Outliers Sequence
            HalfPoint = Math.Round(ReconstructTemp_DepIndDataCount / 2)
            For i = 1 To 12
                'Separate Dependent and Independent Datasets Into Two Parts for Forward and Backward Regression
                'Remove Outliers from Set 1
                For j = 0 To HalfPoint - 1
                    If ReconstructTemp_DepIndData(1, j) = i And ReconstructTemp_DepIndData(4, j) <> -99 And ReconstructTemp_DepIndData(3, j) <> -99 Then
                        Select Case ReconstructTemp_DepIndData(2, j)
                            Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                'Set Model 1 Differences
                                ReDim Preserve DiffModel1(Model1Set1Count)
                                DiffModel1(Model1Set1Count) = ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j)
                                Model1Set1Count = Model1Set1Count + 1
                            Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                'Set Model 2 Differences
                                ReDim Preserve DiffModel2(Model2Set1Count)
                                DiffModel2(Model2Set1Count) = ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j)
                                Model2Set1Count = Model2Set1Count + 1
                            Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                'Set Model 3 Differences
                                ReDim Preserve DiffModel3(Model3Set1Count)
                                DiffModel3(Model3Set1Count) = ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j)
                                Model3Set1Count = Model3Set1Count + 1
                        End Select
                    End If
                Next j
                'Store 10th/90th Percentiles for Model 1
                If DiffModel1.Length <> 1 Then
                    'Sort Array
                    Array.Sort(DiffModel1)
                    'Find 10th Percentile
                    EvenOddCheck = Model1Set1Count Mod (1 / 0.1)
                    If EvenOddCheck = 0 Then
                        '10th Percentile At k
                        k = Model1Set1Count * 0.1
                        Model1Percentile10 = DiffModel1(k - 1)
                    Else
                        '10th Percentile In Between k and k+1
                        k = Math.Truncate(Model1Set1Count * 0.1)
                        Model1Percentile10 = DiffModel1(k)
                    End If
                    'Find 90th Percentile
                    EvenOddCheck = Model1Set1Count Mod (1 / 0.9)
                    If EvenOddCheck = 0 Then
                        '90th Percentile At k
                        k = Model1Set1Count * 0.9
                        Model1Percentile90 = DiffModel1(k - 1)
                    Else
                        '90th Perentile In Between k and k+1
                        k = Math.Truncate(Model1Set1Count * 0.9)
                        Model1Percentile90 = DiffModel1(k - 1)
                    End If
                Else
                    Model1Percentile10 = -99
                    Model1Percentile90 = -99
                End If
                'Store 10th/90th Percentiles for Model 2
                If DiffModel2.Length <> 1 Then
                    'Sort Array
                    Array.Sort(DiffModel2)
                    'Find 10th Percentile
                    EvenOddCheck = Model2Set1Count Mod (1 / 0.1)
                    If EvenOddCheck = 0 Then
                        '10th Percentile At k
                        k = Model2Set1Count * 0.1
                        Model2Percentile10 = DiffModel2(k - 1)
                    Else
                        '10th Percentile In Between k and k+1
                        k = Math.Truncate(Model2Set1Count * 0.1)
                        Model2Percentile10 = DiffModel2(k)
                    End If
                    'Find 90th Percentile
                    EvenOddCheck = Model2Set1Count Mod (1 / 0.9)
                    If EvenOddCheck = 0 Then
                        '90th Percentile At k
                        k = Model2Set1Count * 0.9
                        Model2Percentile90 = DiffModel2(k - 1)
                    Else
                        '90th Perentile In Between k and k+1
                        k = Math.Truncate(Model2Set1Count * 0.9)
                        Model2Percentile90 = DiffModel2(k - 1)
                    End If
                Else
                    Model2Percentile10 = -99
                    Model2Percentile90 = -99
                End If
                'Store 10th/90th Percentiles for Model 3
                If DiffModel3.Length <> 1 Then
                    'Sort Array
                    Array.Sort(DiffModel3)
                    'Find 10th Percentile
                    EvenOddCheck = Model3Set1Count Mod (1 / 0.1)
                    If EvenOddCheck = 0 Then
                        '10th Percentile At k
                        k = Model3Set1Count * 0.1
                        Model3Percentile10 = DiffModel3(k - 1)
                    Else
                        '10th Percentile In Between k and k+1
                        k = Math.Truncate(Model3Set1Count * 0.1)
                        Model3Percentile10 = DiffModel3(k)
                    End If
                    'Find 90th Percentile
                    EvenOddCheck = Model3Set1Count Mod (1 / 0.9)
                    If EvenOddCheck = 0 Then
                        '90th Percentile At k
                        k = Model3Set1Count * 0.9
                        Model3Percentile90 = DiffModel3(k - 1)
                    Else
                        '90th Perentile In Between k and k+1
                        k = Math.Truncate(Model3Set1Count * 0.9)
                        Model3Percentile90 = DiffModel3(k - 1)
                    End If
                Else
                    Model3Percentile10 = -99
                    Model3Percentile90 = -99
                End If
                'Reset Variables
                ReDim DiffModel1(0)
                ReDim DiffModel2(0)
                ReDim DiffModel3(0)
                Model1Set1Count = 0
                Model2Set1Count = 0
                Model3Set1Count = 0
                For j = 0 To HalfPoint - 1
                    If ReconstructTemp_DepIndData(1, j) = i And ReconstructTemp_DepIndData(4, j) <> -99 And ReconstructTemp_DepIndData(3, j) <> -99 Then
                        Select Case ReconstructTemp_DepIndData(2, j)
                            Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                'Check Model 1 Data
                                If Model1Percentile10 <> -99 And Model1Percentile90 <> -99 Then
                                    If ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) < Model1Percentile10 Or ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) > Model1Percentile90 Then
                                        'Perform Inter-Station Screening
                                        CheckDate = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                        For k = 0 To ReconstructTemp_ScreeningStationDataCount - 1
                                            If DateValue(ReconstructTemp_ScreeningStationData(1, k) & "/" & ReconstructTemp_ScreeningStationData(2, k) & "/" & ReconstructTemp_ScreeningStationData(0, k)) = CheckDate Then
                                                TotalTemp = 0
                                                TempCount = 0
                                                For l = 3 To ReconstructTemp_ScreeningStations + 2
                                                    If ReconstructTemp_ScreeningStationData(l, k) <> -99 Then
                                                        TotalTemp = ReconstructTemp_ScreeningStationData(l, k) + TotalTemp
                                                        TempCount = TempCount + 1
                                                    End If
                                                Next l
                                                NetworkAvg = TotalTemp / TempCount
                                                Exit For
                                            End If
                                        Next k
                                        DepvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(3, j))
                                        IndvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(4, j))
                                        Select Case DepvsNetwork
                                            Case Is > IndvsNetwork
                                                'Purge Dependent Station Temperature Value
                                                ReconstructTemp_DepIndData(3, j) = -99
                                            Case Is < IndvsNetwork
                                                'Purge Independent Station Temperature Value
                                                ReconstructTemp_DepIndData(4, j) = -99
                                        End Select
                                    End If
                                End If
                            Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                'Check Model 2 Data
                                If Model2Percentile10 <> -99 And Model2Percentile90 <> -99 Then
                                    If ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) < Model2Percentile10 Or ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) > Model2Percentile90 Then
                                        'Perform Inter-Station Screening
                                        CheckDate = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                        For k = 0 To ReconstructTemp_ScreeningStationDataCount - 1
                                            If DateValue(ReconstructTemp_ScreeningStationData(1, k) & "/" & ReconstructTemp_ScreeningStationData(2, k) & "/" & ReconstructTemp_ScreeningStationData(0, k)) = CheckDate Then
                                                TotalTemp = 0
                                                TempCount = 0
                                                For l = 3 To ReconstructTemp_ScreeningStations + 2
                                                    If ReconstructTemp_ScreeningStationData(l, k) <> -99 Then
                                                        TotalTemp = ReconstructTemp_ScreeningStationData(l, k) + TotalTemp
                                                        TempCount = TempCount + 1
                                                    End If
                                                Next l
                                                NetworkAvg = TotalTemp / TempCount
                                                Exit For
                                            End If
                                        Next k
                                        DepvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(3, j))
                                        IndvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(4, j))
                                        Select Case DepvsNetwork
                                            Case Is > IndvsNetwork
                                                'Purge Dependent Station Temperature Value
                                                ReconstructTemp_DepIndData(3, j) = -99
                                            Case Is < IndvsNetwork
                                                'Purge Independent Station Temperature Value
                                                ReconstructTemp_DepIndData(4, j) = -99
                                        End Select
                                    End If
                                End If
                            Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                'Check Model 3 Data
                                If Model3Percentile10 <> -99 And Model3Percentile90 <> -99 Then
                                    If ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) < Model3Percentile10 Or ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) > Model3Percentile90 Then
                                        'Perform Inter-Station Screening
                                        CheckDate = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                        For k = 0 To ReconstructTemp_ScreeningStationDataCount - 1
                                            If DateValue(ReconstructTemp_ScreeningStationData(1, k) & "/" & ReconstructTemp_ScreeningStationData(2, k) & "/" & ReconstructTemp_ScreeningStationData(0, k)) = CheckDate Then
                                                TotalTemp = 0
                                                TempCount = 0
                                                For l = 3 To ReconstructTemp_ScreeningStations + 2
                                                    If ReconstructTemp_ScreeningStationData(l, k) <> -99 Then
                                                        TotalTemp = ReconstructTemp_ScreeningStationData(l, k) + TotalTemp
                                                        TempCount = TempCount + 1
                                                    End If
                                                Next l
                                                NetworkAvg = TotalTemp / TempCount
                                                Exit For
                                            End If
                                        Next k
                                        DepvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(3, j))
                                        IndvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(4, j))
                                        Select Case DepvsNetwork
                                            Case Is > IndvsNetwork
                                                'Purge Dependent Station Temperature Value
                                                ReconstructTemp_DepIndData(3, j) = -99
                                            Case Is < IndvsNetwork
                                                'Purge Independent Station Temperature Value
                                                ReconstructTemp_DepIndData(4, j) = -99
                                        End Select
                                    End If
                                End If
                        End Select
                    End If
                Next j
                'Remove Outliers from Set 2
                For j = HalfPoint To ReconstructTemp_DepIndDataCount - 1
                    If ReconstructTemp_DepIndData(1, j) = i And ReconstructTemp_DepIndData(4, j) <> -99 And ReconstructTemp_DepIndData(3, j) <> -99 Then
                        Select Case ReconstructTemp_DepIndData(2, j)
                            Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                'Set Model 1 Differences
                                ReDim Preserve DiffModel1(Model1Set2Count)
                                DiffModel1(Model1Set2Count) = ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j)
                                Model1Set2Count = Model1Set2Count + 1
                            Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                'Set Model 2 Differences
                                ReDim Preserve DiffModel2(Model2Set2Count)
                                DiffModel2(Model2Set2Count) = ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j)
                                Model2Set2Count = Model2Set2Count + 1
                            Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                'Set Model 3 Differences
                                ReDim Preserve DiffModel3(Model3Set2Count)
                                DiffModel3(Model3Set2Count) = ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j)
                                Model3Set2Count = Model3Set2Count + 1
                        End Select
                    End If
                Next j
                'Store 10th/90th Percentiles for Model 1
                If DiffModel1.Length <> 1 Then
                    'Sort Array
                    Array.Sort(DiffModel1)
                    'Find 10th Percentile
                    EvenOddCheck = Model1Set2Count Mod (1 / 0.1)
                    If EvenOddCheck = 0 Then
                        '10th Percentile At k
                        k = Model1Set2Count * 0.1
                        Model1Percentile10 = DiffModel1(k - 1)
                    Else
                        '10th Percentile In Between k and k+1
                        k = Math.Truncate(Model1Set2Count * 0.1)
                        Model1Percentile10 = DiffModel1(k)
                    End If
                    'Find 90th Percentile
                    EvenOddCheck = Model1Set2Count Mod (1 / 0.9)
                    If EvenOddCheck = 0 Then
                        '90th Percentile At k
                        k = Model1Set2Count * 0.9
                        Model1Percentile90 = DiffModel1(k - 1)
                    Else
                        '90th Perentile In Between k and k+1
                        k = Math.Truncate(Model1Set2Count * 0.9)
                        Model1Percentile90 = DiffModel1(k - 1)
                    End If
                Else
                    Model1Percentile10 = -99
                    Model1Percentile90 = -99
                End If
                'Store 10th/90th Percentiles for Model 2
                If DiffModel2.Length <> 1 Then
                    'Sort Array
                    Array.Sort(DiffModel2)
                    'Find 10th Percentile
                    EvenOddCheck = Model2Set2Count Mod (1 / 0.1)
                    If EvenOddCheck = 0 Then
                        '10th Percentile At k
                        k = Model2Set2Count * 0.1
                        Model2Percentile10 = DiffModel2(k - 1)
                    Else
                        '10th Percentile In Between k and k+1
                        k = Math.Truncate(Model2Set2Count * 0.1)
                        Model2Percentile10 = DiffModel2(k)
                    End If
                    'Find 90th Percentile
                    EvenOddCheck = Model2Set2Count Mod (1 / 0.9)
                    If EvenOddCheck = 0 Then
                        '90th Percentile At k
                        k = Model2Set2Count * 0.9
                        Model2Percentile90 = DiffModel2(k - 1)
                    Else
                        '90th Perentile In Between k and k+1
                        k = Math.Truncate(Model2Set2Count * 0.9)
                        Model2Percentile90 = DiffModel2(k - 1)
                    End If
                Else
                    Model2Percentile10 = -99
                    Model2Percentile90 = -99
                End If
                If DiffModel3.Length <> 1 Then
                    'Store 10th/90th Percentiles for Model 3
                    'Sort Array
                    Array.Sort(DiffModel3)
                    'Find 10th Percentile
                    EvenOddCheck = Model3Set2Count Mod (1 / 0.1)
                    If EvenOddCheck = 0 Then
                        '10th Percentile At k
                        k = Model3Set2Count * 0.1
                        Model3Percentile10 = DiffModel3(k - 1)
                    Else
                        '10th Percentile In Between k and k+1
                        k = Math.Truncate(Model3Set2Count * 0.1)
                        Model3Percentile10 = DiffModel3(k)
                    End If
                    'Find 90th Percentile
                    EvenOddCheck = Model3Set2Count Mod (1 / 0.9)
                    If EvenOddCheck = 0 Then
                        '90th Percentile At k
                        k = Model3Set2Count * 0.9
                        Model3Percentile90 = DiffModel3(k - 1)
                    Else
                        '90th Perentile In Between k and k+1
                        k = Math.Truncate(Model3Set2Count * 0.9)
                        Model3Percentile90 = DiffModel3(k - 1)
                    End If
                Else
                    Model3Percentile10 = -99
                    Model3Percentile90 = -99
                End If
                'Reset Variables
                ReDim DiffModel1(0)
                ReDim DiffModel2(0)
                ReDim DiffModel3(0)
                Model1Set2Count = 0
                Model2Set2Count = 0
                Model3Set2Count = 0
                For j = HalfPoint To ReconstructTemp_DepIndDataCount - 1
                    If ReconstructTemp_DepIndData(1, j) = i And ReconstructTemp_DepIndData(4, j) <> -99 And ReconstructTemp_DepIndData(3, j) <> -99 Then
                        Select Case ReconstructTemp_DepIndData(2, j)
                            Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                'Check Model 1 Data
                                If Model1Percentile10 <> -99 And Model1Percentile90 <> -99 Then
                                    If ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) < Model1Percentile10 Or ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) > Model1Percentile90 Then
                                        'Perform Inter-Station Screening
                                        CheckDate = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                        For k = 0 To ReconstructTemp_ScreeningStationDataCount - 1
                                            If DateValue(ReconstructTemp_ScreeningStationData(1, k) & "/" & ReconstructTemp_ScreeningStationData(2, k) & "/" & ReconstructTemp_ScreeningStationData(0, k)) = CheckDate Then
                                                TotalTemp = 0
                                                TempCount = 0
                                                For l = 3 To ReconstructTemp_ScreeningStations + 2
                                                    If ReconstructTemp_ScreeningStationData(l, k) <> -99 Then
                                                        TotalTemp = ReconstructTemp_ScreeningStationData(l, k) + TotalTemp
                                                        TempCount = TempCount + 1
                                                    End If
                                                Next l
                                                NetworkAvg = TotalTemp / TempCount
                                                Exit For
                                            End If
                                        Next k
                                        DepvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(3, j))
                                        IndvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(4, j))
                                        Select Case DepvsNetwork
                                            Case Is > IndvsNetwork
                                                'Purge Dependent Station Temperature Value
                                                ReconstructTemp_DepIndData(3, j) = -99
                                            Case Is < IndvsNetwork
                                                'Purge Independent Station Temperature Value
                                                ReconstructTemp_DepIndData(4, j) = -99
                                        End Select
                                    End If
                                End If
                            Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                'Check Model 2 Data
                                If Model2Percentile10 <> -99 And Model2Percentile90 <> -99 Then
                                    If ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) < Model2Percentile10 Or ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) > Model2Percentile90 Then
                                        'Perform Inter-Station Screening
                                        CheckDate = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                        For k = 0 To ReconstructTemp_ScreeningStationDataCount - 1
                                            If DateValue(ReconstructTemp_ScreeningStationData(1, k) & "/" & ReconstructTemp_ScreeningStationData(2, k) & "/" & ReconstructTemp_ScreeningStationData(0, k)) = CheckDate Then
                                                TotalTemp = 0
                                                TempCount = 0
                                                For l = 3 To ReconstructTemp_ScreeningStations + 2
                                                    If ReconstructTemp_ScreeningStationData(l, k) <> -99 Then
                                                        TotalTemp = ReconstructTemp_ScreeningStationData(l, k) + TotalTemp
                                                        TempCount = TempCount + 1
                                                    End If
                                                Next l
                                                NetworkAvg = TotalTemp / TempCount
                                                Exit For
                                            End If
                                        Next k
                                        DepvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(3, j))
                                        IndvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(4, j))
                                        Select Case DepvsNetwork
                                            Case Is > IndvsNetwork
                                                'Purge Dependent Station Temperature Value
                                                ReconstructTemp_DepIndData(3, j) = -99
                                            Case Is < IndvsNetwork
                                                'Purge Independent Station Temperature Value
                                                ReconstructTemp_DepIndData(4, j) = -99
                                        End Select
                                    End If
                                End If
                            Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                'Check Model 3 Data
                                If Model3Percentile10 <> -99 And Model3Percentile90 <> -99 Then
                                    If ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) < Model3Percentile10 Or ReconstructTemp_DepIndData(4, j) - ReconstructTemp_DepIndData(3, j) > Model3Percentile90 Then
                                        'Perform Inter-Station Screening
                                        CheckDate = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                        For k = 0 To ReconstructTemp_ScreeningStationDataCount - 1
                                            If DateValue(ReconstructTemp_ScreeningStationData(1, k) & "/" & ReconstructTemp_ScreeningStationData(2, k) & "/" & ReconstructTemp_ScreeningStationData(0, k)) = CheckDate Then
                                                TotalTemp = 0
                                                TempCount = 0
                                                For l = 3 To ReconstructTemp_ScreeningStations + 2
                                                    If ReconstructTemp_ScreeningStationData(l, k) <> -99 Then
                                                        TotalTemp = ReconstructTemp_ScreeningStationData(l, k) + TotalTemp
                                                        TempCount = TempCount + 1
                                                    End If
                                                Next l
                                                NetworkAvg = TotalTemp / TempCount
                                                Exit For
                                            End If
                                        Next k
                                        DepvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(3, j))
                                        IndvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(4, j))
                                        Select Case DepvsNetwork
                                            Case Is > IndvsNetwork
                                                'Purge Dependent Station Temperature Value
                                                ReconstructTemp_DepIndData(3, j) = -99
                                            Case Is < IndvsNetwork
                                                'Purge Independent Station Temperature Value
                                                ReconstructTemp_DepIndData(4, j) = -99
                                        End Select
                                    End If
                                End If
                        End Select
                    End If
                Next j
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            Next i
            'Next Job Number
            ReconstructTemp_JobNumber = 5
        End If
        If ReconstructTemp_JobNumber = 4 Then
            'Remove Selected Residuals Sequence
            For i = 0 To ReconstructTemp_RemoveResidualDatesCount - 1
                'Perform Inter-Station Screening of Each Date Selected
                For k = 0 To ReconstructTemp_ScreeningStationDataCount - 1
                    If DateValue(ReconstructTemp_ScreeningStationData(1, k) & "/" & ReconstructTemp_ScreeningStationData(2, k) & "/" & ReconstructTemp_ScreeningStationData(0, k)) = ReconstructTemp_RemoveResidualDates(i) Then
                        TotalTemp = 0
                        TempCount = 0
                        For l = 3 To ReconstructTemp_ScreeningStations + 2
                            If ReconstructTemp_ScreeningStationData(l, k) <> -99 Then
                                TotalTemp = ReconstructTemp_ScreeningStationData(l, k) + TotalTemp
                                TempCount = TempCount + 1
                            End If
                        Next l
                        NetworkAvg = TotalTemp / TempCount
                        Exit For
                    End If
                Next k
                For j = 0 To ReconstructTemp_DepIndDataCount - 1
                    If DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j)) = ReconstructTemp_RemoveResidualDates(i) Then
                        DepvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(3, j))
                        IndvsNetwork = Math.Abs(NetworkAvg - ReconstructTemp_DepIndData(4, j))
                        Select Case DepvsNetwork
                            Case Is > IndvsNetwork
                                'Purge Dependent Station Temperature Value
                                ReconstructTemp_DepIndData(3, j) = -99
                            Case Is < IndvsNetwork
                                'Purge Independent Station Temperature Value
                                ReconstructTemp_DepIndData(4, j) = -99
                        End Select
                        Exit For
                    End If
                Next j
            Next i
            'Update Progress Bar
            FileProgress = FileProgress + 1
            Percent = (FileProgress / FileComplete) * 100
            BackgroundRun.ReportProgress(Percent)
            'Next Job Number
            ReconstructTemp_JobNumber = 5
        End If
        If ReconstructTemp_JobNumber = 5 Then
            'Foward and Backward Calibration and Verification Sequence
            HalfPoint = Math.Round(ReconstructTemp_DepIndDataCount / 2)
            FileOpen(4, ReconstructTemp_ResidualsSet1, OpenMode.Output)
            FileOpen(5, ReconstructTemp_ResidualsSet2, OpenMode.Output)
            For i = 1 To 12
                'Separate Dependent and Independent Datasets Into Two Parts for Forward and Backward Regression
                'Set 1
                For j = 0 To HalfPoint - 1
                    If ReconstructTemp_DepIndData(1, j) = i And ReconstructTemp_DepIndData(4, j) <> -99 And ReconstructTemp_DepIndData(3, j) <> -99 Then
                        Select Case ReconstructTemp_DepIndData(2, j)
                            Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                'Set Model 1
                                ReDim Preserve IndModel1Set1(Model1Set1Count)
                                ReDim Preserve DepModel1Set1(Model1Set1Count)
                                ReDim Preserve Model1Set1Dates(Model1Set1Count)
                                IndModel1Set1(Model1Set1Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel1Set1(Model1Set1Count) = ReconstructTemp_DepIndData(3, j)
                                Model1Set1Dates(Model1Set1Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model1Set1Count = Model1Set1Count + 1
                            Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                'Set Model 2
                                ReDim Preserve IndModel2Set1(Model2Set1Count)
                                ReDim Preserve DepModel2Set1(Model2Set1Count)
                                ReDim Preserve Model2Set1Dates(Model2Set1Count)
                                IndModel2Set1(Model2Set1Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel2Set1(Model2Set1Count) = ReconstructTemp_DepIndData(3, j)
                                Model2Set1Dates(Model2Set1Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model2Set1Count = Model2Set1Count + 1
                            Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                'Set Model 3
                                ReDim Preserve IndModel3Set1(Model3Set1Count)
                                ReDim Preserve DepModel3Set1(Model3Set1Count)
                                ReDim Preserve Model3Set1Dates(Model3Set1Count)
                                IndModel3Set1(Model3Set1Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel3Set1(Model3Set1Count) = ReconstructTemp_DepIndData(3, j)
                                Model3Set1Dates(Model3Set1Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model3Set1Count = Model3Set1Count + 1
                        End Select
                    End If
                Next j
                'Set 2
                For j = HalfPoint To ReconstructTemp_DepIndDataCount - 1
                    If ReconstructTemp_DepIndData(1, j) = i And ReconstructTemp_DepIndData(4, j) <> -99 And ReconstructTemp_DepIndData(3, j) <> -99 Then
                        Select Case ReconstructTemp_DepIndData(2, j)
                            Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                'Set Model 1
                                ReDim Preserve IndModel1Set2(Model1Set2Count)
                                ReDim Preserve DepModel1Set2(Model1Set2Count)
                                ReDim Preserve Model1Set2Dates(Model1Set2Count)
                                IndModel1Set2(Model1Set2Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel1Set2(Model1Set2Count) = ReconstructTemp_DepIndData(3, j)
                                Model1Set2Dates(Model1Set2Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model1Set2Count = Model1Set2Count + 1
                            Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                'Set Model 2
                                ReDim Preserve IndModel2Set2(Model2Set2Count)
                                ReDim Preserve DepModel2Set2(Model2Set2Count)
                                ReDim Preserve Model2Set2Dates(Model2Set2Count)
                                IndModel2Set2(Model2Set2Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel2Set2(Model2Set2Count) = ReconstructTemp_DepIndData(3, j)
                                Model2Set2Dates(Model2Set2Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model2Set2Count = Model2Set2Count + 1
                            Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                'Set Model 3
                                ReDim Preserve IndModel3Set2(Model3Set2Count)
                                ReDim Preserve DepModel3Set2(Model3Set2Count)
                                ReDim Preserve Model3Set2Dates(Model3Set2Count)
                                IndModel3Set2(Model3Set2Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel3Set2(Model3Set2Count) = ReconstructTemp_DepIndData(3, j)
                                Model3Set2Dates(Model3Set2Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model3Set2Count = Model3Set2Count + 1
                        End Select
                    End If
                Next j
                'Set Current Month to PrintMonth for Output
                Select Case i
                    Case 1
                        MonthPrint = "Jan"
                    Case 2
                        MonthPrint = "Feb"
                    Case 3
                        MonthPrint = "Mar"
                    Case 4
                        MonthPrint = "Apr"
                    Case 5
                        MonthPrint = "May"
                    Case 6
                        MonthPrint = "Jun"
                    Case 7
                        MonthPrint = "Jul"
                    Case 8
                        MonthPrint = "Aug"
                    Case 9
                        MonthPrint = "Sep"
                    Case 10
                        MonthPrint = "Oct"
                    Case 11
                        MonthPrint = "Nov"
                    Case 12
                        MonthPrint = "Dec"
                End Select
                'Validate that Station Data are Long Enough for Forward/Backward Calibration and Verification
                If IndModel1Set1.Length = 1 Or IndModel2Set1.Length = 1 Or IndModel3Set1.Length = 1 Or IndModel1Set2.Length = 1 Or IndModel2Set2.Length = 1 Or IndModel3Set2.Length = 1 Or DepModel1Set1.Length = 1 Or DepModel2Set1.Length = 1 Or DepModel3Set1.Length = 1 Or DepModel1Set2.Length = 1 Or DepModel2Set2.Length = 1 Or DepModel3Set2.Length = 1 Then
                    MsgBox("Insufficient temperature data to perform forward and backward calibration and verification for the independent, " & ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber) & ", station.  HOB Tools will attempt to derive the final models.", MsgBoxStyle.Exclamation Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
                    ReconstructTemp_RegressionResultsForwardCal = "Forward Calibration:  NA -- Not enough station data to perform forward and backward calibration and verification"
                    ReconstructTemp_RegressionResultsForwardVer = "Forward Verification:  NA"
                    ReconstructTemp_RegressionResultsBackwardCal = "Backward Calibration:  NA"
                    ReconstructTemp_RegressionResultsBackwardVer = "Backward Verification:  NA"
                    'Reset Progress Bar
                    FileProgress = 0
                    FileComplete = 12
                    'Set New Job Number
                    ReconstructTemp_JobNumber = 6
                    Exit For
                End If
                'Forward Calibration
                'Derive Model 1
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model1Set1Count - 1
                    SumXY = SumXY + IndModel1Set1(j) * DepModel1Set1(j)
                    SumX = SumX + IndModel1Set1(j)
                    SumY = SumY + DepModel1Set1(j)
                    SumXSq = SumXSq + IndModel1Set1(j) ^ 2
                    SumYSq = SumYSq + DepModel1Set1(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model1Set1Count
                SXX = SumXSq - SumX ^ 2 / Model1Set1Count
                Model1B1 = SXY / SXX
                Model1B0 = (SumY - Model1B1 * SumX) / Model1Set1Count
                'Compute Model 1 R-Squared
                SSE = SumYSq - Model1B0 * SumY - Model1B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model1Set1Count
                Model1RSq = 1 - SSE / SST
                'Compute Model 1 Durbin-Watson Statistic
                For j = 1 To Model1Set1Count - 1
                    ActualTemp = DepModel1Set1(j)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel1Set1(j - 1)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set1(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model1DurWat = SumError / SSE
                'Compute Model 1 Standard Error
                Model1StdErr = (SSE / (Model1Set1Count - 2)) ^ 0.5
                'Derive Model 2
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model2Set1Count - 1
                    SumXY = SumXY + IndModel2Set1(j) * DepModel2Set1(j)
                    SumX = SumX + IndModel2Set1(j)
                    SumY = SumY + DepModel2Set1(j)
                    SumXSq = SumXSq + IndModel2Set1(j) ^ 2
                    SumYSq = SumYSq + DepModel2Set1(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model2Set1Count
                SXX = SumXSq - SumX ^ 2 / Model2Set1Count
                Model2B1 = SXY / SXX
                Model2B0 = (SumY - Model2B1 * SumX) / Model2Set1Count
                'Compute Model 2 R-Squared and Correlation
                SSE = SumYSq - Model2B0 * SumY - Model2B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model2Set1Count
                Model2RSq = 1 - SSE / SST
                'Compute Model 2 Durbin-Watson Statistic
                For j = 1 To Model2Set1Count - 1
                    ActualTemp = DepModel2Set1(j)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel2Set1(j - 1)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set1(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model2DurWat = SumError / SSE
                'Compute Model 2 Standard Error
                Model2StdErr = (SSE / (Model2Set1Count - 2)) ^ 0.5
                'Derive Model 3
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model3Set1Count - 1
                    SumXY = SumXY + IndModel3Set1(j) * DepModel3Set1(j)
                    SumX = SumX + IndModel3Set1(j)
                    SumY = SumY + DepModel3Set1(j)
                    SumXSq = SumXSq + IndModel3Set1(j) ^ 2
                    SumYSq = SumYSq + DepModel3Set1(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model3Set1Count
                SXX = SumXSq - SumX ^ 2 / Model3Set1Count
                Model3B1 = SXY / SXX
                Model3B0 = (SumY - Model3B1 * SumX) / Model3Set1Count
                'Compute Model 3 R-Squared and Correlation
                SSE = SumYSq - Model3B0 * SumY - Model3B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model3Set1Count
                Model3RSq = 1 - SSE / SST
                'Compute Model 3 Durbin-Watson Statistic
                For j = 1 To Model3Set1Count - 1
                    ActualTemp = DepModel3Set1(j)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel3Set1(j - 1)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set1(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model3DurWat = SumError / SSE
                'Compute Model 3 Standard Error
                Model3StdErr = (SSE / (Model3Set1Count - 2)) ^ 0.5
                'Calculate Residuals and Output to Temporary File
                'Model 1 Set 1 Residuals
                For j = 0 To Model1Set1Count - 1
                    DayValue = Model1Set1Dates(j)
                    IndValue = IndModel1Set1(j)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set1(j)
                    ActualTemp = DepModel1Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(4, MonthPrint & vbTab & "1" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Model 2 Set 1 Residuals
                For j = 0 To Model2Set1Count - 1
                    DayValue = Model2Set1Dates(j)
                    IndValue = IndModel2Set1(j)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set1(j)
                    ActualTemp = DepModel2Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(4, MonthPrint & vbTab & "2" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Model 3 Set 1 Residuals
                For j = 0 To Model3Set1Count - 1
                    DayValue = Model3Set1Dates(j)
                    IndValue = IndModel3Set1(j)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set1(j)
                    ActualTemp = DepModel3Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(4, MonthPrint & vbTab & "3" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Set Regression Results for Display
                If i = 1 Then
                    ReconstructTemp_RegressionResultsForwardCal = "Forward Calibration:"
                    ReconstructTemp_RegressionResultsForwardCal = ReconstructTemp_RegressionResultsForwardCal & vbCrLf & "Month" & vbTab & "Model" & vbTab & "N" & vbTab & "B0" & vbTab & "B1" & vbTab & "StdErr" & vbTab & "R-Sq" & vbTab & "DurWat"
                End If
                ReconstructTemp_RegressionResultsForwardCal = ReconstructTemp_RegressionResultsForwardCal & vbCrLf & MonthPrint & vbTab & "1" & vbTab & Model1Set1Count & vbTab & Math.Round(Model1B0, 3) & vbTab & Math.Round(Model1B1, 3) & vbTab & Math.Round(Model1StdErr, 3) & vbTab & Math.Round(Model1RSq, 3) & vbTab & Math.Round(Model1DurWat, 3)
                ReconstructTemp_RegressionResultsForwardCal = ReconstructTemp_RegressionResultsForwardCal & vbCrLf & MonthPrint & vbTab & "2" & vbTab & Model2Set1Count & vbTab & Math.Round(Model2B0, 3) & vbTab & Math.Round(Model2B1, 3) & vbTab & Math.Round(Model2StdErr, 3) & vbTab & Math.Round(Model2RSq, 3) & vbTab & Math.Round(Model2DurWat, 3)
                ReconstructTemp_RegressionResultsForwardCal = ReconstructTemp_RegressionResultsForwardCal & vbCrLf & MonthPrint & vbTab & "3" & vbTab & Model3Set1Count & vbTab & Math.Round(Model3B0, 3) & vbTab & Math.Round(Model3B1, 3) & vbTab & Math.Round(Model3StdErr, 3) & vbTab & Math.Round(Model3RSq, 3) & vbTab & Math.Round(Model3DurWat, 3)
                'Forward Verification
                'Model 1 Verification
                SumX = 0
                SumY = 0
                SSR = 0
                SXX = 0
                SYY = 0
                SXY = 0
                ReDim VerPredicted(Model1Set2Count - 1)
                ReDim VerActual(Model1Set2Count - 1)
                For j = 0 To Model1Set2Count - 1
                    IndValue = IndModel1Set2(j)
                    VerPredicted(j) = Model1B0 + Model1B1 * IndModel1Set2(j)
                    VerActual(j) = DepModel1Set2(j)
                    SumX = SumX + VerActual(j)
                    SumY = SumY + VerPredicted(j)
                    SSR = SSR + (VerActual(j) - VerPredicted(j)) ^ 2
                Next j
                For j = 0 To Model1Set2Count - 1
                    SXX = SXX + (VerActual(j) - (SumX / Model1Set2Count)) ^ 2
                    SYY = SYY + (VerPredicted(j) - (SumY / Model1Set2Count)) ^ 2
                    SXY = SXY + (VerActual(j) - (SumX / Model1Set2Count)) * (VerPredicted(j) - (SumY / Model1Set2Count))
                Next j
                'Calculate Coefficent of Efficiency (CE)
                CE = Math.Round(1 - (SSR / SXX), 3)
                'Calculate Correlation Between Actual and Predicted Data
                Verr = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
                Verr = Math.Round(Verr, 3)
                'Set Verification Results for Display
                If i = 1 Then
                    ReconstructTemp_RegressionResultsForwardVer = "Forward Verification:"
                    ReconstructTemp_RegressionResultsForwardVer = ReconstructTemp_RegressionResultsForwardVer & vbCrLf & "Month" & vbTab & "Model" & vbTab & "N" & vbTab & "CE" & vbTab & "r"
                End If
                ReconstructTemp_RegressionResultsForwardVer = ReconstructTemp_RegressionResultsForwardVer & vbCrLf & MonthPrint & vbTab & "1" & vbTab & Model1Set2Count & vbTab & CE & vbTab & Verr
                'Model 2 Verification
                SumX = 0
                SumY = 0
                SSR = 0
                SXX = 0
                SYY = 0
                SXY = 0
                ReDim VerPredicted(Model2Set2Count - 1)
                ReDim VerActual(Model2Set2Count - 1)
                For j = 0 To Model2Set2Count - 1
                    IndValue = IndModel2Set2(j)
                    VerPredicted(j) = Model2B0 + Model2B1 * IndModel2Set2(j)
                    VerActual(j) = DepModel2Set2(j)
                    SumX = SumX + VerActual(j)
                    SumY = SumY + VerPredicted(j)
                    SSR = SSR + (VerActual(j) - VerPredicted(j)) ^ 2
                Next j
                For j = 0 To Model2Set2Count - 1
                    SXX = SXX + (VerActual(j) - (SumX / Model2Set2Count)) ^ 2
                    SYY = SYY + (VerPredicted(j) - (SumY / Model2Set2Count)) ^ 2
                    SXY = SXY + (VerActual(j) - (SumX / Model2Set2Count)) * (VerPredicted(j) - (SumY / Model2Set2Count))
                Next j
                'Calculate Coefficent of Efficiency (CE)
                CE = Math.Round(1 - (SSR / SXX), 3)
                'Calculate Correlation Between Actual and Predicted Data
                Verr = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
                Verr = Math.Round(Verr, 3)
                'Set Verification Results for Display
                ReconstructTemp_RegressionResultsForwardVer = ReconstructTemp_RegressionResultsForwardVer & vbCrLf & MonthPrint & vbTab & "2" & vbTab & Model2Set2Count & vbTab & CE & vbTab & Verr
                'Model 3 Verification
                SumX = 0
                SumY = 0
                SSR = 0
                SXX = 0
                SYY = 0
                SXY = 0
                ReDim VerPredicted(Model3Set2Count - 1)
                ReDim VerActual(Model3Set2Count - 1)
                For j = 0 To Model3Set2Count - 1
                    IndValue = IndModel3Set2(j)
                    VerPredicted(j) = Model3B0 + Model3B1 * IndModel3Set2(j)
                    VerActual(j) = DepModel3Set2(j)
                    SumX = SumX + VerActual(j)
                    SumY = SumY + VerPredicted(j)
                    SSR = SSR + (VerActual(j) - VerPredicted(j)) ^ 2
                Next j
                For j = 0 To Model3Set2Count - 1
                    SXX = SXX + (VerActual(j) - (SumX / Model3Set2Count)) ^ 2
                    SYY = SYY + (VerPredicted(j) - (SumY / Model3Set2Count)) ^ 2
                    SXY = SXY + (VerActual(j) - (SumX / Model3Set2Count)) * (VerPredicted(j) - (SumY / Model3Set2Count))
                Next j
                'Calculate Coefficent of Efficiency (CE)
                CE = Math.Round(1 - (SSR / SXX), 3)
                'Calculate Correlation Between Actual and Predicted Data
                Verr = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
                Verr = Math.Round(Verr, 3)
                'Set Verification Results for Display
                ReconstructTemp_RegressionResultsForwardVer = ReconstructTemp_RegressionResultsForwardVer & vbCrLf & MonthPrint & vbTab & "3" & vbTab & Model3Set2Count & vbTab & CE & vbTab & Verr
                'Backward Calibration
                'Derive Model 1
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model1Set2Count - 1
                    SumXY = SumXY + IndModel1Set2(j) * DepModel1Set2(j)
                    SumX = SumX + IndModel1Set2(j)
                    SumY = SumY + DepModel1Set2(j)
                    SumXSq = SumXSq + IndModel1Set2(j) ^ 2
                    SumYSq = SumYSq + DepModel1Set2(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model1Set2Count
                SXX = SumXSq - SumX ^ 2 / Model1Set2Count
                Model1B1 = SXY / SXX
                Model1B0 = (SumY - Model1B1 * SumX) / Model1Set2Count
                'Compute Model 1 R-Squared and Correlation
                SSE = SumYSq - Model1B0 * SumY - Model1B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model1Set2Count
                Model1RSq = 1 - SSE / SST
                'Compute Model 1 Durbin-Watson Statistic
                For j = 1 To Model1Set2Count - 1
                    ActualTemp = DepModel1Set2(j)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set2(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel1Set2(j - 1)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set2(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model1DurWat = SumError / SSE
                'Compute Model 1 Standard Error
                Model1StdErr = (SSE / (Model1Set2Count - 2)) ^ 0.5
                'Derive Model 2
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model2Set2Count - 1
                    SumXY = SumXY + IndModel2Set2(j) * DepModel2Set2(j)
                    SumX = SumX + IndModel2Set2(j)
                    SumY = SumY + DepModel2Set2(j)
                    SumXSq = SumXSq + IndModel2Set2(j) ^ 2
                    SumYSq = SumYSq + DepModel2Set2(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model2Set2Count
                SXX = SumXSq - SumX ^ 2 / Model2Set2Count
                Model2B1 = SXY / SXX
                Model2B0 = (SumY - Model2B1 * SumX) / Model2Set2Count
                'Compute Model 2 R-Squared and Correlation
                SSE = SumYSq - Model2B0 * SumY - Model2B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model2Set2Count
                Model2RSq = 1 - SSE / SST
                'Compute Model 2 Durbin-Watson Statistic
                For j = 1 To Model2Set2Count - 1
                    ActualTemp = DepModel2Set2(j)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set2(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel2Set2(j - 1)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set2(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model2DurWat = SumError / SSE
                'Compute Model 2 Standard Error
                Model2StdErr = (SSE / (Model2Set2Count - 2)) ^ 0.5
                'Derive Model 3
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model3Set2Count - 1
                    SumXY = SumXY + IndModel3Set2(j) * DepModel3Set2(j)
                    SumX = SumX + IndModel3Set2(j)
                    SumY = SumY + DepModel3Set2(j)
                    SumXSq = SumXSq + IndModel3Set2(j) ^ 2
                    SumYSq = SumYSq + DepModel3Set2(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model3Set2Count
                SXX = SumXSq - SumX ^ 2 / Model3Set2Count
                Model3B1 = SXY / SXX
                Model3B0 = (SumY - Model3B1 * SumX) / Model3Set2Count
                'Compute Model 3 R-Squared and Correlation
                SSE = SumYSq - Model3B0 * SumY - Model3B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model3Set2Count
                Model3RSq = 1 - SSE / SST
                'Compute Model 3 Durbin-Watson Statistic
                For j = 1 To Model3Set2Count - 1
                    ActualTemp = DepModel3Set2(j)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set2(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel3Set2(j - 1)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set2(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model3DurWat = SumError / SSE
                'Compute Model 3 Standard Error
                Model3StdErr = (SSE / (Model3Set2Count - 2)) ^ 0.5
                'Calculate Residuals and Output to Temporary File
                'Model 1 Set 2 Residuals
                For j = 0 To Model1Set2Count - 1
                    DayValue = Model1Set2Dates(j)
                    IndValue = IndModel1Set2(j)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set2(j)
                    ActualTemp = DepModel1Set2(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(5, MonthPrint & vbTab & "1" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Model 2 Set 2 Residuals
                For j = 0 To Model2Set2Count - 1
                    DayValue = Model2Set2Dates(j)
                    IndValue = IndModel2Set2(j)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set2(j)
                    ActualTemp = DepModel2Set2(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(5, MonthPrint & vbTab & "2" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Model 3 Set 2 Residuals
                For j = 0 To Model3Set2Count - 1
                    DayValue = Model3Set2Dates(j)
                    IndValue = IndModel3Set2(j)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set2(j)
                    ActualTemp = DepModel3Set2(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(5, MonthPrint & vbTab & "3" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Set Regression Results for Display
                If i = 1 Then
                    ReconstructTemp_RegressionResultsBackwardCal = "Backward Calibration:"
                    ReconstructTemp_RegressionResultsBackwardCal = ReconstructTemp_RegressionResultsBackwardCal & vbCrLf & "Month" & vbTab & "Model" & vbTab & "N" & vbTab & "B0" & vbTab & "B1" & vbTab & "StdErr" & vbTab & "R-Sq" & vbTab & "DurWat"
                End If
                ReconstructTemp_RegressionResultsBackwardCal = ReconstructTemp_RegressionResultsBackwardCal & vbCrLf & MonthPrint & vbTab & "1" & vbTab & Model1Set2Count & vbTab & Math.Round(Model1B0, 3) & vbTab & Math.Round(Model1B1, 3) & vbTab & Math.Round(Model1StdErr, 3) & vbTab & Math.Round(Model1RSq, 3) & vbTab & Math.Round(Model1DurWat, 3)
                ReconstructTemp_RegressionResultsBackwardCal = ReconstructTemp_RegressionResultsBackwardCal & vbCrLf & MonthPrint & vbTab & "2" & vbTab & Model2Set2Count & vbTab & Math.Round(Model2B0, 3) & vbTab & Math.Round(Model2B1, 3) & vbTab & Math.Round(Model2StdErr, 3) & vbTab & Math.Round(Model2RSq, 3) & vbTab & Math.Round(Model2DurWat, 3)
                ReconstructTemp_RegressionResultsBackwardCal = ReconstructTemp_RegressionResultsBackwardCal & vbCrLf & MonthPrint & vbTab & "3" & vbTab & Model3Set2Count & vbTab & Math.Round(Model3B0, 3) & vbTab & Math.Round(Model3B1, 3) & vbTab & Math.Round(Model3StdErr, 3) & vbTab & Math.Round(Model3RSq, 3) & vbTab & Math.Round(Model3DurWat, 3)
                'Backward Verification
                'Model 1 Verification
                SumX = 0
                SumY = 0
                SSR = 0
                SXX = 0
                SYY = 0
                SXY = 0
                ReDim VerPredicted(Model1Set1Count - 1)
                ReDim VerActual(Model1Set1Count - 1)
                For j = 0 To Model1Set1Count - 1
                    IndValue = IndModel1Set1(j)
                    VerPredicted(j) = Model1B0 + Model1B1 * IndModel1Set1(j)
                    VerActual(j) = DepModel1Set1(j)
                    SumX = SumX + VerActual(j)
                    SumY = SumY + VerPredicted(j)
                    SSR = SSR + (VerActual(j) - VerPredicted(j)) ^ 2
                Next j
                For j = 0 To Model1Set1Count - 1
                    SXX = SXX + (VerActual(j) - (SumX / Model1Set1Count)) ^ 2
                    SYY = SYY + (VerPredicted(j) - (SumY / Model1Set1Count)) ^ 2
                    SXY = SXY + (VerActual(j) - (SumX / Model1Set1Count)) * (VerPredicted(j) - (SumY / Model1Set1Count))
                Next j
                'Calculate Coefficent of Efficiency (CE)
                CE = Math.Round(1 - (SSR / SXX), 3)
                'Calculate Correlation Between Actual and Predicted Data
                Verr = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
                Verr = Math.Round(Verr, 3)
                'Set Verification Results for Display
                If i = 1 Then
                    ReconstructTemp_RegressionResultsBackwardVer = "Backward Verification:"
                    ReconstructTemp_RegressionResultsBackwardVer = ReconstructTemp_RegressionResultsBackwardVer & vbCrLf & "Month" & vbTab & "Model" & vbTab & "N" & vbTab & "CE" & vbTab & "r"
                End If
                ReconstructTemp_RegressionResultsBackwardVer = ReconstructTemp_RegressionResultsBackwardVer & vbCrLf & MonthPrint & vbTab & "1" & vbTab & Model1Set1Count & vbTab & CE & vbTab & Verr
                'Model 2 Verification
                SumX = 0
                SumY = 0
                SSR = 0
                SXX = 0
                SYY = 0
                SXY = 0
                ReDim VerPredicted(Model2Set1Count - 1)
                ReDim VerActual(Model2Set1Count - 1)
                For j = 0 To Model2Set1Count - 1
                    IndValue = IndModel2Set1(j)
                    VerPredicted(j) = Model2B0 + Model2B1 * IndModel2Set1(j)
                    VerActual(j) = DepModel2Set1(j)
                    SumX = SumX + VerActual(j)
                    SumY = SumY + VerPredicted(j)
                    SSR = SSR + (VerActual(j) - VerPredicted(j)) ^ 2
                Next j
                For j = 0 To Model2Set1Count - 1
                    SXX = SXX + (VerActual(j) - (SumX / Model2Set1Count)) ^ 2
                    SYY = SYY + (VerPredicted(j) - (SumY / Model2Set1Count)) ^ 2
                    SXY = SXY + (VerActual(j) - (SumX / Model2Set1Count)) * (VerPredicted(j) - (SumY / Model2Set1Count))
                Next j
                'Calculate Coefficent of Efficiency (CE)
                CE = Math.Round(1 - (SSR / SXX), 3)
                'Calculate Correlation Between Actual and Predicted Data
                Verr = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
                Verr = Math.Round(Verr, 3)
                'Set Verification Results for Display
                ReconstructTemp_RegressionResultsBackwardVer = ReconstructTemp_RegressionResultsBackwardVer & vbCrLf & MonthPrint & vbTab & "2" & vbTab & Model2Set1Count & vbTab & CE & vbTab & Verr
                'Model 3 Verification
                SumX = 0
                SumY = 0
                SSR = 0
                SXX = 0
                SYY = 0
                SXY = 0
                ReDim VerPredicted(Model3Set1Count - 1)
                ReDim VerActual(Model3Set1Count - 1)
                For j = 0 To Model3Set1Count - 1
                    IndValue = IndModel3Set1(j)
                    VerPredicted(j) = Model3B0 + Model3B1 * IndModel3Set1(j)
                    VerActual(j) = DepModel3Set1(j)
                    SumX = SumX + VerActual(j)
                    SumY = SumY + VerPredicted(j)
                    SSR = SSR + (VerActual(j) - VerPredicted(j)) ^ 2
                Next j
                For j = 0 To Model3Set1Count - 1
                    SXX = SXX + (VerActual(j) - (SumX / Model3Set1Count)) ^ 2
                    SYY = SYY + (VerPredicted(j) - (SumY / Model3Set1Count)) ^ 2
                    SXY = SXY + (VerActual(j) - (SumX / Model3Set1Count)) * (VerPredicted(j) - (SumY / Model3Set1Count))
                Next j
                'Calculate Coefficent of Efficiency (CE)
                CE = Math.Round(1 - (SSR / SXX), 3)
                'Calculate Correlation Between Actual and Predicted Data
                Verr = SXY / ((SXX ^ 0.5) * (SYY ^ 0.5))
                Verr = Math.Round(Verr, 3)
                'Set Verification Results for Display
                ReconstructTemp_RegressionResultsBackwardVer = ReconstructTemp_RegressionResultsBackwardVer & vbCrLf & MonthPrint & vbTab & "3" & vbTab & Model3Set1Count & vbTab & CE & vbTab & Verr
                'Reset Variables for Next Month
                ReDim IndModel1Set1(0)
                ReDim IndModel2Set1(0)
                ReDim IndModel3Set1(0)
                ReDim DepModel1Set1(0)
                ReDim DepModel2Set1(0)
                ReDim DepModel3Set1(0)
                ReDim Model1Set1Dates(0)
                ReDim Model2Set1Dates(0)
                ReDim Model3Set1Dates(0)
                Model1Set1Count = 0
                Model2Set1Count = 0
                Model3Set1Count = 0
                ReDim IndModel1Set2(0)
                ReDim IndModel2Set2(0)
                ReDim IndModel3Set2(0)
                ReDim DepModel1Set2(0)
                ReDim DepModel2Set2(0)
                ReDim DepModel3Set2(0)
                ReDim Model1Set2Dates(0)
                ReDim Model2Set2Dates(0)
                ReDim Model3Set2Dates(0)
                Model1Set2Count = 0
                Model2Set2Count = 0
                Model3Set2Count = 0
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            Next i
            FileClose(4)
            FileClose(5)
        End If
        If ReconstructTemp_JobNumber = 6 Then
            'Calibrate Final Models Sequence
            FileOpen(4, ReconstructTemp_ResidualsSet1, OpenMode.Output)
            For i = 1 To 12
                For j = 0 To ReconstructTemp_DepIndDataCount - 1
                    If ReconstructTemp_DepIndData(1, j) = i And ReconstructTemp_DepIndData(4, j) <> -99 And ReconstructTemp_DepIndData(3, j) <> -99 Then
                        Select Case ReconstructTemp_DepIndData(2, j)
                            Case 1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31
                                'Set Model 1
                                ReDim Preserve IndModel1Set1(Model1Set1Count)
                                ReDim Preserve DepModel1Set1(Model1Set1Count)
                                ReDim Preserve Model1Set1Dates(Model1Set1Count)
                                IndModel1Set1(Model1Set1Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel1Set1(Model1Set1Count) = ReconstructTemp_DepIndData(3, j)
                                Model1Set1Dates(Model1Set1Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model1Set1Count = Model1Set1Count + 1
                            Case 2, 5, 8, 11, 14, 17, 20, 23, 26, 29
                                'Set Model 2
                                ReDim Preserve IndModel2Set1(Model2Set1Count)
                                ReDim Preserve DepModel2Set1(Model2Set1Count)
                                ReDim Preserve Model2Set1Dates(Model2Set1Count)
                                IndModel2Set1(Model2Set1Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel2Set1(Model2Set1Count) = ReconstructTemp_DepIndData(3, j)
                                Model2Set1Dates(Model2Set1Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model2Set1Count = Model2Set1Count + 1
                            Case 3, 6, 9, 12, 15, 18, 21, 24, 27, 30
                                'Set Model 3
                                ReDim Preserve IndModel3Set1(Model3Set1Count)
                                ReDim Preserve DepModel3Set1(Model3Set1Count)
                                ReDim Preserve Model3Set1Dates(Model3Set1Count)
                                IndModel3Set1(Model3Set1Count) = ReconstructTemp_DepIndData(4, j)
                                DepModel3Set1(Model3Set1Count) = ReconstructTemp_DepIndData(3, j)
                                Model3Set1Dates(Model3Set1Count) = DateValue(ReconstructTemp_DepIndData(1, j) & "/" & ReconstructTemp_DepIndData(2, j) & "/" & ReconstructTemp_DepIndData(0, j))
                                Model3Set1Count = Model3Set1Count + 1
                        End Select
                    End If
                Next j
                'Set Current Month to PrintMonth for Output
                Select Case i
                    Case 1
                        MonthPrint = "Jan"
                    Case 2
                        MonthPrint = "Feb"
                    Case 3
                        MonthPrint = "Mar"
                    Case 4
                        MonthPrint = "Apr"
                    Case 5
                        MonthPrint = "May"
                    Case 6
                        MonthPrint = "Jun"
                    Case 7
                        MonthPrint = "Jul"
                    Case 8
                        MonthPrint = "Aug"
                    Case 9
                        MonthPrint = "Sep"
                    Case 10
                        MonthPrint = "Oct"
                    Case 11
                        MonthPrint = "Nov"
                    Case 12
                        MonthPrint = "Dec"
                End Select
                'Validate that Station Data are Long Enough for Final Model Calibration
                If IndModel1Set1.Length = 1 Or IndModel2Set1.Length = 1 Or IndModel3Set1.Length = 1 Or DepModel1Set1.Length = 1 Or DepModel2Set1.Length = 1 Or DepModel3Set1.Length = 1 Then
                    MsgBox("Insufficient temperature data to calibrate final models for the independent, " & ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber) & ", station.  Please check this station's data and re-run the Reconstruct Temperature tool.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Reconstruct Temperature")
                    'Reset Variables and Arrays for New Run
                    System.IO.File.Delete(ReconstructTemp_ResidualsSet1)
                    System.IO.File.Delete(ReconstructTemp_ResidualsSet2)
                    ReDim ReconstructTemp_ReconstructionStationNames(0)
                    ReDim ReconstructTemp_ReconstructionStationData(0, 0)
                    ReconstructTemp_ReconstructionStationDataCount = 0
                    ReDim ReconstructTemp_ScreeningStationData(0, 0)
                    ReconstructTemp_ScreeningStationDataCount = 0
                    ReDim ReconstructTemp_FinalReconstruction(0, 0)
                    ReconstructTemp_Exception = 1
                    Exit Sub
                End If
                'Derive Model 1
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model1Set1Count - 1
                    SumXY = SumXY + IndModel1Set1(j) * DepModel1Set1(j)
                    SumX = SumX + IndModel1Set1(j)
                    SumY = SumY + DepModel1Set1(j)
                    SumXSq = SumXSq + IndModel1Set1(j) ^ 2
                    SumYSq = SumYSq + DepModel1Set1(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model1Set1Count
                SXX = SumXSq - SumX ^ 2 / Model1Set1Count
                Model1B1 = SXY / SXX
                Model1B0 = (SumY - Model1B1 * SumX) / Model1Set1Count
                'Compute Model 1 R-Squared
                SSE = SumYSq - Model1B0 * SumY - Model1B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model1Set1Count
                Model1RSq = 1 - SSE / SST
                'Compute Model 1 Durbin-Watson Statistic
                For j = 1 To Model1Set1Count - 1
                    ActualTemp = DepModel1Set1(j)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel1Set1(j - 1)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set1(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model1DurWat = SumError / SSE
                'Compute Model 1 Standard Error
                Model1StdErr = (SSE / (Model1Set1Count - 2)) ^ 0.5
                'Derive Model 2
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model2Set1Count - 1
                    SumXY = SumXY + IndModel2Set1(j) * DepModel2Set1(j)
                    SumX = SumX + IndModel2Set1(j)
                    SumY = SumY + DepModel2Set1(j)
                    SumXSq = SumXSq + IndModel2Set1(j) ^ 2
                    SumYSq = SumYSq + DepModel2Set1(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model2Set1Count
                SXX = SumXSq - SumX ^ 2 / Model2Set1Count
                Model2B1 = SXY / SXX
                Model2B0 = (SumY - Model2B1 * SumX) / Model2Set1Count
                'Compute Model 2 R-Squared and Correlation
                SSE = SumYSq - Model2B0 * SumY - Model2B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model2Set1Count
                Model2RSq = 1 - SSE / SST
                'Compute Model 2 Durbin-Watson Statistic
                For j = 1 To Model2Set1Count - 1
                    ActualTemp = DepModel2Set1(j)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel2Set1(j - 1)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set1(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model2DurWat = SumError / SSE
                'Compute Model 2 Standard Error
                Model2StdErr = (SSE / (Model2Set1Count - 2)) ^ 0.5
                'Derive Model 3
                SumXY = 0
                SumX = 0
                SumY = 0
                SumXSq = 0
                SumYSq = 0
                SumError = 0
                For j = 0 To Model3Set1Count - 1
                    SumXY = SumXY + IndModel3Set1(j) * DepModel3Set1(j)
                    SumX = SumX + IndModel3Set1(j)
                    SumY = SumY + DepModel3Set1(j)
                    SumXSq = SumXSq + IndModel3Set1(j) ^ 2
                    SumYSq = SumYSq + DepModel3Set1(j) ^ 2
                Next j
                SXY = SumXY - SumX * SumY / Model3Set1Count
                SXX = SumXSq - SumX ^ 2 / Model3Set1Count
                Model3B1 = SXY / SXX
                Model3B0 = (SumY - Model3B1 * SumX) / Model3Set1Count
                'Compute Model 3 R-Squared and Correlation
                SSE = SumYSq - Model3B0 * SumY - Model3B1 * SumXY
                SST = SumYSq - SumY ^ 2 / Model3Set1Count
                Model3RSq = 1 - SSE / SST
                'Compute Model 3 Durbin-Watson Statistic
                For j = 1 To Model3Set1Count - 1
                    ActualTemp = DepModel3Set1(j)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    ActualTemp = DepModel3Set1(j - 1)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set1(j - 1)
                    Residualm1 = ActualTemp - PredictedTemp
                    SumError = ((Residual - Residualm1) ^ 2) + SumError
                Next j
                Model3DurWat = SumError / SSE
                'Compute Model 3 Standard Error
                Model3StdErr = (SSE / (Model3Set1Count - 2)) ^ 0.5
                'Calculate Residuals and Output to Temporary File
                'Model 1 Set 1 Residuals
                For j = 0 To Model1Set1Count - 1
                    DayValue = Model1Set1Dates(j)
                    IndValue = IndModel1Set1(j)
                    PredictedTemp = Model1B0 + Model1B1 * IndModel1Set1(j)
                    ActualTemp = DepModel1Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(4, MonthPrint & vbTab & "1" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Model 2 Set 1 Residuals
                For j = 0 To Model2Set1Count - 1
                    DayValue = Model2Set1Dates(j)
                    IndValue = IndModel2Set1(j)
                    PredictedTemp = Model2B0 + Model2B1 * IndModel2Set1(j)
                    ActualTemp = DepModel2Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(4, MonthPrint & vbTab & "2" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Model 3 Set 1 Residuals
                For j = 0 To Model3Set1Count - 1
                    DayValue = Model3Set1Dates(j)
                    IndValue = IndModel3Set1(j)
                    PredictedTemp = Model3B0 + Model3B1 * IndModel3Set1(j)
                    ActualTemp = DepModel3Set1(j)
                    Residual = ActualTemp - PredictedTemp
                    PredictedTemp = Math.Round(PredictedTemp, 1)
                    ActualTemp = Math.Round(ActualTemp, 1)
                    Residual = Math.Round(Residual, 1)
                    PrintLine(4, MonthPrint & vbTab & "3" & vbTab & IndValue & vbTab & ActualTemp & vbTab & PredictedTemp & vbTab & Residual & vbTab & DayValue)
                Next j
                'Set Regression Results for Display
                If i = 1 Then
                    ReconstructTemp_RegressionResultsFinal = "Final Calibration:"
                    ReconstructTemp_RegressionResultsFinal = ReconstructTemp_RegressionResultsFinal & vbCrLf & "Month" & vbTab & "Model" & vbTab & "N" & vbTab & "B0" & vbTab & "B1" & vbTab & "StdErr" & vbTab & "R-Sq" & vbTab & "DurWat"
                End If
                ReconstructTemp_RegressionResultsFinal = ReconstructTemp_RegressionResultsFinal & vbCrLf & MonthPrint & vbTab & "1" & vbTab & Model1Set1Count & vbTab & Math.Round(Model1B0, 3) & vbTab & Math.Round(Model1B1, 3) & vbTab & Math.Round(Model1StdErr, 3) & vbTab & Math.Round(Model1RSq, 3) & vbTab & Math.Round(Model1DurWat, 3)
                ReconstructTemp_RegressionResultsFinal = ReconstructTemp_RegressionResultsFinal & vbCrLf & MonthPrint & vbTab & "2" & vbTab & Model2Set1Count & vbTab & Math.Round(Model2B0, 3) & vbTab & Math.Round(Model2B1, 3) & vbTab & Math.Round(Model2StdErr, 3) & vbTab & Math.Round(Model2RSq, 3) & vbTab & Math.Round(Model2DurWat, 3)
                ReconstructTemp_RegressionResultsFinal = ReconstructTemp_RegressionResultsFinal & vbCrLf & MonthPrint & vbTab & "3" & vbTab & Model3Set1Count & vbTab & Math.Round(Model3B0, 3) & vbTab & Math.Round(Model3B1, 3) & vbTab & Math.Round(Model3StdErr, 3) & vbTab & Math.Round(Model3RSq, 3) & vbTab & Math.Round(Model3DurWat, 3)
                'Set Final Models to Arrays
                Select Case i
                    Case 1
                        ReconstructTemp_FinalB0(0) = Model1B0
                        ReconstructTemp_FinalB0(1) = Model2B0
                        ReconstructTemp_FinalB0(2) = Model3B0
                        ReconstructTemp_FinalB1(0) = Model1B1
                        ReconstructTemp_FinalB1(1) = Model2B1
                        ReconstructTemp_FinalB1(2) = Model3B1
                        ReconstructTemp_FinalStdErr(0) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(1) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(2) = Math.Round(Model3StdErr, 3)
                    Case 2
                        ReconstructTemp_FinalB0(3) = Model1B0
                        ReconstructTemp_FinalB0(4) = Model2B0
                        ReconstructTemp_FinalB0(5) = Model3B0
                        ReconstructTemp_FinalB1(3) = Model1B1
                        ReconstructTemp_FinalB1(4) = Model2B1
                        ReconstructTemp_FinalB1(5) = Model3B1
                        ReconstructTemp_FinalStdErr(3) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(4) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(5) = Math.Round(Model3StdErr, 3)
                    Case 3
                        ReconstructTemp_FinalB0(6) = Model1B0
                        ReconstructTemp_FinalB0(7) = Model2B0
                        ReconstructTemp_FinalB0(8) = Model3B0
                        ReconstructTemp_FinalB1(6) = Model1B1
                        ReconstructTemp_FinalB1(7) = Model2B1
                        ReconstructTemp_FinalB1(8) = Model3B1
                        ReconstructTemp_FinalStdErr(6) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(7) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(8) = Math.Round(Model3StdErr, 3)
                    Case 4
                        ReconstructTemp_FinalB0(9) = Model1B0
                        ReconstructTemp_FinalB0(10) = Model2B0
                        ReconstructTemp_FinalB0(11) = Model3B0
                        ReconstructTemp_FinalB1(9) = Model1B1
                        ReconstructTemp_FinalB1(10) = Model2B1
                        ReconstructTemp_FinalB1(11) = Model3B1
                        ReconstructTemp_FinalStdErr(9) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(10) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(11) = Math.Round(Model3StdErr, 3)
                    Case 5
                        ReconstructTemp_FinalB0(12) = Model1B0
                        ReconstructTemp_FinalB0(13) = Model2B0
                        ReconstructTemp_FinalB0(14) = Model3B0
                        ReconstructTemp_FinalB1(12) = Model1B1
                        ReconstructTemp_FinalB1(13) = Model2B1
                        ReconstructTemp_FinalB1(14) = Model3B1
                        ReconstructTemp_FinalStdErr(12) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(13) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(14) = Math.Round(Model3StdErr, 3)
                    Case 6
                        ReconstructTemp_FinalB0(15) = Model1B0
                        ReconstructTemp_FinalB0(16) = Model2B0
                        ReconstructTemp_FinalB0(17) = Model3B0
                        ReconstructTemp_FinalB1(15) = Model1B1
                        ReconstructTemp_FinalB1(16) = Model2B1
                        ReconstructTemp_FinalB1(17) = Model3B1
                        ReconstructTemp_FinalStdErr(15) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(16) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(17) = Math.Round(Model3StdErr, 3)
                    Case 7
                        ReconstructTemp_FinalB0(18) = Model1B0
                        ReconstructTemp_FinalB0(19) = Model2B0
                        ReconstructTemp_FinalB0(20) = Model3B0
                        ReconstructTemp_FinalB1(18) = Model1B1
                        ReconstructTemp_FinalB1(19) = Model2B1
                        ReconstructTemp_FinalB1(20) = Model3B1
                        ReconstructTemp_FinalStdErr(18) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(19) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(20) = Math.Round(Model3StdErr, 3)
                    Case 8
                        ReconstructTemp_FinalB0(21) = Model1B0
                        ReconstructTemp_FinalB0(22) = Model2B0
                        ReconstructTemp_FinalB0(23) = Model3B0
                        ReconstructTemp_FinalB1(21) = Model1B1
                        ReconstructTemp_FinalB1(22) = Model2B1
                        ReconstructTemp_FinalB1(23) = Model3B1
                        ReconstructTemp_FinalStdErr(21) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(22) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(23) = Math.Round(Model3StdErr, 3)
                    Case 9
                        ReconstructTemp_FinalB0(24) = Model1B0
                        ReconstructTemp_FinalB0(25) = Model2B0
                        ReconstructTemp_FinalB0(26) = Model3B0
                        ReconstructTemp_FinalB1(24) = Model1B1
                        ReconstructTemp_FinalB1(25) = Model2B1
                        ReconstructTemp_FinalB1(26) = Model3B1
                        ReconstructTemp_FinalStdErr(24) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(25) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(26) = Math.Round(Model3StdErr, 3)
                    Case 10
                        ReconstructTemp_FinalB0(27) = Model1B0
                        ReconstructTemp_FinalB0(28) = Model2B0
                        ReconstructTemp_FinalB0(29) = Model3B0
                        ReconstructTemp_FinalB1(27) = Model1B1
                        ReconstructTemp_FinalB1(28) = Model2B1
                        ReconstructTemp_FinalB1(29) = Model3B1
                        ReconstructTemp_FinalStdErr(27) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(28) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(29) = Math.Round(Model3StdErr, 3)
                    Case 11
                        ReconstructTemp_FinalB0(30) = Model1B0
                        ReconstructTemp_FinalB0(31) = Model2B0
                        ReconstructTemp_FinalB0(32) = Model3B0
                        ReconstructTemp_FinalB1(30) = Model1B1
                        ReconstructTemp_FinalB1(31) = Model2B1
                        ReconstructTemp_FinalB1(32) = Model3B1
                        ReconstructTemp_FinalStdErr(30) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(31) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(32) = Math.Round(Model3StdErr, 3)
                    Case 12
                        ReconstructTemp_FinalB0(33) = Model1B0
                        ReconstructTemp_FinalB0(34) = Model2B0
                        ReconstructTemp_FinalB0(35) = Model3B0
                        ReconstructTemp_FinalB1(33) = Model1B1
                        ReconstructTemp_FinalB1(34) = Model2B1
                        ReconstructTemp_FinalB1(35) = Model3B1
                        ReconstructTemp_FinalStdErr(33) = Math.Round(Model1StdErr, 3)
                        ReconstructTemp_FinalStdErr(34) = Math.Round(Model2StdErr, 3)
                        ReconstructTemp_FinalStdErr(35) = Math.Round(Model3StdErr, 3)
                End Select
                'Reset Variables for Next Month
                ReDim IndModel1Set1(0)
                ReDim IndModel2Set1(0)
                ReDim IndModel3Set1(0)
                ReDim DepModel1Set1(0)
                ReDim DepModel2Set1(0)
                ReDim DepModel3Set1(0)
                ReDim Model1Set1Dates(0)
                ReDim Model2Set1Dates(0)
                ReDim Model3Set1Dates(0)
                Model1Set1Count = 0
                Model2Set1Count = 0
                Model3Set1Count = 0
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            Next i
            FileClose(4)
        End If
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ReconstructTemp_Exception
            Case 0
                Status_Label.Text = "Status:  Modeling Cycle Completed Successfully"
                DependentStation_Label.Text = "Dependent Station:  " & ReconstructTemp_ReconstructionStationNames(0)
                IndependentStation_Label.Text = "Independent Station:  " & ReconstructTemp_ReconstructionStationNames(ReconstructTemp_StationNumber)
                If ReconstructTemp_JobNumber = 5 Then
                    'Forward and Backward Calibration Last Job
                    RegressionResults.Text = ReconstructTemp_RegressionResultsForwardCal & vbCrLf & vbCrLf & ReconstructTemp_RegressionResultsForwardVer & vbCrLf & vbCrLf & ReconstructTemp_RegressionResultsBackwardCal & vbCrLf & vbCrLf & ReconstructTemp_RegressionResultsBackwardVer
                    Forward.Enabled = True
                    Backward.Enabled = True
                    ViewPlots.Enabled = True
                    Run3.Text = "Proceed to Next Station"
                    Run3.Checked = False
                    Run3.Enabled = False
                    RemoveOutliers.Checked = False
                    RemoveOutliers.Enabled = True
                    Run1.Enabled = True
                    Run2.Enabled = True
                    Run1.Checked = True
                    Run.Text = "Continue Run"
                    If Forward.Checked = False And Backward.Checked = False Then
                        Forward.Checked = True
                    Else
                        If Forward.Checked = True Then
                            'Read Residuals Set 1 Into Residuals List
                            ResidualsList.Items.Clear()
                            Dim ResidualsSet1InFile As New System.IO.StreamReader(ReconstructTemp_ResidualsSet1)
                            Dim ResidualsSet1Line As String
                            Dim ResidualsSet1Fields As String()
                            While ResidualsSet1InFile.Peek <> -1
                                ResidualsSet1Line = ResidualsSet1InFile.ReadLine()
                                ResidualsSet1Fields = ResidualsSet1Line.Split(ControlChars.Tab)
                                ResidualsList.Items.Add(ResidualsSet1Fields(0) & " Model " & ResidualsSet1Fields(1) & " Residual = " & ResidualsSet1Fields(5))
                            End While
                            ResidualsSet1InFile.Close()
                        End If
                        If Backward.Checked = True Then
                            'Read Residuals Set 2 Into Residuals List
                            ResidualsList.Items.Clear()
                            Dim ResidualsSet2InFile As New System.IO.StreamReader(ReconstructTemp_ResidualsSet2)
                            Dim ResidualsSet2Line As String
                            Dim ResidualsSet2Fields As String()
                            While ResidualsSet2InFile.Peek <> -1
                                ResidualsSet2Line = ResidualsSet2InFile.ReadLine()
                                ResidualsSet2Fields = ResidualsSet2Line.Split(ControlChars.Tab)
                                ResidualsList.Items.Add(ResidualsSet2Fields(0) & " Model " & ResidualsSet2Fields(1) & " Residual = " & ResidualsSet2Fields(5))
                            End While
                            ResidualsSet2InFile.Close()
                        End If
                    End If
                End If
                If ReconstructTemp_JobNumber = 6 Then
                    'Calibration of Final Models Last Job
                    RegressionResults.Text = ReconstructTemp_RegressionResultsFinal
                    Run3.Enabled = True
                    Run2.Checked = True
                    RemoveOutliers.Checked = False
                    RemoveOutliers.Enabled = False
                    If ReconstructTemp_StationNumber = ReconstructTemp_ReconstructionStations - 1 Then
                        Run3.Text = "Complete Reconstruction"
                    End If
                    Forward.Enabled = False
                    Forward.Checked = False
                    Backward.Enabled = False
                    Backward.Checked = False
                    ViewPlots.Enabled = True
                    'Read Residuals Set 1 Into Residuals List
                    ResidualsList.Items.Clear()
                    Dim ResidualsSet1InFile As New System.IO.StreamReader(ReconstructTemp_ResidualsSet1)
                    Dim ResidualsSet1Line As String
                    Dim ResidualsSet1Fields As String()
                    While ResidualsSet1InFile.Peek <> -1
                        ResidualsSet1Line = ResidualsSet1InFile.ReadLine()
                        ResidualsSet1Fields = ResidualsSet1Line.Split(ControlChars.Tab)
                        ResidualsList.Items.Add(ResidualsSet1Fields(0) & " Model " & ResidualsSet1Fields(1) & " Residual = " & ResidualsSet1Fields(5))
                    End While
                    ResidualsSet1InFile.Close()
                End If
            Case 1
                Status_Label.Text = "Status:  Run Failed"
                DependentStation_Label.Text = "Dependent Station:  NA"
                IndependentStation_Label.Text = "Independent Station:  NA"
                RegressionResults.Text = ""
                ResidualsList.Items.Clear()
                Forward.Enabled = False
                Backward.Enabled = False
                ViewPlots.Enabled = False
                Run1.Enabled = False
                Run2.Enabled = False
                Run3.Text = "Proceed to First Station"
                RemoveOutliers.Enabled = True
                Run.Text = "Run"
            Case 2
                'Reconstruction Completed
                Status_Label.Text = "Status:  Reconstruction Completed Successfully"
                DependentStation_Label.Text = "Dependent Station:  NA"
                IndependentStation_Label.Text = "Independent Station:  NA"
                RegressionResults.Text = ""
                ResidualsList.Items.Clear()
                Forward.Enabled = False
                Backward.Enabled = False
                ViewPlots.Enabled = False
                Run1.Enabled = False
                Run2.Enabled = False
                Run3.Text = "Proceed to First Station"
                RemoveOutliers.Enabled = True
                Run.Text = "Run"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ReconstructTemp_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ReconstructTempDisp = 0
    End Sub
End Class