Imports ZedGraph
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Public Class ReconstructTempFig

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ReconstructFig_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        HOBToolsHelp.HelpNamespace = HOBToolsUG
        'Set Plot to Default Values
        Month.SelectedIndex = 0
        Model.SelectedIndex = 0
        PlotType.SelectedIndex = 0
        'Call Sub to Plot Residuals
        DrawResidualPlot()
    End Sub

    Private Sub DrawResidualPlot()
        Dim MonthName As String
        Dim ModelNumber As Integer
        Dim PlotName As String
        Dim StoreOnOff As Integer
        StoreOnOff = 0
        Dim PlotList As New PointPairList()
        Dim x As Double
        Dim y As Double
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ReconstructTemp_OutputResiduals)
        Dim Line As String
        Dim Fields As String()
        'Import Data
        While InFile.Peek <> -1
            Line = InFile.ReadLine
            Fields = Line.Split(ControlChars.Tab)
            If Month.SelectedIndex = 0 And Fields(0) = "Jan" Then
                StoreOnOff = 1
                MonthName = "January"
            End If
            If Month.SelectedIndex = 1 And Fields(0) = "Feb" Then
                StoreOnOff = 1
                MonthName = "February"
            End If
            If Month.SelectedIndex = 2 And Fields(0) = "Mar" Then
                StoreOnOff = 1
                MonthName = "March"
            End If
            If Month.SelectedIndex = 3 And Fields(0) = "Apr" Then
                StoreOnOff = 1
                MonthName = "April"
            End If
            If Month.SelectedIndex = 4 And Fields(0) = "May" Then
                StoreOnOff = 1
                MonthName = "May"
            End If
            If Month.SelectedIndex = 5 And Fields(0) = "Jun" Then
                StoreOnOff = 1
                MonthName = "June"
            End If
            If Month.SelectedIndex = 6 And Fields(0) = "Jul" Then
                StoreOnOff = 1
                MonthName = "July"
            End If
            If Month.SelectedIndex = 7 And Fields(0) = "Aug" Then
                StoreOnOff = 1
                MonthName = "August"
            End If
            If Month.SelectedIndex = 8 And Fields(0) = "Sep" Then
                StoreOnOff = 1
                MonthName = "September"
            End If
            If Month.SelectedIndex = 9 And Fields(0) = "Oct" Then
                StoreOnOff = 1
                MonthName = "October"
            End If
            If Month.SelectedIndex = 10 And Fields(0) = "Nov" Then
                StoreOnOff = 1
                MonthName = "November"
            End If
            If Month.SelectedIndex = 11 And Fields(0) = "Dec" Then
                StoreOnOff = 1
                MonthName = "December"
            End If
            If Model.SelectedItem = Fields(1) And StoreOnOff = 1 Then
                ModelNumber = Fields(1)
                Select Case PlotType.SelectedIndex
                    Case 0
                        'Plot X vs. Y
                        x = Fields(2)
                        y = Fields(3)
                        PlotName = "X vs. Y"
                    Case 1
                        'Plot Y vs. Y-Hat
                        x = Fields(3)
                        y = Fields(4)
                        PlotName = "Y vs. Y-Hat"
                    Case 2
                        'Plot Y-Hat vs. Residuals
                        x = Fields(4)
                        y = Fields(5)
                        PlotName = "Y-Hat vs. Residuals"
                    Case 3
                        'Plot X vs. Residuals
                        x = Fields(3)
                        y = Fields(5)
                        PlotName = "X vs. Residuals"
                End Select
                PlotList.Add(x, y)
            End If
            StoreOnOff = 0
        End While
        InFile.Close()
        'Draw Scatter Plot
        'Set Residual Chart Properties
        Dim ResidualPane As GraphPane = ResidualChart.GraphPane
        ResidualPane.CurveList.Clear()
        ResidualChart.ZoomOutAll(ResidualPane)
        ResidualChart.MasterPane.Border.IsVisible = False
        ResidualPane.Border.IsVisible = False
        ResidualPane.Chart.Border.IsVisible = False
        'Set Title
        ResidualPane.Title.Text = ReconstructTemp_IndStation & " > " & ReconstructTemp_DepStation & ControlChars.CrLf & MonthName & " Model " & ModelNumber & " " & PlotName
        ResidualPane.Title.FontSpec.Size = 16
        ResidualPane.Title.FontSpec.IsBold = True
        'Set X-Axis
        ResidualPane.XAxis.Scale.FontSpec.Size = 12
        ResidualPane.XAxis.Scale.FontSpec.IsBold = True
        ResidualPane.XAxis.Scale.LabelGap = 0.2
        Select Case PlotType.SelectedIndex
            Case 0, 3
                ResidualPane.XAxis.Title.Text = "X"
            Case 1
                ResidualPane.XAxis.Title.Text = "Y"
            Case 2
                ResidualPane.XAxis.Title.Text = "Y-Hat"
        End Select
        ResidualPane.XAxis.Title.FontSpec.Size = 14
        ResidualPane.XAxis.Title.FontSpec.IsBold = True
        ResidualPane.XAxis.MajorTic.PenWidth = 2
        ResidualPane.XAxis.MajorTic.IsInside = False
        ResidualPane.XAxis.MajorTic.IsOpposite = False
        ResidualPane.XAxis.MinorTic.IsAllTics = False
        ResidualPane.X2Axis.MajorTic.IsAllTics = False
        ResidualPane.X2Axis.MinorTic.IsAllTics = False
        'Set Y-Axis
        ResidualPane.YAxis.Scale.FontSpec.Size = 12
        ResidualPane.YAxis.Scale.FontSpec.IsBold = True
        ResidualPane.YAxis.Scale.LabelGap = 0.2
        Select Case PlotType.SelectedIndex
            Case 0
                ResidualPane.YAxis.Title.Text = "Y"
            Case 1
                ResidualPane.YAxis.Title.Text = "Y-Hat"
            Case 2, 3
                ResidualPane.YAxis.Title.Text = "Residuals"
        End Select
        ResidualPane.YAxis.Title.FontSpec.Size = 14
        ResidualPane.YAxis.Title.FontSpec.IsBold = True
        Select Case PlotType.SelectedIndex
            Case 0, 1
                ResidualPane.YAxis.MajorGrid.IsZeroLine = False
            Case 2, 3
                ResidualPane.YAxis.MajorGrid.IsZeroLine = True
        End Select
        ResidualPane.YAxis.MajorTic.PenWidth = 2
        ResidualPane.YAxis.MajorTic.IsInside = False
        ResidualPane.YAxis.MajorTic.IsOpposite = False
        ResidualPane.YAxis.MinorTic.IsAllTics = False
        ResidualPane.Y2Axis.MajorTic.IsAllTics = False
        ResidualPane.Y2Axis.MinorTic.IsAllTics = False
        'Set Legend
        ResidualPane.Legend.Border.IsVisible = False
        ResidualPane.Legend.Position = LegendPos.BottomCenter
        ResidualPane.Legend.IsHStack = False
        ResidualPane.Legend.Location.AlignV = AlignV.Top
        ResidualPane.Legend.Location.AlignH = AlignH.Left
        ResidualPane.Legend.FontSpec.Size = 10
        ResidualPane.Legend.FontSpec.IsBold = True
        'Turn off Legend
        ResidualPane.Legend.IsVisible = False
        'Generate Scatter Plot
        Dim ScatterPlot As LineItem
        ScatterPlot = ResidualPane.AddCurve("Scatter Plot", PlotList, Color.Red, SymbolType.Circle)
        ScatterPlot.Line.IsVisible = False
        ScatterPlot.Symbol.Fill = New Fill(Brushes.Red)
        'Calculate Axis Scale Ranges and Refresh Chart
        ResidualPane.AxisChange()
        ResidualChart.Refresh()
        Plot.Text = "Plot"
        Plot.Enabled = True
    End Sub

    Private Sub Plot_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Plot.Click
        Plot.Text = "Plotting..."
        Plot.Enabled = False
        DrawResidualPlot()
    End Sub

    Private Sub CopyBitmap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyBitmap.Click
        Clipboard.SetImage(ResidualChart.MasterPane.GetImage(2307, 1500, 288, False))
    End Sub

    Private Sub CopyMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyMetafile.Click
        Dim g As Graphics = Me.CreateGraphics()
        Dim hdc As System.IntPtr = g.GetHdc()
        Dim metaFile As New Imaging.Metafile(hdc, Imaging.EmfType.EmfOnly)
        g.ReleaseHdc(hdc)
        g.Dispose()
        Dim gMeta As Graphics = Graphics.FromImage(metaFile)
        ResidualChart.MasterPane.Draw(gMeta)
        gMeta.Dispose()
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, metaFile)
    End Sub

    Private Sub SaveChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveChart.Click
        Dim SaveResidualPlot As New SaveFileDialog
        Dim OutputImage As String
        SaveResidualPlot.Title = "Save Regression Model Residual Plot"
        SaveResidualPlot.Filter = "Bitmap (*.bmp)|*.bmp|GIF (*.gif)|*.gif|JPEG (*.jpg)|*.jpg|PNG (*.png)|*.png|TIFF (*.tif)|*.tif"
        If SaveResidualPlot.ShowDialog = Windows.Forms.DialogResult.OK Then
            OutputImage = SaveResidualPlot.FileName
            If SaveResidualPlot.FileName.EndsWith("bmp") Then
                ResidualChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Bmp)
            End If
            If SaveResidualPlot.FileName.EndsWith("gif") Then
                ResidualChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Gif)
            End If
            If SaveResidualPlot.FileName.EndsWith("jpg") Then
                ResidualChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Jpeg)
            End If
            If SaveResidualPlot.FileName.EndsWith("png") Then
                ResidualChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Png)
            End If
            If SaveResidualPlot.FileName.EndsWith("tif") Then
                ResidualChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Tiff)
            End If
        End If
    End Sub

    Private Sub PrintChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrintChart.Click
        ResidualChart.DoPrint()
    End Sub

    Private Sub ResidualChart_ContextMenuBuilder(ByVal sender As ZedGraph.ZedGraphControl, ByVal menuStrip As System.Windows.Forms.ContextMenuStrip, ByVal mousePt As System.Drawing.Point, ByVal objState As ZedGraph.ZedGraphControl.ContextMenuObjectState) Handles ResidualChart.ContextMenuBuilder
        'Remove Specific Functions from Residual Chart Context Menu for Use Elsewhere
        Dim Item As ToolStripMenuItem
        Dim CopyItem As ToolStripMenuItem
        Dim PageSetupItem As ToolStripMenuItem
        Dim PrintItem As ToolStripMenuItem
        Dim SaveAsItem As ToolStripMenuItem
        Dim SetScaleItem As ToolStripMenuItem
        For Each Item In menuStrip.Items
            If Item.Tag = "copy" Then
                CopyItem = Item
            End If
            If Item.Tag = "page_setup" Then
                PageSetupItem = Item
            End If
            If Item.Tag = "print" Then
                PrintItem = Item
            End If
            If Item.Tag = "save_as" Then
                SaveAsItem = Item
            End If
            If Item.Tag = "set_default" Then
                SetScaleItem = Item
            End If
        Next Item
        menuStrip.Items.Remove(CopyItem)
        menuStrip.Items.Remove(PageSetupItem)
        menuStrip.Items.Remove(PrintItem)
        menuStrip.Items.Remove(SaveAsItem)
        menuStrip.Items.Remove(SetScaleItem)
    End Sub

    Private Sub ReconstructTempFig_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ReconstructTempFigDisp = 0
    End Sub
End Class