Public Class ReturnTime

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Form Input/Output Variables
    Private ReturnTime_Input As String
    'Initialize Global Form Variables
    Private ReturnTime_DataRowStart As Integer
    Private ReturnTime_DataColumn As Integer
    Private ReturnTime_CriticalAboveBelow As Integer
    Private ReturnTime_CriticalValue As Double
    Private ReturnTime_NYear As Integer
    Private ReturnTime_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ReturnTime_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Return Time Analysis Tool
        CriticalAboveBelow.SelectedIndex = 0
        ReturnTime_NYear = 100
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ReturnTime_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ReturnTime_Input = FileSelect.FileName
            InputFileLocation.Text = ReturnTime_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                ReturnTime_DataRowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DataColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataColumn.TextChanged
        EntryError.SetError(DataColumn, "")
        If DataColumn.Text <> "" Then
            Try
                ReturnTime_DataColumn = DataColumn.Text
            Catch ex As Exception
                EntryError.SetError(DataColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartYearEntry.TextChanged
        EntryError.SetError(StartYearEntry, "")
        If StartYearEntry.Text <> "" Then
            Try
                ReturnTime_StartYear = StartYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(StartYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndYearEntry.TextChanged
        EntryError.SetError(EndYearEntry, "")
        If EndYearEntry.Text <> "" Then
            Try
                ReturnTime_EndYear = EndYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(EndYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub CriticalValue_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CriticalValue.TextChanged
        EntryError.SetError(CriticalValue, "")
        If CriticalValue.Text <> "" Then
            Try
                ReturnTime_CriticalValue = CriticalValue.Text
            Catch ex As Exception
                EntryError.SetError(CriticalValue, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub NYear_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NYear.TextChanged
        EntryError.SetError(NYear, "")
        If NYear.Text <> "" Then
            Try
                ReturnTime_NYear = NYear.Text
            Catch ex As Exception
                EntryError.SetError(NYear, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ReturnTime_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ReturnTime_Output = FileOutput.FileName
            OutputFileLocation.Text = ReturnTime_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ReturnTime_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ReturnTime_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If DataColumn.Text = "" Then
            EntryError.SetError(DataColumn, "Required input data information missing.")
            Exit Sub
        End If
        If StartYearEntry.Text = "" Then
            EntryError.SetError(StartYearEntry, "Required analysis information missing.")
            Exit Sub
        End If
        If EndYearEntry.Text = "" Then
            EntryError.SetError(EndYearEntry, "Required analysis information missing.")
            Exit Sub
        End If
        If CriticalValue.Text = "" Then
            EntryError.SetError(CriticalValue, "Required analysis information missing.")
            Exit Sub
        End If
        If NYear.Text = "" Then
            EntryError.SetError(NYear, "Required analysis information missing.")
            Exit Sub
        End If
        If ReturnTime_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If ReturnTime_Output = ReturnTime_Input Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        Select Case CriticalAboveBelow.SelectedIndex
            Case 0
                ReturnTime_CriticalAboveBelow = 0
            Case 1
                ReturnTime_CriticalAboveBelow = 1
        End Select
        ReturnTime_Exception = 0
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim FoundStart As Integer
        Dim DataCount As Integer
        Dim i As Integer
        Dim Rank As Integer
        Dim CriticalValueReturnInterval As Double
        Dim CriticalValueReturnProbability As Double
        Dim NYearProb As Double
        'Initialize Observation Array
        Dim ObservationData(0) As Double
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ReturnTime_Input)
        Dim Line As String
        Dim Fields As String()
        'Set StreamReader to Data Start
        If ReturnTime_DataRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ReturnTime_DataRowStart - 2
                Line = InFile.ReadLine()
            Next Row
        End If
        'Initialize Progress Bar
        Dim RunProgress As Integer
        Dim RunComplete As Integer
        Dim Percent As Integer
        RunProgress = 0
        RunComplete = 3
        'First Row Is Column Headers...Not Needed Move Input Line Down
        Line = InFile.ReadLine()
        'Input Data
        Try
            While InFile.Peek <> -1
                Line = InFile.ReadLine()
                Fields = Line.Split(ControlChars.Tab)
                Year = Fields(0)
                If Year = ReturnTime_StartYear Then
                    FoundStart = 1
                End If
                If FoundStart = 1 And Fields(ReturnTime_DataColumn - 1) <> -99 Then
                    ReDim Preserve ObservationData(DataCount)
                    ObservationData(DataCount) = Fields(ReturnTime_DataColumn - 1)
                    DataCount = DataCount + 1
                End If
                If Year = ReturnTime_EndYear Then
                    Exit While
                End If
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the input data near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Return Time Analysis")
            Else
                MsgBox("Error reading the input data near the header.  Make sure the row where the input data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Return Time Analysis")
            End If
            InFile.Close()
            FileClose(1)
            ReturnTime_Exception = 1
            Exit Sub
        End Try
        'Validate that Start Date Has Been Found
        If FoundStart = 0 Then
            MsgBox("Unable to find analysis start date in the input data.  Recheck the analysis start date.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Return Time Analysis")
            InFile.Close()
            FileClose(1)
            ReturnTime_Exception = 1
            Exit Sub
        End If
        'Input of Data Complete...Close File and Update Progress Bar
        InFile.Close()
        RunProgress = RunProgress + 1
        Percent = (RunProgress / RunComplete) * 100
        BackgroundRun.ReportProgress(Percent)
        'Sort Observation Data Array
        Select Case ReturnTime_CriticalAboveBelow
            Case 0
                'Calculate Return Time of Equal or Higher...Sort Observation Data Array High to Low
                Array.Sort(ObservationData)
                Array.Reverse(ObservationData)
            Case 1
                'Calculate Return Time of Equal or Lower...Sort Observation Data Array Low to High
                Array.Sort(ObservationData)
        End Select
        'Find Event Rank in Sorted Observation Data Array
        Rank = 1
        For i = 0 To DataCount - 1
            Select Case ReturnTime_CriticalAboveBelow
                Case 0
                    'Rank of Equal or Higher
                    If ObservationData(i) < ReturnTime_CriticalValue Then
                        'Rank Assigned...Exit For Loop
                        Exit For
                    Else
                        Rank = Rank + 1
                    End If
                Case 1
                    'Rank of Equal or Lower
                    If ObservationData(i) > ReturnTime_CriticalValue Then
                        'Rank Assigned...Exit For Loop
                        Exit For
                    Else
                        Rank = Rank + 1
                    End If
            End Select
        Next i
        'Compute Event Return Interval
        CriticalValueReturnInterval = (DataCount + 1) / Rank
        CriticalValueReturnProbability = 1 / CriticalValueReturnInterval
        'Calculation of Event Return Interval and Probability Complete...Update Progress Bar
        FileClose(1)
        RunProgress = RunProgress + 1
        Percent = (RunProgress / RunComplete) * 100
        BackgroundRun.ReportProgress(Percent)
        'Initialize Final Output File and Set Header
        FileOpen(1, ReturnTime_Output, OpenMode.Output)
        PrintLine(1, ReturnTime_Header)
        PrintLine(1, "")
        'Ouput Return Interval and Probabilities Over N-Years
        PrintLine(1, "Analysis Start Year:" & vbTab & vbTab & ReturnTime_StartYear)
        PrintLine(1, "Analysis End Year:" & vbTab & vbTab & ReturnTime_EndYear)
        PrintLine(1, "")
        PrintLine(1, "Critical Value:" & vbTab & vbTab & vbTab & ReturnTime_CriticalValue)
        PrintLine(1, "Return Interval:" & vbTab & vbTab & Math.Round(CriticalValueReturnInterval, 1) & " years")
        PrintLine(1, "Probability In Any Year:" & vbTab & Math.Round(CriticalValueReturnProbability, 4))
        PrintLine(1, "")
        PrintLine(1, "N-Year" & vbTab & "Probability of Recurrence")
        'Read Probabilities from R and Output
        For i = 1 To ReturnTime_NYear
            NYearProb = 1 - ((1 - CriticalValueReturnProbability) ^ i)
            PrintLine(1, i & vbTab & Math.Round(NYearProb, 4))
        Next i
        'Output of Return Time Analysis Complete...Close Output File and Update Progress Bar
        FileClose(1)
        RunProgress = RunProgress + 1
        Percent = (RunProgress / RunComplete) * 100
        BackgroundRun.ReportProgress(Percent)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ReturnTime_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
                'Show Figure
                If ReturnTimeFigDisp = 0 Then
                    ReturnTimeFigDisp = 1
                    Dim frmReturnTimeFig As New ReturnTimeFig
                    frmReturnTimeFig.MdiParent = MainParent
                    frmReturnTimeFig.Show()
                Else
                    MsgBox("Only one time series can be displayed at a time.  Please close the displayed time series, and run the Screen Interhourly Temperature Ranges Tool check again.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Screen Interhourly Temperature Ranges")
                End If
            Case 1
                Status_Label.Text = "Status:  Run Failed"
            Case 2
                Status_Label.Text = "Status:  Run Completed...Probability Zero"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ReturnTime_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ReturnTimeDisp = 0
    End Sub
End Class