﻿Imports ZedGraph
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Public Class ReturnTimeFig

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ReturnTimeFig_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        HOBToolsHelp.HelpNamespace = HOBToolsUG
        DrawProbPlot()
    End Sub

    Private Sub DrawProbPlot()
        'Initialize Variables
        Dim Year As Integer
        Dim DataValue As Double
        Dim ProbabilitySeriesData As New PointPairList
        Dim ProbabilitySeries As LineItem
        Dim i As Integer
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ReturnTime_Output)
        Dim Line As String
        Dim Fields As String()
        'Set StreamReader to Data Start
        Dim Row As Integer
        For Row = 0 To 9
            Line = InFile.ReadLine()
        Next Row
        'Import Data
        While InFile.Peek <> -1
            Line = InFile.ReadLine
            Fields = Line.Split(ControlChars.Tab)
            Year = Fields(0)
            DataValue = Fields(1)
            ProbabilitySeriesData.Add(Year, DataValue)
            i = i + 1
        End While
        InFile.Close()
        'Set Probability Chart Properties
        Dim ProbPane As GraphPane = ProbChart.GraphPane
        ProbPane.CurveList.Clear()
        ProbChart.ZoomOutAll(ProbPane)
        ProbChart.MasterPane.Border.IsVisible = False
        ProbPane.Border.IsVisible = False
        ProbPane.Chart.Border.IsVisible = False
        'Title
        ProbPane.Title.Text = ReturnTime_Header & Chr(10) & ReturnTime_StartYear & " - " & ReturnTime_EndYear
        ProbPane.Title.FontSpec.Size = 16
        ProbPane.Title.FontSpec.IsBold = True
        'X-Axis
        ProbPane.XAxis.Scale.Max = i
        ProbPane.XAxis.Scale.Min = 0
        ProbPane.XAxis.Scale.FontSpec.Size = 12
        ProbPane.XAxis.Scale.FontSpec.IsBold = True
        ProbPane.XAxis.Scale.LabelGap = 0.2
        ProbPane.XAxis.Title.Text = "N-Year"
        ProbPane.XAxis.Title.FontSpec.Size = 14
        ProbPane.XAxis.Title.FontSpec.IsBold = True
        ProbPane.XAxis.MajorTic.PenWidth = 2
        ProbPane.XAxis.MajorTic.IsInside = False
        ProbPane.XAxis.MajorTic.IsOpposite = False
        ProbPane.XAxis.MajorTic.Size = 10
        ProbPane.XAxis.MinorTic.IsInside = False
        ProbPane.XAxis.MinorTic.IsOpposite = False
        ProbPane.XAxis.MinorTic.Size = 5
        ProbPane.X2Axis.MajorTic.IsAllTics = False
        ProbPane.X2Axis.MinorTic.IsAllTics = False
        'Y-Axis
        ProbPane.YAxis.Scale.Max = 1.0
        ProbPane.YAxis.Scale.Min = 0.0
        ProbPane.YAxis.Scale.FontSpec.Size = 12
        ProbPane.YAxis.Scale.FontSpec.IsBold = True
        ProbPane.YAxis.Scale.LabelGap = 0.2
        ProbPane.YAxis.Title.Text = "Probability of Recurrence"
        ProbPane.YAxis.Title.FontSpec.Size = 14
        ProbPane.YAxis.Title.FontSpec.IsBold = True
        ProbPane.YAxis.MajorGrid.IsZeroLine = False
        ProbPane.YAxis.MajorTic.PenWidth = 2
        ProbPane.YAxis.MajorTic.IsInside = False
        ProbPane.YAxis.MajorTic.IsOpposite = False
        ProbPane.YAxis.MajorTic.Size = 10
        ProbPane.YAxis.MinorTic.IsAllTics = False
        ProbPane.Y2Axis.MajorTic.IsAllTics = False
        ProbPane.Y2Axis.MinorTic.IsAllTics = False
        'Generate Probability Plot
        ProbabilitySeries = ProbPane.AddCurve("Data", ProbabilitySeriesData, Color.Blue, SymbolType.None)
        ProbabilitySeries.Line.SmoothTension = 0
        ProbabilitySeries.Line.Width = 2
        'Turn off Legend
        ProbPane.Legend.IsVisible = False
        'Calculate Axis Scale Ranges and Refresh Chart
        ProbPane.AxisChange()
        ProbChart.Refresh()
    End Sub

    Private Sub CopyBitmap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyBitmap.Click
        Clipboard.SetImage(ProbChart.MasterPane.GetImage(2307, 1500, 288, False))
    End Sub

    Private Sub CopyMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyMetafile.Click
        Dim g As Graphics = Me.CreateGraphics()
        Dim hdc As System.IntPtr = g.GetHdc()
        Dim metaFile As New Imaging.Metafile(hdc, Imaging.EmfType.EmfOnly)
        g.ReleaseHdc(hdc)
        g.Dispose()
        Dim gMeta As Graphics = Graphics.FromImage(metaFile)
        ProbChart.MasterPane.Draw(gMeta)
        gMeta.Dispose()
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, metaFile)
    End Sub

    Private Sub SaveChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveChart.Click
        Dim SaveReturnTimePlot As New SaveFileDialog
        Dim OutputImage As String
        SaveReturnTimePlot.Title = "Save Return Time Analysis Probability Plot"
        SaveReturnTimePlot.Filter = "Bitmap (*.bmp)|*.bmp|GIF (*.gif)|*.gif|JPEG (*.jpg)|*.jpg|PNG (*.png)|*.png|TIFF (*.tif)|*.tif"
        If SaveReturnTimePlot.ShowDialog = Windows.Forms.DialogResult.OK Then
            OutputImage = SaveReturnTimePlot.FileName
            If SaveReturnTimePlot.FileName.EndsWith("bmp") Then
                ProbChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Bmp)
            End If
            If SaveReturnTimePlot.FileName.EndsWith("gif") Then
                ProbChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Gif)
            End If
            If SaveReturnTimePlot.FileName.EndsWith("jpg") Then
                ProbChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Jpeg)
            End If
            If SaveReturnTimePlot.FileName.EndsWith("png") Then
                ProbChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Png)
            End If
            If SaveReturnTimePlot.FileName.EndsWith("tif") Then
                ProbChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Tiff)
            End If
        End If
    End Sub

    Private Sub PrintChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrintChart.Click
        ProbChart.DoPrint()
    End Sub

    Private Sub ProbChart_ContextMenuBuilder(ByVal sender As ZedGraph.ZedGraphControl, ByVal menuStrip As System.Windows.Forms.ContextMenuStrip, ByVal mousePt As System.Drawing.Point, ByVal objState As ZedGraph.ZedGraphControl.ContextMenuObjectState) Handles ProbChart.ContextMenuBuilder
        'Remove Specific Functions from Probability Chart Context Menu for Use Elsewhere
        Dim Item As ToolStripMenuItem
        Dim CopyItem As ToolStripMenuItem
        Dim PageSetupItem As ToolStripMenuItem
        Dim PrintItem As ToolStripMenuItem
        Dim SaveAsItem As ToolStripMenuItem
        Dim SetScaleItem As ToolStripMenuItem
        For Each Item In menuStrip.Items
            If Item.Tag = "copy" Then
                CopyItem = Item
            End If
            If Item.Tag = "page_setup" Then
                PageSetupItem = Item
            End If
            If Item.Tag = "print" Then
                PrintItem = Item
            End If
            If Item.Tag = "save_as" Then
                SaveAsItem = Item
            End If
            If Item.Tag = "set_default" Then
                SetScaleItem = Item
            End If
        Next Item
        menuStrip.Items.Remove(CopyItem)
        menuStrip.Items.Remove(PageSetupItem)
        menuStrip.Items.Remove(PrintItem)
        menuStrip.Items.Remove(SaveAsItem)
        menuStrip.Items.Remove(SetScaleItem)
    End Sub

    Private Sub ReturnTimeFig_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ReturnTimeFigDisp = 0
    End Sub
End Class