Public Class ScreenPrecip

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input Variables
    Private ScreenPrecip_HistoricalInput As String
    'Initialize Global Form Variables
    Private ScreenPrecip_HistRowStart As Integer
    Private ScreenPrecip_PrecipColumn As Integer
    Private ScreenPrecip_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ScreenPrecip_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Screen Precipitation Data Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub HistoricalSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HistoricalSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ScreenPrecip_HistoricalInput
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ScreenPrecip_HistoricalInput = FileSelect.FileName
            InputFileLocation.Text = ScreenPrecip_HistoricalInput
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HistDataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HistDataRowStart.TextChanged
        EntryError.SetError(HistDataRowStart, "")
        If HistDataRowStart.Text <> "" Then
            Try
                ScreenPrecip_HistRowStart = HistDataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(HistDataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub PrecipColumnEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrecipColumnEntry.TextChanged
        EntryError.SetError(PrecipColumnEntry, "")
        If PrecipColumnEntry.Text <> "" Then
            Try
                ScreenPrecip_PrecipColumn = PrecipColumnEntry.Text
            Catch ex As Exception
                EntryError.SetError(PrecipColumnEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartYearEntry.TextChanged
        EntryError.SetError(StartYearEntry, "")
        If StartYearEntry.Text <> "" Then
            Try
                ScreenPrecip_StartYear = StartYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(StartYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartMonthEntry.TextChanged
        EntryError.SetError(StartMonthEntry, "")
        If StartMonthEntry.Text <> "" Then
            Try
                ScreenPrecip_StartMonth = StartMonthEntry.Text
                If ScreenPrecip_StartMonth > 12 Or ScreenPrecip_StartMonth < 1 Then
                    EntryError.SetError(StartMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartDayEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartDayEntry.TextChanged
        EntryError.SetError(StartDayEntry, "")
        If StartDayEntry.Text <> "" Then
            Try
                ScreenPrecip_StartDay = StartDayEntry.Text
                If ScreenPrecip_StartDay < 1 Then
                    EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartDayEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndYearEntry.TextChanged
        EntryError.SetError(EndYearEntry, "")
        If EndYearEntry.Text <> "" Then
            Try
                ScreenPrecip_EndYear = EndYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(EndYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndMonthEntry.TextChanged
        EntryError.SetError(EndMonthEntry, "")
        If EndMonthEntry.Text <> "" Then
            Try
                ScreenPrecip_EndMonth = EndMonthEntry.Text
                If ScreenPrecip_EndMonth > 12 Or ScreenPrecip_EndMonth < 1 Then
                    EntryError.SetError(EndMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndDayEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndDayEntry.TextChanged
        EntryError.SetError(EndDayEntry, "")
        If EndDayEntry.Text <> "" Then
            Try
                ScreenPrecip_EndDay = EndDayEntry.Text
                If ScreenPrecip_EndDay < 1 Then
                    EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndDayEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ScreenPrecip_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ScreenPrecip_Output = FileOutput.FileName
            OutputFileLocation.Text = ScreenPrecip_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ScreenPrecip_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ScreenPrecip_HistoricalInput = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If HistDataRowStart.Text = "" Then
            EntryError.SetError(HistDataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If PrecipColumnEntry.Text = "" Then
            EntryError.SetError(PrecipColumnEntry, "Required input data information missing.")
            Exit Sub
        End If
        If StartYearEntry.Text = "" Then
            EntryError.SetError(StartYearEntry, "Required input data information missing.")
            Exit Sub
        End If
        If StartMonthEntry.Text = "" Then
            EntryError.SetError(StartMonthEntry, "Required input data information missing.")
            Exit Sub
        End If
        If StartDayEntry.Text = "" Then
            EntryError.SetError(StartDayEntry, "Required input data information missing.")
            Exit Sub
        End If
        Select Case ScreenPrecip_StartMonth
            Case 1, 3, 5, 7, 8, 10, 12
                If ScreenPrecip_StartDay > 31 Then
                    EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                    Exit Sub
                End If
            Case 4, 6, 9, 11
                If ScreenPrecip_StartDay > 30 Then
                    EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                    Exit Sub
                End If
            Case 2
                If Date.IsLeapYear(ScreenPrecip_StartYear) = True Then
                    If ScreenPrecip_StartDay > 29 Then
                        EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Else
                    If ScreenPrecip_StartDay > 28 Then
                        EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                End If
        End Select
        If EndYearEntry.Text = "" Then
            EntryError.SetError(EndYearEntry, "Required input data information missing.")
            Exit Sub
        End If
        If EndMonthEntry.Text = "" Then
            EntryError.SetError(EndMonthEntry, "Required input data information missing.")
            Exit Sub
        End If
        If EndDayEntry.Text = "" Then
            EntryError.SetError(EndDayEntry, "Required input data information missing.")
            Exit Sub
        End If
        Select Case ScreenPrecip_EndMonth
            Case 1, 3, 5, 7, 8, 10, 12
                If ScreenPrecip_EndDay > 31 Then
                    EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                    Exit Sub
                End If
            Case 4, 6, 9, 11
                If ScreenPrecip_EndDay > 30 Then
                    EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                    Exit Sub
                End If
            Case 2
                If Date.IsLeapYear(ScreenPrecip_EndYear) = True Then
                    If ScreenPrecip_EndDay > 29 Then
                        EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Else
                    If ScreenPrecip_EndDay > 28 Then
                        EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                End If
        End Select
        If ScreenPrecip_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If ScreenPrecip_HistoricalInput = ScreenPrecip_Output Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        ScreenPrecip_Exception = 0
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim StoreOnOff As Integer
        Dim FoundStart As Integer
        Dim i As Integer
        Dim Amount As Integer
        Year = 0
        StoreOnOff = 0
        FoundStart = 0
        'Initialize Observation Arrays
        Dim CompletePrecipDays(200) As Integer
        Dim JanPrecipDays(200) As Integer
        Dim FebPrecipDays(200) As Integer
        Dim MarPrecipDays(200) As Integer
        Dim AprPrecipDays(200) As Integer
        Dim MayPrecipDays(200) As Integer
        Dim JunPrecipDays(200) As Integer
        Dim JulPrecipDays(200) As Integer
        Dim AugPrecipDays(200) As Integer
        Dim SepPrecipDays(200) As Integer
        Dim OctPrecipDays(200) As Integer
        Dim NovPrecipDays(200) As Integer
        Dim DecPrecipDays(200) As Integer
        'Initialize Output File and Set Header
        FileOpen(1, ScreenPrecip_Output, OpenMode.Output)
        PrintLine(1, ScreenPrecip_Header)
        PrintLine(1, "")
        PrintLine(1, "Month" & vbTab & "Amount" & vbTab & "Days")
        'Initialize StreamReader
        Dim HistoricalInFile As New System.IO.StreamReader(ScreenPrecip_HistoricalInput)
        Dim HistoricalLine As String
        Dim HistoricalFields As String()
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While HistoricalInFile.Peek <> -1
            HistoricalLine = HistoricalInFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReaders
        HistoricalInFile.DiscardBufferedData()
        HistoricalInFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set Historical StreamReader to Data Start
        If ScreenPrecip_HistRowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ScreenPrecip_HistRowStart - 2
                HistoricalLine = HistoricalInFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = FileComplete - (ScreenPrecip_HistRowStart - 1)
        End If
        'Skip Column Headers
        HistoricalLine = HistoricalInFile.ReadLine()
        FileProgress = FileProgress + 1
        'Calculate Precipitation Frequencies
        Try
            While HistoricalInFile.Peek <> -1
                HistoricalLine = HistoricalInFile.ReadLine()
                HistoricalFields = HistoricalLine.Split(ControlChars.Tab)
                Year = HistoricalFields(0)
                Month = HistoricalFields(1)
                Day = HistoricalFields(2)
                If Year = ScreenPrecip_StartYear Then
                    If Month = ScreenPrecip_StartMonth Then
                        If Day = ScreenPrecip_StartDay Then
                            'Turn On Store
                            StoreOnOff = 1
                            FoundStart = 1
                        End If
                    End If
                End If
                If StoreOnOff = 1 Then
                    'Store Precipitation Frequency Data
                    If HistoricalFields(ScreenPrecip_PrecipColumn - 1) <> -99 And HistoricalFields(ScreenPrecip_PrecipColumn - 1) <> -1 And HistoricalFields(ScreenPrecip_PrecipColumn - 1) <= 2 Then
                        'Entire Time Period
                        Amount = (HistoricalFields(ScreenPrecip_PrecipColumn - 1)) * 100
                        CompletePrecipDays(Amount) = CompletePrecipDays(Amount) + 1
                        'Time Period by Month
                        Select Case Month
                            Case 1
                                JanPrecipDays(Amount) = JanPrecipDays(Amount) + 1
                            Case 2
                                FebPrecipDays(Amount) = FebPrecipDays(Amount) + 1
                            Case 3
                                MarPrecipDays(Amount) = MarPrecipDays(Amount) + 1
                            Case 4
                                AprPrecipDays(Amount) = AprPrecipDays(Amount) + 1
                            Case 5
                                MayPrecipDays(Amount) = MayPrecipDays(Amount) + 1
                            Case 6
                                JunPrecipDays(Amount) = JunPrecipDays(Amount) + 1
                            Case 7
                                JulPrecipDays(Amount) = JulPrecipDays(Amount) + 1
                            Case 8
                                AugPrecipDays(Amount) = AugPrecipDays(Amount) + 1
                            Case 9
                                SepPrecipDays(Amount) = SepPrecipDays(Amount) + 1
                            Case 10
                                OctPrecipDays(Amount) = OctPrecipDays(Amount) + 1
                            Case 11
                                NovPrecipDays(Amount) = NovPrecipDays(Amount) + 1
                            Case 12
                                DecPrecipDays(Amount) = DecPrecipDays(Amount) + 1
                        End Select
                    End If
                End If
                If Year = ScreenPrecip_EndYear Then
                    If Month = ScreenPrecip_EndMonth Then
                        If Day = ScreenPrecip_EndDay Then
                            'Turn Off Store with Exit While Flag
                            StoreOnOff = 2
                        End If
                    End If
                End If
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the input data file at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Screen Precipitation Data")
            Else
                MsgBox("Error reading the input data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Screen Precipitation Data")
            End If
            HistoricalInFile.Close()
            FileClose(1)
            ScreenPrecip_Exception = 1
            Exit Sub
        End Try
        If FoundStart = 0 Then
            MsgBox("Unable to find start date in the input data.  Recheck the start date.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Screen Precipitation Data")
            HistoricalInFile.Close()
            FileClose(1)
            ScreenPrecip_Exception = 1
            Exit Sub
        End If
        'Output Results
        For i = 1 To 200
            PrintLine(1, "All" & vbTab & i & vbTab & CompletePrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Jan" & vbTab & i & vbTab & JanPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Feb" & vbTab & i & vbTab & FebPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Mar" & vbTab & i & vbTab & MarPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Apr" & vbTab & i & vbTab & AprPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "May" & vbTab & i & vbTab & MayPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Jun" & vbTab & i & vbTab & JunPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Jul" & vbTab & i & vbTab & JulPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Aug" & vbTab & i & vbTab & AugPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Sep" & vbTab & i & vbTab & SepPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Oct" & vbTab & i & vbTab & OctPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Nov" & vbTab & i & vbTab & NovPrecipDays(i))
        Next i
        For i = 1 To 200
            PrintLine(1, "Dec" & vbTab & i & vbTab & DecPrecipDays(i))
        Next i
        'Close Files
        FileClose(1)
        HistoricalInFile.Close()
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ScreenPrecip_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
                'Show Frequency Histogram
                If ScreenPrecipFigDisp = 0 Then
                    ScreenPrecipFigDisp = 1
                    Dim frmScreenPrecipFig As New ScreenPrecipFig
                    frmScreenPrecipFig.MdiParent = MainParent
                    frmScreenPrecipFig.Show()
                Else
                    MsgBox("Only one frequency histogram can be displayed at a time.  Please close the displayed histogram, and run the Screen Precipitation Tool again.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Screen Precipitation Data")
                End If
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub PrecipAmountsDays_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ScreenPrecipDisp = 0
    End Sub
End Class