Imports ZedGraph
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices
Public Class SpaghettiPlots

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input Variable
    Private SpaghettiPlots_Input As String
    'Initialize Global Form Variables
    Private SpaghettiPlots_RowStart As Integer
    Private SpaghettiPlots_TotalStations As Integer
    Private SpaghettiPlots_Title As String
    Private SpaghettiPlots_YAxisLabel As String
    Private SpaghettiPlots_StartYear As Integer
    Private SpaghettiPlots_StartMonth As Integer
    Private SpaghettiPlots_StartDay As Integer
    Private SpaghettiPlots_EndYear As Integer
    Private SpaghettiPlots_EndMonth As Integer
    Private SpaghettiPlots_EndDay As Integer
    Private SpaghettiPlots_Highlight As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub SpaghettiPlots_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Spaghetti Plot Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = SpaghettiPlots_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            SpaghettiPlots_Input = FileSelect.FileName
            InputFileLocation.Text = SpaghettiPlots_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStartEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStartEntry.TextChanged
        EntryError.SetError(DataRowStartEntry, "")
        If DataRowStartEntry.Text <> "" Then
            Try
                SpaghettiPlots_RowStart = DataRowStartEntry.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStartEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TotalStationsEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TotalStationsEntry.TextChanged
        EntryError.SetError(TotalStationsEntry, "")
        If TotalStationsEntry.Text <> "" Then
            Try
                SpaghettiPlots_TotalStations = TotalStationsEntry.Text
            Catch ex As Exception
                EntryError.SetError(TotalStationsEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TitleEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TitleEntry.TextChanged
        EntryError.SetError(TitleEntry, "")
        SpaghettiPlots_Title = TitleEntry.Text
    End Sub

    Private Sub YAxisEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles YAxisEntry.TextChanged
        EntryError.SetError(YAxisEntry, "")
        SpaghettiPlots_YAxisLabel = YAxisEntry.Text
    End Sub

    Private Sub DailyFormat_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DailyFormat.CheckedChanged
        If DailyFormat.Checked = True Then
            StartMonthEntry_Label.Enabled = True
            StartMonthEntry.Enabled = True
            StartDayEntry_Label.Enabled = True
            StartDayEntry.Enabled = True
            EndMonthEntry_Label.Enabled = True
            EndMonthEntry.Enabled = True
            EndDayEntry_Label.Enabled = True
            EndDayEntry.Enabled = True
        End If
    End Sub

    Private Sub MonthlyFormat_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MonthlyFormat.CheckedChanged
        If MonthlyFormat.Checked = True Then
            StartMonthEntry_Label.Enabled = True
            StartMonthEntry.Enabled = True
            StartDayEntry_Label.Enabled = False
            StartDayEntry.Enabled = False
            EndMonthEntry_Label.Enabled = True
            EndMonthEntry.Enabled = True
            EndDayEntry_Label.Enabled = False
            EndDayEntry.Enabled = False
        End If
    End Sub

    Private Sub AnnualFormat_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AnnualFormat.CheckedChanged
        If AnnualFormat.Checked = True Then
            StartMonthEntry_Label.Enabled = False
            StartMonthEntry.Enabled = False
            StartDayEntry_Label.Enabled = False
            StartDayEntry.Enabled = False
            EndMonthEntry_Label.Enabled = False
            EndMonthEntry.Enabled = False
            EndDayEntry_Label.Enabled = False
            EndDayEntry.Enabled = False
        End If
    End Sub

    Private Sub StartYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartYearEntry.TextChanged
        EntryError.SetError(StartYearEntry, "")
        If StartYearEntry.Text <> "" Then
            Try
                SpaghettiPlots_StartYear = StartYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(StartYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartMonthEntry.TextChanged
        EntryError.SetError(StartMonthEntry, "")
        If StartMonthEntry.Text <> "" Then
            Try
                SpaghettiPlots_StartMonth = StartMonthEntry.Text
                If SpaghettiPlots_StartMonth > 12 Or SpaghettiPlots_StartMonth < 1 Then
                    EntryError.SetError(StartMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartDayEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartDayEntry.TextChanged
        EntryError.SetError(StartDayEntry, "")
        If StartDayEntry.Text <> "" Then
            Try
                SpaghettiPlots_StartDay = StartDayEntry.Text
                If SpaghettiPlots_StartDay < 1 Then
                    EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(StartDayEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndYearEntry.TextChanged
        EntryError.SetError(EndYearEntry, "")
        If EndYearEntry.Text <> "" Then
            Try
                SpaghettiPlots_EndYear = EndYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(EndYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndMonthEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndMonthEntry.TextChanged
        EntryError.SetError(EndMonthEntry, "")
        If EndMonthEntry.Text <> "" Then
            Try
                SpaghettiPlots_EndMonth = EndMonthEntry.Text
                If SpaghettiPlots_EndMonth > 12 Or SpaghettiPlots_EndMonth < 1 Then
                    EntryError.SetError(EndMonthEntry, "Month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndMonthEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndDayEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndDayEntry.TextChanged
        EntryError.SetError(EndDayEntry, "")
        If EndDayEntry.Text <> "" Then
            Try
                SpaghettiPlots_EndDay = EndDayEntry.Text
                If SpaghettiPlots_EndDay < 1 Then
                    EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                End If
            Catch ex As Exception
                EntryError.SetError(EndDayEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub Plot_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Plot.Click
        'Draw New Plot With No Highlighted Station
        SpaghettiPlots_Highlight = 0
        DrawSpaghettiPlot()
    End Sub

    Private Sub StationList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StationList.SelectedIndexChanged
        'ReDraw Plot to Highlight Selected Station
        SpaghettiPlots_Highlight = 1
        DrawSpaghettiPlot()
    End Sub

    Private Sub DrawSpaghettiPlot()
        'Check for User Input Errors
        If SpaghettiPlots_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStartEntry.Text = "" Then
            EntryError.SetError(DataRowStartEntry, "Required input data information missing.")
            Exit Sub
        End If
        If TotalStationsEntry.Text = "" Then
            EntryError.SetError(TotalStationsEntry, "Required input data information missing.")
            Exit Sub
        End If
        If TitleEntry.Text = "" Then
            EntryError.SetError(TitleEntry, "Required title information missing.")
            Exit Sub
        End If
        If YAxisEntry.Text = "" Then
            EntryError.SetError(YAxisEntry, "Required y-axis information missing.")
            Exit Sub
        End If
        If StartYearEntry.Text = "" Then
            EntryError.SetError(StartYearEntry, "Required start year missing.")
            Exit Sub
        End If
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            If StartMonthEntry.Text = "" Then
                EntryError.SetError(StartMonthEntry, "Required start month missing.")
                Exit Sub
            End If
            If EndMonthEntry.Text = "" Then
                EntryError.SetError(EndMonthEntry, "Required end month missing.")
                Exit Sub
            End If
        End If
        If DailyFormat.Checked = True Then
            If StartDayEntry.Text = "" Then
                EntryError.SetError(StartDayEntry, "Required start day missing.")
                Exit Sub
            End If
            Select Case SpaghettiPlots_StartMonth
                Case 1, 3, 5, 7, 8, 10, 12
                    If SpaghettiPlots_StartDay > 31 Then
                        EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 4, 6, 9, 11
                    If SpaghettiPlots_StartDay > 30 Then
                        EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 2
                    If Date.IsLeapYear(SpaghettiPlots_StartYear) = True Then
                        If SpaghettiPlots_StartDay > 29 Then
                            EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    Else
                        If SpaghettiPlots_StartDay > 28 Then
                            EntryError.SetError(StartDayEntry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    End If
            End Select
        End If
        If EndYearEntry.Text = "" Then
            EntryError.SetError(EndYearEntry, "Required end year missing.")
            Exit Sub
        End If
        If DailyFormat.Checked = True Then
            If EndDayEntry.Text = "" Then
                EntryError.SetError(EndDayEntry, "Required end day missing.")
                Exit Sub
            End If
            Select Case SpaghettiPlots_EndMonth
                Case 1, 3, 5, 7, 8, 10, 12
                    If SpaghettiPlots_EndDay > 31 Then
                        EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 4, 6, 9, 11
                    If SpaghettiPlots_EndDay > 30 Then
                        EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                        Exit Sub
                    End If
                Case 2
                    If Date.IsLeapYear(SpaghettiPlots_EndYear) = True Then
                        If SpaghettiPlots_EndDay > 29 Then
                            EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    Else
                        If SpaghettiPlots_EndDay > 28 Then
                            EntryError.SetError(EndDayEntry, "Day of the month does not exist.")
                            Exit Sub
                        End If
                    End If
            End Select
        End If
        'Start Plot
        Plot.Text = "Plotting..."
        Plot.Enabled = False
        'Initialize Variables
        Dim i As Integer
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim StartDate As Date
        Dim EndDate As Date
        Dim DataValue As Double
        Dim CurrentDate As Date
        Dim StoreOnOff As Integer
        StoreOnOff = 0
        Dim StationName(SpaghettiPlots_TotalStations - 1) As String
        Dim StationLine As LineItem
        Dim StationData As PointPairList
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(SpaghettiPlots_Input)
        Dim Line As String
        Dim Fields As String()
        If SpaghettiPlots_Highlight = 0 Then
            StationList.Items.Clear()
        End If
        'Set StreamReader to Data Start
        If SpaghettiPlots_RowStart <> 2 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To SpaghettiPlots_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
        End If
        If DailyFormat.Checked = True Then
            'Set Segment Start and End Dates
            StartDate = DateValue(SpaghettiPlots_StartMonth & "/" & SpaghettiPlots_StartDay & "/" & SpaghettiPlots_StartYear)
            EndDate = DateValue(SpaghettiPlots_EndMonth & "/" & SpaghettiPlots_EndDay & "/" & SpaghettiPlots_EndYear)
        End If
        If MonthlyFormat.Checked = True Then
            'Set Segment Start and End Dates
            StartDate = DateValue(SpaghettiPlots_StartMonth & "/1/" & SpaghettiPlots_StartYear)
            EndDate = DateValue(SpaghettiPlots_EndMonth & "/1/" & SpaghettiPlots_EndYear)
        End If
        'Set Spaghetti Chart Properties
        Dim SpaghettiPane As GraphPane = SpaghettiChart.GraphPane
        SpaghettiPane.CurveList.Clear()
        SpaghettiChart.ZoomOutAll(SpaghettiPane)
        SpaghettiChart.MasterPane.Border.IsVisible = False
        SpaghettiPane.Border.IsVisible = False
        SpaghettiPane.Chart.Border.IsVisible = False
        'Set Title
        If DailyFormat.Checked = True Then
            SpaghettiPane.Title.Text = SpaghettiPlots_Title & ControlChars.CrLf & StartDate.Month & "/" & StartDate.Day & "/" & StartDate.Year & " - " & EndDate.Month & "/" & EndDate.Day & "/" & EndDate.Year
        End If
        If MonthlyFormat.Checked = True Then
            SpaghettiPane.Title.Text = SpaghettiPlots_Title & ControlChars.CrLf & StartDate.Month & "/" & StartDate.Year & " - " & EndDate.Month & "/" & EndDate.Year
        End If
        If AnnualFormat.Checked = True Then
            SpaghettiPane.Title.Text = SpaghettiPlots_Title & ControlChars.CrLf & SpaghettiPlots_StartYear & " - " & SpaghettiPlots_EndYear
        End If
        SpaghettiPane.Title.FontSpec.Size = 16
        SpaghettiPane.Title.FontSpec.IsBold = True
        'Set X-Axis
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            SpaghettiPane.XAxis.Type = AxisType.Date
            SpaghettiPane.XAxis.Scale.Min = StartDate.ToOADate
            SpaghettiPane.XAxis.Scale.Max = EndDate.ToOADate
            Select Case DailyFormat.Checked
                Case True
                    SpaghettiPane.XAxis.Scale.MajorUnit = DateUnit.Day
                Case False
                    SpaghettiPane.XAxis.Scale.MajorUnit = DateUnit.Month
            End Select
        Else
            SpaghettiPane.XAxis.Scale.Min = SpaghettiPlots_StartYear
            SpaghettiPane.XAxis.Scale.Max = SpaghettiPlots_EndYear
        End If
        SpaghettiPane.XAxis.Scale.FontSpec.Size = 12
        SpaghettiPane.XAxis.Scale.FontSpec.IsBold = True
        SpaghettiPane.XAxis.Scale.LabelGap = 0.2
        If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
            SpaghettiPane.XAxis.Title.Text = "Date"
        Else
            SpaghettiPane.XAxis.Title.Text = "Year"
        End If
        SpaghettiPane.XAxis.Title.FontSpec.Size = 14
        SpaghettiPane.XAxis.Title.FontSpec.IsBold = True
        SpaghettiPane.XAxis.MajorTic.PenWidth = 2
        SpaghettiPane.XAxis.MajorTic.IsInside = False
        SpaghettiPane.XAxis.MajorTic.IsOpposite = False
        SpaghettiPane.XAxis.MajorTic.Size = 10
        SpaghettiPane.XAxis.MinorTic.IsAllTics = False
        SpaghettiPane.X2Axis.MajorTic.IsAllTics = False
        SpaghettiPane.X2Axis.MinorTic.IsAllTics = False
        'Set Y-Axis
        SpaghettiPane.YAxis.Scale.FontSpec.Size = 12
        SpaghettiPane.YAxis.Scale.FontSpec.IsBold = True
        SpaghettiPane.YAxis.Scale.LabelGap = 0.2
        SpaghettiPane.YAxis.Title.Text = SpaghettiPlots_YAxisLabel
        SpaghettiPane.YAxis.Title.FontSpec.Size = 14
        SpaghettiPane.YAxis.Title.FontSpec.IsBold = True
        SpaghettiPane.YAxis.MajorGrid.IsZeroLine = False
        SpaghettiPane.YAxis.MajorTic.PenWidth = 2
        SpaghettiPane.YAxis.MajorTic.IsInside = False
        SpaghettiPane.YAxis.MajorTic.IsOpposite = False
        SpaghettiPane.YAxis.MajorTic.Size = 10
        SpaghettiPane.YAxis.MinorTic.IsAllTics = False
        SpaghettiPane.Y2Axis.MajorTic.IsAllTics = False
        SpaghettiPane.Y2Axis.MinorTic.IsAllTics = False
        'Turn off Legend
        SpaghettiPane.Legend.IsVisible = False
        'Import Station Data
        For i = 0 To (SpaghettiPlots_TotalStations - 1)
            'Read Column Headers
            Line = InFile.ReadLine()
            Fields = Line.Split(ControlChars.Tab)
            Try
                If DailyFormat.Checked = True Then
                    StationName(i) = Fields(SpaghettiPlots_TotalStations + 2)
                Else
                    StationName(i) = Fields(SpaghettiPlots_TotalStations)
                End If
            Catch ex As Exception
                MsgBox("The number of input stations input does not match the input file.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Spaghetti Plot")
                Plot.Text = "Plot"
                Plot.Enabled = True
                InFile.Close()
                Exit Sub
            End Try
            If DailyFormat.Checked = True Then
                StationName(i) = Fields(i + 3)
            End If
            If MonthlyFormat.Checked = True Then
                StationName(i) = Fields(i + 2)
            End If
            If AnnualFormat.Checked = True Then
                StationName(i) = Fields(i + 1)
            End If
            If SpaghettiPlots_Highlight = 0 Then
                StationList.Items.Add(StationName(i))
            End If
            If DailyFormat.Checked = True Or MonthlyFormat.Checked = True Then
                'Daily or Monthly Format
                Try
                    StationData = New PointPairList()
                    While InFile.Peek <> -1
                        'Read Station Data
                        Line = InFile.ReadLine()
                        Fields = Line.Split(ControlChars.Tab)
                        Year = Fields(0)
                        Month = Fields(1)
                        Select Case DailyFormat.Checked
                            Case True
                                Day = Fields(2)
                                CurrentDate = DateValue(Month & "/" & Day & "/" & Year)
                            Case False
                                CurrentDate = DateValue(Month & "/1/" & Year)
                        End Select
                        If CurrentDate = StartDate Then
                            'Start Date Reached
                            StoreOnOff = 1
                        End If
                        Select Case DailyFormat.Checked
                            Case True
                                DataValue = Fields(i + 3)
                            Case False
                                DataValue = Fields(i + 2)
                        End Select
                        If StoreOnOff = 1 Then
                            If DataValue <> -99 Then
                                StationData.Add(CurrentDate.ToOADate, DataValue)
                            Else
                                'Denote as Missing
                                StationData.Add(CurrentDate.ToOADate, PointPair.Missing)
                            End If
                        End If
                        If CurrentDate = EndDate Then
                            'End Date Reached
                            StoreOnOff = 0
                        End If
                    End While
                Catch ex As Exception
                    Select Case DailyFormat.Checked
                        Case True
                            MsgBox("Error reading the input data near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Spaghetti Plot")
                        Case False
                            MsgBox("Error reading the input data near the following Year/Month:  " & Year & "/" & Month & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Spaghetti Plot")
                    End Select
                    Plot.Text = "Plot"
                    Plot.Enabled = True
                    InFile.Close()
                    Exit Sub
                End Try
            Else
                'Annual Format
                Try
                    StationData = New PointPairList()
                    While InFile.Peek <> -1
                        'Read Station Temperature Data
                        Line = InFile.ReadLine()
                        Fields = Line.Split(ControlChars.Tab)
                        Year = Fields(0)
                        If Year = SpaghettiPlots_StartYear Then
                            'Start Year Reached
                            StoreOnOff = 1
                        End If
                        DataValue = Fields(i + 1)
                        If StoreOnOff = 1 Then
                            If DataValue <> -99 Then
                                StationData.Add(Year, DataValue)
                            Else
                                'Denote as Missing
                                StationData.Add(Year, PointPair.Missing)
                            End If
                        End If
                        If Year = SpaghettiPlots_EndYear Then
                            'End Year Reached
                            StoreOnOff = 0
                        End If
                    End While
                Catch ex As Exception
                    MsgBox("Error reading the input data near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Spaghetti Plot")
                    Plot.Text = "Plot"
                    Plot.Enabled = True
                    InFile.Close()
                    Exit Sub
                End Try
            End If
            'Add Station Data to Plot
            SpaghettiPane.AddCurve(StationName(i), StationData, Color.Black, SymbolType.None)
            'Reset PointPairList and StreamReader for Next Station
            InFile.DiscardBufferedData()
            InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
            'Set StreamReader to Data Start
            If SpaghettiPlots_RowStart <> 2 Then
                'Header Exists so Move LineInput Down
                Dim Row As Integer
                For Row = 0 To SpaghettiPlots_RowStart - 2
                    Line = InFile.ReadLine()
                Next Row
            End If
        Next i
        'Finalize Curves
        For i = 0 To (SpaghettiPlots_TotalStations - 1)
            StationLine = SpaghettiPane.CurveList(i)
            StationLine.Line.SmoothTension = 0
            StationLine.Line.Width = 1.5
            If StationList.SelectedIndex = i Then
                'Highlight Selected Station Curve in Red
                StationLine.Color = Color.Red
                StationLine.Line.Width = 2
            End If
        Next i
        'Calculate Axis Scale Ranges and Refresh Chart
        SpaghettiPane.AxisChange()
        SpaghettiChart.Refresh()
        If SpaghettiChart.Visible = False Then
            SpaghettiChart.Visible = True
            CopyBitmap.Enabled = True
            CopyMetafile.Enabled = True
            SaveChart.Enabled = True
            PrintChart.Enabled = True
        End If
        'End Run
        InFile.Close()
        Plot.Text = "Plot"
        Plot.Enabled = True
    End Sub

    Private Sub CopyBitmap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyBitmap.Click
        Clipboard.SetImage(SpaghettiChart.MasterPane.GetImage(2307, 1500, 288, False))
    End Sub

    Private Sub CopyMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyMetafile.Click
        Dim g As Graphics = Me.CreateGraphics()
        Dim hdc As System.IntPtr = g.GetHdc()
        Dim metaFile As New Imaging.Metafile(hdc, Imaging.EmfType.EmfOnly)
        g.ReleaseHdc(hdc)
        g.Dispose()
        Dim gMeta As Graphics = Graphics.FromImage(metaFile)
        SpaghettiChart.MasterPane.Draw(gMeta)
        gMeta.Dispose()
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, metaFile)
    End Sub

    Private Sub SaveChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveChart.Click
        Dim SaveIntTempRangePlot As New SaveFileDialog
        Dim OutputImage As String
        SaveIntTempRangePlot.Title = "Save Spaghetti Plot"
        SaveIntTempRangePlot.Filter = "Bitmap (*.bmp)|*.bmp|GIF (*.gif)|*.gif|JPEG (*.jpg)|*.jpg|PNG (*.png)|*.png|TIFF (*.tif)|*.tif"
        If SaveIntTempRangePlot.ShowDialog = Windows.Forms.DialogResult.OK Then
            OutputImage = SaveIntTempRangePlot.FileName
            If SaveIntTempRangePlot.FileName.EndsWith("bmp") Then
                SpaghettiChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Bmp)
            End If
            If SaveIntTempRangePlot.FileName.EndsWith("gif") Then
                SpaghettiChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Gif)
            End If
            If SaveIntTempRangePlot.FileName.EndsWith("jpg") Then
                SpaghettiChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Jpeg)
            End If
            If SaveIntTempRangePlot.FileName.EndsWith("png") Then
                SpaghettiChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Png)
            End If
            If SaveIntTempRangePlot.FileName.EndsWith("tif") Then
                SpaghettiChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Tiff)
            End If
        End If
    End Sub

    Private Sub PrintChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrintChart.Click
        SpaghettiChart.DoPrint()
    End Sub

    Private Sub TempChart_ContextMenuBuilder(ByVal sender As ZedGraph.ZedGraphControl, ByVal menuStrip As System.Windows.Forms.ContextMenuStrip, ByVal mousePt As System.Drawing.Point, ByVal objState As ZedGraph.ZedGraphControl.ContextMenuObjectState) Handles SpaghettiChart.ContextMenuBuilder
        'Remove Specific Functions from Spaghetti Chart Context Menu for Use Elsewhere
        Dim Item As ToolStripMenuItem
        Dim CopyItem As ToolStripMenuItem
        Dim PageSetupItem As ToolStripMenuItem
        Dim PrintItem As ToolStripMenuItem
        Dim SaveAsItem As ToolStripMenuItem
        Dim SetScaleItem As ToolStripMenuItem
        For Each Item In menuStrip.Items
            If Item.Tag = "copy" Then
                CopyItem = Item
            End If
            If Item.Tag = "page_setup" Then
                PageSetupItem = Item
            End If
            If Item.Tag = "print" Then
                PrintItem = Item
            End If
            If Item.Tag = "save_as" Then
                SaveAsItem = Item
            End If
            If Item.Tag = "set_default" Then
                SetScaleItem = Item
            End If
        Next Item
        menuStrip.Items.Remove(CopyItem)
        menuStrip.Items.Remove(PageSetupItem)
        menuStrip.Items.Remove(PrintItem)
        menuStrip.Items.Remove(SaveAsItem)
        menuStrip.Items.Remove(SetScaleItem)
    End Sub

    Private Sub SpaghettiPlots_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        SpaghettiPlotsDisp = 0
    End Sub
End Class