Public Class TempUnits

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private TempUnits_Input As String
    Private TempUnits_Output As String
    'Initialize Global Form Variables
    Private TempUnits_RowStart As Integer
    Private TempUnits_DataColumn As Integer
    Private TempUnits_ErrorColumn As Integer
    Private TempUnits_InputDegF As Integer
    Private TempUnits_InputDegC As Integer
    Private TempUnits_ErrorBars As Integer
    Private TempUnits_OutputDegF As Integer
    Private TempUnits_OutputDegC As Integer
    Private TempUnits_Header As String
    Private TempUnits_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub TempUnits_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Convert Temperature Units Tool
        InputDegC.Text = Chr(176) & "C"
        InputDegF.Text = Chr(176) & "F"
        OutputDegC.Text = Chr(176) & "C"
        OutputDegF.Text = Chr(176) & "F"
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Temperature Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = TempUnits_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            TempUnits_Input = FileSelect.FileName
            InputFileLocation.Text = TempUnits_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                TempUnits_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DataColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataColumn.TextChanged
        EntryError.SetError(DataColumn, "")
        If DataColumn.Text <> "" Then
            Try
                TempUnits_DataColumn = DataColumn.Text
            Catch ex As Exception
                EntryError.SetError(DataColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub InputDegC_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InputDegC.CheckedChanged
        EntryError.SetError(OutputDegF, "")
    End Sub

    Private Sub InputDegF_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InputDegF.CheckedChanged
        EntryError.SetError(OutputDegF, "")
    End Sub

    Private Sub ErrorBars_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorBars.CheckedChanged
        Select Case ErrorBars.Checked
            Case True
                ErrorColumn_Label.Enabled = True
                ErrorColumn.Enabled = True
            Case False
                ErrorColumn_Label.Enabled = False
                ErrorColumn.Enabled = False
        End Select
    End Sub

    Private Sub ErrorColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorColumn.TextChanged
        EntryError.SetError(ErrorColumn, "")
        If ErrorColumn.Text <> "" Then
            Try
                TempUnits_ErrorColumn = ErrorColumn.Text
            Catch ex As Exception
                EntryError.SetError(ErrorColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = TempUnits_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            TempUnits_Output = FileOutput.FileName
            OutputFileLocation.Text = TempUnits_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub OutputDegC_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OutputDegC.CheckedChanged
        EntryError.SetError(OutputDegF, "")
    End Sub

    Private Sub OutputDegF_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OutputDegF.CheckedChanged
        EntryError.SetError(OutputDegF, "")
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        TempUnits_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If TempUnits_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If DataColumn.Text = "" Then
            EntryError.SetError(DataColumn, "Required input data information missing.")
            Exit Sub
        End If
        If ErrorBars.Checked = True Then
            If ErrorColumn.Text = "" Then
                EntryError.SetError(ErrorColumn, "Required input data information missing.")
                Exit Sub
            End If
        End If
        If TempUnits_Output = "" Then
            EntryError.SetError(OutputFileLocation, "Output dataset missing.")
            Exit Sub
        End If
        If TempUnits_Output = TempUnits_Input Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If (InputDegF.Checked = True And OutputDegF.Checked = True) Or (InputDegC.Checked = True And OutputDegC.Checked = True) Then
            EntryError.SetError(OutputDegF, "Input and output units cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        TempUnits_Exception = 0
        If InputDegF.Checked = True Then
            TempUnits_InputDegF = 1
        Else
            TempUnits_InputDegF = 0
        End If
        If InputDegC.Checked = True Then
            TempUnits_InputDegC = 1
        Else
            TempUnits_InputDegC = 0
        End If
        If ErrorBars.Checked = True Then
            TempUnits_ErrorBars = 1
        Else
            TempUnits_ErrorBars = 0
        End If
        If OutputDegF.Checked = True Then
            TempUnits_OutputDegF = 1
        Else
            TempUnits_OutputDegF = 0
        End If
        If OutputDegC.Checked = True Then
            TempUnits_OutputDegC = 1
        Else
            TempUnits_OutputDegC = 0
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim Day As Integer
        Dim TempValue As Double
        Dim ErrorValue As Double
        'Initialize Output File and Set Header
        FileOpen(1, TempUnits_Output, OpenMode.Output)
        PrintLine(1, TempUnits_Header)
        PrintLine(1, "")
        If TempUnits_ErrorBars = 1 Then
            PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & "Temp" & vbTab & "Error")
        Else
            PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Day" & vbTab & "Temp")
        End If
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(TempUnits_Input)
        Dim Line As String
        Dim Fields As String()
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set Input Temperature StreamReader to Data Start
        If TempUnits_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To TempUnits_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = FileComplete - (TempUnits_RowStart - 1)
        End If
        'Skip Column Headers
        Line = InFile.ReadLine()
        FileProgress = FileProgress + 1
        'Convert Temperature Units
        Try
            While InFile.Peek <> -1
                Line = InFile.ReadLine()
                Fields = Line.Split(ControlChars.Tab)
                Year = Fields(0)
                Month = Fields(1)
                Day = Fields(2)
                TempValue = Fields(TempUnits_DataColumn - 1)
                If TempUnits_ErrorBars = 1 Then
                    ErrorValue = Fields(TempUnits_ErrorColumn - 1)
                End If
                'Convert Temperature Value
                If TempValue <> -99 Then
                    If TempUnits_OutputDegF = 1 And TempUnits_InputDegC = 1 Then
                        'Convert from DegC to DegF
                        TempValue = DegCDegF(TempValue)
                    End If
                    If TempUnits_OutputDegC = 1 And TempUnits_InputDegF = 1 Then
                        'Convert from DegF to DegC
                        TempValue = DegFDegC(TempValue)
                    End If
                    TempValue = Math.Round(TempValue, 1)
                End If
                'Convert Temperature Error Value (+/- Error so Equation is Different)
                If TempUnits_ErrorBars = 1 Then
                    If TempValue <> -99 Then
                        If TempUnits_OutputDegF = 1 And TempUnits_InputDegC = 1 Then
                            'Convert from DegC to DegF
                            ErrorValue = 1.8 * ErrorValue
                        End If
                        If TempUnits_OutputDegC = 1 And TempUnits_InputDegF = 1 Then
                            'Convert from DegF to DegC
                            ErrorValue = (ErrorValue) / 1.8
                        End If
                        ErrorValue = Math.Round(ErrorValue, 1)
                    End If
                End If
                'Output New Temperature
                If TempUnits_ErrorBars = 1 Then
                    PrintLine(1, Year & vbTab & Month & vbTab & Day & vbTab & TempValue & vbTab & ErrorValue)
                Else
                    PrintLine(1, Year & vbTab & Month & vbTab & Day & vbTab & TempValue)
                End If
                'Update Progress Bar
                FileProgress = FileProgress + 1
                Percent = (FileProgress / FileComplete) * 100
                BackgroundRun.ReportProgress(Percent)
            End While
        Catch ex As Exception
            If Year <> 0 Then
                MsgBox("Error reading the temperature data file at or near the following Year/Month/Day:  " & Year & "/" & Month & "/" & Day & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Convert Temperature Units")
            Else
                MsgBox("Error reading the temperature data file near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Convert Temperature Units")
            End If
            InFile.Close()
            FileClose(1)
            TempUnits_Exception = 1
            Exit Sub
        End Try
        'Close Files
        InFile.Close()
        FileClose(1)
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case TempUnits_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub TempUnits_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        TempUnitsDisp = 0
    End Sub
End Class