Public Class ThresholdAnalysis

    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input/Output Variables
    Private ThresholdAnalysis_Input As String
    Private ThresholdAnalysis_Output As String
    'Initialize Global Form Variables
    Private ThresholdAnalysis_RowStart As Integer
    Private ThresholdAnalysis_DataColumn As Integer
    Private ThresholdAnalysis_ErrorBars As Integer
    Private ThresholdAnalysis_ErrorColumn As Integer
    Private ThresholdAnalysis_Daily As Integer
    Private ThresholdAnalysis_Monthly As Integer
    Private ThresholdAnalysis_Threshold As Double
    Private ThresholdAnalysis_ThresholdAbove As Integer
    Private ThresholdAnalysis_ThresholdBelow As Integer
    Private ThresholdAnalysis_Percentile As Integer
    Private ThresholdAnalysis_ComputeMonth As Integer
    Private ThresholdAnalysis_ComputeRange As Integer
    Private ThresholdAnalysis_Month As Integer
    Private ThresholdAnalysis_StartMonth As Integer
    Private ThresholdAnalysis_EndMonth As Integer
    Private ThresholdAnalysis_ThresholdPersistence As Integer
    Private ThresholdAnalysis_PersistentSpells As Integer
    Private ThresholdAnalysis_FirstLastOccurrence As Integer
    Private ThresholdAnalysis_Header As String
    Private ThresholdAnalysis_Exception As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub ThresholdAnalysis_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Threshold Analysis Tool
        ThresholdAboveBelow.SelectedIndex = 0
        Month.SelectedIndex = 0
        StartMonth.SelectedIndex = 0
        EndMonth.SelectedIndex = 0
        ThresholdAnalysis_PersistentSpells = 6
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        EntryError.SetError(OutputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = ThresholdAnalysis_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            ThresholdAnalysis_Input = FileSelect.FileName
            InputFileLocation.Text = ThresholdAnalysis_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
    End Sub

    Private Sub DataRowStart_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStart.TextChanged
        EntryError.SetError(DataRowStart, "")
        If DataRowStart.Text <> "" Then
            Try
                ThresholdAnalysis_RowStart = DataRowStart.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStart, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DataColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataColumn.TextChanged
        EntryError.SetError(DataColumn, "")
        If DataColumn.Text <> "" Then
            Try
                ThresholdAnalysis_DataColumn = DataColumn.Text
            Catch ex As Exception
                EntryError.SetError(DataColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DailyButton_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DailyButton.CheckedChanged
        Select Case DailyButton.Checked
            Case True
                ComputeMonth.Enabled = True
                ThresholdPersistence.Enabled = True
                PersistentSpells_Label.Enabled = True
                If ComputeRange.Checked = True Then
                    FirstLastOccurrence.Enabled = True
                End If
            Case False
                ComputeMonth.Checked = False
                ComputeMonth.Enabled = False
                ComputeRange.Checked = True
                ThresholdPersistence.Checked = False
                ThresholdPersistence.Enabled = False
                PersistentSpells_Label.Enabled = False
                PersistentSpells.Enabled = False
                FirstLastOccurrence.Checked = False
                FirstLastOccurrence.Enabled = False
        End Select
    End Sub

    Private Sub ErrorBars_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorBars.CheckedChanged
        Select Case ErrorBars.Checked
            Case True
                ErrorColumn_Label.Enabled = True
                ErrorColumn.Enabled = True
            Case False
                ErrorColumn_Label.Enabled = False
                ErrorColumn.Enabled = False
        End Select
    End Sub

    Private Sub ErrorColumn_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ErrorColumn.TextChanged
        EntryError.SetError(ErrorColumn, "")
        If ErrorColumn.Text <> "" Then
            Try
                ThresholdAnalysis_ErrorColumn = ErrorColumn.Text
            Catch ex As Exception
                EntryError.SetError(ErrorColumn, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ThresholdEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ThresholdEntry.TextChanged
        EntryError.SetError(ThresholdEntry, "")
        If ThresholdEntry.Text <> "" Then
            Try
                ThresholdAnalysis_Threshold = ThresholdEntry.Text
            Catch ex As Exception
                EntryError.SetError(ThresholdEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub ComputeMonth_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComputeMonth.CheckedChanged
        Select Case ComputeMonth.Checked
            Case True
                Month.Enabled = True
            Case False
                Month.Enabled = False
        End Select
    End Sub

    Private Sub Month_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Month.SelectedIndexChanged
        EntryError.SetError(Month, "")
        ThresholdAnalysis_Month = Month.SelectedIndex
    End Sub

    Private Sub ComputeRange_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComputeRange.CheckedChanged
        Select Case ComputeRange.Checked
            Case True
                StartMonth.Enabled = True
                EndMonth.Enabled = True
                FirstLastOccurrence.Enabled = True
            Case False
                StartMonth.Enabled = False
                EndMonth.Enabled = False
                FirstLastOccurrence.Enabled = False
        End Select
    End Sub

    Private Sub LastFirstOccurrence_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FirstLastOccurrence.CheckedChanged
        If FirstLastOccurrence.Checked = True Then
            ThresholdPersistence.Checked = False
            ThresholdPersistence.Enabled = False
            PersistentSpells.Enabled = False
        Else
            If MonthlyButton.Checked = False Then
                ThresholdPersistence.Enabled = True
            End If
        End If
    End Sub

    Private Sub StartMonth_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartMonth.SelectedIndexChanged
        EntryError.SetError(StartMonth, "")
        EntryError.SetError(EndMonth, "")
        ThresholdAnalysis_StartMonth = StartMonth.SelectedIndex
    End Sub

    Private Sub EndMonth_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndMonth.SelectedIndexChanged
        EntryError.SetError(StartMonth, "")
        EntryError.SetError(EndMonth, "")
        ThresholdAnalysis_EndMonth = EndMonth.SelectedIndex
    End Sub

    Private Sub ThresholdPersistence_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ThresholdPersistence.CheckedChanged
        If ThresholdPersistence.Checked = True Then
            PersistentSpells.Enabled = True
        End If
        If ThresholdPersistence.Checked = False Then
            PersistentSpells.Enabled = False
        End If
    End Sub

    Private Sub PersistentSpells_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PersistentSpells.TextChanged
        EntryError.SetError(PersistentSpells, "")
        If PersistentSpells.Text <> "" Then
            Try
                ThresholdAnalysis_PersistentSpells = PersistentSpells.Text
            Catch ex As Exception
                EntryError.SetError(PersistentSpells, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub FileOutput_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileOutput_Button.Click
        EntryError.SetError(OutputFileLocation, "")
        Dim FileOutput As New SaveFileDialog
        FileOutput.Title = "Select Location and Enter Name for Output File"
        FileOutput.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If OutputFileLocation.Text <> "" Then
            FileOutput.InitialDirectory = ThresholdAnalysis_Output
        End If
        If FileOutput.ShowDialog = Windows.Forms.DialogResult.OK Then
            ThresholdAnalysis_Output = FileOutput.FileName
            OutputFileLocation.Text = ThresholdAnalysis_Output
            OutputFileLocation.SelectionStart = OutputFileLocation.Text.Length
        End If
    End Sub

    Private Sub HeaderInfo_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HeaderInfo.TextChanged
        EntryError.SetError(HeaderInfo, "")
        ThresholdAnalysis_Header = HeaderInfo.Text
    End Sub

    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click
        'Check for User Input Errors
        If ThresholdAnalysis_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Exit Sub
        End If
        If DataRowStart.Text = "" Then
            EntryError.SetError(DataRowStart, "Required input data information missing.")
            Exit Sub
        End If
        If DataColumn.Text = "" Then
            EntryError.SetError(DataColumn, "Required input data information missing.")
            Exit Sub
        End If
        If ErrorBars.Checked = True Then
            If ErrorColumn.Text = "" Then
                EntryError.SetError(ErrorColumn, "Required input data information missing.")
                Exit Sub
            End If
        End If
        If ThresholdEntry.Text = "" Then
            EntryError.SetError(ThresholdEntry, "Required threshold information missing.")
            Exit Sub
        End If
        If ComputeMonth.Checked = True Then
            If Month.SelectedIndex = 0 Then
                EntryError.SetError(Month, "Required month not selected.")
                Exit Sub
            End If
        End If
        If ComputeRange.Checked = True Then
            If StartMonth.SelectedIndex = 0 Then
                EntryError.SetError(StartMonth, "Required start month not selected.")
                Exit Sub
            End If
            If EndMonth.SelectedIndex = 0 Then
                EntryError.SetError(EndMonth, "Required end month not selected.")
                Exit Sub
            End If
            If StartMonth.SelectedIndex = EndMonth.SelectedIndex Then
                EntryError.SetError(EndMonth, "Start and end months cannot be the same.")
                Exit Sub
            End If
        End If
        If ThresholdPersistence.Checked = True Then
            If PersistentSpells.Text = "" Then
                EntryError.SetError(PersistentSpells, "Required persistent daily spell information missing.")
                Exit Sub
            End If
        End If
        If ThresholdAnalysis_Input = ThresholdAnalysis_Output Then
            EntryError.SetError(OutputFileLocation, "Input and output datasets cannot be the same.")
            Exit Sub
        End If
        If HeaderInfo.Text = "" Then
            EntryError.SetError(HeaderInfo, "Required output header information missing.")
            Exit Sub
        End If
        'Start Run
        Run.Enabled = False
        Status_Label.Text = "Status:  Running..."
        'Force Reset of Threshold Value
        ThresholdAnalysis_Threshold = ThresholdEntry.Text
        ThresholdAnalysis_Exception = 0
        If DailyButton.Checked = True Then
            ThresholdAnalysis_Daily = 1
        Else
            ThresholdAnalysis_Daily = 0
        End If
        If MonthlyButton.Checked = True Then
            ThresholdAnalysis_Monthly = 1
        Else
            ThresholdAnalysis_Monthly = 0
        End If
        If ErrorBars.Checked = True Then
            ThresholdAnalysis_ErrorBars = 1
        Else
            ThresholdAnalysis_ErrorBars = 0
        End If
        If ThresholdAboveBelow.SelectedIndex = 0 Then
            ThresholdAnalysis_ThresholdAbove = 1
        Else
            ThresholdAnalysis_ThresholdAbove = 0
        End If
        If ThresholdAboveBelow.SelectedIndex = 1 Then
            ThresholdAnalysis_ThresholdBelow = 1
        Else
            ThresholdAnalysis_ThresholdBelow = 0
        End If
        If Percentile.Checked = True Then
            ThresholdAnalysis_Percentile = 1
        Else
            ThresholdAnalysis_Percentile = 0
        End If
        If ComputeMonth.Checked = True Then
            ThresholdAnalysis_ComputeMonth = 1
        Else
            ThresholdAnalysis_ComputeMonth = 0
        End If
        If ComputeRange.Checked = True Then
            ThresholdAnalysis_ComputeRange = 1
        Else
            ThresholdAnalysis_ComputeRange = 0
        End If
        If ThresholdPersistence.Checked = True Then
            ThresholdAnalysis_ThresholdPersistence = 1
        Else
            ThresholdAnalysis_ThresholdPersistence = 0
        End If
        If FirstLastOccurrence.Checked = True Then
            ThresholdAnalysis_FirstLastOccurrence = 1
        Else
            ThresholdAnalysis_FirstLastOccurrence = 0
        End If
        BackgroundRun.RunWorkerAsync()
    End Sub

    Private Sub BackgroundRun_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundRun.DoWork
        'Initialize Variables
        Dim Year As Integer
        Dim Month As Integer
        Dim Flags As Integer
        Dim CriticalValue As Double
        Dim Values(0) As Double
        Dim MainValue As Double
        Dim LowValue As Double
        Dim HighValue As Double
        Dim MonthCheck As Integer
        Dim AtStartMonth As Integer
        Dim StoreOnOff As Integer
        Dim EvenOddCheck As Double
        Dim i As Integer
        Dim TimeThrough As Integer
        Dim ThresholdCount As Integer
        Dim ThresholdCountLowErr As Integer
        Dim ThresholdCountHighErr As Integer
        Dim CurrentDayHit As Integer
        Dim CurrentDayHitLowErr As Integer
        Dim CurrentDayHitHighErr As Integer
        Dim ContDays As Integer
        Dim ContDaysLowErr As Integer
        Dim ContDaysHighErr As Integer
        Dim SpellIndex As Integer
        Dim SpellIndexLowErr As Integer
        Dim SpellIndexHighErr As Integer
        Dim LongestSpell As Integer
        Dim LongestSpellLowErr As Integer
        Dim LongestSpellHighErr As Integer
        Dim LastDate(0) As Date
        Dim LastDateLowErr(0) As Date
        Dim LastDateHighErr(0) As Date
        Dim FirstDate(0) As Date
        Dim FirstDateLowErr(0) As Date
        Dim FirstDateHighErr(0) As Date
        Dim IntervalOnOff As Integer
        Dim IntervalOnOffLowErr As Integer
        Dim IntervalOnOffHighErr As Integer
        Dim IntervalCount As Integer
        Dim IntervalCountLowErr As Integer
        Dim IntervalCountHighErr As Integer
        Dim IntervalDays(0) As Integer
        Dim IntervalDaysLowErr(0) As Integer
        Dim IntervalDaysHighErr(0) As Integer
        Dim LastDay As Integer
        Dim LastDayLowErr As Integer
        Dim LastDayHighErr As Integer
        Dim FirstDay As Integer
        Dim FirstDayLowErr As Integer
        Dim FirstDayHighErr As Integer
        Dim TotalDays As Integer
        Dim TotalDaysLowErr As Integer
        Dim TotalDaysHighErr As Integer
        MonthCheck = 0
        AtStartMonth = 0
        StoreOnOff = 0
        ThresholdCount = 0
        CurrentDayHit = 0
        Flags = 0
        TimeThrough = 0
        'Initialize Output File and Set Header
        FileOpen(1, ThresholdAnalysis_Output, OpenMode.Output)
        PrintLine(1, ThresholdAnalysis_Header)
        PrintLine(1, "")
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(ThresholdAnalysis_Input)
        Dim Line As String
        Dim Fields As String()
        'Initialize Progress Bar
        Dim FileProgress As Integer
        Dim FileComplete As Integer
        Dim Percent As Integer
        FileProgress = 0
        FileComplete = 0
        While InFile.Peek <> -1
            Line = InFile.ReadLine()
            FileComplete = FileComplete + 1
        End While
        'Reset StreamReader
        InFile.DiscardBufferedData()
        InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
        'Set StreamReader to Data Start
        If ThresholdAnalysis_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To ThresholdAnalysis_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
            'Reset Progress Bar Completion
            FileComplete = FileComplete - (ThresholdAnalysis_RowStart - 1)
        End If
        'Skip Column Headers
        Line = InFile.ReadLine()
        FileProgress = FileProgress + 1
        If ThresholdAnalysis_Percentile = 1 Then
            'Calculate Percentile to Find Threshold Critical Value (Any Error Bars Input Not Used to Find Critical Value)
            FileComplete = FileComplete * 2
            If ThresholdAnalysis_ComputeMonth = 1 Then
                'Compute Percentile by Month
                Try
                    While InFile.Peek <> -1
                        Line = InFile.ReadLine()
                        Fields = Line.Split(ControlChars.Tab)
                        Year = Fields(0)
                        Month = Fields(1)
                        'Save Value to Array if Needed
                        If ThresholdAnalysis_Month = Month And Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                            Values(ThresholdCount) = Fields(ThresholdAnalysis_DataColumn - 1)
                            ThresholdCount = ThresholdCount + 1
                            ReDim Preserve Values(ThresholdCount)
                        End If
                        'Update Progress Bar
                        FileProgress = FileProgress + 1
                        Percent = (FileProgress / FileComplete) * 100
                        BackgroundRun.ReportProgress(Percent)
                    End While
                Catch ex As Exception
                    If Year <> 0 Then
                        MsgBox("Error reading the temperature data at or near the following Year/Month/Day:  " & Fields(0) & "/" & Fields(1) & "/" & Fields(2) & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                    Else
                        MsgBox("Error reading the temperature data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                    End If
                    FileClose(1)
                    InFile.Close()
                    ThresholdAnalysis_Exception = 1
                    Exit Sub
                End Try
            End If
            If ThresholdAnalysis_ComputeRange = 1 Then
                'Compute Percentile by Range
                Try
                    While InFile.Peek <> -1
                        Line = InFile.ReadLine()
                        Fields = Line.Split(ControlChars.Tab)
                        Year = Fields(0)
                        Month = Fields(1)
                        If TimeThrough = 0 Then
                            'First Time Through While Loop...Check If Initial Month Is After Start Month but Before or At End Month
                            TimeThrough = 1
                            MonthCheck = ThresholdAnalysis_EndMonth - ThresholdAnalysis_StartMonth
                            Select Case MonthCheck
                                Case Is > 0
                                    If Month > ThresholdAnalysis_StartMonth And Month <= ThresholdAnalysis_EndMonth Then
                                        'Begin Store
                                        StoreOnOff = 1
                                    End If
                                Case Is < 0
                                    If Month > ThresholdAnalysis_StartMonth And Month <= 12 Then
                                        'Begin Store
                                        StoreOnOff = 1
                                    End If
                                    If Month >= 1 And Month <= ThresholdAnalysis_EndMonth Then
                                        'Begin Store
                                        StoreOnOff = 1
                                    End If
                            End Select
                        End If
                        'Check for End Month
                        If ThresholdAnalysis_EndMonth = 12 Then
                            MonthCheck = 1
                        Else
                            MonthCheck = ThresholdAnalysis_EndMonth + 1
                        End If
                        If Month = MonthCheck And StoreOnOff = 1 And AtStartMonth = 0 Then
                            'End Month Reached...End Store
                            StoreOnOff = 0
                        End If
                        'Check for StartMonth
                        If Month = ThresholdAnalysis_StartMonth Then
                            'Start Month Reached...Begin Store
                            StoreOnOff = 1
                            'Force No Save of Start Month
                            AtStartMonth = 1
                        Else
                            AtStartMonth = 0
                        End If
                        'Save Temperature Value to Array if Needed
                        If StoreOnOff = 1 And Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                            Values(ThresholdCount) = Fields(ThresholdAnalysis_DataColumn - 1)
                            ThresholdCount = ThresholdCount + 1
                            ReDim Preserve Values(ThresholdCount)
                        End If
                        'Update Progress Bar
                        FileProgress = FileProgress + 1
                        Percent = (FileProgress / FileComplete) * 100
                        BackgroundRun.ReportProgress(Percent)
                    End While
                Catch ex As Exception
                    If Year <> 0 Then
                        If ThresholdAnalysis_Daily = 1 Then
                            MsgBox("Error reading the temperature data at or near the following Year/Month/Day:  " & Fields(0) & "/" & Fields(1) & "/" & Fields(2) & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                        End If
                        If ThresholdAnalysis_Monthly = 1 Then
                            MsgBox("Error reading the temperature data at or near the following Year/Month:  " & Fields(0) & "/" & Fields(1) & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                        End If
                    Else
                        MsgBox("Error reading the temperature data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                    End If
                    FileClose(1)
                    InFile.Close()
                    ThresholdAnalysis_Exception = 1
                    Exit Sub
                End Try
            End If
            'Sort Values Array
            Array.Sort(Values)
            'Change Entered Whole Percent into Decimal
            ThresholdAnalysis_Threshold = ThresholdAnalysis_Threshold / 100
            ThresholdAnalysis_Threshold = Math.Round(ThresholdAnalysis_Threshold, 2)
            'Calculate Percentile and Set Critical Value
            EvenOddCheck = ThresholdCount Mod (1 / ThresholdAnalysis_Threshold)
            If EvenOddCheck = 0 Then
                'Percentile At i
                i = ThresholdCount * ThresholdAnalysis_Threshold
                CriticalValue = Values(i - 1)
            Else
                'Percentile In Between i and i+1
                i = Math.Truncate(ThresholdCount * ThresholdAnalysis_Threshold)
                CriticalValue = Values(i - 1)
            End If
            'Reset StreamReader
            InFile.DiscardBufferedData()
            InFile.BaseStream.Seek(0, IO.SeekOrigin.Begin)
            'Set StreamReader to Data Start
            If ThresholdAnalysis_RowStart <> 1 Then
                'Header Exists so Move LineInput Down
                Dim Row As Integer
                For Row = 0 To ThresholdAnalysis_RowStart - 2
                    Line = InFile.ReadLine()
                Next Row
            End If
            'Skip Column Headers
            Line = InFile.ReadLine()
            'Reset Variables
            TimeThrough = 0
            MonthCheck = 0
            AtStartMonth = 0
            StoreOnOff = 0
        Else
            'Entered Value is the Threshold Critical Value
            CriticalValue = ThresholdAnalysis_Threshold
        End If
        'Output Critical Threshold Value
        PrintLine(1, "Critical Threshold Value = " & Math.Round(CriticalValue, 1))
        PrintLine(1, "")
        ThresholdCount = 0
        Year = 0
        'Perform Threshold Analysis
        If ThresholdAnalysis_ComputeMonth = 1 Then
            'Count Threshold by Month
            Select Case ThresholdAnalysis_ThresholdPersistence
                Case 0
                    If ThresholdAnalysis_ErrorBars = 1 Then
                        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Days" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                    Else
                        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Days" & vbTab & "Flags")
                    End If
                Case 1
                    If ThresholdAnalysis_ErrorBars = 1 Then
                        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Days" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Cont." & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "LCont." & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                    Else
                        PrintLine(1, "Year" & vbTab & "Month" & vbTab & "Days" & vbTab & "Cont." & vbTab & "LCont." & vbTab & "Flags")
                    End If
            End Select
            Try
                While InFile.Peek <> -1
                    Line = InFile.ReadLine()
                    Fields = Line.Split(ControlChars.Tab)
                    If TimeThrough = 1 Then
                        'Check For Month Change
                        If Month <> Fields(1) Then
                            'New Month...Save Previous Month's Data
                            If ThresholdAnalysis_Month = Month Then
                                Select Case ThresholdAnalysis_ThresholdPersistence
                                    Case 0
                                        If ThresholdAnalysis_ErrorBars = 1 Then
                                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & Flags)
                                        Else
                                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & Flags)
                                        End If
                                    Case 1
                                        If ThresholdAnalysis_ErrorBars = 1 Then
                                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & SpellIndex & vbTab & SpellIndexLowErr & vbTab & SpellIndexHighErr & vbTab & LongestSpell & vbTab & LongestSpellLowErr & vbTab & LongestSpellHighErr & vbTab & Flags)
                                        Else
                                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & SpellIndex & vbTab & LongestSpell & vbTab & Flags)
                                        End If
                                End Select
                            End If
                            'Reset to New Month
                            Year = Fields(0)
                            Month = Fields(1)
                            CurrentDayHit = 0
                            CurrentDayHitLowErr = 0
                            CurrentDayHitHighErr = 0
                            ThresholdCount = 0
                            ThresholdCountLowErr = 0
                            ThresholdCountHighErr = 0
                            SpellIndex = 0
                            SpellIndexLowErr = 0
                            SpellIndexHighErr = 0
                            LongestSpell = 0
                            LongestSpellLowErr = 0
                            LongestSpellHighErr = 0
                            ContDays = 0
                            ContDaysLowErr = 0
                            ContDaysHighErr = 0
                            Flags = 0
                            'Restart Count By Month
                            If ThresholdAnalysis_ThresholdAbove = 1 Then
                                If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                    MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                    If MainValue > CriticalValue Then
                                        CurrentDayHit = 1
                                        ThresholdCount = ThresholdCount + 1
                                    End If
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                            LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            If LowValue > CriticalValue Then
                                                CurrentDayHitLowErr = 1
                                                ThresholdCountLowErr = ThresholdCountLowErr + 1
                                            End If
                                            If HighValue > CriticalValue Then
                                                CurrentDayHitHighErr = 1
                                                ThresholdCountHighErr = ThresholdCountHighErr + 1
                                            End If
                                        Else
                                            Flags = Flags + 1
                                        End If
                                    End If
                                Else
                                    Flags = Flags + 1
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        Flags = Flags + 1
                                    End If
                                End If
                            End If
                            If ThresholdAnalysis_ThresholdBelow = 1 Then
                                If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                    MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                    If MainValue < CriticalValue Then
                                        CurrentDayHit = 1
                                        ThresholdCount = ThresholdCount + 1
                                    End If
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                            LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            If LowValue < CriticalValue Then
                                                CurrentDayHitHighErr = 1
                                                ThresholdCountHighErr = ThresholdCountHighErr + 1
                                            End If
                                            If HighValue < CriticalValue Then
                                                CurrentDayHitLowErr = 1
                                                ThresholdCountLowErr = ThresholdCountLowErr + 1
                                            End If
                                        Else
                                            Flags = Flags + 1
                                        End If
                                    End If
                                Else
                                    Flags = Flags + 1
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        Flags = Flags + 1
                                    End If
                                End If
                            End If
                        Else
                            'Continue Count By Month
                            If ThresholdAnalysis_ThresholdAbove = 1 Then
                                If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                    MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                    If MainValue > CriticalValue Then
                                        CurrentDayHit = 1
                                        ThresholdCount = ThresholdCount + 1
                                    End If
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                            LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            If LowValue > CriticalValue Then
                                                CurrentDayHitLowErr = 1
                                                ThresholdCountLowErr = ThresholdCountLowErr + 1
                                            End If
                                            If HighValue > CriticalValue Then
                                                CurrentDayHitHighErr = 1
                                                ThresholdCountHighErr = ThresholdCountHighErr + 1
                                            End If
                                        Else
                                            Flags = Flags + 1
                                        End If
                                    End If
                                Else
                                    Flags = Flags + 1
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        Flags = Flags + 1
                                    End If
                                End If
                            End If
                            If ThresholdAnalysis_ThresholdBelow = 1 Then
                                If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                    MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                    If MainValue < CriticalValue Then
                                        CurrentDayHit = 1
                                        ThresholdCount = ThresholdCount + 1
                                    End If
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                            LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                            If LowValue < CriticalValue Then
                                                CurrentDayHitHighErr = 1
                                                ThresholdCountHighErr = ThresholdCountHighErr + 1
                                            End If
                                            If HighValue < CriticalValue Then
                                                CurrentDayHitLowErr = 1
                                                ThresholdCountLowErr = ThresholdCountLowErr + 1
                                            End If
                                        Else
                                            Flags = Flags + 1
                                        End If
                                    End If
                                Else
                                    Flags = Flags + 1
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        Flags = Flags + 1
                                    End If
                                End If
                            End If
                        End If
                    End If
                    If TimeThrough = 0 Then
                        'First Time Through While Loop So Set Initial Conditions
                        TimeThrough = 1
                        Year = Fields(0)
                        Month = Fields(1)
                        'Start Count By Month
                        If ThresholdAnalysis_ThresholdAbove = 1 Then
                            If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                If MainValue > CriticalValue Then
                                    CurrentDayHit = 1
                                    ThresholdCount = ThresholdCount + 1
                                End If
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                        LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        If LowValue > CriticalValue Then
                                            CurrentDayHitLowErr = 1
                                            ThresholdCountLowErr = ThresholdCountLowErr + 1
                                        End If
                                        If HighValue > CriticalValue Then
                                            CurrentDayHitHighErr = 1
                                            ThresholdCountHighErr = ThresholdCountHighErr + 1
                                        End If
                                    Else
                                        Flags = Flags + 1
                                    End If
                                End If
                            Else
                                Flags = Flags + 1
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags + 1
                                End If
                            End If
                        End If
                        If ThresholdAnalysis_ThresholdBelow = 1 Then
                            If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                If MainValue < CriticalValue Then
                                    CurrentDayHit = 1
                                    ThresholdCount = ThresholdCount + 1
                                End If
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                        LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        If LowValue < CriticalValue Then
                                            CurrentDayHitHighErr = 1
                                            ThresholdCountHighErr = ThresholdCountHighErr + 1
                                        End If
                                        If HighValue < CriticalValue Then
                                            CurrentDayHitLowErr = 1
                                            ThresholdCountLowErr = ThresholdCountLowErr + 1
                                        End If
                                    Else
                                        Flags = Flags + 1
                                    End If
                                End If
                            Else
                                Flags = Flags + 1
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags + 1
                                End If
                            End If
                        End If
                    End If
                    'Reset ContDays and CurrentDayHit Variables for Next Day
                    If CurrentDayHit = 1 Then
                        ContDays = ContDays + 1
                    Else
                        If ContDays >= ThresholdAnalysis_PersistentSpells Then
                            SpellIndex = SpellIndex + ContDays
                            If ContDays > LongestSpell Then
                                LongestSpell = ContDays
                            End If
                        End If
                        ContDays = 0
                    End If
                    CurrentDayHit = 0
                    If ThresholdAnalysis_ErrorBars = 1 Then
                        If CurrentDayHitLowErr = 1 Then
                            ContDaysLowErr = ContDaysLowErr + 1
                        Else
                            If ContDaysLowErr >= ThresholdAnalysis_PersistentSpells Then
                                SpellIndexLowErr = SpellIndexLowErr + ContDaysLowErr
                                If ContDaysLowErr > LongestSpellLowErr Then
                                    LongestSpellLowErr = ContDaysLowErr
                                End If
                            End If
                            ContDaysLowErr = 0
                        End If
                        CurrentDayHitLowErr = 0
                        If CurrentDayHitHighErr = 1 Then
                            ContDaysHighErr = ContDaysHighErr + 1
                        Else
                            If ContDaysHighErr >= ThresholdAnalysis_PersistentSpells Then
                                SpellIndexHighErr = SpellIndexHighErr + ContDaysHighErr
                                If ContDaysHighErr > LongestSpellHighErr Then
                                    LongestSpellHighErr = ContDaysHighErr
                                End If
                            End If
                            ContDaysHighErr = 0
                        End If
                        CurrentDayHitHighErr = 0
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    MsgBox("Error reading the temperature data at or near the following Year/Month/Day:  " & Fields(0) & "/" & Fields(1) & "/" & Fields(2) & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                Else
                    MsgBox("Error reading the temperature data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                End If
                FileClose(1)
                InFile.Close()
                ThresholdAnalysis_Exception = 1
                Exit Sub
            End Try
            'Finalize Output File
            If ThresholdAnalysis_Month = Month Then
                Select Case ThresholdAnalysis_ThresholdPersistence
                    Case 0
                        If ThresholdAnalysis_ErrorBars = 1 Then
                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & Flags)
                        Else
                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & Flags)
                        End If
                    Case 1
                        If ThresholdAnalysis_ErrorBars = 1 Then
                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & SpellIndex & vbTab & SpellIndexLowErr & vbTab & SpellIndexHighErr & vbTab & LongestSpell & vbTab & LongestSpellLowErr & vbTab & LongestSpellHighErr & vbTab & Flags)
                        Else
                            PrintLine(1, Year & vbTab & Month & vbTab & ThresholdCount & vbTab & SpellIndex & vbTab & LongestSpell & vbTab & Flags)
                        End If
                End Select
            End If
        End If
        If ThresholdAnalysis_ComputeRange = 1 Then
            'Count Threshold by Range
            If ThresholdAnalysis_Daily = 1 Then
                Select Case ThresholdAnalysis_FirstLastOccurrence
                    Case 0
                        Select Case ThresholdAnalysis_ThresholdPersistence
                            Case 0
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    PrintLine(1, "Year" & vbTab & "Days" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                                Else
                                    PrintLine(1, "Year" & vbTab & "Days" & vbTab & "Flags")
                                End If
                            Case 1
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    PrintLine(1, "Year" & vbTab & "Days" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Cont." & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "LCont." & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                                Else
                                    PrintLine(1, "Year" & vbTab & "Days" & vbTab & "Cont." & vbTab & "LCont." & vbTab & "Flags")
                                End If
                        End Select
                    Case 1
                        If ThresholdAnalysis_ErrorBars = 1 Then
                            PrintLine(1, "Year" & vbTab & "Start" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "End" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Days" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                        Else
                            PrintLine(1, "Year" & vbTab & "Start" & vbTab & "End" & vbTab & "Days" & vbTab & "Flags")
                        End If
                End Select
            End If
            If ThresholdAnalysis_Monthly = 1 Then
                If ThresholdAnalysis_ErrorBars = 1 Then
                    PrintLine(1, "Year" & vbTab & "Months" & vbTab & "LowErr" & vbTab & "HighErr" & vbTab & "Flags")
                Else
                    PrintLine(1, "Year" & vbTab & "Months" & vbTab & "Flags")
                End If
            End If
            Try
                While InFile.Peek <> -1
                    Line = InFile.ReadLine()
                    Fields = Line.Split(ControlChars.Tab)
                    Year = Fields(0)
                    Month = Fields(1)
                    If TimeThrough = 0 Then
                        'First Time Through While Loop...Check If Initial Month Is After Start Month but Before or At End Month
                        TimeThrough = 1
                        MonthCheck = ThresholdAnalysis_EndMonth - ThresholdAnalysis_StartMonth
                        Select Case MonthCheck
                            Case Is > 0
                                If Month > ThresholdAnalysis_StartMonth And Month <= ThresholdAnalysis_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                End If
                            Case Is < 0
                                If Month > ThresholdAnalysis_StartMonth And Month <= 12 Then
                                    'Begin Store
                                    StoreOnOff = 1
                                End If
                                If Month >= 1 And Month <= ThresholdAnalysis_EndMonth Then
                                    'Begin Store
                                    StoreOnOff = 1
                                    'Reset Year
                                    Year = Year - 1
                                End If
                        End Select
                        If StoreOnOff = 1 Then
                            'Reset Flags
                            MonthCheck = ThresholdAnalysis_StartMonth
                            If ThresholdAnalysis_Daily = 1 Then
                                Do
                                    If MonthCheck = Month Then
                                        Exit Do
                                    End If
                                    If MonthCheck = 1 Or MonthCheck = 3 Or MonthCheck = 5 Or MonthCheck = 7 Or MonthCheck = 8 Or MonthCheck = 10 Or MonthCheck = 12 Then
                                        Flags = Flags + 31
                                    End If
                                    If MonthCheck = 4 Or MonthCheck = 6 Or MonthCheck = 9 Or MonthCheck = 11 Then
                                        Flags = Flags + 30
                                    End If
                                    If MonthCheck = 2 Then
                                        If Date.IsLeapYear(Year) = True Then
                                            Flags = Flags + 29
                                        Else
                                            Flags = Flags + 28
                                        End If
                                    End If
                                    If MonthCheck = 12 Then
                                        MonthCheck = 1
                                        Year = Year + 1
                                    Else
                                        MonthCheck = MonthCheck + 1
                                    End If
                                Loop
                                'Double Value If Error Bars are Selected
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags * 2
                                End If
                            End If
                            If ThresholdAnalysis_Monthly = 1 Then
                                Do
                                    If MonthCheck = Month Then
                                        Exit Do
                                    Else
                                        Flags = Flags + 1
                                    End If
                                    If MonthCheck = 12 Then
                                        MonthCheck = 1
                                        Year = Year + 1
                                    Else
                                        MonthCheck = MonthCheck + 1
                                    End If
                                Loop
                                'Double Value If Error Bars are Selected
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags * 2
                                End If
                            End If
                        End If
                    End If
                    'Check for End Month
                    If ThresholdAnalysis_EndMonth = 12 Then
                        MonthCheck = 1
                    Else
                        MonthCheck = ThresholdAnalysis_EndMonth + 1
                    End If
                    If Month = MonthCheck And StoreOnOff = 1 And AtStartMonth = 0 Then
                        'End Month Reached...End Store
                        StoreOnOff = 0
                        'Save Data
                        If ThresholdAnalysis_EndMonth = 12 Then
                            'New Year Read So Go Back One for Output
                            Year = Year - 1
                        End If
                        Select Case ThresholdAnalysis_FirstLastOccurrence
                            Case 0
                                'No Interval Output
                                Select Case ThresholdAnalysis_ThresholdPersistence
                                    Case 0
                                        If ThresholdAnalysis_ErrorBars = 1 Then
                                            PrintLine(1, Year & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & Flags)
                                        Else
                                            PrintLine(1, Year & vbTab & ThresholdCount & vbTab & Flags)
                                        End If
                                    Case 1
                                        If ThresholdAnalysis_ErrorBars = 1 Then
                                            PrintLine(1, Year & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & SpellIndex & vbTab & SpellIndexLowErr & vbTab & SpellIndexHighErr & vbTab & LongestSpell & vbTab & LongestSpellLowErr & vbTab & LongestSpellHighErr & vbTab & Flags)
                                        Else
                                            PrintLine(1, Year & vbTab & ThresholdCount & vbTab & SpellIndex & vbTab & LongestSpell & vbTab & Flags)
                                        End If
                                End Select
                            Case 1
                                'Output Largest Interval
                                If IntervalCount = 0 Then
                                    FirstDay = 0
                                    LastDay = 0
                                    TotalDays = 0
                                Else
                                    TotalDays = 0
                                    For i = 0 To IntervalCount - 1
                                        If IntervalDays(i) > TotalDays Then
                                            FirstDay = FirstDate(i).DayOfYear
                                            LastDay = LastDate(i).DayOfYear
                                            TotalDays = IntervalDays(i)
                                        End If
                                    Next i
                                End If
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    If IntervalCountLowErr = 0 Then
                                        FirstDayLowErr = 0
                                        LastDayLowErr = 0
                                        TotalDaysLowErr = 0
                                    Else
                                        TotalDaysLowErr = 0
                                        For i = 0 To IntervalCountLowErr - 1
                                            If IntervalDaysLowErr(i) > TotalDaysLowErr Then
                                                FirstDayLowErr = FirstDateLowErr(i).DayOfYear
                                                LastDayLowErr = LastDateLowErr(i).DayOfYear
                                                TotalDaysLowErr = IntervalDaysLowErr(i)
                                            End If
                                        Next i
                                    End If
                                    If IntervalCountHighErr = 0 Then
                                        FirstDayHighErr = 0
                                        LastDayHighErr = 0
                                        TotalDaysHighErr = 0
                                    Else
                                        TotalDaysHighErr = 0
                                        For i = 0 To IntervalCountHighErr - 1
                                            If IntervalDaysHighErr(i) > TotalDaysHighErr Then
                                                FirstDayHighErr = FirstDateHighErr(i).DayOfYear
                                                LastDayHighErr = LastDateHighErr(i).DayOfYear
                                                TotalDaysHighErr = IntervalDaysHighErr(i)
                                            End If
                                        Next i
                                    End If
                                End If
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    PrintLine(1, Year & vbTab & FirstDay & vbTab & FirstDayLowErr & vbTab & FirstDayHighErr & vbTab & LastDay & vbTab & LastDayLowErr & vbTab & LastDayHighErr & vbTab & TotalDays & vbTab & TotalDaysLowErr & vbTab & TotalDaysHighErr & vbTab & Flags)
                                Else
                                    PrintLine(1, Year & vbTab & FirstDay & vbTab & LastDay & vbTab & TotalDays & vbTab & Flags)
                                End If
                        End Select
                        'Reset for Next Range
                        CurrentDayHit = 0
                        CurrentDayHitLowErr = 0
                        CurrentDayHitHighErr = 0
                        ThresholdCount = 0
                        ThresholdCountLowErr = 0
                        ThresholdCountHighErr = 0
                        SpellIndex = 0
                        SpellIndexLowErr = 0
                        SpellIndexHighErr = 0
                        LongestSpell = 0
                        LongestSpellLowErr = 0
                        LongestSpellHighErr = 0
                        ContDays = 0
                        ContDaysLowErr = 0
                        ContDaysHighErr = 0
                        ReDim FirstDate(0)
                        ReDim FirstDateLowErr(0)
                        ReDim FirstDateHighErr(0)
                        ReDim LastDate(0)
                        ReDim LastDateLowErr(0)
                        ReDim LastDateHighErr(0)
                        ReDim IntervalDays(0)
                        ReDim IntervalDaysLowErr(0)
                        ReDim IntervalDaysHighErr(0)
                        IntervalOnOff = 0
                        IntervalOnOffLowErr = 0
                        IntervalOnOffHighErr = 0
                        IntervalCount = 0
                        IntervalCountLowErr = 0
                        IntervalCountHighErr = 0
                        Flags = 0
                    End If
                    'Check for StartMonth
                    If Month = ThresholdAnalysis_StartMonth Then
                        'Start Month Reached...Begin Store
                        StoreOnOff = 1
                        'Force No Save of Start Month
                        AtStartMonth = 1
                    Else
                        AtStartMonth = 0
                    End If
                    If StoreOnOff = 1 Then
                        If ThresholdAnalysis_ThresholdAbove = 1 Then
                            If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                If MainValue > CriticalValue Then
                                    CurrentDayHit = 1
                                    ThresholdCount = ThresholdCount + 1
                                End If
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                        LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        If LowValue > CriticalValue Then
                                            CurrentDayHitLowErr = 1
                                            ThresholdCountLowErr = ThresholdCountLowErr + 1
                                        End If
                                        If HighValue > CriticalValue Then
                                            CurrentDayHitHighErr = 1
                                            ThresholdCountHighErr = ThresholdCountHighErr + 1
                                        End If
                                    Else
                                        Flags = Flags + 1
                                    End If
                                End If
                            Else
                                Flags = Flags + 1
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags + 1
                                End If
                            End If
                        End If
                        If ThresholdAnalysis_ThresholdBelow = 1 Then
                            If Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                                MainValue = Fields(ThresholdAnalysis_DataColumn - 1)
                                If MainValue < CriticalValue Then
                                    CurrentDayHit = 1
                                    ThresholdCount = ThresholdCount + 1
                                End If
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                        LowValue = MainValue - Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        HighValue = MainValue + Fields(ThresholdAnalysis_ErrorColumn - 1)
                                        If LowValue < CriticalValue Then
                                            CurrentDayHitHighErr = 1
                                            ThresholdCountHighErr = ThresholdCountHighErr + 1
                                        End If
                                        If HighValue < CriticalValue Then
                                            CurrentDayHitLowErr = 1
                                            ThresholdCountLowErr = ThresholdCountLowErr + 1
                                        End If
                                    Else
                                        Flags = Flags + 1
                                    End If
                                End If
                            Else
                                Flags = Flags + 1
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags + 1
                                End If
                            End If
                        End If
                        'Check Interval If Necessary
                        If ThresholdAnalysis_FirstLastOccurrence = 1 And Fields(ThresholdAnalysis_DataColumn - 1) <> -99 Then
                            'Check for Current Day Hit
                            Select Case CurrentDayHit
                                Case 0
                                    If IntervalOnOff = 1 Then
                                        IntervalOnOff = 0
                                        LastDate(IntervalCount) = Fields(1) & "/" & Fields(2) & "/" & Fields(0)
                                        LastDate(IntervalCount) = LastDate(IntervalCount).AddDays(-1)
                                        IntervalDays(IntervalCount) = DateDiff(DateInterval.Day, FirstDate(IntervalCount), LastDate(IntervalCount)) + 1
                                        IntervalCount = IntervalCount + 1
                                        ReDim Preserve FirstDate(IntervalCount)
                                        ReDim Preserve LastDate(IntervalCount)
                                        ReDim Preserve IntervalDays(IntervalCount)
                                    End If
                                Case 1
                                    If IntervalOnOff = 0 Then
                                        IntervalOnOff = 1
                                        FirstDate(IntervalCount) = Fields(1) & "/" & Fields(2) & "/" & Fields(0)
                                    End If
                            End Select
                            If ThresholdAnalysis_ErrorBars = 1 Then
                                If Fields(ThresholdAnalysis_ErrorColumn - 1) <> -99 Then
                                    'Check for Current Day Hit Low Error
                                    Select Case CurrentDayHitLowErr
                                        Case 0
                                            If IntervalOnOffLowErr = 1 Then
                                                IntervalOnOffLowErr = 0
                                                LastDateLowErr(IntervalCountLowErr) = Fields(1) & "/" & Fields(2) & "/" & Fields(0)
                                                LastDateLowErr(IntervalCountLowErr) = LastDateLowErr(IntervalCountLowErr).AddDays(-1)
                                                IntervalDaysLowErr(IntervalCountLowErr) = DateDiff(DateInterval.Day, FirstDateLowErr(IntervalCountLowErr), LastDateLowErr(IntervalCountLowErr)) + 1
                                                IntervalCountLowErr = IntervalCountLowErr + 1
                                                ReDim Preserve FirstDateLowErr(IntervalCountLowErr)
                                                ReDim Preserve LastDateLowErr(IntervalCountLowErr)
                                                ReDim Preserve IntervalDaysLowErr(IntervalCountLowErr)
                                            End If
                                        Case 1
                                            If IntervalOnOffLowErr = 0 Then
                                                IntervalOnOffLowErr = 1
                                                FirstDateLowErr(IntervalCountLowErr) = Fields(1) & "/" & Fields(2) & "/" & Fields(0)
                                            End If
                                    End Select
                                    'Check for Current Day Hit High Error
                                    Select Case CurrentDayHitHighErr
                                        Case 0
                                            If IntervalOnOffHighErr = 1 Then
                                                IntervalOnOffHighErr = 0
                                                LastDateHighErr(IntervalCountHighErr) = Fields(1) & "/" & Fields(2) & "/" & Fields(0)
                                                LastDateHighErr(IntervalCountHighErr) = LastDateHighErr(IntervalCountHighErr).AddDays(-1)
                                                IntervalDaysHighErr(IntervalCountHighErr) = DateDiff(DateInterval.Day, FirstDateHighErr(IntervalCountHighErr), LastDateHighErr(IntervalCountHighErr)) + 1
                                                IntervalCountHighErr = IntervalCountHighErr + 1
                                                ReDim Preserve FirstDateHighErr(IntervalCountHighErr)
                                                ReDim Preserve LastDateHighErr(IntervalCountHighErr)
                                                ReDim Preserve IntervalDaysHighErr(IntervalCountHighErr)
                                            End If
                                        Case 1
                                            If IntervalOnOffHighErr = 0 Then
                                                IntervalOnOffHighErr = 1
                                                FirstDateHighErr(IntervalCountHighErr) = Fields(1) & "/" & Fields(2) & "/" & Fields(0)
                                            End If
                                    End Select
                                End If
                            End If
                        End If
                        'Reset ContDays and CurrentDayHit Variables for Next Day
                        If CurrentDayHit = 1 Then
                            ContDays = ContDays + 1
                        Else
                            If ContDays >= ThresholdAnalysis_PersistentSpells Then
                                SpellIndex = SpellIndex + ContDays
                                If ContDays > LongestSpell Then
                                    LongestSpell = ContDays
                                End If
                            End If
                            ContDays = 0
                        End If
                        CurrentDayHit = 0
                        If ThresholdAnalysis_ErrorBars = 1 Then
                            If CurrentDayHitLowErr = 1 Then
                                ContDaysLowErr = ContDaysLowErr + 1
                            Else
                                If ContDaysLowErr >= ThresholdAnalysis_PersistentSpells Then
                                    SpellIndexLowErr = SpellIndexLowErr + ContDaysLowErr
                                    If ContDaysLowErr > LongestSpellLowErr Then
                                        LongestSpellLowErr = ContDaysLowErr
                                    End If
                                End If
                                ContDaysLowErr = 0
                            End If
                            CurrentDayHitLowErr = 0
                            If CurrentDayHitHighErr = 1 Then
                                ContDaysHighErr = ContDaysHighErr + 1
                            Else
                                If ContDaysHighErr >= ThresholdAnalysis_PersistentSpells Then
                                    SpellIndexHighErr = SpellIndexHighErr + ContDaysHighErr
                                    If ContDaysHighErr > LongestSpellHighErr Then
                                        LongestSpellHighErr = ContDaysHighErr
                                    End If
                                End If
                                ContDaysHighErr = 0
                            End If
                            CurrentDayHitHighErr = 0
                        End If
                    End If
                    'Update Progress Bar
                    FileProgress = FileProgress + 1
                    Percent = (FileProgress / FileComplete) * 100
                    BackgroundRun.ReportProgress(Percent)
                End While
            Catch ex As Exception
                If Year <> 0 Then
                    If ThresholdAnalysis_Daily = 1 Then
                        MsgBox("Error reading the temperature data at or near the following Year/Month/Day:  " & Fields(0) & "/" & Fields(1) & "/" & Fields(2) & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                    End If
                    If ThresholdAnalysis_Monthly = 1 Then
                        MsgBox("Error reading the temperature data at or near the following Year/Month:  " & Fields(0) & "/" & Fields(1) & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                    End If
                Else
                    MsgBox("Error reading the temperature data near the header.  Make sure the row where the data start is correct.", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Threshold Analysis")
                End If
                FileClose(1)
                InFile.Close()
                ThresholdAnalysis_Exception = 1
                Exit Sub
            End Try
            'Finalize Output File
            If StoreOnOff = 1 Then
                'Tabulate Final Flags
                If Month <> ThresholdAnalysis_EndMonth Then
                    If Month = 12 Then
                        MonthCheck = 1
                    Else
                        MonthCheck = Month + 1
                    End If
                    If ThresholdAnalysis_Daily = 1 Then
                        Do
                            If MonthCheck = 1 Then
                                Year = Year + 1
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags + 62
                                Else
                                    Flags = Flags + 31
                                End If
                            End If
                            If MonthCheck = 3 Or MonthCheck = 5 Or MonthCheck = 7 Or MonthCheck = 8 Or MonthCheck = 10 Or MonthCheck = 12 Then
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags + 62
                                Else
                                    Flags = Flags + 31
                                End If
                            End If
                            If MonthCheck = 4 Or MonthCheck = 6 Or MonthCheck = 9 Or MonthCheck = 11 Then
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    Flags = Flags + 60
                                Else
                                    Flags = Flags + 30
                                End If
                            End If
                            If MonthCheck = 2 Then
                                If Date.IsLeapYear(Year) = True Then
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        Flags = Flags + 58
                                    Else
                                        Flags = Flags + 29
                                    End If
                                Else
                                    If ThresholdAnalysis_ErrorBars = 1 Then
                                        Flags = Flags + 56
                                    Else
                                        Flags = Flags + 28
                                    End If
                                End If
                            End If
                            If MonthCheck = ThresholdAnalysis_EndMonth Then
                                Exit Do
                            Else
                                If MonthCheck = 12 Then
                                    MonthCheck = 1
                                Else
                                    MonthCheck = MonthCheck + 1
                                End If
                            End If
                        Loop
                    End If
                    If ThresholdAnalysis_Monthly = 1 Then
                        Do
                            If ThresholdAnalysis_ErrorBars = 1 Then
                                Flags = Flags + 2
                            Else
                                Flags = Flags + 1
                            End If
                            If MonthCheck = ThresholdAnalysis_EndMonth Then
                                Exit Do
                            Else
                                If MonthCheck = 12 Then
                                    MonthCheck = 1
                                Else
                                    MonthCheck = MonthCheck + 1
                                End If
                            End If
                        Loop
                    End If
                End If
                'Save Data
                Select Case ThresholdAnalysis_FirstLastOccurrence
                    Case 0
                        'No Interval Output
                        Select Case ThresholdAnalysis_ThresholdPersistence
                            Case 0
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    PrintLine(1, Year & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & Flags)
                                Else
                                    PrintLine(1, Year & vbTab & ThresholdCount & vbTab & Flags)
                                End If
                            Case 1
                                If ThresholdAnalysis_ErrorBars = 1 Then
                                    PrintLine(1, Year & vbTab & ThresholdCount & vbTab & ThresholdCountLowErr & vbTab & ThresholdCountHighErr & vbTab & SpellIndex & vbTab & SpellIndexLowErr & vbTab & SpellIndexHighErr & vbTab & LongestSpell & vbTab & LongestSpellLowErr & vbTab & LongestSpellHighErr & vbTab & Flags)
                                Else
                                    PrintLine(1, Year & vbTab & ThresholdCount & vbTab & SpellIndex & vbTab & LongestSpell & vbTab & Flags)
                                End If
                        End Select
                    Case 1
                        'Output Largest Interval
                        If IntervalCount = 0 Then
                            FirstDay = 0
                            LastDay = 0
                            TotalDays = 0
                        Else
                            TotalDays = 0
                            For i = 0 To IntervalCount - 1
                                If IntervalDays(i) > TotalDays Then
                                    FirstDay = FirstDate(i).DayOfYear
                                    LastDay = LastDate(i).DayOfYear
                                    TotalDays = IntervalDays(i)
                                End If
                            Next i
                        End If
                        If ThresholdAnalysis_ErrorBars = 1 Then
                            If IntervalCountLowErr = 0 Then
                                FirstDayLowErr = 0
                                LastDayLowErr = 0
                                TotalDaysLowErr = 0
                            Else
                                TotalDaysLowErr = 0
                                For i = 0 To IntervalCountLowErr - 1
                                    If IntervalDaysLowErr(i) > TotalDaysLowErr Then
                                        FirstDayLowErr = FirstDateLowErr(i).DayOfYear
                                        LastDayLowErr = LastDateLowErr(i).DayOfYear
                                        TotalDaysLowErr = IntervalDaysLowErr(i)
                                    End If
                                Next i
                            End If
                            If IntervalCountHighErr = 0 Then
                                FirstDayHighErr = 0
                                LastDayHighErr = 0
                                TotalDaysHighErr = 0
                            Else
                                TotalDaysHighErr = 0
                                For i = 0 To IntervalCountHighErr - 1
                                    If IntervalDaysHighErr(i) > TotalDaysHighErr Then
                                        FirstDayHighErr = FirstDateHighErr(i).DayOfYear
                                        LastDayHighErr = LastDateHighErr(i).DayOfYear
                                        TotalDaysHighErr = IntervalDaysHighErr(i)
                                    End If
                                Next i
                            End If
                        End If
                        If ThresholdAnalysis_ErrorBars = 1 Then
                            PrintLine(1, Year & vbTab & FirstDay & vbTab & FirstDayLowErr & vbTab & FirstDayHighErr & vbTab & LastDay & vbTab & LastDayLowErr & vbTab & LastDayHighErr & vbTab & TotalDays & vbTab & TotalDaysLowErr & vbTab & TotalDaysHighErr & vbTab & Flags)
                        Else
                            PrintLine(1, Year & vbTab & FirstDay & vbTab & LastDay & vbTab & TotalDays & vbTab & Flags)
                        End If
                End Select
            End If
        End If
        'Close Files
        FileClose(1)
        InFile.Close()
    End Sub

    Private Sub BackgroundRun_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundRun.ProgressChanged
        ProgressBar.Value = e.ProgressPercentage
    End Sub

    Private Sub BackgroundRun_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundRun.RunWorkerCompleted
        'End Run
        Select Case ThresholdAnalysis_Exception
            Case 0
                Status_Label.Text = "Status:  Run Completed Successfully"
            Case 1
                Status_Label.Text = "Status:  Run Failed"
        End Select
        Run.Enabled = True
        ProgressBar.Value = 0
    End Sub

    Private Sub ThresholdAnalysis_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        ThresholdAnalysisDisp = 0
    End Sub
End Class