Imports ZedGraph
Imports System.Drawing
Imports System.Runtime.InteropServices
Public Class TimeSeriesPlot
    'HOB Tools.  A collection of quality check, adjustment, reconstruction, and analysis tools for historical weather observations.
    'Copyright (C) 2013  Dorian J. Burnette

    'This file is part of HOB Tools.

    'HOB Tools is free software: you can redistribute it and/or modify
    'it under the terms of the GNU General Public License as published by
    'the Free Software Foundation, either version 3 of the License, or
    '(at your option) any later version.

    'HOB Tools is distributed in the hope that it will be useful,
    'but WITHOUT ANY WARRANTY; without even the implied warranty of
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    'GNU General Public License for more details.

    'You should have received a copy of the GNU General Public License
    'along with HOB Tools.  If not, see <http://www.gnu.org/licenses/>.

    'Initialize Input Variable
    Private TimeSeriesPlot_Input As String
    'Initialize Global Form Variables
    Private TimeSeriesPlot_RowStart As Integer
    Private TimeSeriesPlot_DataColumn As Integer
    Private TimeSeriesPlot_StartYear As Integer
    Private TimeSeriesPlot_EndYear As Integer
    Private TimeSeriesPlot_Title As String
    Private TimeSeriesPlot_YAxis As String
    Private TimeSeriesPlot_ReSizeDraw As Integer

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Create Background Gradient
        MyBase.OnPaint(e)
        Dim baseBackground As System.Drawing.Drawing2D.LinearGradientBrush
        baseBackground = New System.Drawing.Drawing2D.LinearGradientBrush(New Point(0, 0), New Point(ClientSize.Width, ClientSize.Height), Color.White, Color.LightSlateGray)
        e.Graphics.FillRectangle(baseBackground, ClientRectangle)
    End Sub

    Private Sub TimeSeriesPlot_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize Time Series Plot Tool
        HOBToolsHelp.HelpNamespace = HOBToolsUG
    End Sub

    Private Sub FileSelect_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileSelect_Button.Click
        EntryError.SetError(InputFileLocation, "")
        Dim FileSelect As New OpenFileDialog
        FileSelect.Title = "Select Input Data File"
        FileSelect.Filter = "Text Files (*.txt)|*.txt|Data Files (*.dat)|*.dat|All Files (*.*)|*.*"
        If InputFileLocation.Text <> "" Then
            FileSelect.InitialDirectory = TimeSeriesPlot_Input
        End If
        If FileSelect.ShowDialog = Windows.Forms.DialogResult.OK Then
            TimeSeriesPlot_Input = FileSelect.FileName
            InputFileLocation.Text = TimeSeriesPlot_Input
            InputFileLocation.SelectionStart = InputFileLocation.Text.Length
        End If
        TimeSeriesPlot_ReSizeDraw = 1
    End Sub

    Private Sub DataRowStartEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataRowStartEntry.TextChanged
        EntryError.SetError(DataRowStartEntry, "")
        If DataRowStartEntry.Text <> "" Then
            Try
                TimeSeriesPlot_RowStart = DataRowStartEntry.Text
            Catch ex As Exception
                EntryError.SetError(DataRowStartEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub DataColumnEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DataColumnEntry.TextChanged
        EntryError.SetError(DataColumnEntry, "")
        If DataRowStartEntry.Text <> "" Then
            Try
                TimeSeriesPlot_DataColumn = DataColumnEntry.Text
            Catch ex As Exception
                EntryError.SetError(DataColumnEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub StartYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartYearEntry.TextChanged
        EntryError.SetError(StartYearEntry, "")
        If StartYearEntry.Text <> "" Then
            Try
                TimeSeriesPlot_StartYear = StartYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(StartYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub EndYearEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EndYearEntry.TextChanged
        EntryError.SetError(EndYearEntry, "")
        If EndYearEntry.Text <> "" Then
            Try
                TimeSeriesPlot_EndYear = EndYearEntry.Text
            Catch ex As Exception
                EntryError.SetError(EndYearEntry, "Numerical values are the only valid characters.")
            End Try
        End If
    End Sub

    Private Sub TitleEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TitleEntry.TextChanged
        EntryError.SetError(TitleEntry, "")
        TimeSeriesPlot_Title = TitleEntry.Text
    End Sub

    Private Sub YAxisEntry_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles YAxisEntry.TextChanged
        EntryError.SetError(YAxisEntry, "")
        TimeSeriesPlot_YAxis = YAxisEntry.Text
    End Sub

    Private Sub Plot_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Plot.Click
        'Draw New Time Series Plot
        Plot.Text = "Plotting..."
        Plot.Enabled = False
        DrawTimeSeriesPlot()
    End Sub

    Private Sub DrawTimeSeriesPlot()
        'Check for User Input Errors
        If TimeSeriesPlot_Input = "" Then
            EntryError.SetError(InputFileLocation, "Input dataset missing.")
            Plot.Text = "Plot"
            Plot.Enabled = True
            Exit Sub
        End If
        If DataRowStartEntry.Text = "" Then
            EntryError.SetError(DataRowStartEntry, "Required input data information missing.")
            Plot.Text = "Plot"
            Plot.Enabled = True
            Exit Sub
        End If
        If DataColumnEntry.Text = "" Then
            EntryError.SetError(DataColumnEntry, "Required input data information missing.")
            Plot.Text = "Plot"
            Plot.Enabled = True
            Exit Sub
        End If
        If StartYearEntry.Text = "" Then
            EntryError.SetError(StartYearEntry, "Required start year missing.")
            Plot.Text = "Plot"
            Plot.Enabled = True
            Exit Sub
        End If
        If EndYearEntry.Text = "" Then
            EntryError.SetError(EndYearEntry, "Required end year missing.")
            Plot.Text = "Plot"
            Plot.Enabled = True
            Exit Sub
        End If
        If TitleEntry.Text = "" Then
            EntryError.SetError(TitleEntry, "Required title information missing.")
            Plot.Text = "Plot"
            Plot.Enabled = True
            Exit Sub
        End If
        If YAxisEntry.Text = "" Then
            EntryError.SetError(YAxisEntry, "Required y-axis information missing.")
            Plot.Text = "Plot"
            Plot.Enabled = True
            Exit Sub
        End If
        'Initialize Variables
        Dim i As Integer
        Dim Year As Integer
        Dim DataValue As Double
        Dim DataCount As Integer
        Dim TimeSeriesData As New PointPairList
        Dim TimeSeries As LineItem
        Dim TimeSeriesTrendData As New PointPairList
        Dim TimeSeriesTrend As LineItem
        Dim TimeSeriesMeanData As New PointPairList
        Dim TimeSeriesMean As LineItem
        Dim SumXY As Double
        Dim SumX As Double
        Dim SumY As Double
        Dim SumXSq As Double
        Dim SumYSq As Double
        Dim SXY As Double
        Dim SXX As Double
        Dim SSE As Double
        Dim B0 As Double
        Dim B1 As Double
        Dim TestStat As Double
        Dim TDist As New MathNet.Numerics.Distributions.StudentsTDistribution
        Dim p As Double
        Dim StdErr As Double
        Dim TSMean As Double
        Dim TSStdDev As Double
        'Initialize Observation Arrays
        Dim YearArray(0) As Integer
        Dim DataArray(0) As Double
        'Initialize StreamReader
        Dim InFile As New System.IO.StreamReader(TimeSeriesPlot_Input)
        Dim Line As String
        Dim Fields As String()
        'Set StreamReader to Data Start
        If TimeSeriesPlot_RowStart <> 1 Then
            'Header Exists so Move LineInput Down
            Dim Row As Integer
            For Row = 0 To TimeSeriesPlot_RowStart - 2
                Line = InFile.ReadLine()
            Next Row
        End If
        'Skip Column Headers
        Line = InFile.ReadLine()
        'Set Time Series Chart Properties
        Dim TimeSeriesPane As GraphPane = TimeSeriesChart.GraphPane
        TimeSeriesPane.CurveList.Clear()
        TimeSeriesChart.ZoomOutAll(TimeSeriesPane)
        TimeSeriesChart.MasterPane.Border.IsVisible = False
        TimeSeriesPane.Border.IsVisible = False
        TimeSeriesPane.Chart.Border.IsVisible = False
        'Set Title
        TimeSeriesPane.Title.Text = TimeSeriesPlot_Title & Chr(10) & TimeSeriesPlot_StartYear & " - " & TimeSeriesPlot_EndYear
        TimeSeriesPane.Title.FontSpec.Size = 16
        TimeSeriesPane.Title.FontSpec.IsBold = True
        'Set X-Axis
        TimeSeriesPane.XAxis.Scale.Min = TimeSeriesPlot_StartYear
        TimeSeriesPane.XAxis.Scale.Max = TimeSeriesPlot_EndYear
        TimeSeriesPane.XAxis.Scale.FontSpec.Size = 12
        TimeSeriesPane.XAxis.Scale.FontSpec.IsBold = True
        TimeSeriesPane.XAxis.Scale.LabelGap = 0.2
        TimeSeriesPane.XAxis.Title.Text = "Year"
        TimeSeriesPane.XAxis.Title.FontSpec.Size = 14
        TimeSeriesPane.XAxis.Title.FontSpec.IsBold = True
        TimeSeriesPane.XAxis.MajorTic.PenWidth = 2
        TimeSeriesPane.XAxis.MajorTic.IsInside = False
        TimeSeriesPane.XAxis.MajorTic.IsOpposite = False
        TimeSeriesPane.XAxis.MajorTic.Size = 10
        TimeSeriesPane.XAxis.MinorTic.IsInside = False
        TimeSeriesPane.XAxis.MinorTic.IsOpposite = False
        TimeSeriesPane.XAxis.MinorTic.Size = 5
        TimeSeriesPane.X2Axis.MajorTic.IsAllTics = False
        TimeSeriesPane.X2Axis.MinorTic.IsAllTics = False
        'Set Y-Axis
        TimeSeriesPane.YAxis.Scale.FontSpec.Size = 12
        TimeSeriesPane.YAxis.Scale.FontSpec.IsBold = True
        TimeSeriesPane.YAxis.Scale.LabelGap = 0.2
        TimeSeriesPane.YAxis.Title.Text = TimeSeriesPlot_YAxis
        TimeSeriesPane.YAxis.Title.FontSpec.Size = 14
        TimeSeriesPane.YAxis.Title.FontSpec.IsBold = True
        TimeSeriesPane.YAxis.MajorGrid.IsZeroLine = False
        TimeSeriesPane.YAxis.MajorTic.PenWidth = 2
        TimeSeriesPane.YAxis.MajorTic.IsInside = False
        TimeSeriesPane.YAxis.MajorTic.IsOpposite = False
        TimeSeriesPane.YAxis.MajorTic.Size = 10
        TimeSeriesPane.YAxis.MinorTic.IsAllTics = False
        TimeSeriesPane.Y2Axis.MajorTic.IsAllTics = False
        TimeSeriesPane.Y2Axis.MinorTic.IsAllTics = False
        'Import Time Series Data
        Try
            While InFile.Peek <> -1
                'Read Station Precipitation Data
                Line = InFile.ReadLine()
                Fields = Line.Split(ControlChars.Tab)
                Year = Fields(0)
                If Year >= TimeSeriesPlot_StartYear And Year <= TimeSeriesPlot_EndYear Then
                    DataValue = Fields(TimeSeriesPlot_DataColumn - 1)
                    If DataValue <> -99 Then
                        TimeSeriesData.Add(Year, DataValue)
                        ReDim Preserve YearArray(DataCount)
                        ReDim Preserve DataArray(DataCount)
                        YearArray(DataCount) = Year
                        DataArray(DataCount) = DataValue
                        DataCount = DataCount + 1
                    Else
                        TimeSeriesData.Add(Year, PointPair.Missing)
                    End If
                End If
            End While
        Catch ex As Exception
            MsgBox("Error reading the time series data near the following Year:  " & Year & ".", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "HOB Tools - Time Series Plot")
            Plot.Text = "Plot"
            Plot.Enabled = True
            InFile.Close()
            Exit Sub
        End Try
        'Derive Linear Trend Line
        For i = 0 To DataCount - 1
            SumXY = SumXY + YearArray(i) * DataArray(i)
            SumX = SumX + YearArray(i)
            SumY = SumY + DataArray(i)
            SumXSq = SumXSq + YearArray(i) ^ 2
            SumYSq = SumYSq + DataArray(i) ^ 2
        Next i
        SXY = SumXY - SumX * SumY / DataCount
        SXX = SumXSq - SumX ^ 2 / DataCount
        B1 = SXY / SXX
        B0 = (SumY - B1 * SumX) / DataCount
        'Compute Mean and Standard Deviation
        TSMean = SumY / DataCount
        TSStdDev = ((SumYSq - SumY ^ 2 / DataCount) / DataCount) ^ 0.5
        'Compute Standard Error
        SSE = SumYSq - B0 * SumY - B1 * SumXY
        StdErr = ((SSE / (DataCount - 2)) ^ 0.5) / (SXX) ^ 0.5
        'Calculate P-Value (Two-Tailed)
        TestStat = B1 / StdErr
        TDist.SetDistributionParameters(DataCount - 2)
        p = TDist.CumulativeDistribution(TestStat)
        If TestStat > 0 Then
            p = 1 - p
        End If
        p = p * 2
        If PlotTrendLine.Checked = True Then
            'Generate Time Series Trend Plot
            For i = TimeSeriesPlot_StartYear To TimeSeriesPlot_EndYear
                DataValue = i * B1 + B0
                TimeSeriesTrendData.Add(i, DataValue)
            Next i
            TimeSeriesTrend = TimeSeriesPane.AddCurve("Trend", TimeSeriesTrendData, Color.Red, SymbolType.None)
            TimeSeriesTrend.Line.SmoothTension = 0
            TimeSeriesTrend.Line.Width = 3
        End If
        If PlotMeanLine.Checked = True Then
            'Generate Time Series Mean Plot
            For i = TimeSeriesPlot_StartYear To TimeSeriesPlot_EndYear
                TimeSeriesMeanData.Add(i, TSMean)
            Next i
            TimeSeriesMean = TimeSeriesPane.AddCurve("Mean", TimeSeriesMeanData, Color.Black, SymbolType.None)
            TimeSeriesMean.Line.SmoothTension = 0
            TimeSeriesMean.Line.Width = 3
        End If
        'Generate Time Series Plot
        TimeSeries = TimeSeriesPane.AddCurve("Data", TimeSeriesData, Color.Blue, SymbolType.None)
        TimeSeries.Line.SmoothTension = 0
        TimeSeries.Line.Width = 2
        If PlotTrendLine.Checked = True Or PlotMeanLine.Checked = True Then
            'Turn on Legend
            TimeSeriesPane.Legend.IsVisible = True
            TimeSeriesPane.Legend.Border.IsVisible = False
            TimeSeriesPane.Legend.Position = LegendPos.BottomCenter
            TimeSeriesPane.Legend.FontSpec.Size = 10
            TimeSeriesPane.Legend.FontSpec.IsBold = True
        Else
            'Turn off Legend
            TimeSeriesPane.Legend.IsVisible = False
        End If
        'Output Time Series Summary Statistics
        TSMean = Math.Round(TSMean, 2)
        TSStdDev = Math.Round(TSStdDev, 2)
        B1 = Math.Round(B1, 4)
        StdErr = Math.Round(StdErr, 4)
        p = Math.Round(p, 4)
        If p = 0 Then
            PlotStats.Text = "Mean = " & TSMean & vbCrLf & "Standard Deviation = " & TSStdDev & vbCrLf & "Trend per Year = " & B1 & " " & Chr(177) & " " & StdErr & vbCrLf & "Slope Test p < 0.0001"
        Else
            PlotStats.Text = "Mean = " & TSMean & vbCrLf & "Standard Deviation = " & TSStdDev & vbCrLf & "Trend per Year = " & B1 & " " & Chr(177) & " " & StdErr & vbCrLf & "Slope Test p = " & p
        End If
        'Calculate Axis Scale Ranges and Refresh Chart
        TimeSeriesPane.AxisChange()
        TimeSeriesChart.Refresh()
        If TimeSeriesChart.Visible = False Then
            TimeSeriesChart.Visible = True
            CopyBitmap.Enabled = True
            CopyMetafile.Enabled = True
            SaveChart.Enabled = True
            PrintChart.Enabled = True
        End If
        'End Run
        InFile.Close()
        Plot.Text = "Plot"
        Plot.Enabled = True
    End Sub

    Private Sub CopyBitmap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyBitmap.Click
        Clipboard.SetImage(TimeSeriesChart.MasterPane.GetImage(2307, 1500, 288, False))
    End Sub

    Private Sub CopyMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CopyMetafile.Click
        Dim g As Graphics = Me.CreateGraphics()
        Dim hdc As System.IntPtr = g.GetHdc()
        Dim metaFile As New Imaging.Metafile(hdc, Imaging.EmfType.EmfOnly)
        g.ReleaseHdc(hdc)
        g.Dispose()
        Dim gMeta As Graphics = Graphics.FromImage(metaFile)
        TimeSeriesChart.MasterPane.Draw(gMeta)
        gMeta.Dispose()
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, metaFile)
    End Sub

    Private Sub SaveChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveChart.Click
        Dim SaveTimeSeriesPlot As New SaveFileDialog
        Dim OutputImage As String
        SaveTimeSeriesPlot.Title = "Save Time Series Plot"
        SaveTimeSeriesPlot.Filter = "Bitmap (*.bmp)|*.bmp|GIF (*.gif)|*.gif|JPEG (*.jpg)|*.jpg|PNG (*.png)|*.png|TIFF (*.tif)|*.tif"
        If SaveTimeSeriesPlot.ShowDialog = Windows.Forms.DialogResult.OK Then
            OutputImage = SaveTimeSeriesPlot.FileName
            If SaveTimeSeriesPlot.FileName.EndsWith("bmp") Then
                TimeSeriesChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Bmp)
            End If
            If SaveTimeSeriesPlot.FileName.EndsWith("gif") Then
                TimeSeriesChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Gif)
            End If
            If SaveTimeSeriesPlot.FileName.EndsWith("jpg") Then
                TimeSeriesChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Jpeg)
            End If
            If SaveTimeSeriesPlot.FileName.EndsWith("png") Then
                TimeSeriesChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Png)
            End If
            If SaveTimeSeriesPlot.FileName.EndsWith("tif") Then
                TimeSeriesChart.MasterPane.GetImage(2307, 1500, 288, False).Save(OutputImage, System.Drawing.Imaging.ImageFormat.Tiff)
            End If
        End If
    End Sub

    Private Sub PrintChart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PrintChart.Click
        TimeSeriesChart.DoPrint()
    End Sub

    Private Sub TimeSeriesChart_ContextMenuBuilder(ByVal sender As ZedGraph.ZedGraphControl, ByVal menuStrip As System.Windows.Forms.ContextMenuStrip, ByVal mousePt As System.Drawing.Point, ByVal objState As ZedGraph.ZedGraphControl.ContextMenuObjectState) Handles TimeSeriesChart.ContextMenuBuilder
        'Remove Specific Functions from Time Series Chart Context Menu for Use Elsewhere
        Dim Item As ToolStripMenuItem
        Dim CopyItem As ToolStripMenuItem
        Dim PageSetupItem As ToolStripMenuItem
        Dim PrintItem As ToolStripMenuItem
        Dim SaveAsItem As ToolStripMenuItem
        Dim SetScaleItem As ToolStripMenuItem
        For Each Item In menuStrip.Items
            If Item.Tag = "copy" Then
                CopyItem = Item
            End If
            If Item.Tag = "page_setup" Then
                PageSetupItem = Item
            End If
            If Item.Tag = "print" Then
                PrintItem = Item
            End If
            If Item.Tag = "save_as" Then
                SaveAsItem = Item
            End If
            If Item.Tag = "set_default" Then
                SetScaleItem = Item
            End If
        Next Item
        menuStrip.Items.Remove(CopyItem)
        menuStrip.Items.Remove(PageSetupItem)
        menuStrip.Items.Remove(PrintItem)
        menuStrip.Items.Remove(SaveAsItem)
        menuStrip.Items.Remove(SetScaleItem)
    End Sub

    Private Sub TimeSeriesPlot_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        'Allow Tool to be Open Again
        TimeSeriesPlotDisp = 0
        'Free Used Resources
        Me.Dispose()
    End Sub
End Class